SystemOrganization addCategory: #'ST80-Editors'!
SystemOrganization addCategory: #'ST80-Framework'!
SystemOrganization addCategory: #'ST80-Kernel-Remnants'!
SystemOrganization addCategory: #'ST80-Paths'!
SystemOrganization addCategory: #'ST80-Pluggable Views'!
SystemOrganization addCategory: #'ST80-Support'!
SystemOrganization addCategory: #'ST80-Support-Tests'!
SystemOrganization addCategory: #'ST80-Symbols'!
SystemOrganization addCategory: #'ST80-Views'!
SystemOrganization addCategory: #'ST80-Menus'!
SystemOrganization addCategory: #'ST80-Menus-Tests'!

DisplayText subclass: #Paragraph
	instanceVariableNames: 'clippingRectangle compositionRectangle destinationForm rule mask marginTabsLevel lines lastLine'
	classVariableNames: ''
	poolDictionaries: 'TextConstants'
	category: 'ST80-Support'!

!Paragraph commentStamp: '<historical>' prior: 0!
I represent displayable text that has been decoraged with margin alignment, line leading, and tab settings.!

Paragraph subclass: #ListParagraph
	instanceVariableNames: ''
	classVariableNames: 'ListStyle'
	poolDictionaries: ''
	category: 'ST80-Support'!

!ListParagraph commentStamp: '<historical>' prior: 0!
I represent a special type of Paragraph that is used in the list panes of a browser.  I  avoid all the composition done by more general Paragraphs, because I know the structure of my Text.!

!ListParagraph classSide methodsFor: 'initialization' stamp: 'nk 9/1/2004 10:27'!
initialize 
	"ListParagraph initialize"
	| aFont |
	"Allow different line spacing for lists"
	aFont := Preferences standardListFont.
	ListStyle := TextStyle fontArray: { aFont }.
	ListStyle gridForFont: 1 withLead: 1! !

!ListParagraph classSide methodsFor: 'style' stamp: 'sw 12/10/1999 10:43'!
standardListStyle
	^ ListStyle! !

!ListParagraph classSide methodsFor: 'instance creation' stamp: 'jm 9/20/1998 17:10'!
withArray: anArray style: aTextStyleOrNil
	"Convert an array of strings into a ListParagraph using the given TextStyle."

	aTextStyleOrNil
		ifNil: [^ (super withText: Text new style: ListStyle) withArray: anArray]
		ifNotNil: [^ (super withText: Text new style: aTextStyleOrNil) withArray: anArray].
! !

!ListParagraph methodsFor: 'composition' stamp: ''!
composeAll
	"No composition is necessary once the ListParagraph is created."
	
	lastLine isNil ifTrue: [lastLine := 0].	
		"Because composeAll is called once in the process of creating the ListParagraph."
	^compositionRectangle width! !

!ListParagraph methodsFor: 'private' stamp: ''!
trimLinesTo: lastLineInteger
	"Since ListParagraphs are not designed to be changed, we can cut back the
		lines field to lastLineInteger."
	lastLine := lastLineInteger.
	lines := lines copyFrom: 1 to: lastLine! !

!ListParagraph methodsFor: 'private' stamp: 'di 7/13/97 16:56'!
withArray: anArray 
	"Modifies self to contain the list of strings in anArray"
	| startOfLine endOfLine lineIndex aString |
	lines := Array new: 20.
	lastLine := 0.
	startOfLine := 1.
	endOfLine := 1.
	lineIndex := 0.
	anArray do: 
		[:item | 
		endOfLine := startOfLine + item size.		"this computation allows for a cr after each line..."
												"...but later we will adjust for no cr after last line"
		lineIndex := lineIndex + 1.
		self lineAt: lineIndex put:
			((TextLineInterval start: startOfLine stop: endOfLine
				internalSpaces: 0 paddingWidth: 0)
				lineHeight: textStyle lineGrid baseline: textStyle baseline).
		startOfLine := endOfLine + 1].
	endOfLine := endOfLine - 1.		"endOfLine is now the total size of the text"
	self trimLinesTo: lineIndex.
	aString := String new: endOfLine.
	anArray with: lines do: 
		[:item :interval | 
		aString
			replaceFrom: interval first
			to: interval last - 1
			with: item asString
			startingAt: 1.
		interval last <= endOfLine ifTrue: [aString at: interval last put: Character cr]].
	lineIndex > 0 ifTrue: [(lines at: lineIndex) stop: endOfLine].	"adjust for no cr after last line"
	self text: aString asText.
	anArray with: lines do: 
		[:item :interval |  item isText ifTrue:
			[text replaceFrom: interval first to: interval last - 1 with: item]].
	self updateCompositionHeight! !

!Paragraph class methodsFor: 'examples' stamp: 'tk 9/30/96'!
example
	"This simple example illustrates how to display a few lines of text on the screen at the current cursor point.  
	Fixed. "

	| para point |
	point := Sensor waitButton.
	para := 'This is the first line of characters
and this is the second line.' asParagraph.
	para displayOn: Display at: point.

	"Paragraph example"! !

!Paragraph class methodsFor: 'instance creation' stamp: ''!
new
	"Do not allow an uninitialized view. Create with text that has no
	characters."

	^self withText: '' asText! !

!Paragraph class methodsFor: 'instance creation' stamp: ''!
withText: aText 
	"Answer an instance of me with text set to aText and style set to the 
	system's default text style."

	^self withText: aText style: DefaultTextStyle copy! !

!Paragraph class methodsFor: 'instance creation' stamp: ''!
withText: aText style: aTextStyle 
	"Answer an instance of me with text set to aText and style set to 
	aTextStyle."

	^super new setWithText: aText style: aTextStyle! !

!Paragraph class methodsFor: 'instance creation' stamp: ''!
withText: aText style: aTextStyle compositionRectangle: compRect clippingRectangle: clipRect foreColor: c1 backColor: c2
	"Answer an instance of me with text set to aText and style set to 
	aTextStyle, composition rectangle is compRect and the clipping rectangle 
	is clipRect."
	| para |
	para := super new.
	para setWithText: aText
		style: aTextStyle
		compositionRectangle: compRect
		clippingRectangle: clipRect
		foreColor: c1 backColor: c2.
	^para! !

!Paragraph methodsFor: 'converting' stamp: 'yo 6/23/2003 19:05'!
asForm
	"Answer a Form made up of the bits that represent the receiver's displayable text."
	| theForm oldBackColor oldForeColor |
	textStyle isTTCStyle ifTrue: [
		theForm :=  (Form extent: compositionRectangle extent depth: 32)
		offset: offset.
	] ifFalse: [
		theForm := (ColorForm extent: compositionRectangle extent)
			offset: offset;
			colors: (Array
				with: (backColor == nil ifTrue: [Color transparent] ifFalse: [backColor])
				with: (foreColor == nil ifTrue: [Color black] ifFalse: [foreColor])).
	].
	oldBackColor := backColor.
	oldForeColor := foreColor.
	backColor := Color white.
	foreColor := Color black.
	self displayOn: theForm
		at: 0@0
		clippingBox: theForm boundingBox
		rule: Form over
		fillColor: nil.
	backColor := oldBackColor.
	foreColor := oldForeColor.
	^ theForm

"Example:
| p |
p := 'Abc' asParagraph.
p foregroundColor: Color red backgroundColor: Color black.
p asForm displayOn: Display at: 30@30 rule: Form over"
! !

!Paragraph methodsFor: 'converting' stamp: ''!
asString
	"Answer the string of characters of the receiver's text."

	^text string! !

!Paragraph methodsFor: 'converting' stamp: ''!
asText
	"Answer the receiver's text."

	^text! !

!Paragraph methodsFor: 'accessing' stamp: ''!
backgroundColor
	backColor == nil ifTrue: [^ Color white].
	^ backColor! !

!Paragraph methodsFor: 'private' stamp: ''!
bottomAtLineIndex: lineIndex 
	"Answer the bottom y of given line."
	| y |
	y := compositionRectangle top.
	lastLine = 0 ifTrue: [^ y + textStyle lineGrid].
	1 to: (lineIndex min: lastLine) do:
		[:i | y := y + (lines at: i) lineHeight].
	^ y
! !

!Paragraph methodsFor: 'display box access' stamp: ''!
boundingBox

	^offset extent: compositionRectangle extent! !

!Paragraph methodsFor: 'selecting' stamp: 'ar 5/28/2000 12:10'!
caretFormForDepth: depth
	"Return a caret form for the given depth."
	"(Paragraph new caretFormForDepth: Display depth) displayOn: Display at: 0@0 rule: Form reverse"

	| box f bb map |
	box := CaretForm boundingBox.
	f := Form extent: box extent depth: depth.
	map := (Color cachedColormapFrom: CaretForm depth to: depth) copy.
	map at: 1 put: (Color transparent pixelValueForDepth: depth).
	map at: 2 put: (Color quickHighLight: depth) first.  "pixel value for reversing"
	bb := BitBlt current toForm: f.
	bb
		sourceForm: CaretForm;
		sourceRect: box;
		destOrigin: 0@0;
		colorMap: map;
 		combinationRule: Form over;
		copyBits.
	^ f! !

!Paragraph methodsFor: 'alignment' stamp: ''!
centered 
	"Set the alignment for the style with which the receiver displays its text 
	so that text is centered in the composition rectangle."

	textStyle alignment: Centered! !

!Paragraph methodsFor: 'character location' stamp: 'ar 5/18/2000 18:33'!
characterBlockAtPoint: aPoint 
	"Answer a CharacterBlock for characters in the text at point aPoint. It is 
	assumed that aPoint has been transformed into coordinates appropriate to 
	the receiver's destinationForm rectangle and the compositionRectangle."

	^CharacterBlockScanner new characterBlockAtPoint: aPoint in: self! !

!Paragraph methodsFor: 'character location' stamp: 'ar 5/18/2000 18:33'!
characterBlockForIndex: targetIndex 
	"Answer a CharacterBlock for character in the text at targetIndex. The 
	coordinates in the CharacterBlock will be appropriate to the intersection 
	of the destinationForm rectangle and the compositionRectangle."

	^CharacterBlockScanner new characterBlockForIndex: targetIndex in: self! !

!Paragraph methodsFor: 'utilities' stamp: ''!
clearVisibleRectangle 
	"Display the area in which the receiver presents its text so that the area 
	is all one tone--in this case, all white."

	destinationForm
	  fill: clippingRectangle
	  rule: rule
	  fillColor: self backgroundColor! !

!Paragraph methodsFor: 'selecting' stamp: 'dvf 10/1/2003 13:28'!
clickAt: clickPoint for: model controller: aController
	"Give sensitive text a chance to fire.  Display flash: (100@100 extent: 100@100)."
	| startBlock action range box boxes |
	action := false.
	startBlock := self characterBlockAtPoint: clickPoint.
	(text attributesAt: startBlock stringIndex forStyle: textStyle) 
		do: [:att | att mayActOnClick ifTrue:
				[range := text rangeOf: att startingAt: startBlock stringIndex.
				boxes := self selectionRectsFrom: (self characterBlockForIndex: range first) 
							to: (self characterBlockForIndex: range last+1).
				box := boxes detect: [:each | each containsPoint: clickPoint]
							ifNone: [^ action].
				Utilities awaitMouseUpIn: box repeating: []
					ifSucceed: [aController terminateAndInitializeAround:
								[(att actOnClickFor: model in: self at: clickPoint editor: aController) ifTrue: [action := true]]]]].
	^ action! !

!Paragraph methodsFor: 'accessing' stamp: ''!
clippingRectangle 
	"Answer the rectangle, defined in absolute coordinates, whose 
	intersection with the destinationForm is the area in which the characters 
	are constrained to display."

	^clippingRectangle! !

!Paragraph methodsFor: 'accessing' stamp: 'di 10/5/97 15:33'!
clippingRectangle: clipRect 
	clippingRectangle := clipRect! !

!Paragraph methodsFor: 'composition' stamp: 'yo 1/23/2003 22:47'!
composeAll
	"Compose a collection of characters into a collection of lines."

	| startIndex stopIndex lineIndex maximumRightX compositionScanner |
	lines := Array new: 32.
	lastLine := 0.
	maximumRightX := 0.
	text size = 0
		ifTrue:
			[compositionRectangle := compositionRectangle withHeight: 0.
			^maximumRightX].
	startIndex := lineIndex := 1.
	stopIndex := text size.
	compositionScanner := MultiCompositionScanner new forParagraph: self.
	[startIndex > stopIndex] whileFalse: 
		[self lineAt: lineIndex 
				put: (compositionScanner composeLine: lineIndex 
										fromCharacterIndex: startIndex 
										inParagraph: self).
		 maximumRightX := compositionScanner rightX max: maximumRightX.
		 startIndex := (lines at: lineIndex) last + 1.
		 lineIndex := lineIndex + 1].
	self updateCompositionHeight.
	self trimLinesTo: lineIndex - 1.
	^ maximumRightX! !

!Paragraph methodsFor: 'accessing' stamp: ''!
compositionRectangle
	"Answer the rectangle whose width is the dimension, modified by 
	indents and tabsLevels, against which line wraparound is measured. The 
	height of the compositionRectangle is reset each time recomposition is 
	required."

	^compositionRectangle! !

!Paragraph methodsFor: 'accessing' stamp: ''!
compositionRectangle: compRectangle 
	"Set the rectangle whose width is the dimension, modified by indents and 
	tabsLevels, against which line wraparound is measured."

	compositionRectangle := compRectangle.
	self composeAll! !

!Paragraph methodsFor: 'private' stamp: 'tk 9/30/96'!
compositionRectangle: compositionRect text: aText style: aTextStyle offset: aPoint

	compositionRectangle := compositionRect copy.
	text := aText.
	textStyle := aTextStyle.
	rule := DefaultRule.
	mask := nil.		"was DefaultMask "
	marginTabsLevel := 0.
	destinationForm := Display.
	offset := aPoint.
	^self composeAll! !

!Paragraph methodsFor: 'private' stamp: ''!
compositionRectangleDelta
	"A handy number -- mostly for scrolling."

	^compositionRectangle top - clippingRectangle top! !

!Paragraph methodsFor: 'display box access' stamp: ''!
computeBoundingBox

	^offset extent: compositionRectangle extent! !

!Paragraph methodsFor: 'utilities' stamp: ''!
deepCopy
	"Don't want to copy the destForm (Display) or fonts in the TextStyle.  9/13/96 tk"

	| new |
	new := self copy.
	new textStyle: textStyle copy.
	new destinationForm: destinationForm.
	new lines: lines copy.
	new text: text deepCopy.
	^ new! !

!Paragraph methodsFor: 'character location' stamp: 'di 10/5/1998 12:59'!
defaultCharacterBlock
	^ CharacterBlock new stringIndex: 1 text: text
			topLeft: compositionRectangle topLeft extent: 0 @ 0! !

!Paragraph methodsFor: 'accessing' stamp: ''!
destinationForm 
	 "Answer the Form into which the characters are scanned."

	^destinationForm! !

!Paragraph methodsFor: 'utilities' stamp: ''!
destinationForm: destForm
	destinationForm := destForm! !

!Paragraph methodsFor: 'private' stamp: ''!
displayLines: linesInterval 
	^ self displayLines: linesInterval
		affectedRectangle: self visibleRectangle! !

!Paragraph methodsFor: 'private' stamp: 'yo 1/23/2003 22:48'!
displayLines: linesInterval affectedRectangle: affectedRectangle
	"This is the first level workhorse in the display portion of the TextForm routines.
	It checks to see which lines in the interval are actually visible, has the
	CharacterScanner display only those, clears out the areas in which display will
	occur, and clears any space remaining in the visibleRectangle following the space
	occupied by lastLine."

	| lineGrid topY firstLineIndex lastLineIndex lastLineIndexBottom |

	"Save some time by only displaying visible lines"
	firstLineIndex := self lineIndexOfTop: affectedRectangle top.
	firstLineIndex < linesInterval first ifTrue: [firstLineIndex := linesInterval first].
	lastLineIndex := self lineIndexOfTop: affectedRectangle bottom - 1.
	lastLineIndex > linesInterval last ifTrue:
			[linesInterval last > lastLine
		 		ifTrue: [lastLineIndex := lastLine]
		  		ifFalse: [lastLineIndex := linesInterval last]].
	lastLineIndexBottom := (self bottomAtLineIndex: lastLineIndex).
	((Rectangle 
		origin: affectedRectangle left @ (topY := self topAtLineIndex: firstLineIndex) 
		corner: affectedRectangle right @ lastLineIndexBottom)
	  intersects: affectedRectangle)
		ifTrue: [ " . . . (skip to clear-below if no lines displayed)"
				MultiDisplayScanner new
					displayLines: (firstLineIndex to: lastLineIndex)
					in: self clippedBy: affectedRectangle].
	lastLineIndex = lastLine ifTrue: 
		 [destinationForm  "Clear out white space below last line"
		 	fill: (affectedRectangle left @ (lastLineIndexBottom max: affectedRectangle top)
				corner: affectedRectangle bottomRight)
		 	rule: rule fillColor: self backgroundColor]! !

!Paragraph methodsFor: 'displaying' stamp: ''!
displayOn: aDisplayMedium
	"Because Paragraphs cache so much information, computation is avoided
	and displayAt: 0@0 is not appropriate here."

	self displayOn: aDisplayMedium
		at: compositionRectangle topLeft
		clippingBox: clippingRectangle
		rule: rule
		fillColor: mask! !

!Paragraph methodsFor: 'displaying' stamp: ''!
displayOn: aDisplayMedium at: aPoint
	"Use internal clippingRect; destination cliping is done during actual display."

	self displayOn: aDisplayMedium at: aPoint
		clippingBox: (clippingRectangle translateBy: aPoint - compositionRectangle topLeft)
		rule: rule fillColor: mask! !

!Paragraph methodsFor: 'displaying' stamp: ''!
displayOn: aDisplayMedium at: aDisplayPoint clippingBox: clipRectangle rule: ruleInteger fillColor: aForm
	"Default display message when aDisplayPoint is in absolute screen
	coordinates."

	rule := ruleInteger.
	mask := aForm.
	clippingRectangle := clipRectangle.
	compositionRectangle := aDisplayPoint extent: compositionRectangle extent.
	(lastLine == nil or: [lastLine < 1]) ifTrue: [self composeAll].
	self displayOn: aDisplayMedium lines: (1 to: lastLine)! !

!Paragraph methodsFor: 'private' stamp: ''!
displayOn: aDisplayMedium lines: lineInterval

	| saveDestinationForm |
	saveDestinationForm := destinationForm.
	destinationForm := aDisplayMedium.
	self displayLines: lineInterval.
	destinationForm := saveDestinationForm! !

!Paragraph methodsFor: 'displaying' stamp: ''!
displayOn: aDisplayMedium transformation: displayTransformation clippingBox: clipRectangle align: alignmentPoint with: relativePoint rule: ruleInteger fillColor: aForm 

	self				"Assumes offset has been set!!!!!!!!!!"
	  displayOn: aDisplayMedium
	  at: (offset 
			+ (displayTransformation applyTo: relativePoint) 
			- alignmentPoint) rounded
	  clippingBox: clipRectangle
	  rule: ruleInteger
	  fillColor: aForm.
	! !

!Paragraph methodsFor: 'selecting' stamp: ''!
extendSelectionAt: beginBlock endBlock: endBlock 
	"Answer with an Array of two CharacterBlocks that represent the text 
	selection that the user makes."
	
	(self characterBlockAtPoint: Sensor cursorPoint) <= beginBlock
		ifTrue: [^self mouseMovedFrom: beginBlock 
					pivotBlock: endBlock
					showingCaret: (beginBlock = endBlock)]
		ifFalse: [^self mouseMovedFrom: endBlock 
					pivotBlock: beginBlock
					showingCaret: (beginBlock = endBlock)]
! !

!Paragraph methodsFor: 'selecting' stamp: 'th 9/19/2002 17:27'!
extendSelectionMark: markBlock pointBlock: pointBlock 
	"Answer with an Array of two CharacterBlocks that represent the text 
	selection that the user makes."
	true 
		ifTrue:[^self mouseMovedFrom: pointBlock
					pivotBlock: markBlock
					showingCaret:(pointBlock = markBlock)]
		ifFalse:
		[	| beginBlock endBlock |
			beginBlock := markBlock min: pointBlock.
			endBlock := markBlock max: endBlock.
	
			(self characterBlockAtPoint: Sensor cursorPoint) <= beginBlock
				ifTrue: [^self mouseMovedFrom: beginBlock 
							pivotBlock: endBlock
							showingCaret: (beginBlock = endBlock)]
				ifFalse: [^self mouseMovedFrom: endBlock 
							pivotBlock: beginBlock
							showingCaret: (beginBlock = endBlock)]
		]
! !

!Paragraph methodsFor: 'accessing' stamp: ''!
fillColor 
	"Answer the Form with which each character is combined by the scanner 
	before applying the rule for display."

	^mask! !

!Paragraph methodsFor: 'accessing' stamp: ''!
fillColor: maskForm 
	"Set the argument, maskForm, to be the form with which each character 
	is combined by the scanner before applying the rule for display."

	mask := maskForm! !

!Paragraph methodsFor: 'utilities' stamp: ''!
fit
	"Make the bounding rectangle of the receiver contain all the text without 
	changing the width of the receiver's composition rectangle."

	[(self lineIndexOfTop: clippingRectangle top) = 1]
		whileFalse: [self scrollBy: (0-1)*textStyle lineGrid].
	self updateCompositionHeight.
	clippingRectangle := clippingRectangle withBottom: compositionRectangle bottom! !

!Paragraph methodsFor: 'indicating' stamp: ''!
flash 
	"Complement twice the visible area in which the receiver displays."

	Display flash: clippingRectangle! !

!Paragraph methodsFor: 'accessing' stamp: ''!
height 
	"Answer the height of the composition rectangle."

	^compositionRectangle height! !

!Paragraph methodsFor: 'selecting' stamp: 'jm 7/1/1999 12:31'!
hiliteRect: rect

	| highlightColor |
	highlightColor := Color quickHighLight: destinationForm depth.
	rect ifNotNil: [
		destinationForm
			fill: rect
			rule: Form reverse
			fillColor: highlightColor.
		"destinationForm
			fill: (rect translateBy: 1@1)
			rule: Form reverse
			fillColor: highlightColor" ].
! !

!Paragraph methodsFor: 'accessing' stamp: ''!
indentationOfLineIndex: lineIndex ifBlank: aBlock
	"Answer the number of leading tabs in the line at lineIndex.  If there are
	 no visible characters, pass the number of tabs to aBlock and return its value.
	 If the line is word-wrap overflow, back up a line and recur."

	| arrayIndex first last reader leadingTabs lastSeparator cr tab ch |
	cr := Character cr.
	tab := Character tab.
	arrayIndex := lineIndex.
	[first := (lines at: arrayIndex) first.
	 first > 1 and: [(text string at: first - 1) ~~ cr]] whileTrue: "word wrap"
		[arrayIndex := arrayIndex - 1].
	last := (lines at: lastLine) last.
	reader := ReadStream on: text string from: first to: last.
	leadingTabs := 0.
	[reader atEnd not and: [(ch := reader next) == tab]]
		whileTrue: [leadingTabs := leadingTabs + 1].
	lastSeparator := first - 1 + leadingTabs.
	[reader atEnd not and: [ch isSeparator and: [ch ~~ cr]]]
		whileTrue: [lastSeparator := lastSeparator + 1. ch := reader next].
	lastSeparator = last | (ch == cr)
		ifTrue: [^aBlock value: leadingTabs].
	^leadingTabs! !

!Paragraph methodsFor: 'alignment' stamp: ''!
justified 
	"Set the alignment for the style with which the receiver displays its text 
	so that the characters in each of text end on an even border in the 
	composition rectangle."

	textStyle alignment: Justified! !

!Paragraph methodsFor: 'alignment' stamp: ''!
leftFlush 
	"Set the alignment for the style with which the receiver displays its text 
	so that the characters in each of text begin on an even border in the 
	composition rectangle. This is also known as ragged-right."

	textStyle alignment: LeftFlush! !

!Paragraph methodsFor: 'private' stamp: ''!
leftMarginForCompositionForLine: lineIndex 
	"Build the left margin for composition of a line. Depends upon
	marginTabsLevel and the indent."

	| indent |
	lineIndex = 1
		ifTrue: [indent := textStyle firstIndent]
		ifFalse: [indent := textStyle restIndent].
	^indent + (textStyle leftMarginTabAt: marginTabsLevel)! !

!Paragraph methodsFor: 'private' stamp: 'ar 12/15/2001 23:29'!
leftMarginForDisplayForLine: lineIndex alignment: alignment
	"Build the left margin for display of a line. Depends upon
	leftMarginForComposition, compositionRectangle left and the alignment."

	| pad |
	(alignment = LeftFlush or: [alignment = Justified])
		ifTrue: 
			[^compositionRectangle left 
				+ (self leftMarginForCompositionForLine: lineIndex)].
	"When called from character location code and entire string has been cut,
	there are no valid lines, hence following nil check."
	(lineIndex <= lines size and: [(lines at: lineIndex) notNil])
		ifTrue: 
			[pad := (lines at: lineIndex) paddingWidth]
		ifFalse: 
			[pad := 
				compositionRectangle width - textStyle firstIndent - textStyle rightIndent].
	alignment = Centered 
		ifTrue: 
			[^compositionRectangle left 
				+ (self leftMarginForCompositionForLine: lineIndex) + (pad // 2)].
	alignment = RightFlush 
		ifTrue:
			[^compositionRectangle left 
				+ (self leftMarginForCompositionForLine: lineIndex) + pad].
	self error: ['no such alignment']! !

!Paragraph methodsFor: 'private' stamp: ''!
lineAt: indexInteger put: aTextLineInterval 
	"Store a line, track last, and grow lines if necessary."
	indexInteger > lastLine ifTrue: [lastLine := indexInteger].
	lastLine > lines size ifTrue: [lines := lines , (Array new: lines size)].
	^lines at: indexInteger put: aTextLineInterval! !

!Paragraph methodsFor: 'private' stamp: ''!
lineIndexOfCharacterIndex: characterIndex 
	"Answer the line index for a given characterIndex."

	1 to: lastLine do: 
		[:lineIndex | 
		(lines at: lineIndex) last >= characterIndex ifTrue: [^lineIndex]].
	^lastLine! !

!Paragraph methodsFor: 'private' stamp: ''!
lineIndexOfTop: top 
	"Answer the line index at a given top y."
	| y line |
	lastLine = 0 ifTrue: [^ 1].
	y := compositionRectangle top.
	1 to: lastLine do:
		[:i | line := lines at: i.
		(y := y + line lineHeight) > top ifTrue: [^ i]].
	^ lastLine
! !

!Paragraph methodsFor: 'private' stamp: ''!
lines

	^lines! !

!Paragraph methodsFor: 'utilities' stamp: ''!
lines: lineArray
	lines := lineArray! !

!Paragraph methodsFor: 'accessing' stamp: ''!
mask 
	"Answer the Form with which each character is combined by the scanner 
	before applying the rule for display."

	^mask! !

!Paragraph methodsFor: 'selecting' stamp: 'jm 7/8/97 12:25'!
mouseMovedFrom: beginBlock pivotBlock: pivotBlock showingCaret: caretOn 
	| startBlock stopBlock showingCaret |
	stopBlock := startBlock := beginBlock.
	showingCaret := caretOn.
	[Sensor redButtonPressed]
		whileTrue: 
			[stopBlock := self characterBlockAtPoint: Sensor cursorPoint.
			stopBlock = startBlock
				ifFalse: 
					[showingCaret
						ifTrue: 
							[showingCaret := false.
							self reverseFrom: pivotBlock to: pivotBlock].
			((startBlock >= pivotBlock and: [stopBlock >= pivotBlock])
				or: [startBlock <= pivotBlock and: [stopBlock <= pivotBlock]])
				ifTrue: 
					[self reverseFrom: startBlock to: stopBlock.
					startBlock := stopBlock]
				ifFalse: 
					[self reverseFrom: startBlock to: pivotBlock.
					self reverseFrom: pivotBlock to: stopBlock.
					startBlock := stopBlock].
			(clippingRectangle containsRect: stopBlock) ifFalse:
				[stopBlock top < clippingRectangle top
				ifTrue: [self scrollBy: stopBlock top - clippingRectangle top
						withSelectionFrom: pivotBlock to: stopBlock]
				ifFalse: [self scrollBy: stopBlock bottom + textStyle lineGrid - clippingRectangle bottom
						withSelectionFrom: pivotBlock to: stopBlock]]]].
	pivotBlock = stopBlock ifTrue:
		[showingCaret ifFalse:  "restore caret"
			[self reverseFrom: pivotBlock to: pivotBlock]].
	^ Array with: pivotBlock with: stopBlock! !

!Paragraph methodsFor: 'selecting' stamp: ''!
mouseSelect
	"Answer with an Array of two CharacterBlocks that represent the text 
	selection that the user makes.  Return quickly if the button is noticed up
	to make double-click more responsive."

	| pivotBlock startBlock stopBlock origPoint stillDown |
	stillDown := Sensor redButtonPressed.
	pivotBlock := startBlock := stopBlock :=
		self characterBlockAtPoint: (origPoint := Sensor cursorPoint).
	stillDown := stillDown and: [Sensor redButtonPressed].
	self reverseFrom: startBlock to: startBlock.
	[stillDown and: [Sensor cursorPoint = origPoint]] whileTrue:
		[stillDown := Sensor redButtonPressed].
	(stillDown and: [clippingRectangle containsPoint: Sensor cursorPoint])
		ifFalse: [^Array with: pivotBlock with: stopBlock].
	^ self mouseMovedFrom: startBlock 
		pivotBlock: pivotBlock
		showingCaret: true! !

!Paragraph methodsFor: 'selecting' stamp: ''!
mouseSelect: clickPoint 
	"Track text selection and answer with an Array of two CharacterBlocks."
	| startBlock |
	startBlock := self characterBlockAtPoint: clickPoint.
	self reverseFrom: startBlock to: startBlock.
	^ self mouseMovedFrom: startBlock 
		pivotBlock: startBlock
		showingCaret: true! !

!Paragraph methodsFor: 'private' stamp: ''!
moveBy: delta
	compositionRectangle := compositionRectangle translateBy: delta.
	clippingRectangle := clippingRectangle translateBy: delta.
! !

!Paragraph methodsFor: 'accessing' stamp: ''!
numberOfLines 
	"Answer the number of lines of text in the receiver."

	^lastLine! !

!Paragraph methodsFor: 'indicating' stamp: ''!
outline 
	"Display a border around the visible area in which the receiver presents 
	its text."

	clippingRectangle bottom <= compositionRectangle bottom
	  ifTrue: [Display 
				border: (clippingRectangle intersect: compositionRectangle) 
				width: 2]
	  ifFalse: [Display 
				border: (clippingRectangle intersect: destinationForm boundingBox)
				width: 2].
	! !

!Paragraph methodsFor: 'accessing' stamp: 'ar 5/18/2000 18:34'!
replaceFrom: start to: stop with: aText displaying: displayBoolean
	"Replace the receiver's text starting at position start, stopping at stop, by 
	the characters in aText. It is expected that most requirements for 
	modifications to the receiver will call this code. Certainly all cut's or 
	paste's." 

	| compositionScanner obsoleteLines obsoleteLastLine firstLineIndex lastLineIndex
	startLine stopLine replacementRange visibleRectangle startIndex newLine done
	newStop obsoleteY newY moveRectangle |

	text replaceFrom: start to: stop with: aText.		"Update the text."
	lastLine = 0 ifTrue:
		["if lines have never been set up, measure them and display
		all the lines falling in the visibleRectangle"
		self composeAll.
		displayBoolean ifTrue: [^ self displayLines: (1 to: lastLine)]].

	"save -- things get pretty mashed as we go along"
	obsoleteLines := lines copy.
	obsoleteLastLine := lastLine.

	"find the starting and stopping lines"
	firstLineIndex := startLine := self lineIndexOfCharacterIndex: start.
	stopLine := self lineIndexOfCharacterIndex: stop.

	"how many characters being inserted or deleted
		-- negative if aText size is < characterInterval size."
	replacementRange := aText size - (stop - start + 1).
	"Give ourselves plenty of elbow room."
	compositionRectangle := compositionRectangle withHeight: (textStyle lineGrid * 9999).
	"build a boundingBox of the actual screen space in question -- we'll need it later"
	visibleRectangle := (clippingRectangle intersect: compositionRectangle)
							intersect: destinationForm boundingBox.
	compositionScanner := CompositionScanner new forParagraph: self.		"Initialize a scanner."

	"If the starting line is not also the first line, then measuring must commence from line preceding the one in which characterInterval start appears.  For example, deleting a line with only a carriage return may move characters following the deleted portion of text into the line preceding the deleted line."
	startIndex := (lines at: firstLineIndex) first.
	startLine > 1
		ifTrue: 	[newLine := compositionScanner composeLine: startLine - 1
						fromCharacterIndex: (lines at: startLine - 1) first
						inParagraph: self.
				(lines at: startLine - 1) = newLine
					ifFalse:	["start in line preceding the one with the starting character"
							startLine := startLine - 1.
							self lineAt: startLine put: newLine.
							startIndex := newLine last + 1]].
	startIndex > text size ifTrue:
		["nil lines after a deletion -- remeasure last line below"
		self trimLinesTo: (firstLineIndex - 1 max: 0).
		text size = 0 ifTrue:
			["entire text deleted -- clear visibleRectangle and return."
			displayBoolean ifTrue: [destinationForm fill: visibleRectangle rule: rule fillColor: self backgroundColor].
			self updateCompositionHeight.
			^self]].

	"Now we really get to it."
	done := false.
	lastLineIndex := stopLine.
	[done or: [startIndex > text size]]
		whileFalse: 
		[self lineAt: firstLineIndex put:
			(newLine := compositionScanner composeLine: firstLineIndex
							fromCharacterIndex: startIndex inParagraph: self).
		[(lastLineIndex > obsoleteLastLine
			or: ["no more old lines to compare with?"
				newLine last <
					(newStop := (obsoleteLines at: lastLineIndex) last + replacementRange)])
			  	or: [done]]
			whileFalse: 
			[newStop = newLine last
				ifTrue:	["got the match"
						"get source and dest y's for moving the unchanged lines"
						obsoleteY := self topAtLineIndex: lastLineIndex + 1
									using: obsoleteLines and: obsoleteLastLine.
						newY := self topAtLineIndex: firstLineIndex + 1.
						stopLine := firstLineIndex.
						done := true.
							"Fill in the new line vector with the old unchanged lines.
							Update their starting and stopping indices on the way."
						((lastLineIndex := lastLineIndex + 1) to: obsoleteLastLine) do:
							[:upDatedIndex | 
							self lineAt: (firstLineIndex := firstLineIndex + 1) 
								put: ((obsoleteLines at: upDatedIndex)
							  		slide: replacementRange)].
							"trim off obsolete lines, if any"
						self trimLinesTo: firstLineIndex]
				ifFalse:	[lastLineIndex := lastLineIndex + 1]].
		startIndex := newLine last + 1.
		firstLineIndex := firstLineIndex + 1].

	"Now the lines are up to date -- Whew!!.  What remains is to move
	the 'unchanged' lines and display those which have changed."
	displayBoolean   "Not much to do if not displaying"
		ifFalse: [^ self updateCompositionHeight].
	startIndex > text size ifTrue:
		["If at the end of previous lines simply display lines from the line in
		which the first character of the replacement occured through the
		end of the paragraph."
		self updateCompositionHeight.
		self displayLines:
			(startLine to: (stopLine := firstLineIndex min: lastLine)).
		destinationForm  "Clear out area at the bottom"
			fill: ((visibleRectangle left @ (self topAtLineIndex: lastLine + 1)
						extent: visibleRectangle extent)
					intersect: visibleRectangle)
			rule: rule fillColor: self backgroundColor]
		ifFalse:
		[newY ~= obsoleteY ifTrue:
			["Otherwise first move the unchanged lines within
			the visibleRectangle with a good old bitblt."
			moveRectangle :=
				visibleRectangle left @ (obsoleteY max: visibleRectangle top)
					corner: visibleRectangle corner.
			destinationForm copyBits: moveRectangle from: destinationForm
				at: moveRectangle origin + (0 @ (newY-obsoleteY))
				clippingBox: visibleRectangle
				rule: Form over fillColor: nil].

		"Then display the altered lines."
		self displayLines: (startLine to: stopLine).

		newY < obsoleteY
			ifTrue:
			[(self topAtLineIndex: obsoleteLastLine+1 using: obsoleteLines and: obsoleteLastLine) > visibleRectangle bottom
				ifTrue:
				["A deletion may have 'pulled' previously undisplayed lines
				into the visibleRectangle.  If so, display them."
				self displayLines:
					((self lineIndexOfTop: visibleRectangle bottom - (obsoleteY - newY))
						to: (self lineIndexOfTop: visibleRectangle bottom))].
			"Clear out obsolete material at the bottom of the visibleRectangle."
			destinationForm
				fill: ((visibleRectangle left @ ((self bottomAtLineIndex: lastLine) + 1)
						extent: visibleRectangle extent)
					intersect: visibleRectangle)  "How about just corner: ??"
				rule: rule fillColor: self backgroundColor].

		(newY > obsoleteY and: [obsoleteY < visibleRectangle top])
			ifTrue:
				["An insertion may have 'pushed' previously undisplayed lines
				into the visibleRectangle.  If so, display them."
				self displayLines:
					((self lineIndexOfTop: visibleRectangle top)
						to: (self lineIndexOfTop: visibleRectangle top + (newY-obsoleteY)))].

		self updateCompositionHeight]! !

!Paragraph methodsFor: 'selecting' stamp: ''!
reverseFrom: characterBlock1 to: characterBlock2 
	"Reverse area between the two character blocks given as arguments."
	| visibleRectangle initialRectangle interiorRectangle finalRectangle lineNo baseline caret |
	characterBlock1 = characterBlock2 ifTrue:
		[lineNo := self lineIndexOfCharacterIndex: characterBlock1 stringIndex.
		baseline := lineNo = 0 ifTrue: [textStyle baseline]
							ifFalse: [(lines at: lineNo) baseline].
		caret := self caretFormForDepth: Display depth.
		^ caret  "Use a caret to indicate null selection"
				displayOn: destinationForm
				at: characterBlock1 topLeft + (-3 @ baseline)
				clippingBox: clippingRectangle
				rule: (false "Display depth>8" ifTrue: [9 "not-reverse"]
									ifFalse: [Form reverse])
				fillColor: nil].
	visibleRectangle := 
		(clippingRectangle intersect: compositionRectangle)
			"intersect: destinationForm boundingBox" "not necessary".
	characterBlock1 top = characterBlock2 top
		ifTrue: [characterBlock1 left < characterBlock2 left
					ifTrue: 
						[initialRectangle := 
							(characterBlock1 topLeft corner: characterBlock2 bottomLeft)
								intersect: visibleRectangle]
					ifFalse: 
						[initialRectangle := 
							(characterBlock2 topLeft corner: characterBlock1 bottomLeft)
								intersect: visibleRectangle]]
		ifFalse: [characterBlock1 top < characterBlock2 top
					ifTrue: 
						[initialRectangle := 
							(characterBlock1 topLeft 
								corner: visibleRectangle right @ characterBlock1 bottom)
								intersect: visibleRectangle.
						characterBlock1 bottom = characterBlock2 top
							ifTrue: 
								[finalRectangle := 
									(visibleRectangle left @ characterBlock2 top 
										corner: characterBlock2 bottomLeft)
										intersect: visibleRectangle]
							ifFalse: 
								[interiorRectangle := 
									(visibleRectangle left @ characterBlock1 bottom
										corner: visibleRectangle right 
														@ characterBlock2 top)
										intersect: visibleRectangle.
								finalRectangle := 
									(visibleRectangle left @ characterBlock2 top 
										corner: characterBlock2 bottomLeft)
										intersect: visibleRectangle]]
				ifFalse: 
					[initialRectangle := 
						(visibleRectangle left @ characterBlock1 top 
							corner: characterBlock1 bottomLeft)
							intersect: visibleRectangle.
					characterBlock1 top = characterBlock2 bottom
						ifTrue: 
							[finalRectangle := 
								(characterBlock2 topLeft 
									corner: visibleRectangle right 
												@ characterBlock2 bottom)
									intersect: visibleRectangle]
						ifFalse: 
							[interiorRectangle := 
								(visibleRectangle left @ characterBlock2 bottom 
									corner: visibleRectangle right @ characterBlock1 top)
									intersect: visibleRectangle.
							finalRectangle := 
								(characterBlock2 topLeft 
									corner: visibleRectangle right 
												@ characterBlock2 bottom)
									intersect: visibleRectangle]]].
	self hiliteRect: initialRectangle.
	self hiliteRect: interiorRectangle.
	self hiliteRect: finalRectangle.! !

!Paragraph methodsFor: 'alignment' stamp: ''!
rightFlush 
	"Set the alignment for the style with which the receiver displays its text 
	so that the characters in each of text end on an even border in the 
	composition rectangle but the beginning of each line does not. This is 
	also known as ragged-left."

	textStyle alignment: RightFlush! !

!Paragraph methodsFor: 'private' stamp: ''!
rightMarginForComposition
	"Build the right margin for a line. Depends upon compositionRectangle
	width, marginTabsLevel, and right indent."

	^compositionRectangle width 
		- (textStyle rightMarginTabAt: marginTabsLevel) 
		- textStyle rightIndent! !

!Paragraph methodsFor: 'private' stamp: ''!
rightMarginForDisplay 
	"Build the right margin for a line. Depends upon compositionRectangle
	rightSide, marginTabsLevel, and right indent."

	^compositionRectangle right - 
		textStyle rightIndent - (textStyle rightMarginTabAt: marginTabsLevel)! !

!Paragraph methodsFor: 'accessing' stamp: ''!
rule 
	"Answer the rule according to which character display behaves. For 
	example, rule may equal over, under, reverse."

	^rule! !

!Paragraph methodsFor: 'accessing' stamp: ''!
rule: ruleInteger 
	"Set the rule according to which character display behaves."

	rule := ruleInteger! !

!Paragraph methodsFor: 'scrolling' stamp: ''!
scrollBy: heightToMove 
	^ self scrollBy: heightToMove withSelectionFrom: nil to: nil! !

!Paragraph methodsFor: 'scrolling' stamp: 'hmm 9/16/2000 21:30'!
scrollBy: heightToMove withSelectionFrom: startBlock to: stopBlock 
	"Translate the composition rectangle up (dy<0) by heightToMove.
	Repainting text as necessary, and selection if blocks not nil.
	Return true unless scrolling limits have been reached."
	| max min amount |
	max := 0 max: "cant scroll up more than dist to (top of) bottom line"
		compositionRectangle bottom - textStyle lineGrid - clippingRectangle top.
	min := 0 min: "cant scroll down more than top is above clipRect"
		compositionRectangle top - clippingRectangle top.
	amount := ((heightToMove truncateTo: textStyle lineGrid) min: max) max: min.
	amount ~= 0
		ifTrue: [destinationForm deferUpdatesIn: clippingRectangle while: [
					self scrollUncheckedBy: amount
						withSelectionFrom: startBlock to: stopBlock].
				^ true]
		ifFalse: [^ false]! !

!Paragraph methodsFor: 'scrolling' stamp: ''!
scrollDelta
	"By comparing this before and after, you know if scrolling happened"
	^ clippingRectangle top - compositionRectangle top! !

!Paragraph methodsFor: 'scrolling' stamp: ''!
scrollUncheckedBy: heightToMove withSelectionFrom: startBlock to: stopBlock 
	"Scroll by the given amount.  Copy bits where possible, display the rest.
	If selection blocks are not nil, then select the newly visible text as well."
	| savedClippingRectangle delta |
	delta := 0 @ (0 - heightToMove).
	compositionRectangle := compositionRectangle translateBy: delta.
	startBlock == nil ifFalse:
		[startBlock moveBy: delta.
		stopBlock moveBy: delta].
	savedClippingRectangle := clippingRectangle.
	clippingRectangle := clippingRectangle intersect: Display boundingBox.
	heightToMove abs >= clippingRectangle height
	  ifTrue: 
		["Entire visible region must be repainted"
		self displayLines: (1 to: lastLine) affectedRectangle: clippingRectangle]
	  ifFalse:
		["Copy bits where possible / display the rest"
		destinationForm
			copyBits: clippingRectangle from: destinationForm
			at: clippingRectangle topLeft + delta
			clippingBox: clippingRectangle
			rule: Form over fillColor: nil.
		"Set clippingRectangle to 'vacated' area for lines 'pulled' into view."
		clippingRectangle := heightToMove < 0
			ifTrue:  "On the top"
				[clippingRectangle topLeft corner: clippingRectangle topRight + delta]
			ifFalse:  "At the bottom"
				[clippingRectangle bottomLeft + delta corner: clippingRectangle bottomRight].
		self displayLines: (1 to: lastLine)   "Refresh vacated region"
			affectedRectangle: clippingRectangle].
	startBlock == nil ifFalse:
		[self reverseFrom: startBlock to: stopBlock].
	"And restore the clippingRectangle to its original value. "
	clippingRectangle := savedClippingRectangle! !

!Paragraph methodsFor: 'selecting' stamp: 'di 12/1/97 04:43'!
selectionRectsFrom: characterBlock1 to: characterBlock2 
	"Return an array of rectangles representing the area between the two character blocks given as arguments."
	| visibleRectangle initialRectangle interiorRectangle finalRectangle lineNo baseline |
	characterBlock1 = characterBlock2 ifTrue:
		[lineNo := self lineIndexOfCharacterIndex: characterBlock1 stringIndex.
		baseline := lineNo = 0 ifTrue: [textStyle baseline]
							ifFalse: [(lines at: lineNo) baseline].
		^ Array with: (characterBlock1 topLeft extent: 1 @ baseline)].
	visibleRectangle := clippingRectangle intersect: compositionRectangle.
	characterBlock1 top = characterBlock2 top
		ifTrue: [characterBlock1 left < characterBlock2 left
					ifTrue: 
						[initialRectangle := 
							(characterBlock1 topLeft corner: characterBlock2 bottomLeft)
								intersect: visibleRectangle]
					ifFalse: 
						[initialRectangle := 
							(characterBlock2 topLeft corner: characterBlock1 bottomLeft)
								intersect: visibleRectangle]]
		ifFalse: [characterBlock1 top < characterBlock2 top
					ifTrue: 
						[initialRectangle := 
							(characterBlock1 topLeft 
								corner: visibleRectangle right @ characterBlock1 bottom)
								intersect: visibleRectangle.
						characterBlock1 bottom = characterBlock2 top
							ifTrue: 
								[finalRectangle := 
									(visibleRectangle left @ characterBlock2 top 
										corner: characterBlock2 bottomLeft)
										intersect: visibleRectangle]
							ifFalse: 
								[interiorRectangle := 
									(visibleRectangle left @ characterBlock1 bottom
										corner: visibleRectangle right 
														@ characterBlock2 top)
										intersect: visibleRectangle.
								finalRectangle := 
									(visibleRectangle left @ characterBlock2 top 
										corner: characterBlock2 bottomLeft)
										intersect: visibleRectangle]]
				ifFalse: 
					[initialRectangle := 
						(visibleRectangle left @ characterBlock1 top 
							corner: characterBlock1 bottomLeft)
							intersect: visibleRectangle.
					characterBlock1 top = characterBlock2 bottom
						ifTrue: 
							[finalRectangle := 
								(characterBlock2 topLeft 
									corner: visibleRectangle right 
												@ characterBlock2 bottom)
									intersect: visibleRectangle]
						ifFalse: 
							[interiorRectangle := 
								(visibleRectangle left @ characterBlock2 bottom 
									corner: visibleRectangle right @ characterBlock1 top)
									intersect: visibleRectangle.
							finalRectangle := 
								(characterBlock2 topLeft 
									corner: visibleRectangle right 
												@ characterBlock2 bottom)
									intersect: visibleRectangle]]].
	^ (Array with: initialRectangle with: interiorRectangle with: finalRectangle)
			select: [:rect | rect notNil]! !

!Paragraph methodsFor: 'private' stamp: ''!
setWithText: aText style: aTextStyle 
	"Set text and adjust bounding rectangles to fit."

	| shrink compositionWidth unbounded |
	unbounded := Rectangle origin: 0 @ 0 extent: 9999@9999.
	compositionWidth := self
		setWithText: aText style: aTextStyle compositionRectangle: unbounded clippingRectangle: unbounded.
	compositionRectangle := compositionRectangle withWidth: compositionWidth.
	clippingRectangle := compositionRectangle copy.
	shrink := unbounded width - compositionWidth.
	"Shrink padding widths accordingly"
	1 to: lastLine do:
		[:i | (lines at: i) paddingWidth: (lines at: i) paddingWidth - shrink]! !

!Paragraph methodsFor: 'private' stamp: ''!
setWithText: aText style: aTextStyle compositionRectangle: compRect clippingRectangle: clipRect 
	"Set text and using supplied parameters. Answer max composition width."

	clippingRectangle := clipRect copy.
	^self
		compositionRectangle: compRect
		text: aText
		style: aTextStyle
		offset: 0 @ 0! !

!Paragraph methodsFor: 'private' stamp: ''!
setWithText: aText style: aTextStyle compositionRectangle: compRect clippingRectangle: clipRect foreColor: cf backColor: cb
	"Set text and using supplied parameters. Answer max composition width."

	clippingRectangle := clipRect copy.
	self foregroundColor: cf backgroundColor: cb.
	^ self
		compositionRectangle: compRect
		text: aText
		style: aTextStyle
		offset: 0 @ 0! !

!Paragraph methodsFor: 'accessing' stamp: 'sw 10/29/1999 18:11'!
stringAtLineNumber: aNumber
	(aNumber > lastLine or: [aNumber < 1]) ifTrue: [^ nil].
	^ (text string copyFrom: (lines at: aNumber) first to: (lines at: aNumber) last) copyWithout: Character cr! !

!Paragraph methodsFor: 'accessing' stamp: ''!
text: aText 
	"Set the argument, aText, to be the text for the receiver."

	text := aText.
	self composeAll! !

!Paragraph methodsFor: 'alignment' stamp: ''!
toggleAlignment 
	"Set the alignment for the style with which the receiver displays its text 
	so that it moves from centered to justified to leftFlush to rightFlush and 
	back to centered again."

	textStyle alignment: textStyle alignment + 1! !

!Paragraph methodsFor: 'private' stamp: ''!
topAtLineIndex: lineIndex 
	"Answer the top y of given line."
	| y |
	y := compositionRectangle top.
	lastLine = 0 ifTrue: [lineIndex > 0 ifTrue: [^ y + textStyle lineGrid]. ^ y].
	1 to: (lineIndex-1 min: lastLine) do:
		[:i | y := y + (lines at: i) lineHeight].
	^ y
! !

!Paragraph methodsFor: 'private' stamp: ''!
topAtLineIndex: lineIndex using: otherLines and: otherLastLine
	"Answer the top y of given line."
	| y |
	y := compositionRectangle top.
	otherLastLine = 0 ifTrue: [^ y].
	1 to: (lineIndex-1 min: otherLastLine) do:
		[:i | y := y + (otherLines at: i) lineHeight].
	^ y
! !

!Paragraph methodsFor: 'private' stamp: ''!
trimLinesTo: lastLineInteger

	(lastLineInteger + 1 to: lastLine) do: [:i | lines at: i put: nil].
	(lastLine := lastLineInteger) < (lines size // 2) 
		ifTrue: [lines := lines copyFrom: 1 to: lines size - (lines size // 2)]! !

!Paragraph methodsFor: 'private' stamp: ''!
updateCompositionHeight
	"Mainly used to insure that intersections with compositionRectangle work." 

	compositionRectangle := compositionRectangle withHeight:
		(self bottomAtLineIndex: lastLine) - compositionRectangle top.
	(text size ~= 0 and: [(text at: text size) = CR])
		ifTrue: [compositionRectangle := compositionRectangle withHeight:
					compositionRectangle height + (lines at: lastLine) lineHeight]! !

!Paragraph methodsFor: 'utilities' stamp: ''!
visibleRectangle 
	"May be less than the clippingRectangle if text ends part way down.
	Also some fearful history includes Display intersection;
	it shouldn't be necessary"

	^ (clippingRectangle intersect: compositionRectangle)
		intersect: destinationForm boundingBox! !

!Paragraph methodsFor: 'private' stamp: 'di 8/30/97 11:14'!
withClippingRectangle: clipRect do: aBlock
	| saveClip |
	saveClip := clippingRectangle.
	clippingRectangle := clipRect.
		aBlock value.
	clippingRectangle := saveClip! !

!Paragraph methodsFor: 'composition' stamp: ''!
wrappingBox: compositionRect clippingBox: clippingRect 
	"Set the composition rectangle for the receiver so that the lines wrap 
	within the rectangle, compositionRect, and the display of the text is 
	clipped by the rectangle, clippingRect."

	self compositionRectangle: compositionRect copy
				text: text
				style: textStyle
				offset: offset.
	clippingRectangle := clippingRect copy! !

OrderedCollection variableSubclass: #GraphicSymbol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Symbols'!

!GraphicSymbol commentStamp: '<historical>' prior: 0!
I represent a structured picture built from primitive display objects and other instances of me.!

!GraphicSymbol methodsFor: 'accessing' stamp: 'jrm 2/6/2000 11:01'!
computeBoundingBox
	"Compute a boundingBox that encloses all of the Paths in this symbol"

	^Rectangle merging: (self collect: [:each | each computeBoundingBox])
! !

!GraphicSymbol methodsFor: 'displaying' stamp: ''!
displayOn: aDisplayMedium transformation: aTransformation clippingBox: clipRect rule: anInteger fillColor: aForm 
	"Display the receiver on the Display where aTransformation is provided 
	as an argument, rule is anInteger and mask is aForm. No translation. 
	Information to be displayed must be confined to the area that intersects 
	with clipRect."

	self do: 
		[:element | 
		element
			displayOn: aDisplayMedium
			transformation: aTransformation
			clippingBox: clipRect
			rule: anInteger
			fillColor: aForm]! !

!GraphicSymbol methodsFor: 'displaying' stamp: ''!
displayTransformation: aTransformation clippingBox: clipRect rule: anInteger fillColor: aForm
	"Display the receiver where aTransformation is provided as an argument, 
	rule is anInteger and mask is aForm. No translation. Information to be 
	displayed must be confined to the area that intersects with clipRect."

	self displayOn: Display transformation: aTransformation clippingBox: clipRect
		rule: anInteger fillColor: aForm! !

Model subclass: #StringHolder
	instanceVariableNames: 'contents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Kernel-Remnants'!

!StringHolder commentStamp: '<historical>' prior: 0!
I am a kind of Model that includes a piece of text.  In some cases, the text can be edited, and in some the text is a method.

Categories 'code pane menu' and 'message list menu' are messages that may be called by my menus when the text is a method, and when some pane is a list of methods.  Other of my subclasses may ignore these two catagories altogether.!

StringHolder subclass: #FillInTheBlank
	instanceVariableNames: 'acceptOnCR done responseUponCancel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Menus'!

!FillInTheBlank commentStamp: '<historical>' prior: 0!
I represent a prompt for string input from the user. The user is asked to type in and edit a string. The resulting string is supplied as the argument to a client-supplied action block.
!

!FillInTheBlank classSide methodsFor: 'private' stamp: 'sma 6/18/2000 10:39'!
fillInTheBlankMorphClass
	"By factoring out this class references, it becomes possible to discard 
	morphic by simply removing this class.  All calls to this method needs
	to be protected by 'Smalltalk isMorphic' tests."

	^ FillInTheBlankMorph! !

!FillInTheBlank classSide methodsFor: 'private' stamp: 'sma 6/18/2000 10:47'!
fillInTheBlankViewClass
	"By factoring out this class references, it becomes possible to discard 
	MVC by simply removing this class.  All calls to this method needs
	to be protected by 'Smalltalk isMorphic' tests."

	^ FillInTheBlankView! !

!FillInTheBlank classSide methodsFor: 'instance creation' stamp: 'nk 7/30/2004 21:50'!
multiLineRequest: queryString centerAt: aPoint initialAnswer: defaultAnswer answerHeight: answerHeight 
	"Create a multi-line instance of me whose question is queryString with
	the given initial answer. Invoke it centered at the given point, and
	answer the string the user accepts.  Answer nil if the user cancels.  An
	empty string returned means that the ussr cleared the editing area and
	then hit 'accept'.  Because multiple lines are invited, we ask that the user
	use the ENTER key, or (in morphic anyway) hit the 'accept' button, to 
	submit; that way, the return key can be typed to move to the next line.
	NOTE: The ENTER key does not work on Windows platforms."

	"FillInTheBlank
		multiLineRequest:
'Enter several lines; end input by accepting
or canceling via menu or press Alt+s/Alt+l'
		centerAt: Display center
		initialAnswer: 'Once upon a time...'
		answerHeight: 200"

	| model fillInView |
	Smalltalk isMorphic 
		ifTrue: 
			[^self fillInTheBlankMorphClass 
				request: queryString
				initialAnswer: defaultAnswer
				centerAt: aPoint
				inWorld: self currentWorld
				onCancelReturn: nil
				acceptOnCR: false].
	model := self new.
	model contents: defaultAnswer.
	model responseUponCancel: nil.
	model acceptOnCR: false.
	fillInView := self fillInTheBlankViewClass 
				multiLineOn: model
				message: queryString
				centerAt: aPoint
				answerHeight: answerHeight.
	^model show: fillInView! !

!FillInTheBlank classSide methodsFor: 'instance creation' stamp: 'ar 3/18/2001 00:53'!
request: queryString 
	"Create an instance of me whose question is queryString. Invoke it 
	centered at the cursor, and answer the string the user accepts. Answer 
	the empty string if the user cancels."

	"FillInTheBlank request: 'Your name?'"

	^ self
		request: queryString
		initialAnswer: ''
		centerAt: (ActiveHand ifNil:[Sensor]) cursorPoint! !

!FillInTheBlank classSide methodsFor: 'instance creation' stamp: 'ar 3/18/2001 00:54'!
request: queryString initialAnswer: defaultAnswer 
	"Create an instance of me whose question is queryString with the given 
	initial answer. Invoke it centered at the given point, and answer the 
	string the user accepts. Answer the empty string if the user cancels."

	"FillInTheBlank 
		request: 'What is your favorite color?' 
		initialAnswer: 'red, no blue. Ahhh!!'"

	^ self
		request: queryString
		initialAnswer: defaultAnswer
		centerAt: (ActiveHand ifNil:[Sensor]) cursorPoint! !

!FillInTheBlank classSide methodsFor: 'instance creation' stamp: 'nk 7/30/2004 21:50'!
request: queryString initialAnswer: defaultAnswer centerAt: aPoint 
	"Create an instance of me whose question is queryString with the given
	initial answer. Invoke it centered at the given point, and answer the
	string the user accepts. Answer the empty string if the user cancels."

	"FillInTheBlank
		request: 'Type something, then type CR.'
		initialAnswer: 'yo ho ho!!'
		centerAt: Display center"

	| model fillInView |
	Smalltalk isMorphic 
		ifTrue: 
			[^self fillInTheBlankMorphClass 
				request: queryString
				initialAnswer: defaultAnswer
				centerAt: aPoint].
	model := self new.
	model contents: defaultAnswer.
	fillInView := self fillInTheBlankViewClass 
				on: model
				message: queryString
				centerAt: aPoint.
	^model show: fillInView! !

!FillInTheBlank classSide methodsFor: 'instance creation' stamp: 'nk 7/30/2004 21:50'!
requestPassword: queryString 
	"Create an instance of me whose question is queryString. Invoke it centered
	at the cursor, and answer the string the user accepts. Answer the empty 
	string if the user cancels."

	"FillInTheBlank requestPassword: 'POP password'"

	| model fillInView |
	Smalltalk isMorphic 
		ifTrue: [^self fillInTheBlankMorphClass requestPassword: queryString].
	model := self new.
	model contents: ''.
	fillInView := self fillInTheBlankViewClass 
				requestPassword: model
				message: queryString
				centerAt: Sensor cursorPoint
				answerHeight: 40.
	^model show: fillInView! !

!FillInTheBlank methodsFor: 'accessing' stamp: 'jm 4/28/1998 06:18'!
acceptOnCR
	"Answer whether a carriage return should cause input to be accepted."

	^ acceptOnCR
! !

!FillInTheBlank methodsFor: 'accessing' stamp: 'jm 4/28/1998 06:18'!
acceptOnCR: aBoolean

	acceptOnCR := aBoolean.
! !

!FillInTheBlank methodsFor: 'object fileIn' stamp: 'RAA 12/20/2000 17:59'!
convertToCurrentVersion: varDict refStream: smartRefStrm
	
	varDict at: 'responseUponCancel' ifAbsent: [responseUponCancel := ''].
	^super convertToCurrentVersion: varDict refStream: smartRefStrm.

! !

!FillInTheBlank methodsFor: 'accessing' stamp: 'jm 5/6/1998 15:13'!
done
	"Answer whether the user has ended the interaction."

	^ done
! !

!FillInTheBlank methodsFor: 'accessing' stamp: 'jm 5/6/1998 15:13'!
done: aBoolean

	done := aBoolean.
! !

!FillInTheBlank methodsFor: 'initialize-release' stamp: 'sw 1/31/2000 14:42'!
initialize

	super initialize.
	acceptOnCR := false.
	done := false.
	responseUponCancel := ''
! !

!FillInTheBlank methodsFor: 'accessing' stamp: 'sw 1/31/2000 14:45'!
responseUponCancel: resp
	responseUponCancel := resp! !

!FillInTheBlank methodsFor: 'accessing' stamp: 'sw 1/31/2000 14:47'!
setResponseForCancel
	self contents: responseUponCancel! !

!FillInTheBlank methodsFor: 'private' stamp: 'sma 6/18/2000 10:54'!
show: fillInView
	| savedArea |
	savedArea := Form fromDisplay: fillInView displayBox.
	fillInView display.
	contents isEmpty
		ifFalse: [fillInView lastSubView controller selectFrom: 1 to: contents size].
	(fillInView lastSubView containsPoint: Sensor cursorPoint)
		ifFalse: [fillInView lastSubView controller centerCursorInView].
	fillInView controller startUp.
	fillInView release.
	savedArea displayOn: Display at: fillInView viewport topLeft.
	^ contents! !

!StringHolder class methodsFor: 'class initialization' stamp: ''!
initialize
	"The class variables were initialized once, and subsequently filled with
	information. Re-executing this method is therefore dangerous." 
	 
	"workSpace := StringHolder new"

	"StringHolder initialize"! !

!StringHolder class methodsFor: 'instance creation' stamp: 'ar 9/27/2005 20:48'!
open
	(Smalltalk at: #Workspace ifAbsent:[self]) new openLabel: 'Workspace'
		"Not to be confused with our own class var 'Workspace'"! !

!StringHolder class methodsFor: 'instance creation' stamp: 'tk 5/4/1998 16:41'!
openLabel: aString

	self new openLabel: aString! !

!StringHolder class methodsFor: 'window color' stamp: 'sw 2/26/2002 14:44'!
windowColorSpecification
	"Answer a WindowColorSpec object that declares my preference"

	^ WindowColorSpec classSymbol: self name wording: 'Workspace' brightColor: #lightYellow pastelColor: #paleYellow helpMessage: 'A place for text in a window.'! !

!StringHolder methodsFor: 'accessing' stamp: 'di 5/19/1998 15:34'!
acceptContents: aString 
	"Set aString to be the contents of the receiver.  Return true cuz happy"

	self contents: aString.
	^ true! !

!StringHolder methodsFor: 'toolbuilder' stamp: 'ar 2/11/2005 20:36'!
buildWith: builder
	| windowSpec textSpec |
	windowSpec := builder pluggableWindowSpec new.
	windowSpec model: self.
	windowSpec label: 'Workspace'.
	windowSpec children: OrderedCollection new.
	textSpec := builder pluggableTextSpec new.
	textSpec 
		model: self;
		getText: #contents; 
		setText: #acceptContents:; 
		selection: nil; 
		menu: #codePaneMenu:shifted:;
		frame: (0@0corner: 1@1).
	windowSpec children add: textSpec.

	^builder build: windowSpec! !

!StringHolder methodsFor: 'accessing' stamp: 'nk 4/29/2004 12:32'!
classCommentIndicated
	"Answer true iff we're viewing the class comment."
	^false! !

!StringHolder methodsFor: 'user edits' stamp: 'di 4/21/1998 11:30'!
clearUserEditFlag
	"Clear the hasUnacceptedEdits flag in all my dependent views."

	self changed: #clearUserEdits! !

!StringHolder methodsFor: 'code pane menu' stamp: 'dgd 10/1/2004 13:43'!
codePaneMenu: aMenu shifted: shifted 
	"Note that unless we override perform:orSendTo:, 
	PluggableTextController will respond to all menu items in a 
	text pane"
	| donorMenu |
	donorMenu := shifted
				ifTrue: [ParagraphEditor shiftedYellowButtonMenu]
				ifFalse: [ParagraphEditor yellowButtonMenu].
	^ aMenu addAllFrom: donorMenu! !

!StringHolder methodsFor: 'accessing' stamp: ''!
contents
	"Answer the contents that the receiver is holding--presumably a string."

	^contents! !

!StringHolder methodsFor: 'accessing' stamp: 'sw 1/12/1999 11:47'!
contents: textOrString 
	"Set textOrString to be the contents of the receiver."

	contents := textOrString "asString"! !

!StringHolder methodsFor: 'accessing' stamp: 'tk 4/3/98 22:50'!
contentsSelection
	"Return the interval of text in the code pane to select when I set the pane's contents"

	^ 1 to: 0  "null selection"! !

!StringHolder methodsFor: 'initialize-release' stamp: ''!
defaultContents

	^''! !

!StringHolder methodsFor: 'evaluation' stamp: ''!
doItContext
	"Answer the context in which a text selection can be evaluated."

	^nil! !

!StringHolder methodsFor: 'evaluation' stamp: ''!
doItReceiver
	"Answer the object that should be informed of the result of evaluating a 
	text selection."

	^nil! !

!StringHolder methodsFor: 'initialize-release' stamp: 'sw 10/16/1998 11:36'!
embeddedInMorphicWindowLabeled: labelString
	| window |
	window := (SystemWindow labelled: labelString) model: self.
	window addMorph: (PluggableTextMorph on: self text: #contents accept: #acceptContents:
			readSelection: nil menu: #codePaneMenu:shifted:)
		frame: (0@0 corner: 1@1).
	^ window! !

!StringHolder methodsFor: 'initialize-release' stamp: 'jm 3/24/98 17:56'!
initialize
	"Initialize the state of the receiver with its default contents."

	contents := self defaultContents.
! !

!StringHolder methodsFor: 'accessing' stamp: 'sw 12/9/2000 23:59'!
noteAcceptanceOfCodeFor: aSelector
	"A method has possibly been submitted for the receiver with aSelector as its selector; If the receiver wishes to take soem action here is a chance for it to do so"
! !

!StringHolder methodsFor: 'user edits' stamp: 'tk 4/13/1998 23:07'!
okToChange

	self canDiscardEdits ifTrue: [^ true].
	self changed: #wantToChange.  "Solicit cancel from view"
	^ self canDiscardEdits
! !

!StringHolder methodsFor: 'initialize-release' stamp: 'sw 10/16/1998 11:37'!
openAsMorphLabel: labelString 
	"Workspace new openAsMorphLabel: 'Workspace'"
	(self embeddedInMorphicWindowLabeled: labelString) openInWorld! !

!StringHolder methodsFor: 'initialize-release' stamp: 'sw 12/22/1998 00:16'!
openLabel: aString 
	"Create a standard system view of the model, me, a StringHolder and open it.  If in mvc, terminate the active controller so that the new window will immediately be activated."
	self openLabel: aString andTerminate: true! !

!StringHolder methodsFor: 'initialize-release' stamp: 'sma 4/30/2000 10:15'!
openLabel: aString andTerminate: terminateBoolean
	"Create a standard system view of the model, me, a StringHolder and open it.; do not terminate the active process if in mvc"
	| topView codeView |

	Smalltalk isMorphic ifTrue: [^ self openAsMorphLabel: aString].

	topView := (StandardSystemView new) model: self.
	topView borderWidth: 1.
	topView label: aString.
	topView minimumSize: 100 @ 50.

	codeView := PluggableTextView on: self 
			text: #contents accept: #acceptContents:
			readSelection: #contentsSelection menu: #codePaneMenu:shifted:.
	codeView window: (0@0 extent: 200@200).
	topView addSubView: codeView.
	"self contents size > 0 ifTrue: [
			codeView hasUnacceptedEdits: true].  Is it already saved or not??"
	terminateBoolean
		ifTrue:
			[topView controller open]
		ifFalse:
			[topView controller openNoTerminate]! !

!StringHolder methodsFor: 'tiles' stamp: 'di 11/4/2000 11:07'!
openSyntaxView
	"Open a syntax view on the current method"

	| class selector |

	(selector := self selectedMessageName) ifNotNil: [
		class := self selectedClassOrMetaClass.
		SyntaxMorph testClass: class andMethod: selector.
	]! !

!StringHolder methodsFor: 'code pane menu' stamp: 'wod 5/29/1998 16:35'!
perform: selector orSendTo: otherTarget
	"Selector was just chosen from a menu by a user.  If can respond, then
perform it on myself. If not, send it to otherTarget, presumably the
editPane from which the menu was invoked."

	(self respondsTo: selector)
		ifTrue: [^ self perform: selector]
		ifFalse: [^ otherTarget perform: selector]! !

!StringHolder methodsFor: 'accessing' stamp: 'sw 12/1/2000 11:04'!
reformulateList
	"If the receiver has a way of reformulating its message list, here is a chance for it to do so"! !

!StringHolder methodsFor: 'accessing' stamp: 'sw 12/6/2000 17:48'!
reformulateListNoting: newSelector
	"A method has possibly been submitted for the receiver with newSelector as its selector; If the receiver has a way of reformulating its message list, here is a chance for it to do so"

	^ self reformulateList! !

!StringHolder methodsFor: 'accessing' stamp: 'tk 4/18/1998 14:59'!
selectedClassName
	"I may know what class is currently selected"

	self selectedClass ifNotNil: [^ self selectedClass name].
	^ nil! !

!StringHolder methodsFor: 'accessing' stamp: 'tk 4/18/1998 15:01'!
selectedClassOrMetaClass

	^ self selectedClass	"I don't know any better"! !

!StringHolder methodsFor: 'accessing' stamp: 'tk 4/18/1998 15:22'!
selectedMessageName

	^ nil! !

!StringHolder methodsFor: 'code pane menu' stamp: 'tk 4/6/98 11:43'!
showBytecodes
	"We don't know how to do this"

	^ self changed: #flash! !

!StringHolder methodsFor: 'code pane menu' stamp: 'ar 9/27/2005 20:47'!
spawn: contentsString

	UIManager default edit: contentsString label: 'Workspace'
! !

!StringHolder methodsFor: 'accessing' stamp: 'di 11/23/1998 15:21'!
textContents: aStringOrText 
	"Set aStringOrText to be the contents of the receiver."

	contents := aStringOrText! !

!StringHolder methodsFor: 'optional panes' stamp: 'sw 1/24/2001 21:25'!
wantsAnnotationPane
	"Answer whether the receiver, seen in some browser window, would like to have the so-called  annotationpane included.  By default, various browsers defer to the global preference 'optionalButtons' -- but individual subclasses can insist to the contrary."

	^ Preferences annotationPanes! !

!StringHolder methodsFor: 'optional panes' stamp: 'sw 1/24/2001 18:57'!
wantsOptionalButtons
	"Answer whether the receiver, seen in some browser window, would like to have the so-called optional button pane included.  By default, various browsers defer to the global preference 'optionalButtons' -- but individual subclasses can insist to the contrary."

	^ Preferences optionalButtons! !

Model subclass: #Switch
	instanceVariableNames: 'on onAction offAction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Menus'!

!Switch commentStamp: '<historical>' prior: 0!
I represent a selection setting and actions to take depending on a change in the setting. An instance has three attributes: state, which is either on or off; on action; and off action. The on and off actions are blocks of code that execute whenever the instance changes state. I am typically used as a menu item in conjunction with a SwitchView and a SwitchController.
1/24/96 sw: made this a subclass of Model, for faster dependents handling!

Switch subclass: #Button
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Menus'!

!Button commentStamp: '<historical>' prior: 0!
I am a Switch that turns off automatically after being turned on, that is, I act like a push-button switch.!

!Button classSide methodsFor: 'instance creation' stamp: ''!
newOn 
	"Refer to the comment in Switch|newOn."

	self error: 'Buttons cannot be created in the on state'.
	^nil! !

!Button methodsFor: 'state' stamp: ''!
turnOff
	"Sets the state of the receiver to 'off'. The off action of the receiver is not  
	executed."

	on := false! !

!Button methodsFor: 'state' stamp: ''!
turnOn
	"The receiver remains in the 'off' state'."

	self doAction: onAction.
	self doAction: offAction! !

Switch subclass: #OneOnSwitch
	instanceVariableNames: 'connection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Menus'!

!OneOnSwitch commentStamp: '<historical>' prior: 0!
I am a kind of Switch that can be connected to some related object, typically to a collection of my instances. When my instance is created, its connection is set to a particular object. When the object changes because an Switch it refers to is turned on, an update message is broadcasted. All the connected OneOnSwitches, except the changed one, turn off. This allows OneOnSwitches to maintain the constraint that at most one of them will be on at any time. OneOnSwitches can thus be made to act like "car radio" switches.!

!OneOnSwitch methodsFor: 'connection' stamp: ''!
connection
	"Answer the object that connects the receiver to other Switches."

	^connection! !

!OneOnSwitch methodsFor: 'connection' stamp: ''!
connection: anObject 
	"Set anObject to be the connection among two or more Switches. Make the 
	receiver a dependent of the argument, anObject."

	connection := anObject.
	connection addDependent: self! !

!OneOnSwitch methodsFor: 'connection' stamp: ''!
isConnectionSet
	"Answer whether the receiver is connected to an object that coordinates 
	updates among switches."

	connection == nil
		ifTrue: [^false]
		ifFalse: [^true]! !

!OneOnSwitch methodsFor: 'connection' stamp: ''!
notifyConnection
	"Send the receiver's connection (if it exists) the message 'changed: self' in 
	order for the connection to broadcast the change to other objects 
	connected by the connection."
	
	self isConnectionSet ifTrue: [self connection changed: self]! !

!OneOnSwitch methodsFor: 'initialize-release' stamp: ''!
release

	super release.
	self isConnectionSet ifTrue: [connection removeDependent: self]! !

!OneOnSwitch methodsFor: 'state' stamp: ''!
turnOn
	"Does nothing if it is already on. If it is not, it is set to 'on', its
	dependents are 	notified of the change, its connection is notified, and
	its action is executed."

	self isOff
		ifTrue: 
			[on := true.
			self changed.
			self notifyConnection.
			self doAction: onAction]! !

!OneOnSwitch methodsFor: 'updating' stamp: ''!
update: aOneOnSwitch 
	"Does nothing if aOneOnSwitch is identical to this object. If it is not, this 
	object is turned off. This message is sent by the connection (an Object)
	when some related OneOnSwitch (possibly this one) has changed. This
	allows a group of related OneOnSwitches to maintain the constraint that
	at most one will be on at any time."

	self ~~ aOneOnSwitch ifTrue: [self turnOff]! !

!Switch classSide methodsFor: 'instance creation' stamp: ''!
new
	"Answer an instance of me such that the on and off actions are set to nil
	('no action'), and the state is set to 'off'."

	^self newOff! !

!Switch classSide methodsFor: 'instance creation' stamp: ''!
newOff
	"Answer an instance of me such that the on and off actions are set to nil 
	('no action'), and the state is set to 'off'."

	^super new initializeOff! !

!Switch classSide methodsFor: 'instance creation' stamp: ''!
newOn
	"Answer an instance of me such that the on and off actions are set to nil 
	('no action'), and the state is set to 'on'."

	^super new initializeOn! !

!Switch methodsFor: 'state' stamp: ''!
clear
	"Set the state of the receiver to 'off'. If the state of the receiver was 
	previously 'on', then 'self change' is sent. The receiver's off action is 
	NOT executed."

	self isOn
		ifTrue: 
			[on := false.
			self changed]! !

!Switch methodsFor: 'action' stamp: ''!
doAction: anAction 
	"Execute anAction if it is non-nil."

	anAction == nil ifFalse: [anAction value]! !

!Switch methodsFor: 'private' stamp: ''!
initializeOff

	on := false. 
	onAction := nil.
	offAction := nil! !

!Switch methodsFor: 'private' stamp: ''!
initializeOn

	on := true. 
	onAction := nil.
	offAction := nil! !

!Switch methodsFor: 'state' stamp: ''!
isOff
	"Answer whether the receiver is set off or not."

	^on not! !

!Switch methodsFor: 'state' stamp: ''!
isOn
	"Answer whether the receiver is set on or not."

	^on! !

!Switch methodsFor: 'action' stamp: ''!
offAction: anAction 
	"Set the off action of the receiver to anAction."

	offAction := anAction fixTemps! !

!Switch methodsFor: 'action' stamp: ''!
onAction: anAction 
	"Set the on action of the receiver to anAction."

	onAction := anAction fixTemps! !

!Switch methodsFor: 'converting' stamp: 'md 9/18/2004 19:51'!
printOn: aStream
	self isOn
		ifTrue: [aStream nextPutAll: 'ON-Switch']
		ifFalse: [aStream nextPutAll: 'OFF-Switch']! !

!Switch methodsFor: 'state' stamp: ''!
set
	"Set the state of the receiver to 'on'. If the state of the receiver was 
	previously 'off', then 'self change' is sent. The receiver's on action is 
	NOT executed."

	self isOff
		ifTrue: 
			[on := true.
			self changed]! !

!Switch methodsFor: 'state' stamp: ''!
switch
	"Change the state of the receiver from 'on' to 'off' or from 'off' to 'on' (see 
	Switch|turnOn, Switch|turnOff)."

	self isOn
		ifTrue: [self turnOff]
		ifFalse: [self turnOn]! !

!Switch methodsFor: 'state' stamp: ''!
turnOff
	"Set the state of the receiver to 'off'. If the state of the receiver was 
	previously 'on', then 'self change' is sent and the receiver's off action is 
	executed."

	self isOn
		ifTrue: 
			[on := false.
			self changed.
			self doAction: offAction]! !

!Switch methodsFor: 'state' stamp: ''!
turnOn
	"Set the state of the receiver to 'on'. If the state of the receiver was 
	previously 'off', then 'self change' is sent and the receiver's on action is 
	executed."

	self isOff
		ifTrue: 
			[on := true.
			self changed.
			self doAction: onAction]! !

Model subclass: #ValueHolder
	instanceVariableNames: 'contents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Kernel-Remnants'!

!ValueHolder methodsFor: 'as yet unclassified' stamp: 'ls 8/5/1998 07:49'!
contents
	^contents! !

!ValueHolder methodsFor: 'as yet unclassified' stamp: 'sw 1/28/1999 12:35'!
contents: newContents
	contents := newContents.
	self contentsChanged! !

DisplayObject subclass: #Path
	instanceVariableNames: 'form collectionOfPoints'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Paths'!

!Path commentStamp: '<historical>' prior: 0!
I am the abstract superclass of the Graphic spatial primitives. I represent an ordered sequence of Points. Spatial primitives are used to generate "trajectories" such as lines and circles.!

Path subclass: #Arc
	instanceVariableNames: 'quadrant radius center'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Paths'!

!Arc commentStamp: '<historical>' prior: 0!
Arcs are an unusual implementation of splines due to Ted Kaehler.  Imagine two lines that meet at a corner. Now imagine two moving points; one moves from the corner to the end on one line, the other moves from the end of the other line in to the corner.  Now imagine a series of lines drawn between those moving points at each step along the way (they form a sort of spider web pattern).  By connecting segments of the intersecting lines, a smooth curve is achieved that is tangent to both of the original lines.  Voila.!

!Arc class methodsFor: 'examples' stamp: ''!
example
	"Click the button somewhere on the screen. The designated point will
	be the center of an Arc with radius 50 in the 4th quadrant."

	| anArc aForm |
	aForm := Form extent: 1 @ 30.	"make a long thin Form for display"
	aForm fillBlack.						"turn it black"
	anArc := Arc new.
	anArc form: aForm.					"set the form for display"
	anArc radius: 50.0.
	anArc center: Sensor waitButton.
	anArc quadrant: 4.
	anArc displayOn: Display.
	Sensor waitButton

	"Arc example"! !

!Arc methodsFor: 'accessing' stamp: ''!
center
	"Answer the point at the center of the receiver."

	^center! !

!Arc methodsFor: 'accessing' stamp: ''!
center: aPoint 
	"Set aPoint to be the receiver's center."

	center := aPoint! !

!Arc methodsFor: 'accessing' stamp: ''!
center: aPoint radius: anInteger 
	"The receiver is defined by a point at the center and a radius. The 
	quadrant is not reset."

	center := aPoint.
	radius := anInteger! !

!Arc methodsFor: 'accessing' stamp: ''!
center: aPoint radius: anInteger quadrant: section 
	"Set the receiver's quadrant to be the argument, section. The size of the 
	receiver is defined by the center and its radius."

	center := aPoint.
	radius := anInteger.
	quadrant := section! !

!Arc methodsFor: 'display box access' stamp: ''!
computeBoundingBox
	| aRectangle aPoint |
	aRectangle := center - radius + form offset extent: form extent + (radius * 2) asPoint.
	aPoint := center + form extent.
	quadrant = 1 ifTrue: [^ aRectangle encompass: center x @ aPoint y].
	quadrant = 2 ifTrue: [^ aRectangle encompass: aPoint x @ aPoint y].
	quadrant = 3 ifTrue: [^ aRectangle encompass: aPoint x @ center y].
	quadrant = 4 ifTrue: [^ aRectangle encompass: center x @ center y]! !

!Arc methodsFor: 'displaying' stamp: ''!
displayOn: aDisplayMedium at: aPoint clippingBox: clipRect rule: anInteger fillColor: aForm

	| nSegments line angle sin cos xn yn xn1 yn1 |
	nSegments := 12.0.
	line := Line new.
	line form: self form.
	angle := 90.0 / nSegments.
	sin := (angle * (2 * Float pi / 360.0)) sin.
	cos := (angle * (2 * Float pi / 360.0)) cos.
	quadrant = 1
		ifTrue: 
			[xn := radius asFloat.
			yn := 0.0].
	quadrant = 2
		ifTrue: 
			[xn := 0.0.
			yn := 0.0 - radius asFloat].
	quadrant = 3
		ifTrue: 
			[xn := 0.0 - radius asFloat.
			yn := 0.0].
	quadrant = 4
		ifTrue: 
			[xn := 0.0.
			yn := radius asFloat].
	nSegments asInteger
		timesRepeat: 
			[xn1 := xn * cos + (yn * sin).
			yn1 := yn * cos - (xn * sin).
			line beginPoint: center + (xn asInteger @ yn asInteger).
			line endPoint: center + (xn1 asInteger @ yn1 asInteger).
			line
				displayOn: aDisplayMedium
				at: aPoint
				clippingBox: clipRect
				rule: anInteger
				fillColor: aForm.
			xn := xn1.
			yn := yn1]! !

!Arc methodsFor: 'displaying' stamp: ''!
displayOn: aDisplayMedium transformation: aTransformation clippingBox: clipRect rule: anInteger fillColor: aForm

	| newArc tempCenter |
	newArc := Arc new.
	tempCenter := aTransformation applyTo: self center.
	newArc center: tempCenter x asInteger @ tempCenter y asInteger.
	newArc quadrant: self quadrant.
	newArc radius: (self radius * aTransformation scale x) asInteger.
	newArc form: self form.
	newArc
		displayOn: aDisplayMedium
		at: 0 @ 0
		clippingBox: clipRect
		rule: anInteger
		fillColor: aForm! !

!Arc methodsFor: 'accessing' stamp: ''!
quadrant
	"Answer the part of the circle represented by the receiver."
	^quadrant! !

!Arc methodsFor: 'accessing' stamp: ''!
quadrant: section 
	"Set the part of the circle represented by the receiver to be the argument, 
	section."

	quadrant := section! !

!Arc methodsFor: 'accessing' stamp: ''!
radius
	"Answer the receiver's radius."

	^radius! !

!Arc methodsFor: 'accessing' stamp: ''!
radius: anInteger 
	"Set the receiver's radius to be the argument, anInteger."

	radius := anInteger! !

Arc subclass: #Circle
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Paths'!

!Circle commentStamp: '<historical>' prior: 0!
I represent a full circle. I am made from four Arcs.!

!Circle class methodsFor: 'examples' stamp: ''!
exampleOne 
	"Click any button somewhere on the screen. The point will be the center
	of the circcle of radius 150."

	| aCircle aForm |
	aForm := Form extent: 1@30.
	aForm fillBlack.
	aCircle := Circle new.
	aCircle form: aForm.
	aCircle radius: 150.
	aCircle center: Sensor waitButton.
	aCircle displayOn: Display
	
	"Circle exampleOne"! !

!Circle class methodsFor: 'examples' stamp: ''!
exampleTwo
	"Designate a rectangular area that should be used as the brush for
	displaying the circle. Click any button at a point on the screen which
	will be the center location for the circle. The curve will be displayed
	with a long black form."

	| aCircle aForm |
	aForm := Form fromUser.
	aCircle := Circle new.
	aCircle form: aForm.
	aCircle radius: 150.
	aCircle center: Sensor waitButton.
	aCircle displayOn: Display at: 0 @ 0 rule: Form reverse
 
	 "Circle exampleTwo"! !

!Circle methodsFor: 'display box access' stamp: ''!
computeBoundingBox

	^center - radius + form offset extent: form extent + (radius * 2) asPoint! !

!Circle methodsFor: 'displaying' stamp: ''!
displayOn: aDisplayMedium at: aPoint clippingBox: clipRect rule: anInteger fillColor: aForm

	1 to: 4 do:
		[:i |
		super quadrant: i.
		super displayOn: aDisplayMedium
			at: aPoint
			clippingBox: clipRect
			rule: anInteger
			fillColor: aForm]! !

!Circle methodsFor: 'displaying' stamp: ''!
displayOn: aDisplayMedium transformation: aTransformation clippingBox: clipRect rule: anInteger fillColor: aForm

	1 to: 4 do:
		[:i |
		super quadrant: i.
		super displayOn: aDisplayMedium
			transformation: aTransformation
			clippingBox: clipRect
			rule: anInteger
			fillColor: aForm]! !

Path subclass: #CurveFitter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Paths'!

!CurveFitter commentStamp: '<historical>' prior: 0!
I represent a conic section determined by three points p1, p2 and p3. I interpolate p1 and p3 and am tangent to line p1, p2 at p1 and line p3, p2 at p3.!

!CurveFitter class methodsFor: 'examples' stamp: '6/9/97 10:16 di'!
example
	"Designate three locations on the screen by clicking any button. The
	curve determined by the points will be displayed with a long black form."

	| aCurveFitter aForm |  
	aForm := Form extent: 1@30.			"make a long thin Form for display "
	aForm fillBlack.							"turn it black"
	aCurveFitter := CurveFitter new.
	aCurveFitter form: aForm.						"set the form for display"
				"collect three Points and show them on the dispaly"
	aCurveFitter firstPoint: Sensor waitButton. Sensor waitNoButton.
	aForm displayOn: Display at: aCurveFitter firstPoint.
	aCurveFitter secondPoint: Sensor waitButton. Sensor waitNoButton.
	aForm displayOn: Display at: aCurveFitter secondPoint.
	aCurveFitter thirdPoint: Sensor waitButton. Sensor waitNoButton.
	aForm displayOn: Display at: aCurveFitter thirdPoint.

	aCurveFitter displayOn: Display					"display the CurveFitter"

	"CurveFitter example"! !

!CurveFitter class methodsFor: 'instance creation' stamp: ''!
new

	| newSelf | 
	newSelf := super new: 3.
	newSelf add: 0@0.
	newSelf add: 0@0.
	newSelf add: 0@0.
	^newSelf! !

!CurveFitter methodsFor: 'displaying' stamp: ''!
displayOn: aDisplayMedium at: aPoint clippingBox: clipRect rule: anInteger fillColor: aForm

	| pa pb k s p1 p2 p3 line |
	line := Line new.
	line form: self form.
	collectionOfPoints size < 3 ifTrue: [self error: 'Curve must have three points'].
	p1 := self firstPoint.
	p2 := self secondPoint.
	p3 := self thirdPoint.
	s := Path new.
	s add: p1.
	pa := p2 - p1.
	pb := p3 - p2.
	k := 5 max: pa x abs + pa y abs + pb x abs + pb y abs // 20.
	"k is a guess as to how many line segments to use to approximate 
	the curve."
	1 to: k do: 
		[:i | 
		s add: pa * i // k + p1 * (k - i) + (pb * (i - 1) // k + p2 * (i - 1)) // (k - 1)].
	s add: p3.
	1 to: s size - 1 do: 
		[:i | 
		line beginPoint: (s at: i).
		line endPoint: (s at: i + 1).
		line displayOn: aDisplayMedium
			at: aPoint
			clippingBox: clipRect
			rule: anInteger
			fillColor: aForm]! !

!CurveFitter methodsFor: 'displaying' stamp: '6/9/97 10:16 di'!
displayOn: aDisplayMedium transformation: aTransformation clippingBox: clipRect rule: anInteger fillColor: aForm

	| transformedPath newCurveFitter |
	transformedPath := aTransformation applyTo: self.
	newCurveFitter := CurveFitter new.
	newCurveFitter firstPoint: transformedPath firstPoint.
	newCurveFitter secondPoint: transformedPath secondPoint.
	newCurveFitter thirdPoint: transformedPath thirdPoint.
	newCurveFitter form: self form.
	newCurveFitter
		displayOn: aDisplayMedium
		at: 0 @ 0
		clippingBox: clipRect
		rule: anInteger
		fillColor: aForm! !

Path subclass: #Line
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Paths'!

!Line commentStamp: '<historical>' prior: 0!
I represent the line segment specified by two points.!

!Line class methodsFor: 'examples' stamp: ''!
example
	"Designate two places on the screen by clicking any mouse button. A
	straight path with a square black form will be displayed connecting the
	two selected points."

	| aLine aForm |  
	aForm := Form extent: 20@20.		"make a form one quarter of inch square"
	aForm fillBlack.							"turn it black"
	aLine := Line new.
	aLine form: aForm.						"use the black form for display"
	aLine beginPoint: Sensor waitButton. Sensor waitNoButton.
	aForm displayOn: Display at: aLine beginPoint.	
	aLine endPoint: Sensor waitButton.
	aLine displayOn: Display.				"display the line"

	"Line example"! !

!Line class methodsFor: 'instance creation' stamp: ''!
from: beginPoint to: endPoint withForm: aForm 
	"Answer an instance of me with end points begingPoint and endPoint; 
	the source form for displaying the line is aForm."

	| newSelf | 
	newSelf := super new: 2.
	newSelf add: beginPoint.
	newSelf add: endPoint.
	newSelf form: aForm.
	^newSelf! !

!Line class methodsFor: 'instance creation' stamp: ''!
new

	| newSelf | 
	newSelf := super new: 2.
	newSelf add: 0@0.
	newSelf add: 0@0.
	^newSelf! !

!Line methodsFor: 'accessing' stamp: ''!
beginPoint
	"Answer the first end point of the receiver."

	^self first! !

!Line methodsFor: 'accessing' stamp: ''!
beginPoint: aPoint 
	"Set the first end point of the receiver to be the argument, aPoint. 
	Answer aPoint."

	self at: 1 put: aPoint.
	^aPoint! !

!Line methodsFor: 'displaying' stamp: ''!
displayOn: aDisplayMedium at: aPoint clippingBox: clipRect rule: anInteger fillColor: aForm 
	"The form associated with this Path will be displayed, according  
	to one of the sixteen functions of two logical variables (rule), at  
	each point on the Line. Also the source form will be first anded  
	with aForm as a mask. Does not effect the state of the Path."

	collectionOfPoints size < 2 ifTrue: [self error: 'a line must have two points'].
	aDisplayMedium
		drawLine: self form
		from: self beginPoint + aPoint
		to: self endPoint + aPoint
		clippingBox: clipRect
		rule: anInteger
		fillColor: aForm! !

!Line methodsFor: 'displaying' stamp: ''!
displayOn: aDisplayMedium transformation: aTransformation clippingBox: clipRect rule: anInteger fillColor: aForm

	| newPath newLine |
	newPath := aTransformation applyTo: self.
	newLine := Line new.
	newLine beginPoint: newPath firstPoint.
	newLine endPoint: newPath secondPoint.
	newLine form: self form.
	newLine
		displayOn: aDisplayMedium
		at: 0 @ 0
		clippingBox: clipRect
		rule: anInteger
		fillColor: aForm! !

!Line methodsFor: 'displaying' stamp: ''!
displayOnPort: aPort at: aPoint 
	aPort sourceForm: self form; combinationRule: Form under; fillColor: nil.
	aPort drawFrom: collectionOfPoints first + aPoint
		to: collectionOfPoints last + aPoint! !

!Line methodsFor: 'accessing' stamp: ''!
endPoint
	"Answer the last end point of the receiver."

	^self last! !

!Line methodsFor: 'accessing' stamp: ''!
endPoint: aPoint 
	"Set the first end point of the receiver to be the argument, aPoint. 
	Answer aPoint."

	self at: 2 put: aPoint.
	^aPoint! !

Path subclass: #LinearFit
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Paths'!

!LinearFit commentStamp: '<historical>' prior: 0!
I represent a piece-wise linear approximation to a set of points in the plane.!

!LinearFit class methodsFor: 'examples' stamp: ''!
example
	"Select points on a Path using the red button. Terminate by selecting
	any other button. Creates a Path from the points and displays it as a
	piece-wise linear approximation." 

	| aLinearFit aForm flag |
	aLinearFit := LinearFit new.
	aForm := Form extent: 1 @ 40.
	aForm  fillBlack.
	aLinearFit form: aForm.
	flag := true.
	[flag] whileTrue:
		[Sensor waitButton.
		 Sensor redButtonPressed
			ifTrue: [aLinearFit add: Sensor waitButton. Sensor waitNoButton.
					aForm displayOn: Display at: aLinearFit last]
			ifFalse: [flag:=false]].
	aLinearFit displayOn: Display

	"LinearFit example"! !

!LinearFit methodsFor: 'displaying' stamp: 'jrm 9/7/1999 22:16'!
displayOn: aDisplayMedium at: aPoint clippingBox: clipRect rule: anInteger
fillColor: aForm
 
	| line |
	line := Line new.
	line form: self form.
	1 to: self size - 1 do: 
		[:i | 
		line beginPoint: (self at: i).
		line endPoint: (self at: i + 1).
		line displayOn: aDisplayMedium
			at: aPoint
			clippingBox: clipRect
			rule: anInteger
			fillColor: aForm]! !

!LinearFit methodsFor: 'displaying' stamp: 'jrm 9/7/1999 23:00'!
displayOn: aDisplayMedium transformation: aTransformation clippingBox:
clipRect rule: anInteger fillColor: aForm 

	| transformedPath |
	"get the scaled and translated Path."
	transformedPath := aTransformation applyTo: self.
	transformedPath
		displayOn: aDisplayMedium
		at: 0 @ 0
		clippingBox: clipRect
		rule: anInteger
		fillColor: aForm! !

!Path class methodsFor: 'examples' stamp: ''!
example
	"Creates a Path from mousePoints and displays it several ways on the display screen. Messes up the display. For learning about class Path, just select the code below and execute it to create a path and see it redisplayed in another place on the screen. Each path displays using a different form. A path is indicated by pressing the red mouse button in a sequence; press any other mouse button to terminate. "

	| aPath aForm pl fl flag |
	aForm := Form extent: 2 @ 40.		"creates a form one inch long"
	aForm fillBlack.							"turns it black"
	aPath := Path new.
	aPath form: aForm.						"use the long black form for displaying"
	flag := true.
	[flag]
		whileTrue: 
			[Sensor waitButton.
			Sensor redButtonPressed
				ifTrue: 
					[aPath add: Sensor waitButton.
					Sensor waitNoButton.
					aForm displayOn: Display at: aPath last]
				ifFalse: [flag := false]].
	Display fillWhite.
	aPath displayOn: Display.			"the original path"
	pl := aPath translateBy: 0 @ 100.
	fl := Form extent: 40 @ 40.
	fl fillGray.
	pl form: fl.
	pl displayOn: Display.				"the translated path"
	Sensor waitNoButton

	"Path example"! !

!Path class methodsFor: 'instance creation' stamp: ''!
new

	^self basicNew initializeCollectionOfPoints! !

!Path class methodsFor: 'instance creation' stamp: ''!
new: anInteger

	^self basicNew initializeCollectionOfPoints: anInteger! !

!Path methodsFor: 'adding' stamp: ''!
add: aPoint 
	"Include aPoint as one of the receiver's elements."

	collectionOfPoints add: aPoint! !

!Path methodsFor: 'accessing' stamp: ''!
at: index 
	"Answer the point on the receiver's path at position index."

	^collectionOfPoints at: index! !

!Path methodsFor: 'accessing' stamp: ''!
at: index put: aPoint 
	"Store the argument, aPoint, as the point on the receiver's path at position
	index."

	^collectionOfPoints at: index put: aPoint! !

!Path methodsFor: 'enumerating' stamp: ''!
collect: aBlock 
	"Evaluate aBlock with each of the receiver's elements as the argument. 
	Collect the resulting values into a path that is like the receiver. Answer 
	the new path."

	| newCollection |
	newCollection := collectionOfPoints collect: aBlock.
	newCollection form: self form.
	^newCollection! !

!Path methodsFor: 'display box access' stamp: ''!
computeBoundingBox 
	"Refer to the comment in DisplayObject|computeBoundingBox."

	| box |
	box := Rectangle origin: (self at: 1) extent: 0 @ 0.
	collectionOfPoints do: 
		[:aPoint | box := box merge: (Rectangle origin: aPoint extent: 0 @ 0)].
	^box! !

!Path methodsFor: 'displaying' stamp: ''!
displayOn: aDisplayMedium at: aDisplayPoint clippingBox: clipRectangle rule: ruleInteger fillColor: aForm 
	"Display this Path--offset by aPoint, clipped by clipRect and the form 
	associated with this Path will be displayedr according to one of the sixteen 
	functions of two logical variables (rule). Also the source form will be first 
	anded with aForm as a mask. Does not effect the state of the Path"

	collectionOfPoints do: 
		[:element | 
		self form
			displayOn: aDisplayMedium
			at: element + aDisplayPoint
			clippingBox: clipRectangle
			rule: ruleInteger
			fillColor: aForm]! !

!Path methodsFor: 'displaying' stamp: ''!
displayOn: aDisplayMedium transformation: displayTransformation clippingBox: clipRectangle rule: ruleInteger fillColor: aForm 
	"Displays this path, translated and scaled by aTransformation. Get the
	scaled and translated Path."

	| newPath transformedPath |
	transformedPath := displayTransformation applyTo: self.
	newPath := Path new.
	transformedPath do: [:point | newPath add: point].
	newPath form: self form.
	newPath
		displayOn: aDisplayMedium
		at: 0 @ 0
		clippingBox: clipRectangle
		rule: ruleInteger
		fillColor: aForm! !

!Path methodsFor: 'accessing' stamp: ''!
first
	"Answer the first point on the receiver's path; included to correspond to 
	OrderedCollection protocol."

	^collectionOfPoints first! !

!Path methodsFor: 'accessing' stamp: ''!
firstPoint
	"Answer the first point on the receiver's path."

	^collectionOfPoints first! !

!Path methodsFor: 'accessing' stamp: ''!
firstPoint: aPoint 
	"Replace the first element of the receiver with the new value aPoint. 
	Answer the argument aPoint."

	collectionOfPoints at: 1 put: aPoint.
	^aPoint! !

!Path methodsFor: 'accessing' stamp: ''!
form
	"Answer the receiver's form, or, if form is nil, then answer a 1 x 1 black 
	form (a black dot)."

	| aForm |
	form == nil
		ifTrue: 
			[aForm := Form extent: 1 @ 1.
			aForm fillBlack.
			^aForm]
		ifFalse: 
			[^form]! !

!Path methodsFor: 'accessing' stamp: ''!
form: aForm 
	"Make the argument, aForm, be the receiver's form."

	form := aForm! !

!Path methodsFor: 'private' stamp: ''!
initializeCollectionOfPoints

	collectionOfPoints := OrderedCollection new! !

!Path methodsFor: 'private' stamp: ''!
initializeCollectionOfPoints: anInteger

	collectionOfPoints := OrderedCollection new: anInteger! !

!Path methodsFor: 'testing' stamp: ''!
isEmpty

	^collectionOfPoints isEmpty! !

!Path methodsFor: 'accessing' stamp: ''!
last
	"Answer the last point on the receiver's path; included to correspond to 
	OrderedCollection protocol."

	^collectionOfPoints last! !

!Path methodsFor: 'accessing' stamp: ''!
offset
	"There are basically two kinds of display objects in the system: those
	that, when asked to transform themselves, create a new object; and those
	that side effect themselves by maintaining a record of the transformation
	request (typically an offset). Path, like Rectangle and Point, is a display
	object of the first kind."

	self shouldNotImplement! !

!Path methodsFor: 'removing' stamp: 'di 4/4/2000 12:33'!
removeAllSuchThat: aBlock 
	"Evaluate aBlock for each element of the receiver.
	Remove each element for which aBlock evaluates to true."

	collectionOfPoints removeAllSuchThat: aBlock.
! !

!Path methodsFor: 'transforming' stamp: 'jrm 9/1/1999 21:26'!
scaleBy: aPoint 
	"Answers a new Path scaled by aPoint. Does not affect the current data in 
	this Path."

	| newPath | 
	newPath := self species new: self size. 
	newPath form: self form.
	collectionOfPoints do: [:element | newPath add: (element scaleBy: aPoint)].
	^newPath! !

!Path methodsFor: 'accessing' stamp: ''!
secondPoint
	"Answer the second element of the receiver."

	^collectionOfPoints at: 2! !

!Path methodsFor: 'accessing' stamp: ''!
secondPoint: aPoint 
	"Replace the second element of the receiver with the new value aPoint. 
	Answer the argument aPoint."

	collectionOfPoints at: 2 put: aPoint.
	^aPoint! !

!Path methodsFor: 'enumerating' stamp: ''!
select: aBlock 
	"Evaluate aBlock with each of the receiver's elements as the argument. 
	Collect into a new path like the receiver only those elements for which 
	aBlock evaluates to true. Answer the new path."

	| newCollection |
	newCollection := collectionOfPoints select: aBlock.
	newCollection form: self form.
	^newCollection! !

!Path methodsFor: 'accessing' stamp: ''!
size
	"Answer the length of the receiver."

	^collectionOfPoints size! !

!Path methodsFor: 'accessing' stamp: ''!
thirdPoint
	"Answer the third element of the receiver."

	^collectionOfPoints at: 3! !

!Path methodsFor: 'accessing' stamp: ''!
thirdPoint: aPoint 
	"Replace the third element of the receiver with the new value aPoint. 
	Answer the argument aPoint."

	collectionOfPoints at: 3 put: aPoint.
	^aPoint! !

!Path methodsFor: 'transforming' stamp: 'jrm 9/1/1999 21:28'!
translateBy: aPoint 
	"Answers a new Path whose elements are translated by aPoint. Does not
	affect the elements of this Path."

	| newPath |
	newPath := self species new: self size.
	newPath form: self form.
	collectionOfPoints do: [:element | newPath add: (element translateBy: aPoint)].
	^newPath! !

Path subclass: #Spline
	instanceVariableNames: 'coefficients'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Paths'!

!Spline commentStamp: '<historical>' prior: 0!
I represent a collection of Points through which a cubic spline curve is fitted.!

!Spline class methodsFor: 'examples' stamp: '6/8/97 13:55 di'!
example
	"Designate points on the Path by clicking the red button. Terminate by
	pressing any other button. A curve will be displayed, through the
	selected points, using a long black form."

	| splineCurve aForm flag|
	aForm := Form extent: 2@2.
	aForm  fillBlack.
	splineCurve := Spline new.
	splineCurve form: aForm.
	flag := true.
	[flag] whileTrue:
		[Sensor waitButton.
		 Sensor redButtonPressed
			ifTrue: 
				[splineCurve add: Sensor waitButton. 
				 Sensor waitNoButton.
				 aForm displayOn: Display at: splineCurve last]
			ifFalse: [flag:=false]].
	splineCurve computeCurve.
	splineCurve isEmpty 
		ifFalse: [splineCurve displayOn: Display.
				Sensor waitNoButton].
 
	"Spline example"! !

!Spline methodsFor: 'accessing' stamp: ''!
coefficients
	"Answer an eight-element Array of Arrays each of which is the length 
	of the receiver. The first four arrays are the values, first, second and 
	third derivatives, respectively, for the parametric spline in x. The last 
	four elements are for y."

	^coefficients! !

!Spline methodsFor: 'displaying' stamp: ''!
computeCurve
	"Compute an array for the coefficients."

	| length extras |
	length := self size.
	extras := 0.
	coefficients := Array new: 8.
	1 to: 8 do: [:i | coefficients at: i put: (Array new: length + extras)].
	1 to: 5 by: 4 do: 
		[:k | 
		1 to: length do:
			[:i | (coefficients at: k)
					at: i put: (k = 1
						ifTrue: [(self at: i) x asFloat]
						ifFalse: [(self at: i) y asFloat])].
			1 to: extras do: [:i | (coefficients at: k)
					at: length + i put: ((coefficients at: k)
						at: i + 1)].
			self derivs: (coefficients at: k)
				first: (coefficients at: k + 1)
				second: (coefficients at: k + 2)
				third: (coefficients at: k + 3)].
	extras > 0 
		ifTrue: [1 to: 8 do: 
					[:i | 
					coefficients at: i put: ((coefficients at: i)
											copyFrom: 2 to: length + 1)]]! !

!Spline methodsFor: 'private' stamp: ''!
derivs: a first: point1 second: point2 third: point3
	"Compute the first, second and third derivitives (in coefficients) from
	the Points in this Path (coefficients at: 1 and coefficients at: 5)."

	| l v anArray |
	l := a size.
	l < 2 ifTrue: [^self].
	l > 2
	  ifTrue:
		[v := Array new: l.
		 v  at:  1 put: 4.0.
		 anArray := Array new: l.
		 anArray  at:  1 put: (6.0 * ((a  at:  1) - ((a  at:  2) * 2.0) + (a  at:  3))).
		 2 to: l - 2 do:
			[:i | 
			v  at:  i put: (4.0 - (1.0 / (v  at:  (i - 1)))).
			anArray
				at:  i 
				put: (6.0 * ((a  at:  i) - ((a  at:  (i + 1)) * 2.0) + (a  at:  (i + 2)))
						- ((anArray  at:  (i - 1)) / (v  at:  (i - 1))))].
		 point2  at: (l - 1) put: ((anArray  at:  (l - 2)) / (v  at:  (l - 2))).
		 l - 2 to: 2 by: 0-1 do: 
			[:i | 
			point2 
				at: i 
				put: ((anArray  at:  (i - 1)) - (point2  at:  (i + 1)) / (v  at:  (i - 1)))]].
	point2 at: 1 put: (point2  at:  l put: 0.0).
	1 to: l - 1 do:
		[:i | point1 
				at: i 
				put: ((a at: (i + 1)) - (a  at:  i) - 
						((point2  at:  i) * 2.0 + (point2  at:  (i + 1)) / 6.0)).
		      point3 at: i put: ((point2  at:  (i + 1)) - (point2  at:  i))]! !

!Spline methodsFor: 'displaying' stamp: ''!
displayOn: aDisplayMedium at: aPoint clippingBox: clipRect rule: anInteger fillColor: aForm 
	"Display the receiver, a spline curve, approximated by straight line
	segments."

	| n line t x y x1 x2 x3 y1 y2 y3 |
	collectionOfPoints size < 1 ifTrue: [self error: 'a spline must have at least one point'].
	line := Line new.
	line form: self form.
	line beginPoint: 
		(x := (coefficients at: 1) at: 1) rounded @ (y := (coefficients at: 5) at: 1) rounded.
	1 to: (coefficients at: 1) size - 1 do: 
		[:i | 
		"taylor series coefficients"
		x1 := (coefficients at: 2) at: i.
		y1 := (coefficients at: 6) at: i.
		x2 := ((coefficients at: 3) at: i) / 2.0.
		y2 := ((coefficients at: 7) at: i) / 2.0.
		x3 := ((coefficients at: 4) at: i) / 6.0.
		y3 := ((coefficients at: 8) at: i) / 6.0.
		"guess n"
		n := 5 max: (x2 abs + y2 abs * 2.0 + ((coefficients at: 3)
							at: i + 1) abs + ((coefficients at: 7)
							at: i + 1) abs / 100.0) rounded.
		1 to: n - 1 do: 
			[:j | 
			t := j asFloat / n.
			line endPoint: 
				(x3 * t + x2 * t + x1 * t + x) rounded 
							@ (y3 * t + y2 * t + y1 * t + y) rounded.
			line
				displayOn: aDisplayMedium
				at: aPoint
				clippingBox: clipRect
				rule: anInteger
				fillColor: aForm.
			line beginPoint: line endPoint].
		line beginPoint: 
				(x := (coefficients at: 1) at: i + 1) rounded 
					@ (y := (coefficients at: 5) at: i + 1) rounded.
		line
			displayOn: aDisplayMedium
			at: aPoint
			clippingBox: clipRect
			rule: anInteger
			fillColor: aForm]! !

!Spline methodsFor: 'displaying' stamp: ''!
displayOn: aDisplayMedium transformation: aTransformation clippingBox: clipRect rule: anInteger fillColor: aForm 
	"Get the scaled and translated path of newKnots."

	| newKnots newSpline |
	newKnots := aTransformation applyTo: self.
	newSpline := Spline new.
	newKnots do: [:knot | newSpline add: knot].
	newSpline form: self form.
	newSpline
		displayOn: aDisplayMedium
		at: 0 @ 0
		clippingBox: clipRect
		rule: anInteger
		fillColor: aForm! !

TestCase subclass: #ST80MenusTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Menus-Tests'!

!ST80MenusTest commentStamp: 'tlk 5/2/2006 22:41' prior: 0!
I am an SUnit Test of PopUpMenu and FillInTheBlank.  The original motivation for my creation was the regression of functionality associated with allowing the non-interactive testing of these menus.  

My fixtures are: None

NOTES ABOUT AUTOMATING USER INPUTS (See MethodContextTest also for a discussion of this functionality.)

When executing non-interactive programs you will inevitably run into programs  that require user input during their execution and these sort of problems shoot the whole non-interactiveness of your enclosing program. This is particularly true in doing Sunit tests.

PopUpMenu and FillInTheBlankMorph were modified to first signal a ProvideAnswerNotification and if someone handles that (e.g. the enclosing block) then the user interaction will be avoided and a answer provided by an array will be used instead. PopUpMenu and FillInTheBlankMorph take advantage of BlockContext helper methods have been made available and tests of these helpers are provided in this class to demonstrate that it can intercept  requests for user interaction.  Of course,

 The basic syntax looks like:

	[self confirm: 'Install spyware?'] valueSupplyingAnswer: #('Install spyware?' false)

There a few variants on this theme making it easy to provide a literal list of answers for the block so that you can handle a bunch of questions in a block with appropriate answers.

Additionally, it is possible to suppress Object>>inform: modal dialog boxes as these get in the way of automating anything.  After applying this changeset you should be able to tryout the following code snippets to see the variants on this theme that are available.

Examples:

So you don't need any introduction here -- this one works like usual.
[self inform: 'hello'. #done] value.

Now let's suppress all inform: messages.
[self inform: 'hello'; inform: 'there'. #done] valueSuppressingAllMessages.

Here we can just suppress a single inform: message.
[self inform: 'hi'; inform: 'there'. #done] valueSuppressingMessages: #('there')

Here you see how you can suppress a list of messages.
[self inform: 'hi'; inform: 'there'; inform: 'bill'. #done] valueSuppressingMessages: #('hi' 'there')

Enough about inform:, let's look at confirm:. As you see this one works as expected.
[self confirm: 'You like Squeak?'] value

Let's supply answers to one of the questions -- check out the return value.
[{self confirm: 'You like Smalltalk?'. self confirm: 'You like Squeak?'}]
	valueSupplyingAnswer: #('You like Smalltalk?' true)

Here we supply answers using only substrings of the questions (for simplicity).
[{self confirm: 'You like Squeak?'. self confirm: 'You like MVC?'}]
	valueSupplyingAnswers: #( ('Squeak' true) ('MVC' false) )

This time let's answer all questions exactly the same way.
[{self confirm: 'You like Squeak?'. self confirm: 'You like Morphic?'}]
	valueSupplyingAnswer: true
	
And, of course, we can answer FillInTheBlank questions in the same manner.
[FillInTheBlank request: 'What day is it?']
	valueSupplyingAnswer: 'the first day of the rest of your life'
	
We can also return whatever the initialAnswer of the FillInTheBlank was by using the #default answer.
[FillInTheBlank request: 'What day is it?' initialAnswer: DateAndTime now dayOfWeekName]
	valueSupplyingAnswer: #default
	
Finally, you can also do regex matches on any of the question text (or inform text) (should you have VB-Regex enhancements in your image).
[FillInTheBlank request: 'What day is it?']
	valueSupplyingAnswers: { {'What day.*\?'. DateAndTime now dayOfWeekName} }!

!ST80MenusTest methodsFor: 'testing' stamp: 'tlk 5/2/2006 22:46'!
testSupplyAnswerOfFillInTheBlank

	self should: ['blue' = ([UIManager default request: 'Should not see this message or this test failed?'] 
		valueSupplyingAnswer: #('Should not see this message or this test failed?' 'blue'))]! !

!ST80MenusTest methodsFor: 'tests' stamp: 'tlk 5/2/2006 22:42'!
testSupplySpecificAnswerToQuestion

	self should: [false = ([self confirm: 'Should not see this message or this test failed?'] 
		valueSupplyingAnswer: #('Should not see this message or this test failed?' false))]! !

!ST80MenusTest methodsFor: 'tests' stamp: 'tlk 5/2/2006 22:28'!
testSuppressInform

	self should: [[nil inform: 'Should not see this message or this test failed!!'] valueSuppressingAllMessages isNil]! !

TestCase subclass: #TestIndenting
	instanceVariableNames: 'para'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Support-Tests'!

!TestIndenting methodsFor: 'running' stamp: 'hmm 2/2/2001 14:29'!
setUp
	| text |
	text := 'p	' asText, (Text string: 'word word' attribute: (TextIndent tabs: 1)).
	para := text asParagraph! !

!TestIndenting methodsFor: 'testing' stamp: 'hmm 2/2/2001 14:42'!
testBreak1
	"Checks whether the beginning of a new line starts at the indented position"
	| cb |
	para compositionRectangle: (0@0 extent: para width - 1@100); updateCompositionHeight.
	para clippingRectangle: (0@0 extent: 200@200).
	cb := para characterBlockForIndex: 8.
	self assert: cb top > 0.
	self assert: cb left = 24! !

!TestIndenting methodsFor: 'testing' stamp: 'hmm 2/2/2001 14:45'!
testBreak2
	"When an indented line is broken at a space, the character block must still lie in the line crossing the right margin."
	| cb |
	para compositionRectangle: (0@0 extent: para width - 24 // 2 + 24@100); updateCompositionHeight.
	para clippingRectangle: (0@0 extent: 200@200).
	cb := para characterBlockForIndex: 7.
	self assert: cb top = 0.
	self assert: cb left >= 24! !

!TestIndenting methodsFor: 'testing' stamp: 'tlk 5/7/2006 17:14'!
testBreakAtSpaceLeavesSpaceOnOriginalLine
	"When an indented line is broken at a space, the character block must still lie in the line crossing the right margin."
	| cb |
	para compositionRectangle: (0@0 extent: para width - 24 @100); updateCompositionHeight.
	para clippingRectangle: (0@0 extent: 200@200).
	cb := para characterBlockForIndex: 7.
	self assert: cb top = 0.
	self assert: cb left >= 24! !

!TestIndenting methodsFor: 'testing' stamp: 'hmm 2/2/2001 14:41'!
testCR
	"Checks whether the beginning of a new line starts at the indented position"
	| cb |
	para replaceFrom: 7 to: 7 with: (String with: Character cr) displaying: false.
	para clippingRectangle: (0@0 extent: 200@200).
	cb := para characterBlockForIndex: 8.
	self assert: cb top > 0.
	self assert: cb left = 24! !

!TestIndenting methodsFor: 'testing' stamp: 'hmm 2/2/2001 14:41'!
testCR2
	"Checks whether the drawing of indented text is really indented..."
	| cb |
	para replaceFrom: 7 to: 7 with: (String with: Character cr) displaying: false.
	para clippingRectangle: (0@0 extent: 200@200).
	cb := para characterBlockForIndex: 8.
	self assert: (para asForm copy: (0@cb top extent: 24@cb height)) isAllWhite! !

!TestIndenting methodsFor: 'testing' stamp: 'hmm 2/2/2001 15:17'!
testCR3
	"Checks whether the beginning of a new line starts at the indented position"
	| cb |
	para replaceFrom: 11 to: 11 with: (Text string: (String with: Character cr) attribute: (TextIndent tabs: 1)) displaying: false.
	para clippingRectangle: (0@0 extent: 200@200).
	cb := para characterBlockForIndex: 12.
	self assert: cb top > 0.
	self assert: cb left = 24! !

!TestIndenting methodsFor: 'testing' stamp: 'tlk 5/7/2006 16:52'!
testNewLineAndTabProvidesDoubleIndent
	"Checks whether the beginning of a new line starts at the indented position"
	| cb |
	para replaceFrom: 11 to: 11 with: (Text string: (String with: Character cr) attribute: (TextIndent tabs: 1)) displaying: false.
 
	cb := para characterBlockForIndex: 12.
	self assert: cb top > 0.
	self assert: cb left = 24! !

!TestIndenting methodsFor: 'testing' stamp: 'tlk 5/7/2006 16:52'!
testNewLineLeaveSpacesOnOldLine
	"Checks whether the drawing of indented text is really indented..."
	| cb |
	para replaceFrom: 7 to: 7 with: (String with: Character cr) displaying: false.

	cb := para characterBlockForIndex: 8.
	self assert: (para asForm copy: (0@cb top extent: 24@cb height)) isAllWhite! !

!TestIndenting methodsFor: 'testing' stamp: 'tlk 5/7/2006 16:53'!
testNewLineStartsIndented
	"Checks whether the beginning of a new line starts at the indented position"
	| cb |
	para replaceFrom: 7 to: 7 with: (String with: Character cr) displaying: false.
 
	cb := para characterBlockForIndex: 8.
	self assert: cb top > 0.
	self assert: cb left = 24! !

!TestIndenting methodsFor: 'testing' stamp: 'tlk 5/7/2006 17:11'!
testNewLineStartsIndentedWhenWrapped
	"Checks whether the beginning of a new line starts at the indented position"
	| cb |
	para compositionRectangle: (0@0 extent: para width - 20@100); updateCompositionHeight.
	para clippingRectangle: (0@0 extent: 200@200).
	cb := para characterBlockForIndex: 8.
	self assert: cb top > 0.
	self assert: cb left = 24! !

!TestIndenting methodsFor: 'testing' stamp: 'tlk 5/7/2006 16:36'!
testSetUp
	"just reminding us all what the paragraph looks like to begin with. assuming Accuny12 font "
	| cb |

	
	cb := para characterBlockForIndex: 1.  "p"
	self assert: cb top = 0.
	self assert: cb left = 0.
	self assert: cb right = 7.
	
	
	cb := para characterBlockForIndex: 2.  "the tab"
	self assert: cb top = 0.
	self assert: cb left = 7.
	self assert: cb right = 24.

	
	cb := para characterBlockForIndex: 3.  "w" 
	self assert: cb top = 0.
	self assert: cb left = 24.
	self assert: cb right = 34.
	
	cb := para characterBlockForIndex: 7.  " " "between word and word"
	self assert: cb top = 0.
	self assert: cb left = 52.
	self assert: cb right = 57.
	
	cb := para characterBlockForIndex: 11.  "d" "last char"
	self assert: cb top = 0.
	self assert: cb left = 79.
	self assert: cb right = 85.
	
		
! !

TestCase subclass: #TestParagraphFix
	instanceVariableNames: 'para'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Support-Tests'!

!TestParagraphFix commentStamp: '<historical>' prior: 0!
This class tests whether locating characters past the end of a text is possible in all cases.!

TestParagraphFix subclass: #TestNewParagraphFix
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Support-Tests'!

!TestNewParagraphFix commentStamp: '<historical>' prior: 0!
This class tests the same things as its superclass, but for NewParagraph which is used in the Morphic environment.!

!TestNewParagraphFix methodsFor: 'running' stamp: 'hmm 10/1/2000 17:41'!
setUp
	| morph |
	morph := TextMorph new contents: 'i i'.
	morph fit.
	para := morph paragraph! !

!TestNewParagraphFix methodsFor: 'tests' stamp: 'hmm 10/1/2000 17:42'!
testCharacterBlockAfterReplacingAll
	^super testCharacterBlockAfterReplacingAll! !

!TestNewParagraphFix methodsFor: 'tests' stamp: 'hmm 10/1/2000 17:42'!
testCharacterBlockAfterReplacingOther
	^super testCharacterBlockAfterReplacingOther! !

!TestNewParagraphFix methodsFor: 'tests' stamp: 'hmm 10/1/2000 17:42'!
testCharacterBlockAfterReplacingSpace
	^super testCharacterBlockAfterReplacingSpace! !

!TestNewParagraphFix methodsFor: 'tests' stamp: 'hmm 10/1/2000 17:43'!
testCharacterBlockNormal
	^super testCharacterBlockNormal! !

!TestParagraphFix methodsFor: 'running' stamp: 'hmm 10/1/2000 15:05'!
setUp
	para := 'i i' asParagraph! !

!TestParagraphFix methodsFor: 'tests' stamp: 'hmm 10/1/2000 15:52'!
testCharacterBlockAfterReplacingAll
	para replaceFrom: 1 to: 3 with: 'mmm' displaying: false.
	self assert: (para characterBlockForIndex: 4) stringIndex = 4! !

!TestParagraphFix methodsFor: 'tests' stamp: 'hmm 10/1/2000 15:05'!
testCharacterBlockAfterReplacingOther
	para replaceFrom: 3 to: 3 with: 'm' displaying: false.
	self assert: (para characterBlockForIndex: 4) stringIndex = 4! !

!TestParagraphFix methodsFor: 'tests' stamp: 'hmm 10/1/2000 15:05'!
testCharacterBlockAfterReplacingSpace
	para replaceFrom: 3 to: 3 with: ' ' displaying: false.
	self assert: (para characterBlockForIndex: 4) stringIndex = 4! !

!TestParagraphFix methodsFor: 'tests' stamp: 'hmm 10/1/2000 15:05'!
testCharacterBlockNormal
	self assert: (para characterBlockForIndex: 4) stringIndex = 4! !

Object subclass: #Clipboard
	instanceVariableNames: 'contents recent interpreter'
	classVariableNames: 'Default'
	poolDictionaries: ''
	category: 'ST80-Kernel-Remnants'!

!Clipboard commentStamp: '<historical>' prior: 0!
The Clipboard class implements a basic buffering scheme for text. The currently selected text is also exported to the OS so that text can be copied from and to other applications. Commonly only a single instance is used (the default clipboard) but applications are free to use other than the default clipboard if necessary.!

!Clipboard class methodsFor: 'accessing' stamp: 'ar 1/15/2001 18:45'!
chooseRecentClipping  "Clipboard chooseRecentClipping"
	"Choose by menu from among the recent clippings"
	^self default chooseRecentClipping! !

!Clipboard class methodsFor: 'class initialization' stamp: 'yo 8/11/2003 22:43'!
clearInterpreters

	self allInstances do: [:each | each clearInterpreter].
! !

!Clipboard class methodsFor: 'accessing' stamp: 'ar 1/15/2001 18:35'!
clipboardText "Clipboard clipboardText"
	^self default clipboardText.! !

!Clipboard class methodsFor: 'accessing' stamp: 'ar 1/15/2001 18:35'!
clipboardText: aText 
	^self default clipboardText: aText! !

!Clipboard class methodsFor: 'accessing' stamp: 'ar 1/15/2001 18:33'!
default
	^Default ifNil:[Default := self new].! !

!Clipboard class methodsFor: 'accessing' stamp: 'ar 1/15/2001 18:48'!
default: aClipboard
	"So that clients can switch between different default clipboards"
	Default := aClipboard.! !

!Clipboard class methodsFor: 'class initialization' stamp: 'yo 12/29/2003 01:03'!
startUp

	self clearInterpreters.
! !

!Clipboard methodsFor: 'accessing' stamp: 'ar 1/15/2001 18:32'!
chooseRecentClipping  "Clipboard chooseRecentClipping"
	"Choose by menu from among the recent clippings"

	recent ifNil: [^ nil].
	^ (SelectionMenu
		labelList: (recent collect: [:txt | ((txt asString contractTo: 50)
									copyReplaceAll: Character cr asString with: '\')
									copyReplaceAll: Character tab asString with: '|'])
		selections: recent) startUp.

! !

!Clipboard methodsFor: 'accessing' stamp: 'yo 8/11/2003 19:07'!
clearInterpreter

	interpreter := nil.
! !

!Clipboard methodsFor: 'accessing' stamp: 'yo 8/11/2003 19:04'!
clipboardText
	"Return the text currently in the clipboard. If the system clipboard is empty, or if it differs from the Smalltalk clipboard text, use the Smalltalk clipboard. This is done since (a) the Mac clipboard gives up on very large chunks of text and (b) since not all platforms support the notion of a clipboard."

	| string decodedString |
	string := self primitiveClipboardText.
	(string isEmpty
			or: [string = contents asString])
		ifTrue: [^ contents].
	decodedString := self interpreter fromSystemClipboard: string.
	^ decodedString = contents asString
		ifTrue: [contents]
		ifFalse: [decodedString asText].
! !

!Clipboard methodsFor: 'accessing' stamp: 'yo 8/11/2003 19:12'!
clipboardText: text 

	| string |
	string := text asString.
	self noteRecentClipping: text asText.
	contents := text asText.
	string := self interpreter toSystemClipboard: string.
	self primitiveClipboardText: string.
! !

!Clipboard methodsFor: 'initialize' stamp: 'ar 1/15/2001 18:34'!
initialize
	contents := '' asText.
	recent := OrderedCollection new.! !

!Clipboard methodsFor: 'accessing' stamp: 'yo 8/11/2003 18:23'!
interpreter

	interpreter ifNil: [self setInterpreter].
	^ interpreter.
! !

!Clipboard methodsFor: 'private' stamp: 'ar 1/15/2001 18:34'!
noteRecentClipping: text
	"Keep most recent clippings in a queue for pasteRecent (paste... command)"
	text isEmpty ifTrue: [^ self].
	text size > 50000 ifTrue: [^ self].
	(recent includes: text) ifTrue: [^ self].
	recent addFirst: text.
	[recent size > 5] whileTrue: [recent removeLast].
! !

!Clipboard methodsFor: 'primitives' stamp: 'ar 1/15/2001 18:28'!
primitiveClipboardText
	"Get the current clipboard text. Return the empty string if the primitive fails."
	<primitive: 141>
	^ ''! !

!Clipboard methodsFor: 'primitives' stamp: 'ar 1/15/2001 18:30'!
primitiveClipboardText: aString
	"Set the current clipboard text to the given string."

	<primitive: 141>
	"don't fail if the primitive is not implemented"! !

!Clipboard methodsFor: 'accessing' stamp: 'mir 7/20/2004 15:44'!
setInterpreter

	interpreter := LanguageEnvironment defaultClipboardInterpreter.
	interpreter ifNil: [
		"Should never be reached, but just in case."
		interpreter := NoConversionClipboardInterpreter new].
! !

Object subclass: #ControlManager
	instanceVariableNames: 'scheduledControllers activeController activeControllerProcess screenController newTopClicked'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Kernel-Remnants'!

!ControlManager commentStamp: '<historical>' prior: 0!
I represent the top level control over scheduling which controller of a view on the screen the user is actively using. ScheduledControllers is the global reference to an instance of me, the one attached to the Project currently being used.!

!ControlManager classSide methodsFor: 'exchange' stamp: ''!
newScheduler: controlManager
	"When switching projects, the control scheduler has to be exchanged. The 
	active one is the one associated with the current project."

	Smalltalk at: #ScheduledControllers put: controlManager.
	ScheduledControllers restore.
	controlManager searchForActiveController! !

!ControlManager classSide methodsFor: 'snapshots' stamp: 'di 2/4/1999 15:16'!
shutDown  "Saves space in snapshots"

	Smalltalk isMorphic ifFalse: [ScheduledControllers unCacheWindows]! !

!ControlManager classSide methodsFor: 'snapshots' stamp: 'di 2/4/1999 09:00'!
startUp
	Smalltalk isMorphic ifFalse: [ScheduledControllers restore]! !

!ControlManager methodsFor: 'scheduling' stamp: ''!
activateController: aController
	"Make aController, which must already be a scheduled controller, the active window.  5/8/96 sw"

	self activeController: aController.
	(activeController view labelDisplayBox
		intersect: Display boundingBox) area < 200
			ifTrue: [activeController move].
	Processor terminateActive! !

!ControlManager methodsFor: 'scheduling' stamp: ''!
activateTranscript
	"There is known to be a Transcript open in the current project; activate it.  2/5/96 sw"

	| itsController |
	itsController := scheduledControllers detect:
			[:controller | controller model == Transcript]
		ifNone:
			[^ self].

	self activeController: itsController.
	(activeController view labelDisplayBox
			intersect: Display boundingBox) area < 200
				ifTrue: [activeController move].
	Processor terminateActive! !

!ControlManager methodsFor: 'accessing' stamp: ''!
activeController
	"Answer the currently active controller."

	^activeController! !

!ControlManager methodsFor: 'accessing' stamp: 'ar 6/5/1998 21:49'!
activeController: aController 
	"Set aController to be the currently active controller. Give the user 
	control in it."
	<primitive: 19> "Simulation guard"
	activeController := aController.
	(activeController == screenController)
		ifFalse: [self promote: activeController].
	activeControllerProcess := 
			[activeController startUp.
			self searchForActiveController] newProcess.
	activeControllerProcess priority: Processor userSchedulingPriority.
	activeControllerProcess resume! !

!ControlManager methodsFor: 'accessing' stamp: ''!
activeControllerNoTerminate: aController andProcess: aProcess
	"Set aController to be the currently active controller and aProcess to be 
	the the process that handles controller scheduling activities in the 
	system. This message differs from activeController:andProcess: in that it 
	does not send controlTerminate to the currently active controller."

	self inActiveControllerProcess
		ifTrue: 
			[aController~~nil
				ifTrue: [(scheduledControllers includes: aController)
							ifTrue: [self promote: aController]
							ifFalse: [self error: 'Old controller not scheduled']].
			activeController := aController.
			activeController == nil
				ifFalse: [activeController controlInitialize].
			activeControllerProcess := aProcess.
			activeControllerProcess resume]
		ifFalse: 
			[self error: 'New active controller process must be set from old one'] ! !

!ControlManager methodsFor: 'accessing' stamp: ''!
activeControllerProcess
	"Answer the process that is currently handling controller scheduling 
	activities in the system."

	^activeControllerProcess! !

!ControlManager methodsFor: 'displaying' stamp: ''!
backgroundForm: aForm
	screenController view model: aForm.
	ScheduledControllers restore
"
	QDPen new mandala: 30 diameter: 640.
	ScheduledControllers backgroundForm:
		(Form fromDisplay: Display boundingBox).

	ScheduledControllers backgroundForm:
		(InfiniteForm with: Form gray).
"! !

!ControlManager methodsFor: 'accessing' stamp: ''!
controllerSatisfying: aBlock
	"Return the first scheduled controller which satisfies the 1-argument boolean-valued block, or nil if none.  7/25/96 sw"

	scheduledControllers do:
		[:aController | (aBlock value: aController) == true ifTrue: [^ aController]].
	^ nil! !

!ControlManager methodsFor: 'accessing' stamp: ''!
controllerWhoseModelSatisfies: aBlock
	"Return the first scheduled controller whose model satisfies the 1-argument boolean-valued block, or nil if none.  5/6/96 sw"

	scheduledControllers do:
		[:aController | (aBlock value: aController model) == true ifTrue: [^ aController]].
	^ nil! !

!ControlManager methodsFor: 'accessing' stamp: 'sw 5/4/2001 23:20'!
controllersSatisfying: aBlock
	"Return a list of scheduled controllers satisfying aBlock"

	^ (scheduledControllers ifNil: [^ #()]) select:
		[:aController | (aBlock value: aController) == true]! !

!ControlManager methodsFor: 'scheduling' stamp: 'di 5/19/1998 09:03'!
findWindow
	"Present a menu of window titles, and activate the one that gets chosen."

	^ self findWindowSatisfying: [:c | true]! !

!ControlManager methodsFor: 'scheduling' stamp: 'rbb 2/18/2005 10:50'!
findWindowSatisfying: aBlock
	"Present a menu of window titles, and activate the one that gets chosen"

	| sortAlphabetically controllers listToUse labels index |
	sortAlphabetically := Sensor shiftPressed.
	controllers := OrderedCollection new.
	scheduledControllers do: [:controller |
		controller == screenController ifFalse:
			[(aBlock value: controller) ifTrue: [controllers addLast: controller]]].
	controllers size == 0 ifTrue: [^ self].
	listToUse := sortAlphabetically
		ifTrue: [controllers asSortedCollection: [:a :b | a view label < b view label]]
		ifFalse: [controllers].
	labels := String streamContents:
		[:strm | 
			listToUse do: [:controller | strm nextPutAll: (controller view label contractTo: 40); cr].
		strm skip: -1  "drop last cr"].
	index := (UIManager default chooseFrom: (labels findTokens: Character cr) asArray).
	index > 0 ifTrue:
		[self activateController: (listToUse at: index)].
! !

!ControlManager methodsFor: 'scheduling' stamp: ''!
inActiveControllerProcess
	"Answer whether the active scheduling process is the actual active 
	process in the system."

	^activeControllerProcess == Processor activeProcess! !

!ControlManager methodsFor: 'accessing' stamp: ''!
includes: aController
	^ scheduledControllers includes: aController! !

!ControlManager methodsFor: 'initialize-release' stamp: ''!
initialize
	"Initialize the receiver to refer to only the background controller."
	| screenView |
	screenController := ScreenController new.
	screenView := FormView new.
	screenView model: (InfiniteForm with: Color gray) controller: screenController.
	screenView window: Display boundingBox.
	scheduledControllers := OrderedCollection with: screenController! !

!ControlManager methodsFor: 'scheduling' stamp: 'dtl 4/4/2005 06:42'!
interruptName: labelString
	"Create a Notifier on the active scheduling process with the given label. Make the Notifier the active controller."

	^ self interruptName: labelString preemptedProcess: nil
! !

!ControlManager methodsFor: 'scheduling' stamp: 'dtl 4/6/2005 23:20'!
interruptName: labelString preemptedProcess: theInterruptedProcess
	"Create a Notifier on the active scheduling process with the given label. Make the Notifier the active controller."
	| suspendingList newActiveController preemptedProcess |

	preemptedProcess := theInterruptedProcess ifNil: [Processor preemptedProcess].
	preemptedProcess == activeControllerProcess
		ifFalse: [(suspendingList := preemptedProcess suspendingList) == nil
				ifTrue: [preemptedProcess suspend]
				ifFalse: [suspendingList remove: preemptedProcess.
						preemptedProcess offList]].

	(suspendingList := activeControllerProcess suspendingList) == nil
		ifTrue: [activeControllerProcess == Processor activeProcess
					ifTrue: [activeControllerProcess suspend]]
		ifFalse: [suspendingList remove: activeControllerProcess ifAbsent:[].
				activeControllerProcess offList].

	activeController ~~ nil ifTrue: [
		"Carefully de-emphasis the current window."
		activeController view topView deEmphasizeForDebugger].

	newActiveController :=
		(Debugger
			openInterrupt: labelString
			onProcess: preemptedProcess) controller.
	newActiveController centerCursorInView.
	self activeController: newActiveController.
! !

!ControlManager methodsFor: 'private' stamp: ''!
nextActiveController
	"Answer the controller that would like control.  
	If there was a click outside the active window, it's the top window
	that now has the mouse, otherwise it's just the top window."

	(newTopClicked notNil and: [newTopClicked])
		ifTrue: [newTopClicked := false.
				^ scheduledControllers 
					detect: [:aController | aController isControlWanted]
					ifNone: [scheduledControllers first]]
		ifFalse: [^ scheduledControllers first]! !

!ControlManager methodsFor: 'accessing' stamp: ''!
noteNewTop
	newTopClicked := true! !

!ControlManager methodsFor: 'scheduling' stamp: ''!
promote: aController
	"Make aController be the first scheduled controller in the ordered 
	collection."
	
	scheduledControllers remove: aController.
	scheduledControllers addFirst: aController! !

!ControlManager methodsFor: 'initialize-release' stamp: ''!
release 
	"Refer to the comment in Object|release."

	scheduledControllers == nil
		ifFalse: 
			[scheduledControllers 
				do: [:controller | (controller isKindOf: Controller)
								ifTrue: [controller view release]
								ifFalse: [controller release]].
			scheduledControllers := nil]! !

!ControlManager methodsFor: 'scheduling' stamp: 'RAA 7/7/2000 09:22'!
resetActiveController
	"When saving a morphic project whose parent is mvc, we need to set this up first"

	activeController := nil.
	activeControllerProcess := Processor activeProcess.
! !

!ControlManager methodsFor: 'displaying' stamp: 'di 2/26/98 08:58'!
restore 
	"Clear the screen to gray and then redisplay all the scheduled views.  Try to be a bit intelligent about the view that wants control and not display it twice if possible."

	scheduledControllers first view uncacheBits.  "assure refresh"
	self unschedule: screenController; scheduleOnBottom: screenController.
	screenController view window: Display boundingBox; displayDeEmphasized.
	self scheduledWindowControllers reverseDo:
		[:aController | aController view displayDeEmphasized].
! !

!ControlManager methodsFor: 'displaying' stamp: 'hmm 1/5/2000 07:00'!
restore: aRectangle
	"Restore all windows visible in aRectangle"
	^ self restore: aRectangle without: nil! !

!ControlManager methodsFor: 'displaying' stamp: 'ar 5/28/2000 12:06'!
restore: aRectangle below: index without: aView
	"Restore all windows visible in aRectangle, but without aView"
	| view | 
	view := (scheduledControllers at: index) view.
	view == aView ifTrue: 
		[index >= scheduledControllers size ifTrue: [^ self].
		^ self restore: aRectangle below: index+1 without: aView].
	view displayOn: ((BitBlt current toForm: Display) clipRect: aRectangle).
	index >= scheduledControllers size ifTrue: [^ self].
	(aRectangle areasOutside: view windowBox) do:
		[:rect | self restore: rect below: index + 1 without: aView]! !

!ControlManager methodsFor: 'displaying' stamp: 'hmm 12/30/1999 19:35'!
restore: aRectangle without: aView
	"Restore all windows visible in aRectangle"
	Display deferUpdates: true.
	self restore: aRectangle below: 1 without: aView.
	Display deferUpdates: false; forceToScreen: aRectangle! !

!ControlManager methodsFor: 'scheduling' stamp: 'ar 6/5/1998 21:48'!
scheduleActive: aController 
	"Make aController be scheduled as the active controller. Presumably the 
	active scheduling process asked to schedule this controller and that a 
	new process associated this controller takes control. So this is the last act 
	of the active scheduling process."
	<primitive: 19> "Simulation guard"
	self scheduleActiveNoTerminate: aController.
	Processor terminateActive! !

!ControlManager methodsFor: 'scheduling' stamp: ''!
scheduleActiveNoTerminate: aController 
	"Make aController be the active controller. Presumably the process that 
	requested the new active controller wants to keep control to do more 
	activites before the new controller can take control. Therefore, do not 
	terminate the currently active process."

	self schedulePassive: aController.
	self scheduled: aController
		from: Processor activeProcess! !

!ControlManager methodsFor: 'scheduling' stamp: ''!
scheduleOnBottom: aController 
	"Make aController be scheduled as a scheduled controller, but not the 
	active one. Put it at the end of the ordered collection of controllers."

	scheduledControllers addLast: aController! !

!ControlManager methodsFor: 'scheduling' stamp: ''!
schedulePassive: aController 
	"Make aController be scheduled as a scheduled controller, but not the 
	active one. Put it at the beginning of the ordered collection of 
	controllers."

	scheduledControllers addFirst: aController! !

!ControlManager methodsFor: 'private' stamp: ''!
scheduled: aController from: aProcess

	activeControllerProcess==aProcess
		ifTrue: 
			[activeController ~~ nil
					ifTrue: [activeController controlTerminate].
			aController centerCursorInView.
			self activeController: aController]! !

!ControlManager methodsFor: 'accessing' stamp: ''!
scheduledControllers
	"Answer a copy of the ordered collection of scheduled controllers."

	^scheduledControllers copy! !

!ControlManager methodsFor: 'accessing' stamp: 'di 10/4/97 09:05'!
scheduledWindowControllers
	"Same as scheduled controllers, but without ScreenController.
	Avoids null views just after closing, eg, a debugger."

	^ scheduledControllers select:
		[:c | c ~~ screenController and: [c view ~~ nil]]! !

!ControlManager methodsFor: 'accessing' stamp: ''!
screenController
	^ screenController! !

!ControlManager methodsFor: 'scheduling' stamp: ''!
searchForActiveController
	"Find a scheduled controller that wants control and give control to it. If 
	none wants control, then see if the System Menu has been requested."
	| aController |
	activeController := nil.
	activeControllerProcess := Processor activeProcess.
	self activeController: self nextActiveController.
	Processor terminateActive! !

!ControlManager methodsFor: 'scheduling' stamp: 'ajh 12/31/2001 15:15'!
spawnNewProcess

	self activeController: self screenController! !

!ControlManager methodsFor: 'private' stamp: 'sw 12/6/1999 23:40'!
unCacheWindows
	scheduledControllers ifNotNil: [scheduledControllers do:
		[:aController | aController view uncacheBits]]! !

!ControlManager methodsFor: 'scheduling' stamp: ''!
unschedule: aController
	"Remove the view, aController, from the collection of scheduled 
	controllers."

	scheduledControllers remove: aController ifAbsent: []! !

!ControlManager methodsFor: 'displaying' stamp: ''!
updateGray
	"From Georg Gollmann - 11/96.  tell the Screen Controller's model to use the currently-preferred desktop color."

	"ScheduledControllers updateGray"
	(screenController view model isMemberOf: InfiniteForm)
		ifTrue: [screenController view model: (InfiniteForm with:
Preferences desktopColor)]! !

!ControlManager methodsFor: 'scheduling' stamp: 'rbb 2/18/2005 10:52'!
windowFromUser
	"Present a menu of window titles, and returns the StandardSystemController belonging to the one that gets chosen, or nil if none"
	| controllers labels index |
	controllers := OrderedCollection new.
	labels := String streamContents:
		[:strm |
		scheduledControllers do:
			[:controller | controller == screenController ifFalse:
				[controllers addLast: controller.
				strm nextPutAll: (controller view label contractTo: 40); cr]].
		strm skip: -1  "drop last cr"].
	index := (UIManager default chooseFrom: (labels findTokens: Character cr) asArray).
	^ index > 0
		ifTrue:
			[controllers at: index]
		ifFalse:
			[nil]! !

!ControlManager methodsFor: 'accessing' stamp: ''!
windowOriginsInUse
	"Answer a collection of the origins of windows currently on the screen in the current project.  5/21/96 sw"

	^ self scheduledWindowControllers collect: [:aController | aController view displayBox origin].! !

Object subclass: #Controller
	instanceVariableNames: 'model view sensor lastActivityTime'
	classVariableNames: 'MinActivityLapse'
	poolDictionaries: ''
	category: 'ST80-Kernel-Remnants'!

!Controller commentStamp: '<historical>' prior: 0!
A Controller coordinates a View, its model, and user actions. It provides scheduling (control) behavior to determine when the user wants to communicate with the model or view.!

!Controller classSide methodsFor: 'initialization' stamp: 'ls 7/13/1998 00:47'!
MinActivityLapse: milliseconds
	"minimum time to delay between calls to controlActivity"
	MinActivityLapse := milliseconds ifNotNil: [ milliseconds rounded ].! !

!Controller classSide methodsFor: 'initialization' stamp: 'ls 7/13/1998 00:47'!
initialize
	"Controller initialize"
	self MinActivityLapse: 10.! !

!Controller methodsFor: 'cursor' stamp: ''!
centerCursorInView
	"Position sensor's mousePoint (which is assumed to be connected to the 
	cursor) to the center of its view's inset display box (see 
	Sensor|mousePoint: and View|insetDisplayBox)."

	^sensor cursorPoint: view insetDisplayBox center! !

!Controller methodsFor: 'control defaults' stamp: ''!
controlActivity
	"Pass control to the next control level (that is, to the Controller of a 
	subView of the receiver's view) if possible. It is sent by 
	Controller|controlLoop each time through the main control loop. It should 
	be redefined in a subclass if some other action is needed."

	self controlToNextLevel! !

!Controller methodsFor: 'basic control sequence' stamp: ''!
controlInitialize
	"Sent by Controller|startUp as part of the standard control sequence, it 
	provides a place in the standard control sequence for initializing the 
	receiver (taking into account the current state of its model and view). It 
	should be redefined in subclasses to perform some specific action."

	^self! !

!Controller methodsFor: 'basic control sequence' stamp: 'ls 7/11/1998 06:33'!
controlLoop 
	"Sent by Controller|startUp as part of the standard control sequence. 
	Controller|controlLoop sends the message Controller|isControlActive to test 
	for loop termination. As long as true is returned, the loop continues. 
	When false is returned, the loop ends. Each time through the loop, the 
	message Controller|controlActivity is sent."

	[self isControlActive] whileTrue: [
		self interActivityPause. self controlActivity. Processor yield]! !

!Controller methodsFor: 'basic control sequence' stamp: ''!
controlTerminate
	"Provide a place in the standard control sequence for terminating the 
	receiver (taking into account the current state of its model and view). It 
	should be redefined in subclasses to perform some specific action."

	^self! !

!Controller methodsFor: 'control defaults' stamp: ''!
controlToNextLevel
	"Pass control to the next control level (that is, to the Controller of a 
	subView of the receiver's view) if possible. The receiver finds the 
	subView (if any) of its view whose inset display box (see 
	View|insetDisplayBox) contains the sensor's cursor point. The Controller 
	of this subView is then given control if it answers true in response to 
	the message Controller|isControlWanted."

	| aView |
	aView := view subViewWantingControl.
	aView ~~ nil ifTrue: [aView controller startUp]! !

!Controller methodsFor: 'initialize-release' stamp: ''!
initialize
	"Initialize the state of the receiver. Subclasses should include 'super 
	initialize' when redefining this message to insure proper initialization."

	sensor := InputSensor default! !

!Controller methodsFor: 'view access' stamp: 'apb 7/14/2004 12:50'!
inspectView
	view notNil ifTrue: [^ view inspect; yourself]! !

!Controller methodsFor: 'basic control sequence' stamp: 'RAA 1/30/2001 19:06'!
interActivityPause
	"if we are looping quickly, insert a short delay.  Thus if we are just doing UI stuff, we won't take up much CPU"
	| currentTime wait |
	MinActivityLapse ifNotNil: [
		lastActivityTime ifNotNil: [ 
			currentTime := Time millisecondClockValue.
			wait := lastActivityTime + MinActivityLapse - currentTime.
			wait > 0 ifTrue: [ 
				wait <= MinActivityLapse  "big waits happen after a snapshot"
					ifTrue: [DisplayScreen checkForNewScreenSize.
							(Delay forMilliseconds: wait) wait ]. ]. ]. ].

	lastActivityTime := Time millisecondClockValue.! !

!Controller methodsFor: 'control defaults' stamp: 'sma 3/11/2000 15:23'!
isControlActive
	"Answer whether receiver wishes to continue evaluating its controlLoop 
	method. It is sent by Controller|controlLoop in order to determine when 
	the receiver's control loop should terminate, and should be redefined in 
	a subclass if some special condition for terminating the main control loop 
	is needed."

	^ self viewHasCursor
		and: [sensor blueButtonPressed not
		and: [sensor yellowButtonPressed not]]! !

!Controller methodsFor: 'control defaults' stamp: ''!
isControlWanted
	"Answer whether the cursor is inside the inset display box (see 
	View|insetDisplayBox) of the receiver's view. It is sent by 
	Controller|controlNextLevel in order to determine whether or not control 
	should be passed to this receiver from the Controller of the superView of 
	this receiver's view."

	^self viewHasCursor! !

!Controller methodsFor: 'model access' stamp: ''!
model
	"Answer the receiver's model which is the same as the model of the 
	receiver's view."

	^model! !

!Controller methodsFor: 'model access' stamp: ''!
model: aModel 
	"Controller|model: and Controller|view: are sent by View|controller: in 
	order to coordinate the links between the model, view, and controller. In 
	ordinary usage, the receiver is created and passed as the parameter to 
	View|controller: so that the receiver's model and view links can be set 
	up by the view."

	model := aModel! !

!Controller methodsFor: 'initialize-release' stamp: ''!
release
	"Breaks the cycle between the receiver and its view. It is usually not 
	necessary to send release provided the receiver's view has been properly 
	released independently."

	model := nil.
	view ~~ nil
		ifTrue: 
			[view controller: nil.
			view := nil]! !

!Controller methodsFor: 'sensor access' stamp: ''!
sensor
	"Answer the receiver's sensor. Subclasses may use other objects that are 
	not instances of Sensor or its subclasses if more general kinds of 
	input/output functions are required."

	^sensor! !

!Controller methodsFor: 'sensor access' stamp: ''!
sensor: aSensor
	"Set the receiver's sensor to aSensor."

	sensor := aSensor! !

!Controller methodsFor: 'basic control sequence' stamp: ''!
startUp
	"Give control to the receiver. The default control sequence is to initialize 
	(see Controller|controlInitialize), to loop (see Controller|controlLoop), and 
	then to terminate (see Controller|controlTerminate). After this sequence, 
	control is returned to the sender of Control|startUp. The receiver's control 
	sequence is used to coordinate the interaction of its view and model. In 
	general, this consists of polling the sensor for user input, testing the 
	input with respect to the current display of the view, and updating the 
	model to reflect intended changes."

	self controlInitialize.
	self controlLoop.
	self controlTerminate! !

!Controller methodsFor: 'basic control sequence' stamp: ''!
terminateAndInitializeAround: aBlock
	"1/12/96 sw"
	self controlTerminate.
	aBlock value.
	self controlInitialize! !

!Controller methodsFor: 'view access' stamp: ''!
view
	"Answer the receiver's view."

	^view! !

!Controller methodsFor: 'view access' stamp: ''!
view: aView 
	"Controller|view: and Controller|model: are sent by View|controller: in 
	order to coordinate the links between the model, view, and controller. In 
	ordinary usage, the receiver is created and passed as the parameter to 
	View|controller: and the receiver's model and view links are set up 
	automatically by the view."

	view := aView! !

!Controller methodsFor: 'cursor' stamp: 'sw 7/13/1999 18:42'!
viewHasCursor
	"Answer whether the cursor point of the receiver's sensor lies within the 
	inset display box of the receiver's view (see View|insetDisplayBox). 
	Controller|viewHasCursor is normally used in internal methods."

	^ view ifNotNil: [view containsPoint: sensor cursorPoint] ifNil: [false]! !

Controller subclass: #FormMenuController
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Editors'!

!FormMenuController commentStamp: '<historical>' prior: 0!
I represent a Controller that takes control if a key on the keyboard is depressed or if the cursor is within my rectangular area.!

!FormMenuController methodsFor: 'control defaults' stamp: ''!
controlActivity
	"Pass control to a subView corresponding to a pressed keyboard key or to
	a mouse button pressed, if any."

	sensor keyboardPressed
		ifTrue: [self processMenuKey]
		ifFalse: [self controlToNextLevel]! !

!FormMenuController methodsFor: 'control defaults' stamp: ''!
isControlActive
	"Answer false if the blue mouse button is pressed and the cursor is
	outside of the inset display box of the Controller's view; answer true,
	otherwise."

	^sensor keyboardPressed |
		(view containsPoint: sensor cursorPoint) & sensor blueButtonPressed not! !

!FormMenuController methodsFor: 'control defaults' stamp: ''!
isControlWanted
	"Answer true if the cursor is inside the inset display box (see 
	View|insetDisplayBox) of the receiver's view, and answer false, 
	otherwise. It is sent by Controller|controlNextLevel in order to determine 
	whether or not control should be passed to this receiver from the
	Controller of the superView of this receiver's view."

	^sensor keyboardPressed | self viewHasCursor! !

!FormMenuController methodsFor: 'control defaults' stamp: 'jm 4/7/98 20:59'!
processMenuKey
	"The user typed a key on the keyboard. Perform the action of the button whose shortcut is that key, if any."

	| aView |
	aView := view subViewContainingCharacter: sensor keyboard.
	aView ~~ nil ifTrue: [aView performAction].
! !

Controller subclass: #ModalController
	instanceVariableNames: 'modeActive'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Framework'!

!ModalController commentStamp: '<historical>' prior: 0!
I am a controller that puts the poor user into a mode by not relinquishing control. However, I do pass control onto my underlings. Some underling is expected to end the mode by sending me 'close'. Watch out Larry Tesler, the mode lives on...
!

!ModalController methodsFor: 'as yet unclassified' stamp: 'jm 5/1/1998 07:05'!
close
	"This is how we leave the mode." 

	modeActive := false.
! !

!ModalController methodsFor: 'as yet unclassified' stamp: 'jm 5/1/1998 07:02'!
controlInitialize

	modeActive := true.
	^ super controlInitialize
! !

!ModalController methodsFor: 'as yet unclassified' stamp: 'jm 5/1/1998 07:00'!
isControlActive

	^ modeActive
! !

!ModalController methodsFor: 'as yet unclassified' stamp: 'jm 5/1/1998 07:00'!
isControlWanted

	^ modeActive
! !

Controller subclass: #MouseMenuController
	instanceVariableNames: 'redButtonMenu redButtonMessages'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Kernel-Remnants'!

!MouseMenuController commentStamp: '<historical>' prior: 0!
I am a Controller that modifies the scheduling of user activities so that the three mouse buttons can be used to make selections or display menus. The menu items are unary messages to the value of sending my instance the message menuMessageReceiver.!

MouseMenuController subclass: #BitEditor
	instanceVariableNames: 'scale squareForm color transparent'
	classVariableNames: 'YellowButtonMenu'
	poolDictionaries: ''
	category: 'ST80-Editors'!

!BitEditor commentStamp: '<historical>' prior: 0!
I am a bit-magnifying tool for editing small Forms directly on the display screen. I continue to be active until the user points outside of my viewing area.!

!BitEditor classSide methodsFor: 'private' stamp: 'BG 12/4/2003 10:18'!
bitEdit: aForm at: magnifiedFormLocation scale: scaleFactor remoteView: remoteView
	"Create a BitEditor on aForm. That is, aForm is a small image that will 
	change as a result of the BitEditor changing a second and magnified 
	view of me. magnifiedFormLocation is where the magnified form is to be 
	located on the screen. scaleFactor is the amount of magnification. This 
	method implements a scheduled view containing both a small and 
	magnified view of aForm. Upon accept, aForm is updated."

	| aFormView scaledFormView bitEditor topView extent menuView lowerRightExtent |
	scaledFormView := FormHolderView new model: aForm.
	scaledFormView scaleBy: scaleFactor.
	bitEditor := self new.
	scaledFormView controller: bitEditor.
	bitEditor setColor: Color black.
	topView := ColorSystemView new.
	remoteView == nil ifTrue: [topView label: 'Bit Editor'].
	topView borderWidth: 2.

	topView addSubView: scaledFormView.
	remoteView == nil
		ifTrue:  "If no remote view, then provide a local view of the form"
			[aFormView := FormView new model: scaledFormView workingForm.
			aFormView controller: NoController new.
			aForm height < 50
				ifTrue: [aFormView borderWidthLeft: 0 right: 2 top: 2 bottom: 2]
				ifFalse: [aFormView borderWidthLeft: 0 right: 2 top: 2 bottom: 0].
			topView addSubView: aFormView below: scaledFormView]
		 ifFalse:  "Otherwise, the remote one should view the same form"
			[remoteView model: scaledFormView workingForm].
	lowerRightExtent := remoteView == nil
			ifTrue:
				[(scaledFormView viewport width - aFormView viewport width) @
					(aFormView viewport height max: 50)]
			ifFalse:
				[scaledFormView viewport width @ 50].
	menuView := self buildColorMenu: lowerRightExtent colorCount: 1.
	menuView model: bitEditor.
	menuView borderWidthLeft: 0 right: 0 top: 2 bottom: 0.
	topView
		addSubView: menuView
		align: menuView viewport topRight
		with: scaledFormView viewport bottomRight.
	extent := scaledFormView viewport extent + (0 @ lowerRightExtent y)
			+ (4 @ 4).  "+4 for borders"
	topView minimumSize: extent.
	topView maximumSize: extent.
	topView translateBy: magnifiedFormLocation.
	topView insideColor: Color white.
	^topView! !

!BitEditor classSide methodsFor: 'private' stamp: 'BG 12/5/2003 13:40'!
buildColorMenu: extent colorCount: nColors
	"See BitEditor magnifyWithSmall."

	| menuView form aSwitchView
	 button formExtent highlightForm color leftOffset |
	menuView := FormMenuView new.
	menuView window: (0@0 corner: extent).
	formExtent := 30@30 min: extent//(nColors*2+1@2).  "compute this better"
	leftOffset := extent x-(nColors*2-1*formExtent x)//2.
	highlightForm := Form extent: formExtent.
	highlightForm borderWidth: 4.
	1 to: nColors do: [:index | 
		color := (nColors = 1
			ifTrue: [#(black)]
			ifFalse: [#(black gray)]) at: index.
		form := Form extent: formExtent.
		form fill: form boundingBox fillColor: (Color perform: color).
		form borderWidth: 5.
		form border: form boundingBox width: 4 fillColor: Color white.
		button := Button new.
		aSwitchView := PluggableButtonView
			on: button
			getState: #isOn
			action: #turnOn
			label: #getCurrentColor.

		index = 1
			ifTrue: [button onAction: [menuView model setColor: Color fromUser.
									  aSwitchView label: menuView model getCurrentColor;
									                  displayView
					                     ]
				    ]
			ifFalse: [button onAction: [menuView model setTransparentColor]].

		aSwitchView
			shortcutCharacter: ((nColors=3 ifTrue: ['xvn'] ifFalse: ['xn']) at: index);
			label: form;
			window: (0@0 extent: form extent);
			translateBy: (((index - 1) * 2 * form width) + leftOffset)@(form height // 2);
			borderWidth: 1.
		menuView addSubView: aSwitchView].
	^ menuView
! !

!BitEditor classSide methodsFor: 'class initialization' stamp: 'sma 3/11/2000 14:48'!
initialize
	"The Bit Editor is the only controller to override the use of the blue
	button with a different pop-up menu. Initialize this menu."

	YellowButtonMenu := SelectionMenu
		labels:
'cancel
accept
file out
test'
		lines: #(2 3)
		selections: #(cancel accept fileOut test)

	"BitEditor initialize"! !

!BitEditor classSide methodsFor: 'private' stamp: ''!
locateMagnifiedView: aForm scale: scaleFactor
	"Answer a rectangle at the location where the scaled view of the form,
	aForm, should be displayed."

	^ Rectangle originFromUser: (aForm extent * scaleFactor + (0@50)).
	! !

!BitEditor classSide methodsFor: 'examples' stamp: ''!
magnifyOnScreen
	"Bit editing of an area of the display screen. User designates a 
	rectangular area that is magnified by 8 to allow individual screens dots to
	be modified. red button is used to set a bit to black and yellow button is
	used to set a bit to white. Editor is not scheduled in a view. Original
	screen location is updated immediately. This is the same as FormEditor
	magnify."
	| smallRect smallForm scaleFactor tempRect |
	scaleFactor := 8 @ 8.
	smallRect := Rectangle fromUser.
	smallRect isNil ifTrue: [^self].
	smallForm := Form fromDisplay: smallRect.
	tempRect := self locateMagnifiedView: smallForm scale: scaleFactor.
	"show magnified form size until mouse is depressed"
	self
		openScreenViewOnForm: smallForm 
		at: smallRect topLeft 
		magnifiedAt: tempRect topLeft 
		scale: scaleFactor

	"BitEditor magnifyOnScreen."! !

!BitEditor classSide methodsFor: 'examples' stamp: ''!
magnifyWithSmall
"	Also try:
	BitEditor openOnForm:
		(Form extent: 32@32 depth: Display depth)
	BitEditor openOnForm:
		((MaskedForm extent: 32@32 depth: Display depth)
		withTransparentPixelValue: -1)
"
	"Open a BitEditor viewing an area on the screen which the user chooses"
	| area form |
	area := Rectangle fromUser.
	area isNil ifTrue: [^ self].
	form := Form fromDisplay: area.
	self openOnForm: form

	"BitEditor magnifyWithSmall."! !

!BitEditor classSide methodsFor: 'instance creation' stamp: ''!
openOnForm: aForm 
	"Create and schedule a BitEditor on the form aForm at its top left corner. 
	Show the small and magnified view of aForm."

	| scaleFactor |
	scaleFactor := 8 @ 8.
	^self openOnForm: aForm
		at: (self locateMagnifiedView: aForm scale: scaleFactor) topLeft
		scale: scaleFactor! !

!BitEditor classSide methodsFor: 'instance creation' stamp: ''!
openOnForm: aForm at: magnifiedLocation 
	"Create and schedule a BitEditor on the form aForm at magnifiedLocation. 
	Show the small and magnified view of aForm."

	^self openOnForm: aForm
		at: magnifiedLocation
		scale: 8 @ 8! !

!BitEditor classSide methodsFor: 'instance creation' stamp: ''!
openOnForm: aForm at: magnifiedLocation scale: scaleFactor 
	"Create and schedule a BitEditor on the form aForm. Show the small and 
	magnified view of aForm."

	| aScheduledView |
	aScheduledView := self
				bitEdit: aForm
				at: magnifiedLocation
				scale: scaleFactor
				remoteView: nil.
	aScheduledView controller openDisplayAt:
		aScheduledView displayBox topLeft + (aScheduledView displayBox extent / 2)! !

!BitEditor classSide methodsFor: 'instance creation' stamp: 'sma 3/11/2000 11:29'!
openScreenViewOnForm: aForm at: formLocation magnifiedAt: magnifiedLocation scale: scaleFactor
	"Create and schedule a BitEditor on the form aForm. Show the magnified
	view of aForm in a scheduled window."
	| smallFormView bitEditor savedForm r |
	smallFormView := FormView new model: aForm.
	smallFormView align: smallFormView viewport topLeft with: formLocation.
	bitEditor := self bitEdit: aForm at: magnifiedLocation scale: scaleFactor remoteView: smallFormView.
	savedForm := Form fromDisplay: (r := bitEditor displayBox expandBy: (0@23 corner: 0@0)).
	bitEditor controller startUp.
	savedForm displayOn: Display at: r topLeft.
	bitEditor release.
	smallFormView release.

	"BitEditor magnifyOnScreen."! !

!BitEditor methodsFor: 'menu messages' stamp: ''!
accept
	"The edited information should now be accepted by the view."

	view accept! !

!BitEditor methodsFor: 'menu messages' stamp: ''!
cancel
	"The edited informatin should be forgotten by the view."

	view cancel! !

!BitEditor methodsFor: 'basic control sequence' stamp: ''!
controlInitialize

	super controlInitialize.
	Cursor crossHair show! !

!BitEditor methodsFor: 'basic control sequence' stamp: ''!
controlTerminate

	Cursor normal show! !

!BitEditor methodsFor: 'menu messages' stamp: 'CdG 10/17/2005 20:51'!
fileOut

	| fileName |
	fileName := UIManager default 
		request: 'File name?' translated
		initialAnswer: 'Filename.form'.
	fileName isEmpty ifTrue: [^ self].
	Cursor normal
		showWhile: [model writeOnFileNamed: fileName].
! !

!BitEditor methodsFor: 'menu messages' stamp: 'BG 12/5/2003 13:53'!
getCurrentColor
	| formExtent form c |
	c := Color colorFromPixelValue: color depth: Display depth.
	formExtent := 30@30" min: 10@ 10//(2+1@2)".  "compute this better"
	form := Form extent: formExtent depth: Display depth.
	form borderWidth: 5.
	form border: form boundingBox width: 4 fillColor: Color white.
	form fill: form boundingBox fillColor: c.

	^form! !

!BitEditor methodsFor: 'pluggable menus' stamp: 'sma 3/11/2000 15:04'!
getPluggableYellowButtonMenu: shiftKeyState
	^ YellowButtonMenu! !

!BitEditor methodsFor: 'control defaults' stamp: 'sma 3/11/2000 14:52'!
isControlActive
	^ super isControlActive and: [sensor keyboardPressed not]! !

!BitEditor methodsFor: 'control defaults' stamp: ''!
redButtonActivity
	| formPoint displayPoint |
	model depth = 1 ifTrue:
		["If this is just a black&white form, then set the color to be
		the opposite of what it was where the mouse was clicked"
		formPoint := (view inverseDisplayTransform: sensor cursorPoint - (scale//2)) rounded.
		color := 1-(view workingForm pixelValueAt: formPoint).
		squareForm fillColor: (color=1 ifTrue: [Color black] ifFalse: [Color white])].
	[sensor redButtonPressed]
	  whileTrue: 
		[formPoint := (view inverseDisplayTransform: sensor cursorPoint - (scale//2)) rounded.
		displayPoint := view displayTransform: formPoint.
		squareForm 
			displayOn: Display
			at: displayPoint 
			clippingBox: view insetDisplayBox 
			rule: Form over
			fillColor: nil.
		view changeValueAt: formPoint put: color]! !

!BitEditor methodsFor: 'initialize-release' stamp: ''!
release

	super release.
	squareForm release.
	squareForm := nil! !

!BitEditor methodsFor: 'menu messages' stamp: 'BG 12/5/2003 13:21'!
setColor: aColor 
	"Set the color that the next edited dots of the model to be the argument,  
	aSymbol. aSymbol can be any color changing message understood by a  
	Form, such as white or black."

	color := aColor pixelValueForDepth: Display depth.
	squareForm fillColor: aColor.
	self changed: #getCurrentColor! !

!BitEditor methodsFor: 'menu messages' stamp: 'sma 3/15/2000 21:10'!
setTransparentColor
	squareForm fillColor: Color gray.
	color := Color transparent! !

!BitEditor methodsFor: 'menu messages' stamp: ''!
test
	view workingForm follow: [Sensor cursorPoint] while: [Sensor noButtonPressed].
	Sensor waitNoButton! !

!BitEditor methodsFor: 'view access' stamp: ''!
view: aView

	super view: aView.
	scale := aView transformation scale.	
	scale := scale x rounded @ scale y rounded.
	squareForm := Form extent: scale depth: aView model depth.
	squareForm fillBlack! !

MouseMenuController subclass: #FormEditor
	instanceVariableNames: 'form tool grid togglegrid mode previousTool color unNormalizedColor xgridOn ygridOn hasUnsavedChanges'
	classVariableNames: 'BitEditKey BlackKey BlockKey ChangeGridsKey CurveKey DarkGrayKey EraseKey FlashCursor GrayKey InKey LightGrayKey LineKey OutKey OverKey RepeatCopyKey ReverseKey SelectKey SingleCopyKey TogglexGridKey ToggleyGridKey UnderKey WhiteKey YellowButtonMenu YellowButtonMessages YgridKey'
	poolDictionaries: ''
	category: 'ST80-Editors'!

!FormEditor commentStamp: 'BG 12/5/2003 22:40' prior: 0!
I represent the basic editor for creating and modifying Forms. This is intended to be an easy to use general-purpose picture (bitMap) editor. I am a kind of MouseMenuController that creates a yellow button menu for accepting and canceling edits. My instances give up control if the cursor is outside the FormView or if a key on the keyboard is pressed.

The form to be edited is stored in instance variable model.
The instance variable form references the paint brush.!

!FormEditor classSide methodsFor: 'private' stamp: 'di 1/16/98 15:46'!
createFullScreenForm
	"Create a StandardSystemView for a FormEditor on the form whole screen."
	| formView formEditor menuView topView extent aForm |
	aForm := Form extent: (Display extent x @ (Display extent y - 112)) depth: Display depth.
	formView := FormHolderView new model: aForm.
	formView borderWidthLeft: 0 right: 0 top: 0 bottom: 1.
	formEditor := formView controller.
	menuView := FormMenuView new makeFormEditorMenu model: formEditor.
	formEditor model: menuView controller.
	topView := StandardSystemView new.
	topView backgroundColor: #veryLightGray.
	topView model: aForm.
	topView addSubView: formView.
	topView 
		addSubView: menuView
		align: menuView viewport topCenter
		with: formView viewport bottomCenter + (0@16).
	topView window: 
		(formView viewport 
			merge: (menuView viewport expandBy: (16 @ 0 corner: 16@16))).
	topView label: 'Form Editor'.
	extent := topView viewport extent.
	topView minimumSize: extent.
	topView maximumSize: extent.
	^topView

! !

!FormEditor classSide methodsFor: 'private' stamp: 'BG 12/5/2003 23:18'!
createOnForm: aForm
	"Create a StandardSystemView for a FormEditor on the form aForm."
	| formView formEditor menuView aView topView extent topViewBorder |
	topViewBorder := 2.
	formView := FormHolderView new model: aForm.
	formEditor := formView controller.
	menuView := FormMenuView new makeFormEditorMenu model: formEditor.
	formEditor model: aForm.
	aView := View new.
	aView model: aForm.
	aView addSubView: formView.
	aView 
		addSubView: menuView
		align: menuView viewport topCenter
		with: formView viewport bottomCenter + (0@16).
	aView window: 
		((formView viewport 
			merge: (menuView viewport expandBy: (16 @ 0 corner: 16@16))) 
		  expandBy: (0@topViewBorder corner: 0@0)).
	topView := "ColorSystemView" FormEditorView new.
	topView model: formEditor.
	topView backgroundColor: #veryLightGray.
	topView addSubView: aView.
	topView label: 'Form Editor'.
	topView borderWidth: topViewBorder.
	extent := topView viewport extent.
	topView minimumSize: extent.
	topView maximumSize: extent.
	^topView! !

!FormEditor classSide methodsFor: 'class initialization' stamp: ''!
flashCursor: aBoolean

	FlashCursor := aBoolean

	"FormEditor flashCursor: true"! !

!FormEditor classSide methodsFor: 'examples' stamp: ''!
formFromDisplay
	"Create an instance of me on a new form designated by the user at a
	location designated by the user."

	Form fromUser edit

	"FormEditor formFromDisplay"! !

!FormEditor classSide methodsFor: 'examples' stamp: ''!
fullScreen
	"Create an instance of me on a new form that fills the full size of the
	display screen."

	FormEditor openFullScreenForm

	"FormEditor fullScreen"! !

!FormEditor classSide methodsFor: 'class initialization' stamp: 'sma 3/11/2000 15:06'!
initialize

	FlashCursor := false.
	self setKeyboardMap.
	YellowButtonMenu := SelectionMenu 
		labels:
'accept
cancel
edit
file out'
		lines: #(2)
		selections: #(accept cancel edit fileOut).

	"FormEditor initialize"! !

!FormEditor classSide methodsFor: 'examples' stamp: 'BG 12/5/2003 22:39'!
newForm
	"Create an instance of me on a new form at a location designated by the user. "

	(Form extent: 400 @ 200 depth: Display depth)
	    fillWhite;
	    edit

	"FormEditor newForm"! !

!FormEditor classSide methodsFor: 'instance creation' stamp: ''!
openFullScreenForm
	"Create and schedule an instance of me on the form whose extent is the 
	extent of the display screen."

	| topView |
	topView := self createFullScreenForm.
	topView controller 
		openDisplayAt: (topView viewport extent//2)

	"FormEditor openFullScreenForm."! !

!FormEditor classSide methodsFor: 'instance creation' stamp: ''!
openOnForm: aForm
	"Create and schedule an instance of me on the form aForm."

	| topView |
	topView := self createOnForm: aForm.
	topView controller open

! !

!FormEditor classSide methodsFor: 'private' stamp: ''!
setKeyboardMap
	"Keyboard Mapping."

	SelectKey:=$a.
	SingleCopyKey:=$s.			"tools"
	RepeatCopyKey:=$d.
	LineKey:=$f.
	CurveKey:=$g.
	BlockKey:=$h.
	OverKey:=$j.				"modes"
	UnderKey:=$k.
	ReverseKey:=$l.
	EraseKey:=$;.
	InKey:=$'.					"file In"
	BitEditKey:=$z.
	WhiteKey:=$x.				"colors"
	LightGrayKey:=$c.
	GrayKey:=$v.
	DarkGrayKey:=$b.
	BlackKey:=$n.
	TogglexGridKey:=$m.		"gridding"
	ToggleyGridKey:=$,.
	ChangeGridsKey:=$..
	OutKey:=$/					"file Out"! !

!FormEditor methodsFor: 'menu messages' stamp: 'BG 12/5/2003 22:59'!
accept
	"The edited information should now be accepted by the view."

	view updateDisplay.
	view accept.
	hasUnsavedChanges contents: false.! !

!FormEditor methodsFor: 'editing tools' stamp: 'BG 12/5/2003 23:00'!
block
	"Allow the user to fill a rectangle with the gray tone and mode currently 
	selected."

	| rectangle originRect |
	originRect := (Sensor cursorPoint grid: grid) extent: 2 @ 2.
 	rectangle := Cursor corner showWhile:
		[originRect newRectFrom:
			[:f | f origin corner: (Sensor cursorPoint grid: grid)]].
	rectangle isNil 
		ifFalse:
		  [sensor waitNoButton.
		   Display
					fill: (rectangle intersect: view insetDisplayBox)
					rule: mode
					fillColor: color.
		   hasUnsavedChanges contents: true.]! !

!FormEditor methodsFor: 'menu messages' stamp: 'BG 12/5/2003 22:59'!
cancel
	"The edited information should be forgotten by the view."

	view cancel.
	hasUnsavedChanges contents: false.! !

!FormEditor methodsFor: 'editing tools' stamp: 'rbb 3/1/2005 11:21'!
changeGridding
	"Allow the user to change the values of the horizontal and/or vertical 
	grid modules. Does not change the primary tool."

	| response gridInteger gridX gridY |
	gridX := togglegrid x.
	gridY := togglegrid y.
	response := UIManager default
		request:
'Current horizontal gridding is: ', gridX printString, '.
Type new horizontal gridding.'.
	response isEmpty
		ifFalse: 
			[gridInteger := Integer readFromString: response.
			gridX := ((gridInteger max: 1) min: Display extent x)].
	response := UIManager default
		request:
'Current vertical gridding is: ', gridY printString, '.
Type new vertical gridding.'.
	response isEmpty
		ifFalse: 
			[gridInteger := Integer readFromString: response.
			gridY := ((gridInteger max: 1) min: Display extent y)].
	xgridOn ifTrue: [grid := gridX @ grid y].
	ygridOn ifTrue: [grid := grid x @ gridY].
	togglegrid := gridX @ gridY.
	tool := previousTool! !

!FormEditor methodsFor: 'editing tools' stamp: ''!
changeTool: aCharacter 
	"Change the value of the instance variable tool to be the tool 
	corresponding to aCharacter. Typically sent from a Switch in a 
	FormMenuView."

	previousTool := tool.
	tool := self selectTool: aCharacter.
	(#(singleCopy repeatCopy line curve block) includes: tool)
		ifFalse:
			[self perform: tool]! !

!FormEditor methodsFor: 'editing tools' stamp: ''!
colorBlack
	"Set the mask (color) to black. Leaves the tool set in its previous state."

	self setColor: Color black! !

!FormEditor methodsFor: 'editing tools' stamp: ''!
colorDarkGray
	"Set the mask (color) to dark gray. Leaves the tool set in its previous 
	state."

	self setColor: Color darkGray! !

!FormEditor methodsFor: 'editing tools' stamp: ''!
colorGray
	"Set the color to gray. Leaves the tool set in its previous state."

	self setColor: Color gray.
! !

!FormEditor methodsFor: 'editing tools' stamp: ''!
colorLightGray
	"Set the mask (color) to light gray. Leaves the tool set in its previous 
	state."

	self setColor: Color lightGray! !

!FormEditor methodsFor: 'editing tools' stamp: ''!
colorWhite
	"Set the color to white. Leaves the tool set in its previous state."

	self setColor: Color white! !

!FormEditor methodsFor: 'control defaults' stamp: ''!
controlActivity

	super controlActivity.
	self dragForm! !

!FormEditor methodsFor: 'basic control sequence' stamp: 'sma 4/22/2000 12:56'!
controlInitialize

	Cursor crossHair show.
	self normalizeColor: unNormalizedColor.
	sensor waitNoButton! !

!FormEditor methodsFor: 'basic control sequence' stamp: ''!
controlTerminate
	"Resets the cursor to be the normal Smalltalk cursor."

	Cursor normal show.
	view updateDisplay! !

!FormEditor methodsFor: 'cursor' stamp: ''!
cursorPoint
	"Answer the mouse coordinate data gridded according to the receiver's 
	grid."

	^sensor cursorPoint grid: grid! !

!FormEditor methodsFor: 'editing tools' stamp: 'BG 12/10/2003 16:21'!
curve
	"Conic-section specified by three points designated by: first point--press 
	red button second point--release red button third point--click red button. 
	The resultant curve on the display is displayed according to the current 
	form and mode."

	| firstPoint secondPoint thirdPoint curve drawForm |
	"sensor noButtonPressed ifTrue: [^self]."
	firstPoint := self cursorPoint.
	secondPoint := self rubberBandFrom: firstPoint until: [sensor noButtonPressed].
	thirdPoint :=  self rubberBandFrom: secondPoint until: [sensor redButtonPressed].
	Display depth > 1
	  ifTrue:
	    [self deleteRubberBandFrom: secondPoint to: thirdPoint.
	     self deleteRubberBandFrom: firstPoint to: secondPoint].
	curve := CurveFitter new.
	curve firstPoint: firstPoint.
	curve secondPoint: secondPoint.
	curve thirdPoint: thirdPoint.
	drawForm := form asFormOfDepth: Display depth.
	Display depth > 1 ifTrue:
	  [drawForm mapColor: Color white to: Color transparent; 
	               mapColor: Color black to: color].

	curve form: drawForm.
	curve
		displayOn: Display
		at: 0 @ 0
		clippingBox: view insetDisplayBox
		rule: (Display depth > 1 ifTrue: [mode ~= Form erase ifTrue: [Form paint] ifFalse: [mode]]
										ifFalse: [mode])
		fillColor: (Display depth = 1 ifTrue: [color] ifFalse: [nil]). 
	sensor waitNoButton.
	hasUnsavedChanges contents: true.! !

!FormEditor methodsFor: 'private' stamp: 'BG 12/10/2003 17:02'!
deleteRubberBandFrom: startPoint to: endPoint

	(Line from: startPoint to: endPoint withForm: form)
		displayOn: Display
		at: 0 @ 0
		clippingBox: view insetDisplayBox
		rule: Form reverse
		fillColor: (Display depth = 1 ifTrue: [Color black] ifFalse: [Color gray]).! !

!FormEditor methodsFor: 'private' stamp: ''!
dragForm

	tool = #block
		ifTrue:
			[Cursor origin show.
			[sensor anyButtonPressed
				or: [sensor keyboardPressed
				or: [self viewHasCursor not]]]
				whileFalse: [].
			^self cursorPoint]
		ifFalse:
			[^self trackFormUntil:
				[sensor anyButtonPressed
					or: [sensor keyboardPressed
					or: [self viewHasCursor not]]]]! !

!FormEditor methodsFor: 'menu messages' stamp: ''!
edit
	model edit! !

!FormEditor methodsFor: 'editing tools' stamp: ''!
eraseMode
	"Set the mode for the tools that copy the form onto the display to erase. 
	Leaves the tool set in its previous state."

	mode := 4.
	tool := previousTool! !

!FormEditor methodsFor: 'editing tools' stamp: 'CdG 10/17/2005 20:51'!
fileInForm
	"Ask the user for a file name and then recalls the Form in that file as the current source Form (form). Does not change the tool."

	| fileName |
	fileName := UIManager default
		request: 'File name?' translated
		initialAnswer: 'Filename.form'.
	fileName isEmpty ifTrue: [^ self].
	form := Form fromFileNamed: fileName.
	tool := previousTool.
! !

!FormEditor methodsFor: 'menu messages' stamp: 'CdG 10/17/2005 20:52'!
fileOut

	| fileName |
	fileName := UIManager default
		request: 'File name?' translated
		initialAnswer: 'Filename.form'.
	fileName isEmpty ifTrue: [^ self].
	Cursor normal
		showWhile: [model writeOnFileNamed: fileName].
! !

!FormEditor methodsFor: 'editing tools' stamp: 'CdG 10/17/2005 20:52'!
fileOutForm
	"Ask the user for a file name and save the current source form under that name. Does not change the tool."

	| fileName |
	fileName := UIManager default
		request: 'File name?' translated
		initialAnswer: 'Filename.form'.
	fileName isEmpty ifTrue: [^ self].
	Cursor normal
		showWhile: [form writeOnFileNamed: fileName].
	tool := previousTool.
! !

!FormEditor methodsFor: 'pluggable menus' stamp: 'sma 3/11/2000 15:08'!
getPluggableYellowButtonMenu: shiftKeyState
	^ YellowButtonMenu! !

!FormEditor methodsFor: 'initialize-release' stamp: 'sma 3/11/2000 15:07'!
initialize
	super initialize.
	self setVariables! !

!FormEditor methodsFor: 'control defaults' stamp: 'sma 3/11/2000 15:07'!
isControlActive
	^ super isControlActive and: [sensor keyboardPressed not]! !

!FormEditor methodsFor: 'editing tools' stamp: 'BG 12/12/2003 15:51'!
line
	"Line is specified by two points from the mouse: first point--press red 
	button; second point--release red button. The resultant line is displayed 
	according to the current form and mode."

	| firstPoint endPoint drawForm |
	drawForm := form asFormOfDepth: Display depth.
	
	 Display depth > 1 
	  ifTrue:
	    [drawForm mapColor: Color white to: Color transparent; 
	                 mapColor: Color black to: color].
	           
	firstPoint := self cursorPoint.
	endPoint := self rubberBandFrom: firstPoint until: [sensor noButtonPressed].
	endPoint isNil ifTrue: [^self].
	Display depth > 1 ifTrue: [self deleteRubberBandFrom: firstPoint to: endPoint.].
	(Line from: firstPoint to: endPoint withForm: drawForm)
		displayOn: Display
		at: 0 @ 0
		clippingBox: view insetDisplayBox
		rule: (Display depth > 1 ifTrue: [mode ~= Form erase ifTrue: [Form paint] ifFalse: [mode]]
								ifFalse: [mode])
		fillColor: (Display depth = 1 ifTrue: [color] ifFalse: [nil]).  
		hasUnsavedChanges contents: true.! !

!FormEditor methodsFor: 'editing tools' stamp: ''!
magnify
	"Allow for bit editing of an area of the Form. The user designates a 
	rectangular area that is scaled by 5 to allow individual screens dots to be 
	modified. Red button is used to set a bit to black, and yellow button is 
	used to set a bit to white. Editing continues until the user depresses any 
	key on the keyboard."

	| smallRect smallForm scaleFactor tempRect |
	scaleFactor := 8@8.
	smallRect := (Rectangle fromUser: grid) intersect: view insetDisplayBox.
	smallRect isNil ifTrue: [^self].
	smallForm := Form fromDisplay: smallRect.

	"Do this computation here in order to be able to save the existing display screen."
	tempRect := BitEditor locateMagnifiedView: smallForm scale: scaleFactor.
	BitEditor
		openScreenViewOnForm: smallForm 
		at: smallRect topLeft 
		magnifiedAt: tempRect topLeft 
		scale: scaleFactor.
	tool := previousTool! !

!FormEditor methodsFor: 'editing tools' stamp: 'jm 6/30/1999 15:46'!
newSourceForm
	"Allow the user to define a new source form for the FormEditor. Copying 
	the source form onto the display is the primary graphical operation. 
	Resets the tool to be repeatCopy."
	| dForm interiorPoint interiorColor |

	dForm := Form fromUser: grid.
	"sourceForm must be only 1 bit deep"
	interiorPoint := dForm extent // 2.
	interiorColor := dForm colorAt: interiorPoint.
	form := (dForm makeBWForm: interiorColor) reverse
				findShapeAroundSeedBlock:
					[:f | f pixelValueAt: interiorPoint put: 1].
	form := form trimBordersOfColor: Color white.
	tool := previousTool! !

!FormEditor methodsFor: 'private' stamp: 'jm 12/4/97 10:22'!
normalizeColor: aColor

	color := aColor.
! !

!FormEditor methodsFor: 'window support' stamp: 'rbb 2/16/2005 16:49'!
okToChange

  ^hasUnsavedChanges contents not
	ifFalse:
	  [self confirm:
		'This drawing was not saved.\Is it OK to close this window?' withCRs
	  ]
	ifTrue:
	  [true]
! !

!FormEditor methodsFor: 'editing tools' stamp: ''!
overMode
	"Set the mode for the tools that copy the form onto the display to over. 
	Leaves the tool set in its previous state."

	mode := Form over.
	tool := previousTool! !

!FormEditor methodsFor: 'menu messages' stamp: ''!
redButtonActivity 
	"Refer to the comment in MouseMenuController|redButtonActivity."

	self perform: tool! !

!FormEditor methodsFor: 'initialize-release' stamp: ''!
release
	"Break the cycle between the Controller and its view. It is usually not 
	necessary to send release provided the Controller's view has been properly 
	released independently."

	super release.
	form := nil! !

!FormEditor methodsFor: 'editing tools' stamp: 'BG 12/10/2003 15:59'!
repeatCopy
	"As long as the red button is pressed, copy the source form onto the 
	display screen."
  | drawingWasChanged |
	drawingWasChanged := false.
	[sensor redButtonPressed]
		whileTrue: 
		[(BitBlt current destForm: Display sourceForm: form halftoneForm: color
			combinationRule: (Display depth > 1 ifTrue: [mode ~= Form erase ifTrue: [Form paint] ifFalse: [mode]]
										ifFalse: [mode])
			destOrigin: self cursorPoint sourceOrigin: 0@0 extent: form extent
			clipRect: view insetDisplayBox)
			colorMap: (Bitmap with: 0 with: 16rFFFFFFFF);
			copyBits.
		  drawingWasChanged := true.
		].
	drawingWasChanged
	  ifTrue: [hasUnsavedChanges contents: true.]! !

!FormEditor methodsFor: 'editing tools' stamp: ''!
reverseMode
	"Set the mode for the tools that copy the form onto the display to reverse. 
	Leaves the tool set in its previous state."

	mode := Form reverse.
	tool := previousTool! !

!FormEditor methodsFor: 'private' stamp: 'BG 12/10/2003 16:47'!
rubberBandFrom: startPoint until: aBlock

	| endPoint previousEndPoint |
	previousEndPoint := startPoint.
	[aBlock value] whileFalse:
		[(endPoint := self cursorPoint) = previousEndPoint 
			ifFalse:
			[(Line from: startPoint to: previousEndPoint withForm: form) 
				displayOn: Display
				at: 0 @ 0
				clippingBox: view insetDisplayBox
				rule: Form reverse
				fillColor: Color gray.
			(Line from: startPoint to: endPoint withForm: form)
				displayOn: Display
				at: 0 @ 0
				clippingBox: view insetDisplayBox
				rule: Form reverse
				fillColor: Color gray.
			previousEndPoint  := endPoint]].
	(Line from: startPoint to: previousEndPoint withForm: form)
		displayOn: Display
		at: 0 @ 0
		clippingBox: view insetDisplayBox
		rule: Form reverse
		fillColor: (Display depth = 1 ifTrue: [Color gray] ifFalse: [Color black]).
	^endPoint! !

!FormEditor methodsFor: 'private' stamp: ''!
selectTool: aCharacter
	"A new tool has been selected. It is denoted by aCharacter. Set the tool.
	This code is written out in long hand (i.e., rather than dispatching on a
	table of options) so that it is obvious what is happening."
	
	aCharacter =  SingleCopyKey	ifTrue: [^#singleCopy].
	aCharacter =  RepeatCopyKey	ifTrue: [^#repeatCopy].
	aCharacter =  LineKey			ifTrue: [^#line].					
	aCharacter =  CurveKey			ifTrue: [^#curve].				
	aCharacter =  BlockKey			ifTrue: [^#block].		
	aCharacter =  SelectKey			ifTrue: [^#newSourceForm].		
	aCharacter =  OverKey			ifTrue: [^#overMode].
	aCharacter =  UnderKey			ifTrue: [^#underMode].
	aCharacter =  ReverseKey		ifTrue: [^#reverseMode].
	aCharacter =  EraseKey			ifTrue: [^#eraseMode].
	aCharacter =  ChangeGridsKey	ifTrue: [^#changeGridding].
	aCharacter =  TogglexGridKey	ifTrue: [^#togglexGridding].
	aCharacter =  ToggleyGridKey	ifTrue: [^#toggleyGridding].
	aCharacter =  BitEditKey			ifTrue: [^#magnify].			
	aCharacter =  WhiteKey			ifTrue: [^#colorWhite].			
	aCharacter =  LightGrayKey		ifTrue: [^#colorLightGray].			
	aCharacter =  GrayKey			ifTrue: [^#colorGray].				
	aCharacter =  DarkGrayKey		ifTrue: [^#colorDarkGray].			
	aCharacter =  BlackKey			ifTrue: [^#colorBlack].				
	aCharacter =  OutKey			ifTrue: [^#fileOutForm].			
	aCharacter =  InKey				ifTrue: [^#fileInForm]! !

!FormEditor methodsFor: 'editing tools' stamp: 'BG 2/25/2001 21:36'!
setColor: aColor
	"Set the mask (color) to aColor.
	Hacked to invoke color chooser if not B/W screen.
	Leaves the tool set in its previous state."

	self normalizeColor:  (unNormalizedColor := Display depth > 1
							ifTrue: [Color fromUser]
							ifFalse: [aColor]).
	tool := previousTool! !

!FormEditor methodsFor: 'private' stamp: 'BG 12/5/2003 22:58'!
setVariables
	tool := #repeatCopy.
	previousTool := tool.
	grid := 1 @ 1.
	togglegrid := 8 @ 8.
	xgridOn := false.
	ygridOn := false.
	mode := Form over.
	form := Form extent: 8 @ 8.
	form fillBlack.
	unNormalizedColor := color := Color black.
	hasUnsavedChanges := ValueHolder new contents: false.
! !

!FormEditor methodsFor: 'editing tools' stamp: 'BG 12/10/2003 16:00'!
singleCopy 
	"If the red button is clicked, copy the source form onto the display 
	screen."

   (BitBlt destForm: Display
           sourceForm: form halftoneForm: color
           combinationRule: (Display depth > 1 ifTrue: [mode ~= Form erase ifTrue: [Form paint] ifFalse: [mode]]
                                                     ifFalse: [mode])
           destOrigin: self cursorPoint sourceOrigin: 0@0 extent: form extent
           clipRect: view insetDisplayBox)
           colorMap: (Bitmap with: 0 with: 16rFFFFFFFF);
	copyBits.
	sensor waitNoButton.
	hasUnsavedChanges contents: true.! !

!FormEditor methodsFor: 'editing tools' stamp: ''!
togglexGridding
	"Turn x (horizontal) gridding off, if it is on, and turns it on, if it is off. 
	Does not change the primary tool."

	xgridOn
		ifTrue: 
			[grid := 1 @ grid y.
			xgridOn := false]
		ifFalse: 
			[grid := togglegrid x @ grid y.
			xgridOn := true].
	tool := previousTool! !

!FormEditor methodsFor: 'editing tools' stamp: ''!
toggleyGridding
	"Turn y (vertical) gridding off, if it is on, and turns it on, if it is off. 
	Does not change the primary tool."

	ygridOn
		ifTrue: 
			[grid := grid x @ 1.
			ygridOn := false]
		ifFalse: 
			[grid := grid x @ togglegrid y.
			ygridOn := true].
	tool := previousTool! !

!FormEditor methodsFor: 'private' stamp: 'BG 12/12/2003 15:50'!
trackFormUntil: aBlock

	| previousPoint cursorPoint displayForm |
	previousPoint := self cursorPoint.
	displayForm := Form extent: form extent depth: form depth.
	displayForm copy: (0 @ 0 extent: form extent)
	               from: form
	               to: 0 @ 0
	               rule: Form over.
	Display depth > 1 ifTrue: [displayForm reverse]. 
	displayForm displayOn: Display at: previousPoint rule: Form reverse.
	[aBlock value] whileFalse:
		[cursorPoint := self cursorPoint.
		(FlashCursor or: [cursorPoint ~= previousPoint])
			ifTrue:
			[displayForm displayOn: Display at: previousPoint rule: Form reverse.
			displayForm displayOn: Display at: cursorPoint rule: Form reverse.
			previousPoint := cursorPoint]].
	displayForm displayOn: Display at: previousPoint rule: Form reverse.
	^previousPoint! !

!FormEditor methodsFor: 'editing tools' stamp: ''!
underMode
	"Set the mode for the tools that copy the form onto the display to under. 
	Leaves the tool set in its previous state."

	mode := Form under.
	tool := previousTool! !

!MouseMenuController methodsFor: 'menu messages' stamp: 'sma 3/11/2000 15:01'!
blueButtonActivity
	"This normally opens the window menu. It is a no-op here
	as only the StandardSystemController deals with that kind
	of menus."! !

!MouseMenuController methodsFor: 'control defaults' stamp: 'sma 3/11/2000 15:28'!
controlActivity
	"Refer to the comment in Controller|controlActivity."
	| cursorPoint |
	cursorPoint := sensor cursorPoint.
	super controlActivity.
	(cursorPoint = sensor cursorPoint and: [self viewHasCursor])
		ifTrue: 
			[sensor redButtonPressed ifTrue: [^ self redButtonActivity].
			sensor yellowButtonPressed ifTrue: [^ self yellowButtonActivity].
			sensor blueButtonPressed ifTrue: [^ self blueButtonActivity]]! !

!MouseMenuController methodsFor: 'pluggable menus' stamp: 'sma 3/11/2000 12:36'!
getPluggableYellowButtonMenu: shiftKeyState
	^ view getMenu: shiftKeyState! !

!MouseMenuController methodsFor: 'control defaults' stamp: 'sma 3/11/2000 11:24'!
isControlActive 
	"In contrast to class Controller, only blue button but not yellow button
	events will end the receiver's control loop."

	^ self viewHasCursor and: [sensor blueButtonPressed not]! !

!MouseMenuController methodsFor: 'menu messages' stamp: ''!
performMenuMessage: aSelector
	"Perform a menu command by sending self the message aSelector.
	 Default does nothing special."

	^self perform: aSelector! !

!MouseMenuController methodsFor: 'pluggable menus' stamp: 'sw 2/17/2002 04:35'!
pluggableYellowButtonActivity: shiftKeyState
	"Invoke the model's popup menu."

	| menu |
	(menu := self getPluggableYellowButtonMenu: shiftKeyState)
		ifNil:
			[sensor waitNoButton]
		ifNotNil:
			[self terminateAndInitializeAround:
				[menu invokeOn: model orSendTo: self]]! !

!MouseMenuController methodsFor: 'menu messages' stamp: 'sma 3/11/2000 14:56'!
redButtonActivity
	"Determine which item in the red button pop-up menu is selected. If one 
	is selected, then send the corresponding message to the object designated 
	as the menu message receiver."

	| index |
	redButtonMenu ~~ nil
		ifTrue: 
			[index := redButtonMenu startUp.
			index ~= 0 
				ifTrue: [self perform: (redButtonMessages at: index)]]
		ifFalse: [super controlActivity]! !

!MouseMenuController methodsFor: 'menu setup' stamp: ''!
redButtonMenu: aSystemMenu redButtonMessages: anArray 
	"Initialize the pop-up menu that should appear when the user presses the 
	red mouse button to be aSystemMenu. The corresponding messages that 
	should be sent are listed in the array, anArray."

	redButtonMenu release.
	redButtonMenu := aSystemMenu.
	redButtonMessages := anArray! !

!MouseMenuController methodsFor: 'initialize-release' stamp: 'sma 3/11/2000 15:54'!
release
	super release.
	redButtonMenu release! !

!MouseMenuController methodsFor: 'initialize-release' stamp: 'sma 3/11/2000 15:54'!
reset
	"Eliminate references to all mouse button menus."

	redButtonMenu := nil.
	redButtonMessages := nil! !

!MouseMenuController methodsFor: 'pluggable menus' stamp: 'sw 3/22/2001 12:03'!
shiftedTextPaneMenuRequest
	"The user chose the more... branch from the text-pane menu."

	^ self pluggableYellowButtonActivity: true! !

!MouseMenuController methodsFor: 'pluggable menus' stamp: 'sma 3/11/2000 12:37'!
shiftedYellowButtonActivity
	"Invoke the model's special popup menu."

	^ self pluggableYellowButtonActivity: true! !

!MouseMenuController methodsFor: 'pluggable menus' stamp: 'sma 3/11/2000 12:37'!
unshiftedYellowButtonActivity
	"Invoke the model's normal popup menu."

	^ self pluggableYellowButtonActivity: false! !

!MouseMenuController methodsFor: 'menu messages' stamp: 'sma 3/11/2000 14:59'!
yellowButtonActivity
	"This normally opens a popup menu. Determine the selected
	item and, if one is selected, then send the corresponding message
	to either the model or the receiver."

	^ self pluggableYellowButtonActivity: sensor leftShiftDown! !

MouseMenuController subclass: #ScrollController
	instanceVariableNames: 'scrollBar marker savedArea menuBar savedMenuBarArea'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Kernel-Remnants'!

!ScrollController commentStamp: '<historical>' prior: 0!
I represent control for scrolling using a scrollBar. I am a MouseMenuController that creates a scrollBar, rather than menus. My subclasses add the button menus. I keep control as long as the cursor is inside the view or the scrollBar area.
	
A scrollBar is a rectangular area representing the length of the information being viewed. It contains an inner rectangle whose top y-coordinate represents the relative position of the information visible on the screen with respect to all of the information, and whose size represents the relative amount of that information visible on the screen. The user controls which part of the information is visible by pressing the red button. If the cursor is to the right of the inner rectangle, the window onto the visible information moves upward, if the cursor is to the left, the window moves downward, and if the cursor is inside, the inner rectangle is grabbed and moved to a desired position.!

ScrollController subclass: #ListController
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Support'!

!ListController commentStamp: '<historical>' prior: 0!
I am a kind of ScrollController that assumes that the view is a kind of ListView. Therefore, scrolling means moving the items in a textual list (menu) up or down. In addition, I provide the red button activity of determining when the red button is selecting an item in the list.!

!ListController methodsFor: 'private' stamp: ''!
changeModelSelection: anInteger
	model toggleListIndex: anInteger! !

!ListController methodsFor: 'marker adjustment' stamp: ''!
computeMarkerRegion 
	"Refer to the comment in ScrollController|computeMarkerRegion."

	| viewList |
	viewList := view list.
	viewList compositionRectangle height = 0
		ifTrue: [^ 0@0 extent: Preferences scrollBarWidth@scrollBar inside height].
	^ 0@0 extent: Preferences scrollBarWidth@
			((viewList clippingRectangle height asFloat /
						viewList compositionRectangle height *
							scrollBar inside height)
					rounded min: scrollBar inside height)! !

!ListController methodsFor: 'control defaults' stamp: 'bf 4/14/1999 12:41'!
controlActivity
	self scrollByKeyboard ifTrue: [^self].
	self processKeyboard.
	super controlActivity.
! !

!ListController methodsFor: 'marker adjustment' stamp: ''!
markerDelta

	| viewList |
	viewList := view list.
	viewList compositionRectangle height == 0 ifTrue: [
		^ (marker top - scrollBar inside top) - scrollBar inside height
	].
	^ (marker top - scrollBar inside top) -
		((viewList clippingRectangle top -
				viewList compositionRectangle top) asFloat /
			viewList compositionRectangle height asFloat *
			scrollBar inside height asFloat) rounded
! !

!ListController methodsFor: 'menu messages' stamp: 'acg 9/18/1999 14:09'!
processKeyboard
	"Derived from a Martin Pammer submission, 02/98"

     | keyEvent oldSelection nextSelection max min howMany |
	sensor keyboardPressed ifFalse: [^ self].

     keyEvent := sensor keyboard asciiValue.
     oldSelection := view selection.
     nextSelection := oldSelection.
     max := view maximumSelection.
     min := view minimumSelection.
     howMany := view clippingBox height // view list lineGrid.

     keyEvent == 31 ifTrue:
		["down-arrow; move down one, wrapping to top if needed"
		nextSelection := oldSelection + 1.
		nextSelection > max ifTrue: [nextSelection := 1]].

     keyEvent == 30 ifTrue:
		["up arrow; move up one, wrapping to bottom if needed"
		nextSelection := oldSelection - 1.
		nextSelection < 1 ifTrue: [nextSelection := max]].

     keyEvent == 1  ifTrue: [nextSelection := 1].  "home"
     keyEvent == 4  ifTrue: [nextSelection := max].   "end"
     keyEvent == 11 ifTrue: [nextSelection := min max: (oldSelection -
howMany)].  "page up"
     keyEvent == 12  ifTrue: [nextSelection := (oldSelection + howMany)
min: max].  "page down"
     nextSelection = oldSelection  ifFalse:
		[model okToChange
			ifTrue:
				[self changeModelSelection: nextSelection.
				self moveMarker]]
			! !

!ListController methodsFor: 'selecting' stamp: 'tk 4/1/98 10:33'!
redButtonActivity
	| noSelectionMovement oldSelection selection nextSelection pt scrollFlag firstTime |
	noSelectionMovement := true.
	scrollFlag := false.
	oldSelection := view selection.
	firstTime := true.
	[sensor redButtonPressed | firstTime]
		whileTrue: 
			[selection := view findSelection: (pt := sensor cursorPoint).
			firstTime := false.
			selection == nil ifTrue:  "Maybe out of box - check for auto-scroll"
					[pt y < view insetDisplayBox top ifTrue:
						[self scrollView: view list lineGrid.
						scrollFlag := true.
						selection := view firstShown].
					pt y > view insetDisplayBox bottom ifTrue:
						[self scrollView: view list lineGrid negated.
						scrollFlag := true.
						selection := view lastShown]].
			selection == nil ifFalse:
					[view moveSelectionBox: (nextSelection := selection).
					nextSelection ~= oldSelection
						ifTrue: [noSelectionMovement := false]]].
	nextSelection ~~ nil & (nextSelection = oldSelection
			ifTrue: [noSelectionMovement]
			ifFalse: [true]) ifTrue: [self changeModelSelection: nextSelection].
	scrollFlag ifTrue: [self moveMarker]! !

!ListController methodsFor: 'scrolling' stamp: ''!
scrollAmount 
	"Refer to the comment in ScrollController|scrollAmount."

	^sensor cursorPoint y - scrollBar inside top! !

!ListController methodsFor: 'scrolling' stamp: ''!
scrollView: anInteger 
	"Scroll the view and highlight the selection if it just came into view"
	| wasClipped |
	wasClipped := view isSelectionBoxClipped.
	(view scrollBy: anInteger)
		ifTrue: [view isSelectionBoxClipped ifFalse:
					[wasClipped ifTrue:  "Selection came into view"
						[view displaySelectionBox]].
				^ true]
		ifFalse: [^ false]! !

!ListController methodsFor: 'scrolling' stamp: ''!
viewDelta 
	"Refer to the comment in ScrollController|viewDelta."

	| viewList |
	viewList := view list.
	^(viewList clippingRectangle top -
			viewList compositionRectangle top -
			((marker top - scrollBar inside top) asFloat /
				scrollBar inside height asFloat *
				viewList compositionRectangle height asFloat))
		roundTo: viewList lineGrid! !

ListController subclass: #PluggableListController
	instanceVariableNames: 'terminateDuringSelect'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Pluggable Views'!

!PluggableListController methodsFor: 'private' stamp: 'di 5/25/1998 10:20'!
changeModelSelection: anInteger
	"Let the view handle this."

	terminateDuringSelect ifTrue: [self controlTerminate].
	view changeModelSelection: anInteger.
	terminateDuringSelect ifTrue: [self controlInitialize].! !

!PluggableListController methodsFor: 'initialization' stamp: 'di 5/25/1998 10:15'!
initialize
	super initialize.
	self terminateDuringSelect: false! !

!PluggableListController methodsFor: 'private' stamp: 'sma 3/11/2000 15:38'!
processKeyboard
	sensor keyboardPressed
		ifTrue: [view handleKeystroke: sensor keyboard]
		ifFalse: [super processKeyboard]! !

!PluggableListController methodsFor: 'control defaults' stamp: 'tk 4/1/98 09:40'!
redButtonActivity
	model okToChange   "Don't change selection if model refuses to unlock"
		ifTrue: [^ super redButtonActivity]! !

!PluggableListController methodsFor: 'private' stamp: 'di 5/25/1998 10:14'!
terminateDuringSelect: trueOrFalse
	terminateDuringSelect := trueOrFalse! !

PluggableListController subclass: #PluggableListControllerOfMany
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Pluggable Views'!

!PluggableListControllerOfMany methodsFor: 'control defaults' stamp: 'tpr 10/4/2001 22:19'!
redButtonActivity
	| selection firstHit turningOn lastSelection pt scrollFlag |
	model okToChange ifFalse: [^ self].
		"Don't change selection if model refuses to unlock"
	firstHit := true.
	scrollFlag := false.
	lastSelection := 0.
	[sensor redButtonPressed] whileTrue: 
		[selection := view findSelection: (pt := sensor cursorPoint).
		selection == nil ifTrue:  "Maybe out of box - check for auto-scroll"
			[pt y < view insetDisplayBox top ifTrue:
				[self scrollView: view list lineGrid.
				scrollFlag := true.
				selection := view firstShown].
			pt y > view insetDisplayBox bottom ifTrue:
				[self scrollView: view list lineGrid negated.
				scrollFlag := true.
				selection := view lastShown]].
		(selection == nil or: [selection = lastSelection]) ifFalse: 
			[firstHit ifTrue:
				[firstHit := false.
				turningOn := (view listSelectionAt: selection) not].
			view selection: selection.
			(view listSelectionAt: selection) == turningOn ifFalse:
				[view displaySelectionBox.
				view listSelectionAt: selection put: turningOn].
			lastSelection := selection]].
	selection notNil ifTrue:
		["Normal protocol delivers change, so unchange first (ugh)"
		view listSelectionAt: selection put: (view listSelectionAt: selection) not.
		self changeModelSelection: selection].
	scrollFlag ifTrue: [self moveMarker]! !

!PluggableListControllerOfMany methodsFor: 'scrolling' stamp: 'tk 4/8/98 11:09'!
scrollView: anInteger 
	"Need to minimize the selections which get recomputed"
	| oldLimit |
	oldLimit := anInteger > 0
		ifTrue: [view firstShown]
		ifFalse: [view lastShown].
	(view scrollBy: anInteger)
		ifTrue: [anInteger > 0  "Highlight selections brought into view"
					ifTrue: [view highlightFrom: view firstShown
								to: (oldLimit-1 min: view lastShown)]
					ifFalse: [view highlightFrom: (oldLimit+1 max: view firstShown)
								to: view lastShown].
				^ true]
		ifFalse: [^ false]! !

!ScrollController methodsFor: 'scrolling' stamp: ''!
anyButtonActivity
	"deal with red button down in scrollBar beyond yellowLine"

	self yellowButtonActivity! !

!ScrollController methodsFor: 'cursor' stamp: ''!
changeCursor: aCursor 
	"The current cursor should be set to be aCursor."

	sensor currentCursor ~~ aCursor ifTrue: [aCursor show]! !

!ScrollController methodsFor: 'marker adjustment' stamp: ''!
computeMarkerRegion
	"Answer the rectangular area in which the gray area of the scroll bar 
	should be displayed."

	^0@0 extent: Preferences scrollBarWidth @
			((view window height asFloat /
						view boundingBox height *
							scrollBar inside height)
				 rounded min: scrollBar inside height)! !

!ScrollController methodsFor: 'control defaults' stamp: 'sma 3/11/2000 15:17'!
controlActivity
	self scrollByKeyboard ifTrue: [^ self].
	self scrollBarContainsCursor
		ifTrue: [self scroll]
		ifFalse: [self normalActivity]! !

!ScrollController methodsFor: 'basic control sequence' stamp: ''!
controlInitialize
	"Recompute scroll bars.  Save underlying image unless it is already saved."
	| |
	super controlInitialize.
	scrollBar region: (0 @ 0 extent: 24 @ view apparentDisplayBox height).
	scrollBar insideColor: view backgroundColor.
	marker region: self computeMarkerRegion.
	scrollBar := scrollBar align: scrollBar topRight with: view apparentDisplayBox topLeft.
	marker := marker align: marker topCenter with: self upDownLine @ (scrollBar top + 2).
	savedArea isNil ifTrue: [savedArea := Form fromDisplay: scrollBar].
	scrollBar displayOn: Display.

	"Show a border around yellow-button (menu) region"
"
	yellowBar := Rectangle left: self yellowLine right: scrollBar right + 1
		top: scrollBar top bottom: scrollBar bottom.
	Display border: yellowBar width: 1 mask: Form veryLightGray.
"
	self moveMarker
! !

!ScrollController methodsFor: 'basic control sequence' stamp: ''!
controlTerminate

	super controlTerminate.
	savedArea notNil 	
		ifTrue: 
			[savedArea displayOn: Display at: scrollBar topLeft.
			savedArea := nil].! !

!ScrollController methodsFor: 'scrolling' stamp: ''!
downLine
	"if cursor before downLine, display down cursor and scroll down on button down"

	^scrollBar left + 6 ! !

!ScrollController methodsFor: 'initialize-release' stamp: ''!
initialize
	super initialize.
	scrollBar := Quadrangle new.
	scrollBar borderWidthLeft: 2 right: 0 top: 2 bottom: 2.
	marker := Quadrangle new.
	marker insideColor: Preferences scrollBarColor.
	menuBar := Quadrangle new.
	menuBar borderWidthLeft:  2 right: 0 top: 2 bottom: 2.! !

!ScrollController methodsFor: 'control defaults' stamp: 'ar 3/24/2000 00:45'!
isControlActive 
	super isControlActive ifTrue: [^ true].
	sensor blueButtonPressed ifTrue: [^ false].
	^ (scrollBar inside merge: view insetDisplayBox) containsPoint: sensor cursorPoint! !

!ScrollController methodsFor: 'control defaults' stamp: 'sma 3/11/2000 15:31'!
isControlWanted
	^ self viewHasCursor! !

!ScrollController methodsFor: 'cursor' stamp: ''!
markerContainsCursor
	"Answer whether the gray area inside the scroll bar area contains the 
	cursor."

	^marker inside containsPoint: sensor cursorPoint! !

!ScrollController methodsFor: 'marker adjustment' stamp: ''!
markerDelta
	^ marker top 
		- scrollBar inside top  
		- ((view window top - view boundingBox top) asFloat 
			/ view boundingBox height asFloat *
				scrollBar inside height asFloat) rounded! !

!ScrollController methodsFor: 'marker adjustment' stamp: ''!
markerRegion: aRectangle 
	"Set the area defined by aRectangle as the marker. Fill it with gray tone."

	Display fill: marker fillColor: scrollBar insideColor.
	marker region: aRectangle.
	marker := marker align: marker topCenter 
			with: self upDownLine @ (scrollBar top + 2) ! !

!ScrollController methodsFor: 'cursor' stamp: ''!
menuBarContainsCursor
	"Answer whether the cursor is anywhere within the menu bar area."

	^ menuBar notNil and:
			[menuBar containsPoint: sensor cursorPoint]! !

!ScrollController methodsFor: 'marker adjustment' stamp: ''!
moveMarker
	"The view window has changed. Update the marker."

	self moveMarker: self markerDelta negated anchorMarker: nil! !

!ScrollController methodsFor: 'marker adjustment' stamp: ''!
moveMarker: anInteger anchorMarker: anchorMarker
	"Update the marker so that is is translated by an amount corresponding to 
	a distance of anInteger, constrained within the boundaries of the scroll 
	bar.  If anchorMarker ~= nil, display the border around the area where the
	marker first went down."

	Display fill: marker fillColor: scrollBar insideColor.
	anchorMarker = nil
		ifFalse: [Display border: anchorMarker width: 1 fillColor: Color gray].
	marker := marker translateBy: 0 @
				((anInteger min: scrollBar inside bottom - marker bottom) max:
					scrollBar inside top - marker top).
	marker displayOn: Display! !

!ScrollController methodsFor: 'marker adjustment' stamp: ''!
moveMarkerTo: aRectangle 
	"Same as markerRegion: aRectangle; moveMarker, except a no-op if the marker
	 would not move."

	(aRectangle height = marker height and: [self viewDelta = 0]) ifFalse:
		[self markerRegion: aRectangle.
		self moveMarker]! !

!ScrollController methodsFor: 'control defaults' stamp: 'sma 3/11/2000 15:16'!
normalActivity
	super controlActivity! !

!ScrollController methodsFor: 'scrolling' stamp: 'ls 7/11/1998 06:33'!
scroll
	"Check to see whether the user wishes to jump, scroll up, or scroll down."
	| savedCursor |
	savedCursor := sensor currentCursor.
			[self scrollBarContainsCursor]
				whileTrue: 
					[self interActivityPause.
					sensor cursorPoint x <= self downLine
								ifTrue: [self scrollDown]
								ifFalse: [sensor cursorPoint x <= self upLine
										ifTrue: [self scrollAbsolute]
										ifFalse: [sensor cursorPoint x <= self yellowLine
												ifTrue: [self scrollUp]
												ifFalse: [sensor cursorPoint x <= scrollBar right
														ifTrue: "Might not be, with touch pen"
														[self changeCursor: Cursor menu.
														sensor anyButtonPressed 
														ifTrue: [self changeCursor: savedCursor. 
																self anyButtonActivity]]]]]].
	savedCursor show! !

!ScrollController methodsFor: 'private' stamp: ''!
scrollAbsolute
	| markerOutline oldY markerForm |
	self changeCursor: Cursor rightArrow.

	oldY := -1.
	sensor anyButtonPressed ifTrue: 
	  [markerOutline := marker deepCopy.
	  markerForm := Form fromDisplay: marker.
	  Display fill: marker fillColor: scrollBar insideColor.
	  Display border: markerOutline width: 1 fillColor: Color gray.
	  markerForm 
		follow: 
			[oldY ~= sensor cursorPoint y
				ifTrue: 
					[oldY := sensor cursorPoint y.
					marker := marker translateBy: 
					  0 @ ((oldY - marker center y 
						min: scrollBar inside bottom - marker bottom) 
						max: scrollBar inside top - marker top).
					self scrollView].
				marker origin] 
		while: [sensor anyButtonPressed].

	  Display fill: markerOutline fillColor: scrollBar insideColor.
	  self moveMarker]! !

!ScrollController methodsFor: 'scrolling' stamp: ''!
scrollAmount
	"Answer the number of bits of y-coordinate should be scrolled. This is a 
	default determination based on the view's preset display transformation."

	^((view inverseDisplayTransform: sensor cursorPoint)
		- (view inverseDisplayTransform: scrollBar inside topCenter)) y! !

!ScrollController methodsFor: 'cursor' stamp: ''!
scrollBarContainsCursor
	"Answer whether the cursor is anywhere within the scroll bar area."

	^scrollBar containsPoint: sensor cursorPoint! !

!ScrollController methodsFor: 'scrolling' stamp: 'th 12/11/1999 16:57'!
scrollByKeyboard
	| keyEvent |
	keyEvent := sensor keyboardPeek.
	keyEvent ifNil: [^ false].
	(sensor controlKeyPressed or:[sensor commandKeyPressed]) ifFalse: [^ false].
	keyEvent asciiValue = 30
		ifTrue: 
			[sensor keyboard.
			self scrollViewDown ifTrue: [self moveMarker].
			^ true].
	keyEvent asciiValue = 31
		ifTrue: 
			[sensor keyboard.
			self scrollViewUp ifTrue: [self moveMarker].
			^ true].
	^ false! !

!ScrollController methodsFor: 'private' stamp: ''!
scrollDown
	| markerForm firstTime |
	self changeCursor: Cursor down.
	sensor anyButtonPressed ifTrue:
	  [markerForm := Form fromDisplay: marker.
	  Display fill: marker fillColor: scrollBar insideColor.
	  firstTime := true.
	  markerForm 
		follow: 
			[self scrollViewDown ifTrue:
				[marker := marker translateBy: 0 @
					((self markerDelta negated 
						min: scrollBar inside bottom - marker bottom) 
						max: scrollBar inside top - marker top).
				firstTime
					ifTrue: [
						"pause before scrolling repeatedly"
						(Delay forMilliseconds: 250) wait.
						firstTime := false.
					] ifFalse: [
						(Delay forMilliseconds: 50) wait.
					].
				].
			marker origin] 
		while: [sensor anyButtonPressed].
	  self moveMarker.]! !

!ScrollController methodsFor: 'private' stamp: ''!
scrollUp
	| markerForm firstTime |
	self changeCursor: Cursor up.
	sensor anyButtonPressed ifTrue:
	  [markerForm := Form fromDisplay: marker.
	  Display fill: marker fillColor: scrollBar insideColor.
	  firstTime := true.
	  markerForm 
		follow: 
			[self scrollViewUp ifTrue:
				[marker := marker translateBy: 0 @
					((self markerDelta negated 
						min: scrollBar inside bottom - marker bottom) 
						max: scrollBar inside top - marker top).
				firstTime
					ifTrue: [
						"pause before scrolling repeatedly"
						(Delay forMilliseconds: 250) wait.
						firstTime := false.
					] ifFalse: [
						(Delay forMilliseconds: 50) wait.
					].
				].
			marker origin] 
		while: [sensor anyButtonPressed].
	  self moveMarker.]! !

!ScrollController methodsFor: 'scrolling' stamp: ''!
scrollView
	"The scroll bar jump method was used so that the view should be 
	updated to correspond to the location of the scroll bar gray area.
	Return true only if scrolling took place."
	^ self scrollView: self viewDelta! !

!ScrollController methodsFor: 'scrolling' stamp: ''!
scrollView: anInteger 
	"Tell the reciever's view to scroll by anInteger amount.
	Return true only if scrolling actually resulted."
	(view scrollBy: 0 @ 
				((anInteger min: view window top - view boundingBox top)
						max: view window top - view boundingBox bottom))
		ifTrue: [view clearInside; display.  ^ true]
		ifFalse: [^ false]! !

!ScrollController methodsFor: 'scrolling' stamp: ''!
scrollViewDown
	"Scroll the receiver's view down the default amount.
	Return true only if scrolling actually took place."
	^ self scrollView: self scrollAmount! !

!ScrollController methodsFor: 'scrolling' stamp: ''!
scrollViewUp
	"Scroll the receiver's view up the default amount.
	Return true only if scrolling actually took place."
	^ self scrollView: self scrollAmount negated! !

!ScrollController methodsFor: 'scrolling' stamp: ''!
upDownLine
	"Check to see whether the user wishes to jump, scroll up, or scroll down."

	^scrollBar left + 12! !

!ScrollController methodsFor: 'scrolling' stamp: ''!
upLine
	"if cursor beyond upLine, display up cursor and scroll up on button down"

	^scrollBar left + 12! !

!ScrollController methodsFor: 'scrolling' stamp: ''!
viewDelta
	"Answer an integer that indicates how much the view should be scrolled. 
	The scroll bar has been moved and now the view must be so the amount 
	to scroll is computed as a ratio of the current scroll bar position."

	^view window top - view boundingBox top -
		((marker top - scrollBar inside top) asFloat /
			scrollBar inside height asFloat *
				view boundingBox height asFloat) rounded! !

!ScrollController methodsFor: 'scrolling' stamp: ''!
yellowLine
	"Check to see whether the user wishes to jump, scroll up, or scroll down."

	^scrollBar left + 16! !

MouseMenuController subclass: #StandardSystemController
	instanceVariableNames: 'status'
	classVariableNames: 'HBorderCursor ScheduledBlueButtonMenu ScheduledBlueButtonMessages VBorderCursor'
	poolDictionaries: ''
	category: 'ST80-Support'!

!StandardSystemController commentStamp: '<historical>' prior: 0!
I am a controller for StandardSystemViews, that is, those views that are at the top level of a project in the system user interface. I am a kind of MouseMenuController that creates a blue button menu for moving, framing, collapsing, and closing ScheduledViews, and for selecting views under the view of my instance.!

StandardSystemController subclass: #ProjectController
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Support'!

!ProjectController methodsFor: 'control activity' stamp: 'rbb 2/16/2005 16:24'!
redButtonActivity
	| index |
	view isCollapsed ifTrue: [^ super redButtonActivity].
	(view insetDisplayBox containsPoint: Sensor cursorPoint)
		ifFalse: [^ super redButtonActivity].
	index := (UIManager default chooseFrom: #('enter' 'jump to project...') lines: #()).
	index = 0 ifTrue: [^ self].

	"save size on enter for thumbnail on exit"
	model setViewSize: view insetDisplayBox extent.
	index = 1 ifTrue: [^ model enter: false revert: false saveForRevert: false].
	index = 2 ifTrue: [Project jumpToProject. ^ self].
! !

!StandardSystemController classSide methodsFor: 'class initialization' stamp: 'sma 3/11/2000 11:57'!
initialize
	"StandardSystemController initialize"

	ScheduledBlueButtonMenu := SelectionMenu
		labels:
'edit label
choose color...
two-tone/full color
move
frame
full screen
collapse
close'
	lines: #(3 7)
	selections: #(label chooseColor toggleTwoTone move reframe fullScreen collapse close).

	VBorderCursor := Cursor extent: 16@16 fromArray: #(
		2r1010000000000000
		2r1010000000000000
		2r1010000000000000
		2r1010000000000000
		2r1010000000000000
		2r1010010000100000
		2r1010110000110000
		2r1011111111111000
		2r1010110000110000
		2r1010010000100000
		2r1010000000000000
		2r1010000000000000
		2r1010000000000000
		2r1010000000000000
		2r1010000000000000
		2r1010000000000000)
			offset: 0@0.
	HBorderCursor := Cursor extent: 16@16 fromArray: #(
		2r1111111111111111
		2r0000000000000000
		2r1111111111111111
		2r0000000100000000
		2r0000001110000000
		2r0000011111000000
		2r0000000100000000
		2r0000000100000000
		2r0000000100000000
		2r0000000100000000
		2r0000011111000000
		2r0000001110000000
		2r0000000100000000
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000)
			offset: 0@0.! !

!StandardSystemController methodsFor: 'borders' stamp: 'ls 7/11/1998 07:45'!
adjustPaneBorders 
	| side sub newRect outerFrame |
	outerFrame := view displayBox.
	side := #none.
	VBorderCursor showWhile:
		[ [sub := view subviewWithLongestSide: [:s | side := s]
						near: sensor cursorPoint.
		  self cursorOnBorder and: [(side = #left) | (side = #right)]]
			whileTrue: [
				self interActivityPause.
				sensor redButtonPressed ifTrue:
				[side = #left ifTrue:
					[newRect := sub stretchFrame:
						[:f | (f withLeft: sensor cursorPoint x)
								intersect: outerFrame]
						startingWith: sub displayBox].
				side = #right ifTrue:
					[newRect := sub stretchFrame:
						[:f | (f withRight: sensor cursorPoint x)
								intersect: outerFrame]
						startingWith: sub displayBox].
				view reframePanesAdjoining: sub along: side to: newRect]]].
	HBorderCursor showWhile:
		[ [sub := view subviewWithLongestSide: [:s | side := s]
						near: sensor cursorPoint.
		  self cursorOnBorder and: [(side = #top) | (side = #bottom)]]
			whileTrue: [
				self interActivityPause.
				sensor redButtonPressed ifTrue:
				[side = #top ifTrue:
					[newRect := sub stretchFrame:
						[:f | (f withTop: sensor cursorPoint y)
								intersect: outerFrame]
						startingWith: sub displayBox].
				side = #bottom ifTrue:
					[newRect := sub stretchFrame:
						[:f | (f withBottom: sensor cursorPoint y)
								intersect: outerFrame]
						startingWith: sub displayBox].
				view reframePanesAdjoining: sub along: side to: newRect]]]! !

!StandardSystemController methodsFor: 'borders' stamp: 'di 11/16/2001 22:22'!
adjustWindowBorders 
	| side noClickYet |
	noClickYet := true.
	VBorderCursor showWhile:
		[ [side := view displayBox sideNearestTo: sensor cursorPoint.
		  self cursorOnBorder
			and: [(side = #left) | (side = #right)
			and: [noClickYet or: [sensor redButtonPressed]]]]
			whileTrue:
			[sensor redButtonPressed ifTrue:
				[noClickYet := false.
				side = #left ifTrue:
					[view newFrame: [:f | f withLeft: sensor cursorPoint x]].
				side = #right ifTrue:
					[view newFrame: [:f | f withRight: sensor cursorPoint x]]].
			self interActivityPause]].
	HBorderCursor showWhile:
		[ [side := view displayBox sideNearestTo: sensor cursorPoint.
		  self cursorOnBorder
			and: [(side = #top) | (side = #bottom)
			and: [noClickYet or: [sensor redButtonPressed]]]]
			whileTrue:
			[sensor redButtonPressed ifTrue:
				[noClickYet := false.
				side = #top ifTrue:
					[view newFrame: [:f | f withTop: sensor cursorPoint y]].
				side = #bottom ifTrue:
					[view newFrame: [:f | f withBottom: sensor cursorPoint y]]].
		  self interActivityPause]]! !

!StandardSystemController methodsFor: 'borders' stamp: 'ls 7/11/1998 07:38'!
adjustWindowCorners 
	| box cornerBox p clicked f2 |
	box := view windowBox.
	clicked := false.
	#(topLeft topRight bottomRight bottomLeft)
		do: [:readCorner |
			cornerBox := ((box insetBy: 2) perform: readCorner) - (10@10) extent: 20@20.
			(cornerBox containsPoint: sensor cursorPoint)
				ifTrue: 
				["Display reverse: cornerBox."
				(Cursor perform: readCorner) showWhile:
					[[(cornerBox containsPoint: (p := sensor cursorPoint))
						and: [(clicked := sensor anyButtonPressed) not]]
						whileTrue: [ self interActivityPause ].
				"Display reverse: cornerBox."
				clicked ifTrue:
					[view newFrame:
						[:f | p := sensor cursorPoint.
						readCorner = #topLeft ifTrue:
							[f2 := p corner: f bottomRight].
						readCorner = #bottomLeft ifTrue:
							[f2 := (f withBottom: p y) withLeft: p x].
						readCorner = #bottomRight ifTrue:
							[f2 := f topLeft corner: p].
						readCorner = #topRight ifTrue:
							[f2 := (f withTop: p y) withRight: p x].
						f2]]]]].
	^ clicked! !

!StandardSystemController methodsFor: 'control defaults' stamp: 'sma 3/11/2000 12:01'!
blueButtonActivity
	ScheduledBlueButtonMenu ifNil: [^ super controlActivity].
	ScheduledBlueButtonMenu invokeOn: self! !

!StandardSystemController methodsFor: 'borders' stamp: 'di 11/16/2001 22:30'!
checkForReframe
	| cp |
	view isCollapsed ifTrue: [^ self].
	cp := sensor cursorPoint.
	((view closeBoxFrame expandBy: 2) containsPoint: cp)
		| ((view growBoxFrame expandBy: 2) containsPoint: cp)
		ifTrue: [^ self].  "Dont let reframe interfere with close/grow"
	self adjustWindowCorners.
	self cursorOnBorder ifFalse: [^ self].
	((view insetDisplayBox insetBy: 2@2) containsPoint: cp)
		ifFalse: [^ self adjustWindowBorders].
	view subViews size <= 1 ifTrue: [^ self].
	(view subviewWithLongestSide: [:s | ] near: cp) == nil
		ifFalse: [^ self adjustPaneBorders].! !

!StandardSystemController methodsFor: 'menu messages' stamp: ''!
chooseColor
	"Allow the user to specify a new background color for the receiver's window.  5/6/96 sw.
	 7/31/96 sw: use Color fromUser"

	view backgroundColor: Color fromUser; uncacheBits; display! !

!StandardSystemController methodsFor: 'menu messages' stamp: ''!
close
	"The receiver's view should be removed from the screen and from the 
	collection of scheduled views."

	model okToChange ifFalse: [^self].
	status := #closed.
	view erase! !

!StandardSystemController methodsFor: 'scheduling' stamp: 'sw 9/30/97 22:04'!
closeAndUnschedule
	"Erase the receiver's view and remove it from the collection of scheduled 
	views."

	status := #closed.
	view erase.
	view release.
	ScheduledControllers unschedule: self; searchForActiveController
! !

!StandardSystemController methodsFor: 'scheduling' stamp: ''!
closeAndUnscheduleNoErase
	"Remove the scheduled view from the collection of scheduled views. Set 
	its status to closed but do not erase."

	status := #closed.
	view release.
	ScheduledControllers unschedule: self! !

!StandardSystemController methodsFor: 'scheduling' stamp: 'jm 3/18/98 19:21'!
closeAndUnscheduleNoTerminate
	"Erase the receiver's view and remove it from the collection of scheduled views, but do not terminate the current process."

	status := #closed.
	view erase.
	view release.
	ScheduledControllers unschedule: self.
! !

!StandardSystemController methodsFor: 'menu messages' stamp: ''!
collapse
	"Get the receiver's view to change to a collapsed view on the screen."
	view collapseToPoint: view chooseCollapsePoint! !

!StandardSystemController methodsFor: 'control defaults' stamp: 'sma 3/11/2000 15:20'!
controlActivity
	self checkForReframe.
	^ super controlActivity! !

!StandardSystemController methodsFor: 'basic control sequence' stamp: 'sw 10/20/1999 09:52'!
controlInitialize
	view displayEmphasized.
	view uncacheBits.  "Release cached bitmap while active"
	model windowActiveOnFirstClick ifFalse: [sensor waitNoButton].
	status := #active.
	view isCollapsed ifFalse: [model modelWakeUpIn: view]! !

!StandardSystemController methodsFor: 'basic control sequence' stamp: 'di 5/11/1999 22:05'!
controlTerminate
	status == #closed
		ifTrue: 
			[view ~~ nil ifTrue: [view release].
			ScheduledControllers unschedule: self.
			^self].
	view deEmphasize; cacheBits.
	view isCollapsed ifFalse: [model modelSleep].! !

!StandardSystemController methodsFor: 'borders' stamp: ''!
cursorOnBorder 
	| cp i box |
	view isCollapsed ifTrue: [^ false].
	cp := sensor cursorPoint.
	((view labelDisplayBox insetBy: (0@2 corner: 0@-2)) containsPoint: cp)
		ifTrue: [^ false].
	(i := view subViews findFirst: [:v | v displayBox containsPoint: cp]) = 0
		ifTrue: [box := view windowBox]
		ifFalse: [box := (view subViews at: i) insetDisplayBox].
	^ ((box insetBy: 3) containsPoint: cp) not
		and: [(box expandBy: 4) containsPoint: cp]! !

!StandardSystemController methodsFor: 'menu messages' stamp: ''!
expand
	"The receiver's view was collapsed; open it again and ask the user to 
	designate its rectangular area."
	view expand; emphasize! !

!StandardSystemController methodsFor: 'borders' stamp: ''!
fullScreen
	"Make the receiver's window occupy jes' about the full screen.  6/10/96 sw"

	view fullScreen! !

!StandardSystemController methodsFor: 'pluggable menus' stamp: 'sma 3/11/2000 15:12'!
getPluggableYellowButtonMenu: shiftKeyState
	^ nil! !

!StandardSystemController methodsFor: 'initialize-release' stamp: 'sma 3/11/2000 11:48'!
initialize
	super initialize.
	status := #inactive! !

!StandardSystemController methodsFor: 'control defaults' stamp: ''!
isControlActive
	status == #active ifFalse: [^ false].
	sensor anyButtonPressed ifFalse: [^ true].
	self viewHasCursor
		ifTrue: [^ true]
		ifFalse: [ScheduledControllers noteNewTop.
				^ false]! !

!StandardSystemController methodsFor: 'menu messages' stamp: 'rbb 3/1/2005 11:14'!
label

	| newLabel |
	newLabel := UIManager default
		request: 'Edit the label, then type RETURN'
		initialAnswer: view label.
	newLabel isEmpty ifFalse: [view relabel: newLabel].
! !

!StandardSystemController methodsFor: 'cursor' stamp: ''!
labelHasCursor
	"Answer true if the cursor is within the window's label"
	^view labelContainsPoint: sensor cursorPoint! !

!StandardSystemController methodsFor: 'menu messages' stamp: ''!
move
	"Ask the user to designate a new origin position for the receiver's view.
	6/10/96 sw: tell the view that it has moved"

	| oldBox | 
	oldBox := view windowBox.
	view uncacheBits.
	view align: view windowBox topLeft
		with: view chooseMoveRectangle topLeft.
	view displayEmphasized.
	view moved.  "In case its model wishes to take note."
	(oldBox areasOutside: view windowBox) do:
		[:rect | ScheduledControllers restore: rect]! !

!StandardSystemController methodsFor: 'scheduling' stamp: ''!
open
	"Create an area on the screen in which the receiver's scheduled view can 
	be displayed. Make it the active view."

	view resizeInitially.
	status := #open.
	ScheduledControllers scheduleActive: self! !

!StandardSystemController methodsFor: 'scheduling' stamp: ''!
openDisplayAt: aPoint 
	"Create an area with origin aPoint in which the receiver's scheduled 
	view can be displayed. Make it the active view."

	view align: view viewport center with: aPoint.
	view translateBy:
		(view displayBox amountToTranslateWithin: Display boundingBox).
	status := #open.
	ScheduledControllers scheduleActive: self! !

!StandardSystemController methodsFor: 'scheduling' stamp: 'jm 10/22/97 08:16'!
openNoTerminate
	"Create an area in which the receiver's scheduled view can be displayed. 
	Make it the active view. Do not terminate the currently active process."

	view resizeInitially.
	status := #open.
	ScheduledControllers scheduleActiveNoTerminate: self! !

!StandardSystemController methodsFor: 'scheduling' stamp: ''!
openNoTerminateDisplayAt: aPoint 
	"Create an area with origin aPoint in which the receiver's scheduled 
	view can be displayed. Make it the active view. Do not terminate the 
	currently active process."

	view resizeMinimumCenteredAt: aPoint.
	status := #open.
	ScheduledControllers scheduleActiveNoTerminate: self! !

!StandardSystemController methodsFor: 'control defaults' stamp: 'sma 3/15/2000 22:19'!
redButtonActivity
	"If cursor is in label of a window when red button is pushed,
	check for closeBox or growBox, else drag the window frame
	or edit the label."

	| box p |
	p := sensor cursorPoint.
	self labelHasCursor ifFalse: [super redButtonActivity. ^ self].
	((box := view closeBoxFrame) containsPoint: p)
		ifTrue:
			[Utilities
				awaitMouseUpIn: box
				repeating: []
				ifSucceed: [self close. ^ self].
			^ self].
	((box := view growBoxFrame) containsPoint: p)
		ifTrue:
			[Utilities
				awaitMouseUpIn: box
				repeating: []
				ifSucceed:
					[Sensor controlKeyPressed ifTrue: [^ self expand; fullScreen].
					^ view isCollapsed
						ifTrue: [self expand]
						ifFalse: [self collapse]].
			^ self].
	(((box := view labelTextRegion expandBy: 1) containsPoint: p)
			and: [Preferences clickOnLabelToEdit or: [sensor leftShiftDown]])
		ifTrue:
			[Utilities
				awaitMouseUpIn: box
				repeating: []
				ifSucceed: [^ self label].
			^ self].
	self move! !

!StandardSystemController methodsFor: 'menu messages' stamp: ''!
reframe
	^ view reframeTo: view getFrame! !

!StandardSystemController methodsFor: 'scheduling' stamp: ''!
status: aSymbol
	status := aSymbol! !

!StandardSystemController methodsFor: 'menu messages' stamp: ''!
toggleTwoTone
	(view isMemberOf: StandardSystemView) ifTrue:
		[^ view become: (view as: ColorSystemView)].
	(view isMemberOf: ColorSystemView) ifTrue:
		[^ view become: (view as: StandardSystemView)].
! !

!StandardSystemController methodsFor: 'menu messages' stamp: ''!
under
	"Deactive the receiver's scheduled view and pass control to any view that 
	might be positioned directly underneath it and the cursor."

	status := #inactive! !

Controller subclass: #NoController
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Framework'!

!NoController commentStamp: '<historical>' prior: 0!
I represent a controller that never wants control. I am the controller for views that are non-interactive.!

!NoController methodsFor: 'control defaults' stamp: ''!
isControlActive 
	"Refer to the comment in Controller|isControlActive."

	^false! !

!NoController methodsFor: 'control defaults' stamp: ''!
isControlWanted 
	"Refer to the comment in Controller|isControlWanted."

	^false! !

!NoController methodsFor: 'basic control sequence' stamp: ''!
startUp
	"I do nothing."

	^self! !

Controller subclass: #PluggableButtonController
	instanceVariableNames: 'selector arguments shownAsComplemented'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Pluggable Views'!

!PluggableButtonController commentStamp: '<historical>' prior: 0!
The controller for Buttons.  Not meant to be used with buttons that have mouseOver feeback when the button is not pressed.  Use mouseEnter/mouseLeave for that. !

!PluggableButtonController methodsFor: 'control defaults' stamp: 'sma 5/28/2000 16:29'!
controlActivity 

	shownAsComplemented ifNil: [^ self].
	shownAsComplemented = self viewHasCursor
		ifFalse:
			[view ifNotNil: [view toggleMouseOverFeedback]. 
			shownAsComplemented := shownAsComplemented not]! !

!PluggableButtonController methodsFor: 'basic control sequence' stamp: 'acg 3/13/2000 08:49'!
controlInitialize 
	"Provide feedback indicating that button has been entered with the mouse down. If triggerOnMouseDown is true, then do the button action on mouse down--and don't bother with the feedback since the action happens immediately."

	sensor anyButtonPressed ifFalse: [^ self].
	view triggerOnMouseDown
		ifTrue: [sensor yellowButtonPressed 
			ifTrue: [self yellowButtonActivity]
			ifFalse: [view performAction]]
		ifFalse: [view toggleMouseOverFeedback.
				 shownAsComplemented := true]! !

!PluggableButtonController methodsFor: 'basic control sequence' stamp: 'acg 3/13/2000 08:44'!
controlTerminate 
	"Reverse the feedback displayed by controlInitialize, if any. Perform the button action if necessary."

	view ifNotNil:
		[view triggerOnMouseDown
			ifFalse:
				[shownAsComplemented ifTrue: [view toggleMouseOverFeedback].
				self viewHasCursor ifTrue: [view performAction]]]! !

!PluggableButtonController methodsFor: 'control defaults' stamp: 'acg 3/13/2000 08:26'!
isControlActive 

	^ sensor anyButtonPressed! !

!PluggableButtonController methodsFor: 'control defaults' stamp: 'tk 4/29/1998 10:47'!
isControlWanted

	"sensor flushKeyboard."
	self viewHasCursor & sensor anyButtonPressed ifFalse: [^ false].
	view askBeforeChanging
		ifTrue: [^ model okToChange]  "ask before changing"
		ifFalse: [^ true].
! !

!PluggableButtonController methodsFor: 'button activity' stamp: 'di 9/7/1999 08:44'!
yellowButtonActivity
	"Invoke the model's menu.  This is option-click, NOT the normal button press."
	| menu |
	menu := view getMenu: false.
	menu == nil
		ifTrue: [sensor waitNoButton]
		ifFalse: [self terminateAndInitializeAround: [menu invokeOn: model]].
! !

Controller subclass: #ScreenController
	instanceVariableNames: ''
	classVariableNames: 'LastScreenModeSelected'
	poolDictionaries: ''
	category: 'ST80-Kernel-Remnants'!

!ScreenController commentStamp: '<historical>' prior: 0!
I am the controller for the parts of the display screen that have no view on them. I only provide a standard yellow button menu. I view (a FormView of) an infinite gray form.  (ScheduledControllers screenController) is the way to find me.!

!ScreenController classSide methodsFor: 'as yet unclassified' stamp: 'RAA 7/15/2000 09:28'!
lastScreenModeSelected

	^LastScreenModeSelected! !

!ScreenController methodsFor: 'menu messages' stamp: 'nk 7/29/2004 10:12'!
aboutThisSystem 
	SmalltalkImage current aboutThisSystem! !

!ScreenController methodsFor: 'menu messages' stamp: 'sw 10/13/1998 16:03'!
absorbUpdatesFromServer 
	Utilities updateFromServer! !

!ScreenController methodsFor: 'nested menus' stamp: 'sw 12/10/1999 11:29'!
appearanceMenu 
	"Answer the appearance menu to be put up as a screen submenu"

	^ SelectionMenu labelList:
		#(	'window colors...'
			'system fonts...'
			'full screen on'
			'full screen off'
			'set display depth...'
			'set desktop color...' ) 

		lines: #(2 4)
		selections: #(windowSpecificationPanel configureFonts
fullScreenOn fullScreenOff setDisplayDepth setDesktopColor)
"
ScreenController new appearanceMenu startUp
"! !

!ScreenController methodsFor: 'menu messages' stamp: 'dao 10/1/2004 13:23'!
beIsolated
	"Establish this project as an isolation layer.
	Further method changes made here will be revoked when you leave the project."

	Project current beIsolated
	! !

!ScreenController methodsFor: 'menu messages' stamp: 'sw 6/11/1999 20:25'!
bitCachingString
	^ StandardSystemView cachingBits
		ifTrue: ['don''t save bits (compact)']
		ifFalse: ['save bits (fast)']! !

!ScreenController methodsFor: 'menu messages' stamp: 'sd 5/23/2003 15:17'!
browseChangedMessages
	ChangeSet  browseChangedMessages! !

!ScreenController methodsFor: 'menu messages' stamp: ''!
browseRecentSubmissions
	"Open a method-list browser on recently-submitted methods.  5/16/96 sw"

	Utilities browseRecentSubmissions! !

!ScreenController methodsFor: 'cursor' stamp: ''!
centerCursorInView
	"Override so this doesn't happen when taking control"! !

!ScreenController methodsFor: 'menu messages' stamp: 'sw 6/11/1999 20:50'!
changeWindowPolicy
	Preferences toggleWindowPolicy! !

!ScreenController methodsFor: 'nested menus' stamp: 'sw 6/4/2000 23:47'!
changesMenu
	"Answer a menu for changes-related items"
	^ SelectionMenu labelList:
		#(	'keep this menu up'

			'simple change sorter'
			'dual change sorter'

			'file out current change set'
			'create new change set...'
			'browse changed methods'
			'check change set for slips'

			'isolate methods of this project'
			'propagate changes above'

			'browse recent submissions'
			'recently logged changes...'
			'recent log file...'
			)
		lines: #(1 3 7 9)
		selections: #(durableChangesMenu
openSimpleChangeSorter openChangeManager
fileOutChanges newChangeSet browseChangedMessages lookForSlips
beIsolated propagateChanges
browseRecentSubmissions browseRecentLog fileForRecentLog)
"
ScreenController new changesMenu startUp
"! !

!ScreenController methodsFor: 'menu messages' stamp: 'tk 4/13/1998 23:13'!
chooseDirtyWindow
	"Put up a list of windows with unaccepted edits and let the user chose one to activate."
	"ScheduledControllers screenController chooseDirtyWindow"

	ScheduledControllers findWindowSatisfying:
		[:c | c model canDiscardEdits not].
! !

!ScreenController methodsFor: 'menu messages' stamp: 'tk 4/13/1998 23:13'!
closeUnchangedWindows
	"Close any window that doesn't have unaccepted input."

	| clean |
	(SelectionMenu confirm:
'Do you really want to close all windows
except those with unaccepted edits?')
		ifFalse: [^ self].

	clean := ScheduledControllers scheduledControllers select:
		[:c | c model canDiscardEdits and: [(c isKindOf: ScreenController) not]].
	clean do: [:c | c closeAndUnscheduleNoTerminate].
	self restoreDisplay.
! !

!ScreenController methodsFor: 'menu messages' stamp: ''!
collapseAll
	"Collapses all open windows"
	ScheduledControllers scheduledControllers do:
		[:controller | controller == self ifFalse:
			[controller view isCollapsed ifFalse:
					[controller collapse.
					controller view deEmphasize]]]! !

!ScreenController methodsFor: 'menu messages' stamp: ''!
commonRequests 
	"Put up a popup of common requests, and perform whatever the user request.  2/1/96 sw"
	Utilities offerCommonRequests! !

!ScreenController methodsFor: 'menu messages' stamp: 'sw 12/10/1999 11:29'!
configureFonts
	Preferences presentMvcFontConfigurationMenu! !

!ScreenController methodsFor: 'control defaults' stamp: 'sma 3/11/2000 12:21'!
controlActivity
	"Any button opens the screen's menu.
	If the shift key is down, do find window."

	sensor leftShiftDown ifTrue: [^ self findWindow].
	(self projectScreenMenu invokeOn: self) ifNil: [super controlActivity]! !

!ScreenController methodsFor: 'menu messages' stamp: 'di 9/22/1998 15:09'!
durableChangesMenu 
	Utilities windowFromMenu: self changesMenu target: self title: 'Changes'! !

!ScreenController methodsFor: 'menu messages' stamp: 'sw 6/11/1999 20:01'!
durableHelpMenu 
	| aMenu selectionList labelList targetList i wordingList colorPattern |
	aMenu := self helpMenu.
	selectionList := aMenu selections.
	labelList := (1 to: selectionList size) collect:
		[:ind | aMenu labelString lineNumber: ind].
	targetList :=  (1 to: selectionList size) collect: [:ind | self].

	(i := labelList indexOf: 'keep this menu up') > 0 ifTrue:
		[selectionList := selectionList copyReplaceFrom: i to: i with: Array new.
		labelList := labelList copyReplaceFrom: i to: i with: Array new.
		targetList := targetList copyReplaceFrom: i to: i with: Array new].

	colorPattern := #(lightRed lightGreen lightBlue lightYellow lightGray lightCyan lightMagenta lightOrange).

	wordingList := selectionList collect:
		[:aSelection |
			(aSelection == #soundOnOrOff) ifTrue: [#soundEnablingString] ifFalse: [nil]].
	^ Utilities windowMenuWithLabels: labelList colorPattern: colorPattern  targets: targetList selections: selectionList wordingSelectors: wordingList title: 'Help'

! !

!ScreenController methodsFor: 'menu messages' stamp: 'di 11/25/1998 12:21'!
durableOpenMenu 
	| colorPattern |
	colorPattern := #(blue lightGreen lightYellow lightMagenta  lightOrange lightCyan) asOrderedCollection.
	colorPattern add: Project someInstance backgroundColorForMvcProject.
	colorPattern add: Project someInstance backgroundColorForMorphicProject.
	colorPattern add: #orange.
	Utilities windowFromMenu: self openMenu target: self title: 'Openers'
		colorPattern: colorPattern! !

!ScreenController methodsFor: 'menu messages' stamp: 'di 9/22/1998 14:36'!
durableScreenMenu 
	Utilities windowFromMenu: self projectScreenMenu target: self title: 'Squeak'! !

!ScreenController methodsFor: 'menu messages' stamp: 'sw 6/11/1999 20:28'!
durableWindowMenu 
	| aMenu selectionList labelList targetList i wordingList colorPattern |
	aMenu := self windowMenu.
	selectionList := aMenu selections.
	labelList := (1 to: selectionList size) collect:
		[:ind | aMenu labelString lineNumber: ind].
	targetList :=  (1 to: selectionList size) collect: [:ind | self].

	(i := labelList indexOf: 'keep this menu up') > 0 ifTrue:
		[selectionList := selectionList copyReplaceFrom: i to: i with: Array new.
		labelList := labelList copyReplaceFrom: i to: i with: Array new.
		targetList := targetList copyReplaceFrom: i to: i with: Array new].

	colorPattern := #(lightRed lightGreen lightBlue lightYellow lightGray lightCyan lightMagenta lightOrange).

	wordingList := selectionList collect:
		[:aSelection |
			(#(fastWindows changeWindowPolicy) includes: aSelection)
				ifFalse:
					[nil]
				ifTrue:
					[aSelection == #fastWindows
						ifFalse:
							[#staggerPolicyString]
						ifTrue:
							[#bitCachingString]]].
	^ Utilities windowMenuWithLabels: labelList colorPattern: colorPattern  targets: targetList selections: selectionList wordingSelectors: wordingList title: 'windows'

! !

!ScreenController methodsFor: 'menu messages' stamp: ''!
editPreferences
	"Open up a Preferences inspector.  2/7/96 sw"

	Preferences openPreferencesInspector! !

!ScreenController methodsFor: 'menu messages' stamp: ''!
emergencyCollapse
	"Emergency collapse of a selected window"
	| controller |
	(controller := ScheduledControllers windowFromUser) notNil
		ifTrue:
			[controller collapse.
			controller view deEmphasize]! !

!ScreenController methodsFor: 'menu messages' stamp: 'dao 10/1/2004 13:14'!
exitProject 
	"Leave the current Project and enter the Project in which the receiver's 
	view is scheduled."

	Project current exit! !

!ScreenController methodsFor: 'menu messages' stamp: ''!
expandAll
	"Reopens all collapsed windows"
	ScheduledControllers scheduledControllers reverseDo:
		[:controller | controller == self ifFalse:
			[controller view isCollapsed
				ifTrue:  [controller view expand]
				ifFalse: [controller view displayDeEmphasized]]]! !

!ScreenController methodsFor: 'menu messages' stamp: ''!
fastWindows
	StandardSystemView cachingBits
		ifTrue: [StandardSystemView dontCacheBits]
		ifFalse: [StandardSystemView doCacheBits]! !

!ScreenController methodsFor: 'menu messages' stamp: 'sw 2/3/2000 16:23'!
fileForRecentLog
	Smalltalk writeRecentToFile! !

!ScreenController methodsFor: 'menu messages' stamp: 'sd 1/16/2004 21:32'!
fileOutChanges
	ChangeSet current verboseFileOut.! !

!ScreenController methodsFor: 'menu messages' stamp: ''!
findWindow
	"Put up a menu of all windows on the screen, and let the user select one.
	 1/18/96 sw: the real work devolved to ControlManager>>findWindowSatisfying:"

	ScheduledControllers findWindowSatisfying: [:c | true]! !

!ScreenController methodsFor: 'menu messages' stamp: 'nk 6/17/2003 14:39'!
fontSizeSummary
	TextStyle fontSizeSummary! !

!ScreenController methodsFor: 'menu messages' stamp: 'sd 6/7/2003 19:49'!
fullScreenOff

	Display fullScreenMode: (LastScreenModeSelected := false).
	DisplayScreen checkForNewScreenSize.
	self restoreDisplay.
! !

!ScreenController methodsFor: 'menu messages' stamp: 'sd 6/7/2003 19:49'!
fullScreenOn

	Display fullScreenMode: (LastScreenModeSelected := true).
	DisplayScreen checkForNewScreenSize.
	self restoreDisplay.! !

!ScreenController methodsFor: 'menu messages' stamp: 'nk 2/15/2004 09:37'!
garbageCollect
	"Do a garbage collection, and report results to the user."

	Utilities garbageCollectAndReport! !

!ScreenController methodsFor: 'nested menus' stamp: 'md 7/22/2005 21:30'!
helpMenu 
	"Answer the help menu to be put up as a screen submenu"

	^ SelectionMenu labelList:
		#(	'keep this menu up'

			'about this system...'
			'update code from server'
			'preferences...'

			'command-key help'
			'font size summary'
			'useful expressions'
			'view graphical imports'
			'standard graphics library'),

			(Array with: (Preferences soundsEnabled
							ifFalse: ['turn sound on']
							ifTrue: ['turn sound off'])) ,

		#(	'set author initials...'
			'vm statistics'
			'space left')
		lines: #(1 4 6 11)
		selections: #(durableHelpMenu aboutThisSystem absorbUpdatesFromServer
editPreferences  openCommandKeyHelp fontSizeSummary openStandardWorkspace viewImageImports
standardGraphicsLibrary soundOnOrOff setAuthorInitials vmStatistics garbageCollect)
"
ScreenController new helpMenu startUp
"! !

!ScreenController methodsFor: 'control defaults' stamp: 'sma 3/11/2000 12:06'!
isControlActive
	^ self isControlWanted! !

!ScreenController methodsFor: 'control defaults' stamp: 'sma 3/11/2000 12:05'!
isControlWanted
	^ self viewHasCursor and: [sensor anyButtonPressed]! !

!ScreenController methodsFor: 'menu messages' stamp: 'jm 5/20/1998 23:28'!
jumpToProject

	Project jumpToProject.
! !

!ScreenController methodsFor: 'menu messages' stamp: 'sd 5/23/2003 14:41'!
lookForSlips
	ChangeSet current lookForSlips! !

!ScreenController methodsFor: 'menu messages' stamp: 'ar 9/27/2005 20:11'!
newChangeSet
	ChangeSet newChangeSet! !

!ScreenController methodsFor: 'file in/out' stamp: 'tk 9/28/2000 15:47'!
objectForDataStream: refStrm
	| dp |
	"I am about to be written on an object file.  Write a path to me in the other system instead."

	dp := DiskProxy global: #ScheduledControllers selector: #screenController args: #().
	refStrm replace: self with: dp.
	^ dp! !

!ScreenController methodsFor: 'menu messages' stamp: 'ar 9/27/2005 20:33'!
openBrowser 
	"Create and schedule a Browser view for browsing code."
	ToolSet browse: nil selector: nil! !

!ScreenController methodsFor: 'menu messages' stamp: ''!
openCommandKeyHelp
	"1/18/96 sw Open a window that explains command-keys"

	Utilities openCommandKeyHelp! !

!ScreenController methodsFor: 'nested menus' stamp: 'sw 6/5/2000 17:21'!
openMenu
	^ SelectionMenu labelList:
		#(	'keep this menu up'

			'browser'
			'package browser'
			'method finder'
			'workspace'
			'file list'
			'file...'
			'transcript'
			'morphic world'

			'simple change sorter'
			'dual change sorter'

			'mvc project'
			'morphic project'
			)
		lines: #(1 9 11)
		selections: #(durableOpenMenu
openBrowser openPackageBrowser openSelectorBrowser openWorkspace openFileList openFile openTranscript openMorphicWorld
openSimpleChangeSorter openChangeManager
openProject  openMorphicProject  )
"
ScreenController  new openMenu startUp
"! !

!ScreenController methodsFor: 'menu messages' stamp: 'md 8/28/2005 00:20'!
openMorphicProject
	"Open a morphic project from within a MVC project"
	Smalltalk verifyMorphicAvailability ifFalse: [^ self].
	ProjectView open: Project newMorphic.
! !

!ScreenController methodsFor: 'menu messages' stamp: 'di 11/26/1999 08:56'!
openMorphicWorld 
	"Create and schedule a StringHolderView for use as a workspace."
	Smalltalk verifyMorphicAvailability ifFalse: [^ self].
	MorphWorldView openWorld.
! !

!ScreenController methodsFor: 'menu messages' stamp: 'sw 4/24/2001 12:00'!
openProject 
	"Create and schedule a Project."

	| proj |
	Smalltalk at: #ProjectView ifPresent:
		[:c | proj := Project new.
		c open: proj].
! !

!ScreenController methodsFor: 'menu messages' stamp: ''!
openStandardWorkspace
	"Open a standard, throwaway window chock full of useful expressions.  1/17/96 sw"

	Utilities openStandardWorkspace! !

!ScreenController methodsFor: 'menu messages' stamp: 'di 5/8/1998 12:49'!
openTranscript 
	"Create and schedule the System Transcript."
	Transcript open! !

!ScreenController methodsFor: 'menu messages' stamp: 'ar 9/27/2005 20:46'!
openWorkspace 
	"Create and schedule workspace."

	UIManager default edit:'' label: 'Workspace'! !

!ScreenController methodsFor: 'nested menus' stamp: 'sma 3/11/2000 12:23'!
popUpMenuFor: aSymbol
	(self perform: aSymbol) invokeOn: self! !

!ScreenController methodsFor: 'nested menus' stamp: 'sw 7/13/1999 18:07'!
presentAppearanceMenu
	self popUpMenuFor: #appearanceMenu! !

!ScreenController methodsFor: 'nested menus' stamp: 'sw 7/6/1998 21:14'!
presentChangesMenu
	self popUpMenuFor: #changesMenu! !

!ScreenController methodsFor: 'nested menus' stamp: 'sw 7/6/1998 21:15'!
presentHelpMenu
	self popUpMenuFor: #helpMenu! !

!ScreenController methodsFor: 'nested menus' stamp: 'sw 7/6/1998 21:15'!
presentOpenMenu
	self popUpMenuFor: #openMenu! !

!ScreenController methodsFor: 'nested menus' stamp: 'sw 7/6/1998 21:16'!
presentWindowMenu
	self popUpMenuFor: #windowMenu! !

!ScreenController methodsFor: 'nested menus' stamp: 'sw 7/13/1999 18:07'!
projectScreenMenu
	"Answer the project screen menu."

	^ SelectionMenu labelList:
		#(	'keep this menu up'

			'previous project'
			'jump to project...'
			'restore display'

			'open...'
			'windows...'
			'changes...'
			'help...'
			'appearance...'
			'do...'

			'save'
			'save as...'
			'save and quit'
			'quit')
		lines: #(1 4 10)
		selections: #(durableScreenMenu
returnToPreviousProject jumpToProject restoreDisplay
presentOpenMenu presentWindowMenu presentChangesMenu presentHelpMenu presentAppearanceMenu commonRequests
snapshot saveAs snapshotAndQuit quit )
"
ScreenController new projectScreenMenu startUp
"! !

!ScreenController methodsFor: 'menu messages' stamp: 'dao 10/1/2004 13:38'!
propagateChanges
	"The changes made in this isolated project will be propagated to projects above."

	Project current propagateChanges! !

!ScreenController methodsFor: 'menu messages' stamp: 'sd 11/16/2003 14:17'!
quit
	SmalltalkImage current
		snapshot:
			(self confirm: 'Save changes before quitting?' translated
				orCancel: [^ self])
		andQuit: true! !

!ScreenController methodsFor: 'menu messages' stamp: 'ar 3/18/2001 00:00'!
restoreDisplay 
	"Clear the screen to gray and then redisplay all the scheduled views."

	Smalltalk isMorphic ifTrue: [^ World restoreMorphicDisplay].

	Display extent = DisplayScreen actualScreenSize
		ifFalse:
			[DisplayScreen startUp.
			ScheduledControllers unCacheWindows].
	ScheduledControllers restore! !

!ScreenController methodsFor: 'menu messages' stamp: 'jm 5/22/1998 06:45'!
returnToPreviousProject

	Project returnToPreviousProject.
! !

!ScreenController methodsFor: 'menu messages' stamp: 'sd 11/16/2003 13:17'!
saveAs
	^ SmalltalkImage current saveAs! !

!ScreenController methodsFor: 'menu messages' stamp: 'sw 5/10/96'!
setAuthorInitials
	"Put up a dialog allowing the user to specify the author's initials.  "
	Utilities setAuthorInitials! !

!ScreenController methodsFor: 'menu messages' stamp: 'sw 11/26/96'!
setDesktopColor
	"Let the user choose a new color for the desktop.   Based on an idea by Georg Gollmann.   "

	Preferences desktopColor: Color fromUser.
	ScheduledControllers updateGray; restore! !

!ScreenController methodsFor: 'menu messages' stamp: 'bf 9/18/1999 20:01'!
setDisplayDepth
	"Let the user choose a new depth for the display. "
	| result |
	(result := (SelectionMenu selections: Display supportedDisplayDepths) startUpWithCaption: 'Choose a display depth
(it is currently ' , Display depth printString , ')') == nil ifFalse:
		[Display newDepth: result]! !

!ScreenController methodsFor: 'menu messages' stamp: 'sd 11/16/2003 14:17'!
snapshot
	SmalltalkImage current snapshot: true andQuit: false! !

!ScreenController methodsFor: 'menu messages' stamp: 'sd 11/16/2003 14:17'!
snapshotAndQuit
	"Snapshot and quit without bother the user further.  2/4/96 sw"

	SmalltalkImage current
		snapshot: true
		andQuit: true! !

!ScreenController methodsFor: 'menu messages' stamp: 'sw 6/11/1999 20:01'!
soundEnablingString
	^ Preferences soundEnablingString! !

!ScreenController methodsFor: 'menu messages' stamp: 'bf 3/16/2000 18:26'!
soundOnOrOff
	Preferences setPreference: #soundsEnabled
		toValue: Preferences soundsEnabled not! !

!ScreenController methodsFor: 'menu messages' stamp: 'sw 6/11/1999 20:23'!
staggerPolicyString
	^ Preferences staggerPolicyString! !

!ScreenController methodsFor: 'menu messages' stamp: 'sw 5/29/2000 00:26'!
standardGraphicsLibrary
	"Open a standard, throwaway window chock full of useful expressions.  1/17/96 sw"

	ScriptingSystem inspectFormDictionary! !

!ScreenController methodsFor: 'menu messages' stamp: 'sd 5/11/2003 22:04'!
viewImageImports
	"Open an inspector on forms imported from Image files."

	Imports default viewImages! !

!ScreenController methodsFor: 'menu messages' stamp: 'sd 9/30/2003 13:53'!
vmStatistics
	"Open a string view on a report of vm statistics"

	(StringHolder new contents: SmalltalkImage current  vmStatisticsReportString)
		openLabel: 'VM Statistics'! !

!ScreenController methodsFor: 'nested menus' stamp: 'sw 6/11/1999 20:25'!
windowMenu
	"Answer a menu for windows-related items.  "

	^ SelectionMenu labelList:
		#(	'keep this menu up'

			'find window...'
			'find changed browsers...'
			'find changed windows...'

			'collapse all windows'
			'expand all windows'
			'close unchanged windows' ) , 
			(Array
				with: self bitCachingString
				with: self staggerPolicyString)
		lines: #(1 4 7)
		selections: #(durableWindowMenu
findWindow chooseDirtyBrowser chooseDirtyWindow
collapseAll expandAll closeUnchangedWindows
fastWindows changeWindowPolicy)
"
ScreenController new windowMenu startUp
"! !

!ScreenController methodsFor: 'menu messages' stamp: 'sw 7/13/1999 20:19'!
windowSpecificationPanel
	Smalltalk hasMorphic ifFalse:
		[^ self inform: 'Sorry, this feature requires the presence of Morphic.'].
	Preferences windowSpecificationPanel! !

Object subclass: #FormButtonCache
	instanceVariableNames: 'offset form value initialState'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Editors'!

!FormButtonCache commentStamp: '<historical>' prior: 0!
My instances are used to save information needed to construct the switch in a menu for a FormEditor. A collection of my instances is stored as a class variable of FormMenuView.!

!FormButtonCache methodsFor: 'accessing' stamp: ''!
form
	"Answer the receiver's form, the image of the button on the screen."

	^form! !

!FormButtonCache methodsFor: 'accessing' stamp: ''!
form: aForm
	"Set the receiver's form to be the argument."

	form := aForm! !

!FormButtonCache methodsFor: 'accessing' stamp: ''!
initialState
	"Answer the receiver's initial state, on or off."

	^initialState! !

!FormButtonCache methodsFor: 'accessing' stamp: ''!
initialState: aBoolean
	"Set the receiver's initial state, on or off, to be the argument."

	initialState := aBoolean! !

!FormButtonCache methodsFor: 'accessing' stamp: ''!
offset
	"Answer the receiver's offset, its relative position for displaying the 
	button."

	^offset! !

!FormButtonCache methodsFor: 'accessing' stamp: ''!
offset: anInteger
	"Set the receiver's offset."

	offset := anInteger! !

!FormButtonCache methodsFor: 'accessing' stamp: ''!
value
	"Answer the receiver's value, the keyboard key that selects the button."

	^value! !

!FormButtonCache methodsFor: 'accessing' stamp: ''!
value: aCharacter
	"Set the receiver's key character."

	value := aCharacter! !

Object subclass: #GraphicSymbolInstance
	instanceVariableNames: 'transformation graphicSymbol'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Symbols'!

!GraphicSymbolInstance commentStamp: '<historical>' prior: 0!
I represent a display transformation of a GraphicSymbol. Multiple copies of a GraphicSymbol can be displayed at different positions and scales on the screen by making appropriate, multiple, instances of me.!

!GraphicSymbolInstance classSide methodsFor: 'examples' stamp: ''!
example
	"Simply evaluate the method and two GraphicSymbolInstances, each
	displaying a transformation of the same graphic symbol, will be
	presented on the screen. Clears the screen to white."

	| gate instance1 instance2 trans1 trans2 line arc f|
	Display fillWhite.			"clear the Screen."
	f := Form extent: 2@2.
	f fillBlack.
	gate:= GraphicSymbol new.		"make a logic gate out of lines and arcs."
	line:=Line new.  line beginPoint: -20@-20.  line endPoint: 0@-20. line form: f.
	gate add: line.

	line:=Line new.  line beginPoint: -20@20.  line endPoint: 0@20. line form: f.
	gate add: line.

	line:=Line new.  line beginPoint: 0@-40.  line endPoint: 0@40. line form: f.
	gate add: line.

	arc := Arc new. arc center: 0@0 radius: 40 quadrant: 1.
	arc form: f.
	gate add: arc.

	arc := Arc new. arc center: 0@0 radius: 40 quadrant: 4.
	arc form: f.
	gate add: arc.

			"one instance at 1/2 scale."
	trans1:=WindowingTransformation identity.	
	trans1:= trans1 scaleBy: 0.5@0.5.
	trans1:= trans1 translateBy: 100@100.

			"the other instance at 2 times scale"
	trans2:=WindowingTransformation identity.	
	trans2:= trans2 scaleBy: 2.0@2.0.
	trans2:= trans2 translateBy: 200@200.

	instance1 := GraphicSymbolInstance new.
	instance1 transformation: trans1.
	instance1 graphicSymbol: gate.

	instance2 := GraphicSymbolInstance new.
	instance2 transformation: trans2.
	instance2 graphicSymbol: gate.

			"display both instances of the logic gate"
	instance1 displayOn: Display
					transformation: WindowingTransformation identity
					clippingBox: Display boundingBox
					rule: Form under
					fillColor: nil.
	instance2 displayOn: Display
					transformation: WindowingTransformation identity
					clippingBox: Display boundingBox
					rule: Form under
					fillColor: nil

	"GraphicSymbolInstance example"! !

!GraphicSymbolInstance methodsFor: 'displaying' stamp: 'jrm 2/13/2000 10:02'!
displayOn: aDisplayMedium transformation: aTransformation clippingBox:
clipRect rule: anInteger fillColor: aForm 
	"Display the graphic symbol on the Display according to the arguments 
	of this message."

	graphicSymbol
		displayOn: aDisplayMedium
		transformation: aTransformation 
		clippingBox: clipRect
		rule: anInteger
		fillColor: aForm! !

!GraphicSymbolInstance methodsFor: 'displaying' stamp: ''!
displayTransformation: aTransformation clippingBox: clipRect rule: anInteger fillColor: aForm
	"Display the graphic symbol according to the arguments of this message."

	self displayOn: Display transformation: aTransformation clippingBox: clipRect
		rule: anInteger fillColor: aForm! !

!GraphicSymbolInstance methodsFor: 'accessing' stamp: ''!
graphicSymbol
	"Answer the graphic symbol that the receiver displays."

	^graphicSymbol! !

!GraphicSymbolInstance methodsFor: 'accessing' stamp: ''!
graphicSymbol: aGraphicSymbol 
	"Set the argument, aGraphicSymbol, to be the graphic symbol that the 
	receiver displays."

	graphicSymbol := aGraphicSymbol! !

!GraphicSymbolInstance methodsFor: 'transforming' stamp: ''!
transformation
	"Answer the receiver's display transformation."

	^transformation! !

!GraphicSymbolInstance methodsFor: 'transforming' stamp: ''!
transformation: aWindowingTransformation 
	"Set the argument, aWindowingTransformation, to be the receiver's 
	display transformation."

	transformation := aWindowingTransformation! !

Object subclass: #PluggableTest
	instanceVariableNames: 'musicTypeList musicTypeIndex artistList artistIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Pluggable Views'!

!PluggableTest commentStamp: '<historical>' prior: 0!
This class demonstrates how to use PluggableListViews.
!

!PluggableTest classSide methodsFor: 'example' stamp: 'nk 7/30/2004 21:50'!
open
	"PluggableTest open"

	| model listView1 topView listView2 |
	model := self new.
	listView1 := PluggableListView 
				on: model
				list: #musicTypeList
				selected: #musicType
				changeSelected: #musicType:
				menu: #musicTypeMenu:
				keystroke: #musicTypeKeystroke:.
	listView1 menuTitleSelector: #musicTypeListTitle.
	listView2 := PluggableListView 
				on: model
				list: #artistList
				selected: #artist
				changeSelected: #artist:
				menu: nil
				keystroke: #artistKeystroke:.
	topView := (StandardSystemView new)
				label: 'Pluggable Test';
				minimumSize: 300 @ 200;
				borderWidth: 1;
				addSubView: listView1;
				addSubView: listView2 toRightOf: listView1.
	topView borderWidth: 1.
	topView controller open! !

!PluggableTest methodsFor: 'artist pane' stamp: 'jm 5/3/1998 16:20'!
artist

	^ artistIndex
! !

!PluggableTest methodsFor: 'artist pane' stamp: 'jm 5/3/1998 16:20'!
artist: anInteger

	artistIndex := anInteger.
	self changed: #artist.
! !

!PluggableTest methodsFor: 'artist pane' stamp: 'jm 5/3/1998 16:14'!
artistKeystroke: aCharacter

	self artistList doWithIndex: [:artist :i |
		(artist first asLowercase = aCharacter asLowercase) ifTrue: [
			self artist: i]].
! !

!PluggableTest methodsFor: 'artist pane' stamp: 'jm 5/3/1998 16:21'!
artistList

	((musicTypeIndex ~= nil) and:
	 [musicTypeIndex between: 1 and: artistList size])
		ifTrue: [^ artistList at: musicTypeIndex]
		ifFalse: [^ #()].
! !

!PluggableTest methodsFor: 'artist pane' stamp: 'jm 5/3/1998 16:20'!
artistName
	"Answer the name of the currently selected artist, or nil if no artist is selected."

	| artistsForCurrentType |
	artistsForCurrentType := self artistList.
	(artistIndex between: 1 and: artistsForCurrentType size)
		ifTrue: [^ artistsForCurrentType at: artistIndex]
		ifFalse: [^ nil].
! !

!PluggableTest methodsFor: 'menu commands' stamp: 'jm 5/3/1998 16:00'!
earlyCmd

	self musicType: (musicTypeList indexOf: 'early').
! !

!PluggableTest methodsFor: 'menu commands' stamp: ''!
flashCmd

	Display reverse; reverse.! !

!PluggableTest methodsFor: 'menu commands' stamp: ''!
grungeCmd

	SelectionMenu confirm:
		'You mean, like those strange bands from Seattle?'! !

!PluggableTest methodsFor: 'initialization' stamp: 'jm 5/3/1998 16:20'!
initialize

	musicTypeList := #('reggae' 'classical' 'early').
	artistList := #(
		('alpha blondy' 'black uhuru' 'bob marley' 'burning spear')
		('bach' 'beethoven' 'josquin' 'morley' 'mozart' 'telemann')
		('josquin' 'morley' 'telemann')).
	musicTypeIndex := 0.
	artistIndex := 0.
! !

!PluggableTest methodsFor: 'music type pane' stamp: 'jm 5/3/1998 16:21'!
musicType

	^ musicTypeIndex
! !

!PluggableTest methodsFor: 'music type pane' stamp: 'jm 5/3/1998 16:21'!
musicType: anInteger

	| oldArtist |
	oldArtist := self artistName.
	musicTypeIndex := anInteger.  "this changes artists list"
	artistIndex := self artistList indexOf: oldArtist.
	self changed: #musicType.
	self changed: #artistList.
! !

!PluggableTest methodsFor: 'music type pane' stamp: 'jm 5/2/1998 15:18'!
musicTypeKeystroke: aCharacter

	musicTypeList doWithIndex: [:type :i |
		(type first asLowercase = aCharacter asLowercase)
			ifTrue: [self musicType: i]].
! !

!PluggableTest methodsFor: 'music type pane' stamp: 'jm 5/3/1998 16:11'!
musicTypeList

	^ musicTypeList
! !

!PluggableTest methodsFor: 'music type pane' stamp: 'sw 8/18/1998 12:31'!
musicTypeListTitle
	^ 'Choose a command'! !

!PluggableTest methodsFor: 'music type pane' stamp: 'jm 5/3/1998 16:00'!
musicTypeMenu: aMenu

	^ aMenu addList: #(
		(reggae reggaeCmd)
		(early earlyCmd)
		(grunge grungeCmd)
		-
		(flash flashCmd))
! !

!PluggableTest methodsFor: 'music type pane' stamp: 'jm 5/3/1998 16:21'!
musicTypeName
	"Answer the name of the currently selected music type, or nil if no music type is selected."

	(musicTypeIndex between: 1 and: musicTypeList size)
		ifTrue: [^ musicTypeList at: musicTypeIndex]
		ifFalse: [^ nil].
! !

!PluggableTest methodsFor: 'menu commands' stamp: 'sw 8/18/1998 12:29'!
perform: sel orSendTo: otherObject
	(self respondsTo: sel) ifTrue: [self perform: sel] ifFalse: [otherObject perform: sel]! !

!PluggableTest methodsFor: 'menu commands' stamp: 'jm 5/3/1998 15:59'!
reggaeCmd

	self musicType: (musicTypeList indexOf: 'reggae').
! !

Object subclass: #PopUpMenu
	instanceVariableNames: 'labelString font lineArray frame form marker selection'
	classVariableNames: 'CacheMenuForms MenuStyle'
	poolDictionaries: ''
	category: 'ST80-Menus'!

!PopUpMenu commentStamp: '<historical>' prior: 0!
I represent a list of items. My instances are presented on the display screen in a rectangular area. The user points to an item, pressing a mouse button; the item is highlighted. When the button is released, the highlighted item indicates the selection.!

!PopUpMenu classSide methodsFor: 'class initialization' stamp: 'jla 4/2/2001 20:41'!
alignment

	^ MenuStyle alignment! !

!PopUpMenu classSide methodsFor: 'class initialization' stamp: 'jla 4/2/2001 20:46'!
alignment: anAlignment

	^ MenuStyle alignment: anAlignment! !

!PopUpMenu classSide methodsFor: 'dialogs' stamp: 'dgd 9/5/2003 18:24'!
confirm: queryString
	"Put up a yes/no menu with caption queryString. Answer true if the 
	response is yes, false if no. This is a modal question--the user must 
	respond yes or no."

	"PopUpMenu confirm: 'Are you hungry?'"

	^ self confirm: queryString trueChoice: 'Yes' translated falseChoice: 'No' translated! !

!PopUpMenu classSide methodsFor: 'dialogs' stamp: 'dgd 10/12/2004 13:49'!
confirm: queryString orCancel: cancelBlock 
	"Put up a yes/no/cancel menu with caption aString. Answer 
	true if  
	the response is yes, false if no. If cancel is chosen, evaluate  
	cancelBlock. This is a modal question--the user must respond 
	yes or no."
	"PopUpMenu confirm: 'Reboot universe' orCancel: 
	[^'Nevermind'] "
	| menu choice |
	menu := PopUpMenu labelArray: {'Yes' translated. 'No' translated. 'Cancel' translated}.
	choice := menu startUpWithCaption: queryString icon: MenuIcons confirmIcon.
	choice = 1
		ifTrue: [^ true].
	choice = 2
		ifTrue: [^ false].
	^ cancelBlock value! !

!PopUpMenu classSide methodsFor: 'dialogs' stamp: 'dgd 10/12/2004 13:49'!
confirm: queryString trueChoice: trueChoice falseChoice: falseChoice 
	"Put up a yes/no menu with caption queryString. The actual 
	wording  
	for the two choices will be as provided in the trueChoice and  
	falseChoice parameters. Answer true if the response is the 
	true-choice,  
	false if it's the false-choice. 
	This is a modal question -- the user must respond one way or 
	the other."
	"PopUpMenu  
	confirm: 'Are you hungry?' 
	trueChoice: 'yes, I''m famished' 
	falseChoice: 'no, I just ate'"
	| menu choice |
	menu := PopUpMenu labelArray: {trueChoice. falseChoice}.
	[(choice := menu startUpWithCaption: queryString icon: MenuIcons confirmIcon) isNil] whileTrue.
	^ choice = 1! !

!PopUpMenu classSide methodsFor: 'dialogs' stamp: 'dgd 10/12/2004 13:42'!
inform: aString
	"PopUpMenu inform: 'I like Squeak'"

	(PopUpMenu labels: ' OK ' translated)
		startUpWithCaption: aString
		icon: MenuIcons confirmIcon
! !

!PopUpMenu classSide methodsFor: 'class initialization' stamp: 'sw 12/6/1999 13:08'!
initialize  "PopUpMenu initialize"
	(MenuStyle := TextStyle default copy)
		gridForFont: TextStyle default defaultFontIndex withLead: 0;
		centered.
	PopUpMenu allSubInstancesDo: [:m | m rescan]! !

!PopUpMenu classSide methodsFor: 'instance creation' stamp: 'sma 5/28/2000 15:44'!
labelArray: labelArray
	"Answer an instance of me whose items are in labelArray."

	^ self labelArray: labelArray lines: nil! !

!PopUpMenu classSide methodsFor: 'instance creation' stamp: 'sma 5/28/2000 15:43'!
labelArray: labelArray lines: lineArray
	"Answer an instance of me whose items are in labelArray, with lines 
	drawn after each item indexed by anArray. 2/1/96 sw"

	labelArray isEmpty ifTrue: [self error: 'Menu must not be zero size'].
	^ self
		labels: (String streamContents: 
			[:stream |
			labelArray do: [:each | stream nextPutAll: each; cr].
			stream skip: -1 "remove last CR"])
		lines: lineArray

"Example:
	(PopUpMenu labelArray: #('frog' 'and' 'toad') lines: #()) startUp"! !

!PopUpMenu classSide methodsFor: 'instance creation' stamp: 'sma 5/28/2000 15:36'!
labels: aString
	"Answer an instance of me whose items are in aString."

	^ self labels: aString lines: nil! !

!PopUpMenu classSide methodsFor: 'instance creation' stamp: 'sw 12/6/1999 17:55'!
labels: aString lines: anArray
	"Answer an instance of me whose items are in aString, with lines drawn 
	after each item indexed by anArray."

	^ self new
		labels: aString
		font: MenuStyle defaultFont
		lines: anArray! !

!PopUpMenu classSide methodsFor: 'class initialization' stamp: 'jla 4/2/2001 20:56'!
leftFlush

	MenuStyle leftFlush! !

!PopUpMenu classSide methodsFor: 'dialogs' stamp: 'sma 5/28/2000 15:57'!
notify: message
	"Deprecated. Use #inform: instead."

	self inform: message! !

!PopUpMenu classSide methodsFor: 'class initialization' stamp: 'nk 9/1/2004 10:27'!
setMenuFontTo: aFont
	"Set the menu font as indicated"

	MenuStyle := TextStyle fontArray: { aFont }.
	MenuStyle 
		gridForFont: 1 withLead: 0;
		centered.
	self allSubInstancesDo: [:m | m rescan]! !

!PopUpMenu classSide methodsFor: 'instance creation' stamp: 'nk 8/30/2004 07:59'!
withCaption: cap chooseFrom: labels 
	"Simply put up a menu. Get the args in the right order with the caption 
	first. labels may be either an array of items or a string with CRs in it. 
	May use backslashes for returns."

	^ (labels isString
		ifTrue: [self labels: labels withCRs lines: nil]
		ifFalse: [self labelArray: labels lines: nil])
		startUpWithCaption: cap withCRs! !

!PopUpMenu methodsFor: 'accessing' stamp: 'sma 5/28/2000 11:44'!
center
	"Answer the point at the center of the receiver's rectangular area."

	^ frame center! !

!PopUpMenu methodsFor: 'private' stamp: 'BG 8/6/2003 12:34'!
computeForm
	"Compute and answer a Form to be displayed for this menu."

	| borderInset paraForm menuForm inside |
	borderInset := 4@4.
	paraForm := (DisplayText text: labelString asText textStyle: MenuStyle) form.
	menuForm := Form extent: paraForm extent + (borderInset * 2) depth: paraForm depth.
      menuForm fill: (0 @ 0 extent: menuForm  extent)
                        rule: Form over
                        fillColor: Color white.
	menuForm borderWidth: 2.
	paraForm displayOn: menuForm at: borderInset.
	lineArray == nil ifFalse:
		[lineArray do:
			[ :line |
			menuForm fillBlack: (4 @ ((line * font height) + borderInset y)
				extent: (menuForm width - 8 @ 1))]].

	frame := Quadrangle new.
	frame region: menuForm boundingBox.
	frame borderWidth: 4.
	inside := frame inside.
	marker := inside topLeft extent: (inside width @ MenuStyle lineGrid).
	selection := 1.

	^ form := menuForm
! !

!PopUpMenu methodsFor: 'private' stamp: ''!
computeLabelParagraph
	"Answer a Paragraph containing this menu's labels, one per line and centered."

	^ Paragraph withText: labelString asText style: MenuStyle! !

!PopUpMenu methodsFor: 'basic control sequence' stamp: 'sma 6/1/2000 13:04'!
controlActivity
	"Do whatever a menu must do - now with keyboard support."

	| didNotMove downPos |
	didNotMove := true.
	Sensor anyButtonPressed
		ifFalse:
			[didNotMove := false.
			Sensor waitButtonOrKeyboard]. 
	
	Sensor keyboardPressed ifFalse: [self manageMarker].
	(didNotMove and: [selection = 0])
		ifTrue:
			[downPos := Sensor cursorPoint.
			[didNotMove and: [Sensor anyButtonPressed]]
				whileTrue:
					[(downPos dist: Sensor cursorPoint) < 2 ifFalse: [didNotMove := false]].
			didNotMove ifTrue: [Sensor waitButtonOrKeyboard]].

	[Sensor keyboardPressed] whileTrue:
		[self readKeyboard ifTrue: [^ self].
		Sensor waitButtonOrKeyboard].

	[Sensor anyButtonPressed] whileTrue: [self manageMarker]! !

!PopUpMenu methodsFor: 'displaying' stamp: 'sw 12/10/1999 09:55'!
displayAt: aPoint withCaption: captionOrNil during: aBlock
	"Display the receiver just to the right of aPoint while aBlock is evaluated.  If the receiver is forced off screen, display it just to the right."
	| delta savedArea captionForm captionSave outerFrame captionText tFrame frameSaveLoc captionBox |
	marker ifNil: [self computeForm].
	frame := frame align: marker leftCenter with: aPoint + (2@0).
	outerFrame := frame.
	captionOrNil notNil ifTrue:
		[captionText := (DisplayText
				text: captionOrNil asText
				textStyle: MenuStyle copy centered)
					foregroundColor: Color black
					backgroundColor: Color white.
		tFrame := captionText boundingBox insetBy: -2.
		outerFrame := frame merge: (tFrame align: tFrame bottomCenter
					with: frame topCenter + (0@2))].
	delta := outerFrame amountToTranslateWithin: Display boundingBox.
	frame right > Display boundingBox right
		ifTrue: [delta := 0 - frame width @ delta y].
	frame := frame translateBy: delta.
	captionOrNil notNil ifTrue:
		[captionForm := captionText form.
		captionBox := captionForm boundingBox expandBy: 4.
		captionBox := captionBox align: captionBox bottomCenter
								with: frame topCenter + (0@2).
		captionSave := Form fromDisplay: captionBox.
		Display border: captionBox width: 4 fillColor: Color white.
		Display border: captionBox width: 2 fillColor: Color black.
		captionForm displayAt: captionBox topLeft + 4].
	marker := marker align: marker leftCenter with: aPoint + delta +  (2@0).
	savedArea := Form fromDisplay: frame.
	self menuForm displayOn: Display at: (frameSaveLoc := frame topLeft).
	selection ~= 0 ifTrue: [Display reverse: marker].
	Cursor normal showWhile: [aBlock value].
	savedArea displayOn: Display at: frameSaveLoc.
	captionOrNil notNil ifTrue:
		[captionSave displayOn: Display at: captionBox topLeft]! !

!PopUpMenu methodsFor: 'accessing' stamp: 'di 4/20/1999 14:33'!
frameHeight
	"Designed to avoid the entire frame computation (includes MVC form),
	since the menu may well end up being displayed in Morphic anyway."
	| nItems |
	frame ifNotNil: [^ frame height].
	nItems := 1 + (labelString occurrencesOf: Character cr).
	^ (nItems * MenuStyle lineGrid) + 4 "border width"! !

!PopUpMenu methodsFor: 'accessing' stamp: 'sma 5/28/2000 14:55'!
labelString
	^ labelString! !

!PopUpMenu methodsFor: 'private' stamp: 'di 4/13/1999 16:21'!
labels: aString font: aFont lines: anArray

	labelString := aString.
	font := aFont.
	lineArray := anArray.
! !

!PopUpMenu methodsFor: 'accessing' stamp: 'sma 5/28/2000 14:55'!
lineArray
	^ lineArray! !

!PopUpMenu methodsFor: 'marker adjustment' stamp: 'di 4/13/1999 17:42'!
manageMarker
	"If the cursor is inside the receiver's frame, then highlight the marked 
	item. Otherwise no item is to be marked."
	| pt |
	"Don't let pt get far from display box, so scrolling will go all the way"
	pt := Sensor cursorPoint adhereTo: (Display boundingBox expandBy: 1).
	(frame inside containsPoint: pt)
		ifTrue: ["Need to cache the form for reasonable scrolling performance"
				((Display boundingBox insetBy: 0@3) containsPoint: pt)
					ifFalse: [pt := pt - (self scrollIntoView: pt)].
				self markerOn: pt]
		ifFalse: [self markerOff]! !

!PopUpMenu methodsFor: 'marker adjustment' stamp: 'sma 5/28/2000 15:27'!
markerOff
	"No item is selected. Reverse the highlight if any item has been marked 
	as selected."

	self setSelection: 0! !

!PopUpMenu methodsFor: 'marker adjustment' stamp: 'sma 6/1/2000 13:01'!
markerOn: aPoint 
	"The item whose bounding area contains aPoint should be marked as 
	selected. Highlight its area and set the selection to its index."

	selection = 0 | (marker containsPoint: aPoint) not 
		ifTrue: [selection = 0 & (marker containsPoint: aPoint)
					ifTrue: [Display reverse: marker]
					ifFalse: 
						[selection > 0 ifTrue: [Display reverse: marker].
						marker := 
							marker 
								align: marker topLeft 
								with: marker left @ (self markerTop: aPoint).
						Display reverse: marker]].
	selection := marker top - frame top // marker height + 1! !

!PopUpMenu methodsFor: 'marker adjustment' stamp: ''!
markerTop: aPoint 
	"Answer aPoint, gridded to lines in the receiver."

	^(aPoint y - frame inside top truncateTo: font height) + frame inside top! !

!PopUpMenu methodsFor: 'private' stamp: 'di 4/13/1999 17:51'!
menuForm
	"Answer a Form to be displayed for this menu."

	form == nil ifTrue: [self computeForm].
	^ form! !

!PopUpMenu methodsFor: 'accessing' stamp: 'sma 5/28/2000 12:32'!
nItems
	^ (labelString occurrencesOf: Character cr) + 1! !

!PopUpMenu methodsFor: 'basic control sequence' stamp: 'sma 6/1/2000 10:55'!
readKeyboard
	"Keyboard support for menus. ESC will abort the menu, Space or CR
	will select an item. Cursor up and cursor down will change the
	selection. Any other key will either select an item whose label starts
	with that character or select the next matching label.
	Answer true if the menu should be closed and false otherwise."

	| ch labels occurences |
	ch := Sensor keyboard asciiValue.
	(ch = 13 or: [ch = 32]) ifTrue: [^ true].
	ch = 27 ifTrue: [self setSelection: 0. ^ true].
	ch = 30
		ifTrue:
			[self setSelection: (selection <= 1
				ifTrue: [self nItems]
				ifFalse: [selection - 1])].
	ch = 31 ifTrue: [self setSelection: selection \\ self nItems + 1].
	ch := ch asCharacter asLowercase.
	labels := labelString findTokens: Character cr asString.
	occurences := 0.
	1 + selection to: selection + labels size do:
		[:index |
		| i | i := index - 1 \\ labels size + 1.
		(labels at: i) withBlanksTrimmed first asLowercase = ch
			ifTrue: [(occurences := occurences + 1) = 1 ifTrue: [self setSelection: i]]].
	^ occurences = 1! !

!PopUpMenu methodsFor: 'private' stamp: 'sma 2/5/2000 11:56'!
rescan
	"Cause my form to be recomputed after a font change."

	labelString == nil ifTrue: [labelString := 'NoText!!'].
	self labels: labelString font: (MenuStyle fontAt: 1) lines: lineArray.
	frame := marker := form := nil.

	"PopUpMenu allSubInstancesDo: [:m | m rescan]"! !

!PopUpMenu methodsFor: 'marker adjustment' stamp: 'di 3/9/98 19:46'!
scrollIntoView: cursorLoc
	| dy |
	dy := 0.
	cursorLoc y < 2 ifTrue: [dy := font height].
	cursorLoc y > (Display height-3) ifTrue: [dy := font height negated].
	dy = 0 ifTrue: [^ 0@0].
	self markerOff.
	frame := frame translateBy: 0@dy.
	marker := marker translateBy: 0@dy.
	self menuForm displayOn: Display at: frame topLeft.
	^ 0@dy! !

!PopUpMenu methodsFor: 'selecting' stamp: 'sma 5/28/2000 12:27'!
selection
	"Answer the current selection."

	^ selection! !

!PopUpMenu methodsFor: 'selecting' stamp: 'sma 6/1/2000 11:01'!
setSelection: index
	| newSelection |
	selection = index ifTrue: [^ self].
	newSelection := (0 max: index) min: frame height // marker height.
	selection > 0 ifTrue: [Display reverse: marker].
	marker := marker translateBy: 0 @ (newSelection - selection * marker height).
	selection := newSelection.
	selection > 0 ifTrue: [Display reverse: marker]! !

!PopUpMenu methodsFor: 'basic control sequence' stamp: ''!
startUp
	"Display and make a selection from the receiver as long as the button 
	is pressed. Answer the current selection."
	
	^ self startUpWithCaption: nil! !

!PopUpMenu methodsFor: 'basic control sequence' stamp: 'ar 3/18/2001 00:55'!
startUpCenteredWithCaption: captionOrNil
	"Differs from startUpWithCaption: by appearing with cursor in the menu,
	and thus ready to act on mouseUp, without requiring user tweak to confirm"
	^ self startUpWithCaption: captionOrNil at: (ActiveHand ifNil:[Sensor]) cursorPoint - (20@0)! !

!PopUpMenu methodsFor: 'accessing' stamp: 'sw 3/12/2002 21:37'!
startUpLeftFlush
	"Build and invoke this menu with no initial selection.  By Jerry Archibald, 4/01.
	If in MVC, align menus items with the left margin.
	Answer the selection associated with the menu item chosen by the user or nil if none is chosen.  
	The mechanism for getting left-flush appearance in mvc leaves a tiny possibility for misadventure: if the user, in mvc, puts up the jump-to-project menu, then hits cmd period while it is up, then puts up a second jump-to-project menu before dismissing or proceeding through the debugger, it's possible for mvc popup-menus thereafter to appear left-aligned rather than centered; this very unlikely condition can be cleared by evaluating 'PopUpMenu alignment: 2'"

	| saveAlignment result |
	Smalltalk isMorphic ifFalse:
		[saveAlignment := PopUpMenu alignment.
		PopUpMenu leftFlush].
	[result := self startUp] ensure:
		[Smalltalk isMorphic ifFalse:
			[PopUpMenu alignment: saveAlignment]].
	^ result! !

!PopUpMenu methodsFor: 'basic control sequence' stamp: 'yo 3/15/2005 12:55'!
startUpSegmented: segmentHeight withCaption: captionOrNil at: location
	"This menu is too big to fit comfortably on the screen.
	Break it up into smaller chunks, and manage the relative indices.
	Inspired by a special-case solution by Reinier van Loon."
"
(PopUpMenu labels: (String streamContents: [:s | 1 to: 100 do: [:i | s print: i; cr]. s skip: -1])
		lines: (5 to: 100 by: 5)) startUpWithCaption: 'Give it a whirl...'.
"
	| nLines nLinesPer allLabels from to subset subLines index |
	frame ifNil: [self computeForm].
	allLabels := labelString findTokens: Character cr asString.
	nLines := allLabels size.
	lineArray ifNil: [lineArray := Array new].
	nLinesPer := segmentHeight // marker height - 3.
	from := 1.
	[ true ] whileTrue:
		[to := (from + nLinesPer) min: nLines.
		subset := allLabels copyFrom: from to: to.
		subset add: (to = nLines ifTrue: ['start over...' translated] ifFalse: ['more...' translated])
			before: subset first.
		subLines := lineArray select: [:n | n >= from] thenCollect: [:n | n - (from-1) + 1].
		subLines := (Array with: 1) , subLines.
		index := (PopUpMenu labels: subset asStringWithCr lines: subLines)
					startUpWithCaption: captionOrNil at: location.
		index = 1
			ifTrue: [from := to + 1.
					from > nLines ifTrue: [ from := 1 ]]
			ifFalse: [index = 0 ifTrue: [^ 0].
					^ from + index - 2]]! !

!PopUpMenu methodsFor: 'basic control sequence' stamp: 'yo 3/15/2005 12:55'!
startUpSegmented: segmentHeight withCaption: captionOrNil at: location allowKeyboard: aBoolean
	"This menu is too big to fit comfortably on the screen.
	Break it up into smaller chunks, and manage the relative indices.
	Inspired by a special-case solution by Reinier van Loon.  The boolean parameter indicates whether the menu should be given keyboard focus (if in morphic)"

"
(PopUpMenu labels: (String streamContents: [:s | 1 to: 100 do: [:i | s print: i; cr]. s skip: -1])
		lines: (5 to: 100 by: 5)) startUpWithCaption: 'Give it a whirl...'.
"
	| nLines nLinesPer allLabels from to subset subLines index |
	frame ifNil: [self computeForm].
	allLabels := labelString findTokens: Character cr asString.
	nLines := allLabels size.
	lineArray ifNil: [lineArray := Array new].
	nLinesPer := segmentHeight // marker height - 3.
	from := 1.
	[ true ] whileTrue:
		[to := (from + nLinesPer) min: nLines.
		subset := allLabels copyFrom: from to: to.
		subset add: (to = nLines ifTrue: ['start over...' translated] ifFalse: ['more...' translated])
			before: subset first.
		subLines := lineArray select: [:n | n >= from] thenCollect: [:n | n - (from-1) + 1].
		subLines := (Array with: 1) , subLines.
		index := (PopUpMenu labels: subset asStringWithCr lines: subLines)
					startUpWithCaption: captionOrNil at: location allowKeyboard: aBoolean.
		index = 1
			ifTrue: [from := to + 1.
					from > nLines ifTrue: [ from := 1 ]]
			ifFalse: [index = 0 ifTrue: [^ 0].
					^ from + index - 2]]! !

!PopUpMenu methodsFor: 'basic control sequence' stamp: 'ar 3/18/2001 00:55'!
startUpWithCaption: captionOrNil
	"Display the menu, slightly offset from the cursor,
	so that a slight tweak is required to confirm any action."
	^ self startUpWithCaption: captionOrNil at: (ActiveHand ifNil:[Sensor]) cursorPoint! !

!PopUpMenu methodsFor: 'basic control sequence' stamp: 'ar 12/27/2001 22:47'!
startUpWithCaption: captionOrNil at: location
	"Display the menu, with caption if supplied. Wait for the mouse button to go down,
	then track the selection as long as the button is pressed. When the button is released, 
	answer the index of the current selection, or zero if the mouse is not released over 
	any menu item. Location specifies the desired topLeft of the menu body rectangle."

		^ self startUpWithCaption: captionOrNil at: location allowKeyboard: Preferences menuKeyboardControl! !

!PopUpMenu methodsFor: 'basic control sequence' stamp: 'dgd 4/4/2006 14:47'!
startUpWithCaption: captionOrNil at: location allowKeyboard: aBoolean
	"Display the menu, with caption if supplied. Wait for the mouse button to go down, then track the selection as long as the button is pressed. When the button is released,
	Answer the index of the current selection, or zero if the mouse is not released over  any menu item. Location specifies the desired topLeft of the menu body rectangle. The final argument indicates whether the menu should seize the keyboard focus in order to allow the user to navigate it via the keyboard."

	^ self
			startUpWithCaption: captionOrNil
			icon: nil
			at: location
			allowKeyboard: aBoolean! !

!PopUpMenu methodsFor: 'basic control sequence' stamp: 'dgd 10/12/2004 13:43'!
startUpWithCaption: captionOrNil icon: aForm
	"Display the menu, slightly offset from the cursor,
	so that a slight tweak is required to confirm any action."
	^ self
			startUpWithCaption: captionOrNil
			icon: aForm
			at: (ActiveHand ifNil:[Sensor]) cursorPoint
! !

!PopUpMenu methodsFor: 'basic control sequence' stamp: 'dgd 10/12/2004 13:44'!
startUpWithCaption: captionOrNil icon: aForm at: location
	"Display the menu, with caption if supplied. Wait for the mouse button to go down,
	then track the selection as long as the button is pressed. When the button is released, 
	answer the index of the current selection, or zero if the mouse is not released over 
	any menu item. Location specifies the desired topLeft of the menu body rectangle."

	^ self
			startUpWithCaption: captionOrNil
			icon: aForm
			at: location
			allowKeyboard: Preferences menuKeyboardControl
! !

!PopUpMenu methodsFor: 'basic control sequence' stamp: 'tlk 5/2/2006 22:22'!
startUpWithCaption: captionOrNil icon: aForm at: location allowKeyboard: aBoolean
	"Display the menu, with caption if supplied. Wait for the mouse button to go down, then track the selection as long as the button is pressed. When the button is released,
	Answer the index of the current selection, or zero if the mouse is not released over  any menu item. Location specifies the desired topLeft of the menu body rectangle. The final argument indicates whether the menu should seize the keyboard focus in order to allow the user to navigate it via the keyboard."

	| maxHeight |
	(ProvideAnswerNotification signal: captionOrNil) ifNotNilDo:
		[:answer | ^ selection := answer ifTrue: [1] ifFalse: [2]].
		 
	maxHeight := Display height*3//4.
	self frameHeight > maxHeight ifTrue:
		[^ self
			startUpSegmented: maxHeight
			withCaption: captionOrNil
			at: location
			allowKeyboard: aBoolean].

	Smalltalk isMorphic
		ifTrue:[
			selection := Cursor normal showWhile:
				[| menuMorph | 
				menuMorph := MVCMenuMorph from: self title: nil.
				(captionOrNil notNil or:[aForm notNil])
					ifTrue:[menuMorph addTitle: captionOrNil icon: aForm].
				MenuIcons decorateMenu: menuMorph.
				menuMorph
					invokeAt: location 
					in: ActiveWorld
					allowKeyboard: aBoolean].
			^ selection].

	frame ifNil: [self computeForm].
	Cursor normal showWhile:
		[self
			displayAt: location
			withCaption: captionOrNil
			during: [self controlActivity]].
	^ selection! !

!PopUpMenu methodsFor: 'basic control sequence' stamp: 'sw 12/17/2001 17:01'!
startUpWithoutKeyboard
	"Display and make a selection from the receiver as long as the button  is pressed. Answer the current selection.  Do not allow keyboard input into the menu"
	
	^ self startUpWithCaption: nil at: ((ActiveHand ifNil:[Sensor]) cursorPoint) allowKeyboard: false! !

PopUpMenu subclass: #SelectionMenu
	instanceVariableNames: 'selections'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Menus'!

SelectionMenu subclass: #CustomMenu
	instanceVariableNames: 'labels dividers lastDivider title targets arguments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Menus'!

!CustomMenu commentStamp: '<historical>' prior: 0!
I am used to build menus on-the-fly. I maintain lists of menu items, actions (usually symbols to be sent as messages), and menu section dividing lines to which my clients may append new entries and lines by sending me the messages:

	add: aString action: anAction
	addLine

After the menu is constructed, it may be invoked with one of the following messages:

	startUp: initialSelection
	startUp

I am a subclass of ActionMenu, so I inherit a number of instance variables. The ones I am particularly concerned with are:

	items _ an OrderedCollection of strings to appear in the menu
	selectors _ an OrderedCollection of Symbols to be used as message selectors
	lineArray _ an OrderedCollection of line positions
	lastLine _ used to keep track of the last line to avoid making duplicate entries in lineArray!

!CustomMenu classSide methodsFor: 'example' stamp: 'sw 11/8/1999 17:27'!
example
	"CustomMenu example"

	| menu |
	menu := CustomMenu new.
	menu add: 'apples' action: #apples.
	menu add: 'oranges' action: #oranges.
	menu addLine.
	menu addLine.  "extra lines ignored"
	menu add: 'peaches' action: #peaches.
	menu addLine.
	menu add: 'pears' action: #pears.
	menu addLine.
	^ menu startUp: #apples


"NB:  The following is equivalent to the above, but uses the compact #fromArray: consruct:
	(CustomMenu fromArray:
		#(	('apples'		apples)
			('oranges'		oranges)
			-
			-
			('peaches'		peaches)
			-
			('pears'			pears)
			-))
				startUp: #apples"! !

!CustomMenu methodsFor: 'construction' stamp: 'dhhi 9/14/2000 22:39'!
add: aString action: actionItem
	"Add the given string as the next menu item. If it is selected, the given action (usually but not necessarily a symbol) will be returned to the client."

	| s |
	aString ifNil: [^ self addLine].
	s := String new: aString size + 2.
	s at: 1 put: Character space.
	s replaceFrom: 2 to: s size - 1 with: aString.
	s at: s size put: Character space.
	labels addLast: s.
	selections addLast: actionItem.! !

!CustomMenu methodsFor: 'compatibility' stamp: 'ads 2/20/2003 08:59'!
add: aString subMenu: aMenu target: target selector: aSymbol argumentList: argList
	"Create a sub-menu with the given label. This isn't really a sub-menu the way Morphic does it; it'll just pop up another menu."

	self
		add: aString
		target: aMenu
		selector: #invokeOn:
		argumentList: argList asArray.! !

!CustomMenu methodsFor: 'compatibility' stamp: 'sumim 2/10/2002 01:23'!
add: aString target: target selector: aSymbol argument: arg
	"Append a menu item with the given label. If the item is selected, it will send the given selector to the target object with the given argument."

	self add: aString
		target: target
		selector: aSymbol
		argumentList: (Array with: arg)! !

!CustomMenu methodsFor: 'compatibility' stamp: 'sumim 2/10/2002 01:18'!
add: aString target: target selector: aSymbol argumentList: argList
	"Append a menu item with the given label. If the item is selected, it will send the given selector to the target object with the given arguments. If the selector takes one more argument than the number of arguments in the given list, then the triggering event is supplied as as the last argument."

	self add: aString action: aSymbol.
	targets addLast: target.
	arguments addLast: argList asArray
! !

!CustomMenu methodsFor: 'construction' stamp: ''!
addLine
	"Append a line to the menu after the last entry. Suppress duplicate lines."

	(lastDivider ~= selections size) ifTrue: [
		lastDivider := selections size.
		dividers addLast: lastDivider].! !

!CustomMenu methodsFor: 'construction' stamp: 'sw 2/27/2001 07:52'!
addList: listOfTuplesAndDashes
	"Add a menu item to the receiver for each tuple in the given list of the form (<what to show> <selector>). Add a line for each dash (-) in the list.  The tuples may have an optional third element, providing balloon help for the item, but such an element is ignored in mvc."

	listOfTuplesAndDashes do: [:aTuple |
		aTuple == #-
			ifTrue: [self addLine]
			ifFalse: [self add: aTuple first action: aTuple second]]

	"CustomMenu new addList: #(
		('apples' buyApples)
		('oranges' buyOranges)
		-
		('milk' buyMilk)); startUp"

! !

!CustomMenu methodsFor: 'compatibility' stamp: 'nk 2/15/2004 16:19'!
addService: aService for: serviceUser
	"Append a menu item with the given service. If the item is selected, it will perform the given service."

	aService addServiceFor: serviceUser toMenu: self.! !

!CustomMenu methodsFor: 'compatibility' stamp: 'nk 2/15/2004 16:02'!
addServices2: services for: served extraLines: linesArray

	services withIndexDo: [:service :i |
		service addServiceFor: served toMenu: self.
		(linesArray includes: i)  ifTrue: [self addLine] ]! !

!CustomMenu methodsFor: 'compatibility' stamp: 'sumim 2/10/2002 01:20'!
addServices: services for: served extraLines: linesArray

	services withIndexDo: [:service :i |
		self addService: service for: served.
		(linesArray includes: i) | service useLineAfter 
			ifTrue: [self addLine]]! !

!CustomMenu methodsFor: 'construction' stamp: 'sw 8/12/2002 17:14'!
addStayUpItem
	"For compatibility with MenuMorph.  Here it is a no-op"! !

!CustomMenu methodsFor: 'construction' stamp: 'nk 11/25/2003 10:00'!
addTranslatedList: listOfTuplesAndDashes
	"Add a menu item to the receiver for each tuple in the given list of the form (<what to show> <selector>). Add a line for each dash (-) in the list.  The tuples may have an optional third element, providing balloon help for the item, but such an element is ignored in mvc.
	The first element will be translated."

	listOfTuplesAndDashes do: [:aTuple |
		aTuple == #-
			ifTrue: [self addLine]
			ifFalse: [self add: aTuple first translated action: aTuple second]]

	"CustomMenu new addTranslatedList: #(
		('apples' buyApples)
		('oranges' buyOranges)
		-
		('milk' buyMilk)); startUp"

! !

!CustomMenu methodsFor: 'compatibility' stamp: 'sw 2/16/2002 00:57'!
arguments
	"Answer my arguments, initializing them to an empty collection if they're found to be nil."

	^ arguments ifNil: [arguments := OrderedCollection new]! !

!CustomMenu methodsFor: 'construction' stamp: 'sw 7/20/1999 18:47'!
balloonTextForLastItem: aString
	"Vacuous backstop provided for compatibility with MorphicMenu"! !

!CustomMenu methodsFor: 'private' stamp: 'sw 12/10/1999 11:21'!
build
	"Turn myself into an invokable ActionMenu."

	| stream |
	stream := WriteStream on: (String new).
	labels do: [:label | stream nextPutAll: label; cr].
	(labels isEmpty) ifFalse: [stream skip: -1].  "remove final cr"
	super labels: stream contents
		font: MenuStyle defaultFont
		lines: dividers! !

!CustomMenu methodsFor: 'initialize-release' stamp: 'sumim 2/10/2002 01:26'!
initialize

	labels := OrderedCollection new.
	selections := OrderedCollection new.
	dividers := OrderedCollection new.
	lastDivider := 0.
	targets := OrderedCollection new.
	arguments := OrderedCollection new	! !

!CustomMenu methodsFor: 'invocation' stamp: 'sw 2/17/2002 04:48'!
invokeOn: targetObject
	"Pop up this menu and return the result of sending to the target object the selector corresponding to the menu item selected by the user. Return nil if no item is selected.  If the chosen selector has arguments, obtain them from my arguments"

	^ self invokeOn: targetObject orSendTo: nil! !

!CustomMenu methodsFor: 'invocation' stamp: 'jm 11/17/97 16:54'!
invokeOn: targetObject defaultSelection: defaultSelection
	"Invoke the menu with the given default selection (i.e. one of my 'action' symbols). Answer the 'action' selector associated with the menu item chosen by the user or nil if none is chosen."

	| sel |
	sel := self startUp: defaultSelection.
	sel = nil ifFalse: [
		sel numArgs = 0
			ifTrue: [^ targetObject perform: sel]
			ifFalse: [^ targetObject perform: sel with: nil]].
	^ nil
! !

!CustomMenu methodsFor: 'invocation' stamp: 'sw 11/16/2002 23:45'!
invokeOn: targetObject orSendTo: anObject
	"Pop up this menu and return the result of sending to the target object the selector corresponding to the menu item selected by the user. Return  nil if no item is selected.  If the chosen selector has arguments, obtain appropriately.  If the recipient does not respond to the resulting message, send it to the alternate object provided"

	| aSelector anIndex recipient |
	^ (aSelector := self startUp) ifNotNil:
		[anIndex := self selection.
		recipient := ((targets := self targets) isEmptyOrNil or: [anIndex > targets size])
			ifTrue:
				[targetObject]
			ifFalse:
				[targets at: anIndex].
		aSelector numArgs == 0
			ifTrue:
				[recipient perform: aSelector orSendTo: anObject]
			ifFalse:
				[recipient perform: aSelector withArguments: (self arguments at: anIndex)]]! !

!CustomMenu methodsFor: 'construction' stamp: 'jm
 8/20/1998 08:34'!
labels: aString font: aFont lines: anArrayOrNil
	"This method allows the receiver to accept old-style SelectionMenu creation messages. It should be used only for backward compatibility during the MVC-to-Morphic transition. New code should be written using the other menu construction protocol such as addList:."

	| labelList linesArray |
	labelList := (aString findTokens: String cr) asArray.
	anArrayOrNil
		ifNil: [linesArray := #()]
		ifNotNil: [linesArray := anArrayOrNil].
	1 to: labelList size do: [:i |
		self add: (labelList at: i) action: (labelList at: i).
		(linesArray includes: i) ifTrue: [self addLine]].
	font ifNotNil: [font := aFont].
! !

!CustomMenu methodsFor: 'construction' stamp: 'yo 8/28/2002 22:34'!
labels: labelList lines: linesArray selections: selectionsArray
	"This method allows the receiver to accept old-style SelectionMenu creation messages. It should be used only for backward compatibility during the MVC-to-Morphic transition. New code should be written using the other menu construction protocol such as addList:."
	"Labels can be either a sting with embedded crs, or a collection of strings."

	| labelArray |
	labelList isString
		ifTrue: [labelArray := labelList findTokens: String cr]
		ifFalse: [labelArray := labelList].
	1 to: labelArray size do: [:i |
		self add: (labelArray at: i) action: (selectionsArray at: i).
		(linesArray includes: i) ifTrue: [self addLine]].
! !

!CustomMenu methodsFor: 'private' stamp: 'di 4/14/1999 21:28'!
preSelect: action
	"Pre-select and highlight the menu item associated with the given action."

	| i |
	i := selections indexOf: action ifAbsent: [^ self].
	marker ifNil: [self computeForm].
	marker := marker
		align: marker topLeft
		with: (marker left)@(frame inside top + (marker height * (i - 1))).
	selection := i.! !

!CustomMenu methodsFor: 'invocation' stamp: ''!
startUp
	"Build and invoke this menu with no initial selection. Answer the selection associated with the menu item chosen by the user or nil if none is chosen."

	^ self startUp: nil! !

!CustomMenu methodsFor: 'invocation' stamp: 'sw 8/18/1998 12:01'!
startUp: initialSelection
	"Build and invoke this menu with the given initial selection. Answer the selection associated with the menu item chosen by the user or nil if none is chosen."

	^ self startUp: initialSelection withCaption: title! !

!CustomMenu methodsFor: 'invocation' stamp: ''!
startUp: initialSelection withCaption: caption
	"Build and invoke this menu with the given initial selection and caption. Answer the selection associated with the menu item chosen by the user or nil if none is chosen."

	self build.
	(initialSelection notNil) ifTrue: [self preSelect: initialSelection].
	^ super startUpWithCaption: caption! !

!CustomMenu methodsFor: 'invocation' stamp: 'sw 7/31/97 19:31'!
startUpWithCaption: caption
	"Build and invoke this menu with no initial selection. Answer the selection associated with the menu item chosen by the user or nil if none is chosen; use the provided caption"

	^ self startUp: nil withCaption: caption! !

!CustomMenu methodsFor: 'compatibility' stamp: 'sw 2/16/2002 00:57'!
targets
	"Answer my targets, initializing them to an empty collection if found to be nil"

	^ targets ifNil: [targets := OrderedCollection new]! !

!CustomMenu methodsFor: 'initialize-release' stamp: 'sw 8/18/1998 12:01'!
title: aTitle
	title := aTitle! !

SelectionMenu subclass: #EmphasizedMenu
	instanceVariableNames: 'emphases'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Menus'!

!EmphasizedMenu commentStamp: '<historical>' prior: 0!
A selection menu in which individual selections are allowed to have different emphases.  Emphases allowed are: bold, italic, struckThrough, and plain.  Provide an emphasis array, with one element per selection, to use.  Refer to the class method #example.!

!EmphasizedMenu classSide methodsFor: 'examples' stamp: 'fc 2/19/2004 22:06'!
example1
	"EmphasizedMenu example1"

	^ (self
		selections: #('how' 'well' 'does' 'this' 'work?' ) 
		emphases: #(#bold #normal #italic #struckOut #normal ))
			startUpWithCaption: 'A Menu with Emphases'! !

!EmphasizedMenu classSide methodsFor: 'examples' stamp: 'sw 9/11/97 16:14'!
example2
	"EmphasizedMenu example2"

	| aMenu |
	aMenu := EmphasizedMenu selections: #('One' 'Two' 'Three' 'Four').
	aMenu onlyBoldItem: 3.
	^ aMenu startUpWithCaption: 'Only the Bold'! !

!EmphasizedMenu classSide methodsFor: 'examples' stamp: 'fc 2/19/2004 22:08'!
example3
	"EmphasizedMenu example3"

	^ (self
		selectionAndEmphasisPairs: #('how' #bold 'well' #normal 'does' #italic 'this' #struckOut 'work' #normal))
		startUpWithCaption: 'A Menu with Emphases'! !

!EmphasizedMenu classSide methodsFor: 'instance creation' stamp: 'sw 12/23/96'!
selectionAndEmphasisPairs: interleavedList
	"An alternative form of call.  "
	| selList  emphList |
	selList := OrderedCollection new.
	emphList := OrderedCollection new.
	interleavedList pairsDo:
		[:aSel :anEmph |
			selList add: aSel.
			emphList add: anEmph].
	^ self selections:selList emphases: emphList! !

!EmphasizedMenu classSide methodsFor: 'instance creation' stamp: 'sma 5/28/2000 16:14'!
selections: selList emphases: emphList
	"Answer an instance of the receiver with the given selections and 
	emphases."

	^ (self selections: selList) emphases: emphList

"Example:
	(EmphasizedMenu
		selections: #('how' 'well' 'does' 'this' 'work?') 
		emphases: #(bold plain italic struckOut plain)) startUp"! !

!EmphasizedMenu methodsFor: 'emphasis' stamp: ''!
emphases: emphasisArray
	emphases := emphasisArray! !

!EmphasizedMenu methodsFor: 'emphasis' stamp: 'fc 2/19/2004 22:07'!
onlyBoldItem: itemNumber
	"Set up emphasis such that all items are plain except for the given item number.  "

	emphases := (Array new: selections size) atAllPut: #normal.
	emphases at: itemNumber put: #bold! !

!EmphasizedMenu methodsFor: 'private' stamp: 'fc 2/20/2004 11:01'!
setEmphasis
	"Set up the receiver to reflect the emphases in the emphases array.  "

	| selStart selEnd currEmphasis |
	
	labelString := labelString asText.
	emphases isEmptyOrNil ifTrue: [^ self].
	selStart := 1.
	1 to: selections size do:
		[:line |
			selEnd := selStart + (selections at: line) size - 1.
			((currEmphasis := emphases at: line) size > 0 and: [currEmphasis ~~ #normal]) ifTrue:
				[labelString addAttribute: (TextEmphasis perform: currEmphasis)
					from: selStart to: selEnd].
			selStart := selEnd + 2]! !

!EmphasizedMenu methodsFor: 'display' stamp: ''!
startUpWithCaption: captionOrNil
	self setEmphasis.
	^ super startUpWithCaption: captionOrNil! !

!SelectionMenu classSide methodsFor: 'instance creation' stamp: 'sw 11/8/1999 17:52'!
fromArray: anArray
	"Construct a menu from anArray.  The elements of anArray must be either:
	*  A pair of the form: <label> <selector>
or	*  The 'dash' (or 'minus sign') symbol

	Refer to the example at the bottom of the method"

	| labelList lines selections anIndex |
	labelList := OrderedCollection new.
	lines := OrderedCollection new.
	selections := OrderedCollection new.
	anIndex := 0.
	anArray do:
		[:anElement |
			anElement size == 1
				ifTrue:
					[(anElement == #-) ifFalse: [self error: 'badly-formed menu constructor'].
					lines add: anIndex]
				ifFalse:
					[anElement size == 2 ifFalse: [self error: 'badly-formed menu constructor'].
					anIndex := anIndex + 1.
					labelList add: anElement first.
					selections add: anElement second]].
	^ self labelList: labelList lines: lines selections: selections

"(SelectionMenu fromArray:
	#(	('first label'		moja)
		('second label'	mbili)
		-
		('third label' 	tatu)
		-
		('fourth label'	nne)
		('fifth label'	tano))) startUp"! !

!SelectionMenu classSide methodsFor: 'instance creation' stamp: 'sma 5/28/2000 16:04'!
labelList: labelList
	^ self labelArray: labelList! !

!SelectionMenu classSide methodsFor: 'instance creation' stamp: 'sma 5/28/2000 16:04'!
labelList: labelList lines: lines
	^ self labelArray: lines lines: lines! !

!SelectionMenu classSide methodsFor: 'instance creation' stamp: 'sma 5/28/2000 16:04'!
labelList: labelList lines: lines selections: selections
	^ (self labelArray: labelList lines: lines) selections: selections! !

!SelectionMenu classSide methodsFor: 'instance creation' stamp: 'sma 5/28/2000 16:04'!
labelList: labelList selections: selections
	^ self
		labelList: labelList
		lines: #()
		selections: selections! !

!SelectionMenu classSide methodsFor: 'instance creation' stamp: 'yo 8/30/2002 17:00'!
labels: labels lines: linesArray
	"Answer an instance of me whose items are in labels, with lines drawn  
	after each item indexed by linesArray. Labels can be either a string 
	with embedded CRs, or a collection of strings."

	(labels isString)
		ifTrue: [^ super labels: labels lines: linesArray]
		ifFalse: [^ super labelArray: labels lines: linesArray]! !

!SelectionMenu classSide methodsFor: 'instance creation' stamp: 'yo 8/30/2002 17:02'!
labels: labels lines: linesArray selections: selectionsArray
	"Answer an instance of me whose items are in labels, with lines drawn  
	after each item indexed by linesArray. Labels can be either a string  
	with embedded CRs, or a collection of strings. Record the given array of 
	selections corresponding to the items in labels."

	| labelString |
	(labels isString)
		ifTrue: [labelString := labels]
		ifFalse: [labelString := String streamContents:
					[:s |
					labels do: [:l | s nextPutAll: l; cr].
					s skip: -1]].
	^ (self labels: labelString lines: linesArray) selections: selectionsArray
! !

!SelectionMenu classSide methodsFor: 'instance creation' stamp: 'sma 5/28/2000 16:10'!
labels: labels selections: selectionsArray
	"Answer an instance of me whose items are in labels, recording 
	the given array of selections corresponding to the items in labels."

	^ self
		labels: labels
		lines: #()
		selections: selectionsArray! !

!SelectionMenu classSide methodsFor: 'instance creation' stamp: 'sma 5/28/2000 16:10'!
selections: selectionsArray
	"Answer an instance of me whose labels and selections are identical."

	^ self selections: selectionsArray lines: nil! !

!SelectionMenu classSide methodsFor: 'instance creation' stamp: 'sma 5/28/2000 16:10'!
selections: selectionsArray lines: linesArray
	"Answer an instance of me whose labels and selections are identical."

	^ self
		labelList: (selectionsArray collect: [:each | each asString])
		lines: linesArray
		selections: selectionsArray! !

!SelectionMenu methodsFor: 'basic control sequence' stamp: 'sma 5/28/2000 15:28'!
invokeOn: targetObject
	"Pop up this menu and return the result of sending to the target object 
	the selector corresponding to the menu item selected by the user. Return 
	nil if no item is selected."

	| sel |
	sel := self startUp.
	sel = nil ifFalse: [^ targetObject perform: sel].
	^ nil

"Example:
	(SelectionMenu labels: 'sin
cos
neg' lines: #() selections: #(sin cos negated)) invokeOn: 0.7"! !

!SelectionMenu methodsFor: 'invocation' stamp: 'sw 11/18/2002 16:24'!
invokeOn: targetObject orSendTo: anObject
	"Pop up the receiver, obtaining a selector; return the result of having the target object perform the selector.  If it dos not understand the selector, give the alternate object a chance"

	| aSelector |
	^ (aSelector := self startUp) ifNotNil:
		[(targetObject respondsTo: aSelector)
			ifTrue:
				[targetObject perform: aSelector]
			ifFalse:
				[anObject perform: aSelector]]! !

!SelectionMenu methodsFor: 'accessing' stamp: 'sma 5/28/2000 11:38'!
selections
	^ selections! !

!SelectionMenu methodsFor: 'accessing' stamp: 'sma 5/28/2000 11:38'!
selections: selectionArray
	selections := selectionArray! !

!SelectionMenu methodsFor: 'basic control sequence' stamp: 'sw 12/17/2001 17:26'!
startUpWithCaption: captionOrNil at: location allowKeyboard: aBoolean
	"Overridden to return value returned by manageMarker.  The boolean parameter indicates whether the menu should be given keyboard focus (if in morphic)"

	| index |
	index := super startUpWithCaption: captionOrNil at: location allowKeyboard: aBoolean.
	(selections = nil or: [(index between: 1 and: selections size) not])
		ifTrue: [^ nil].
	^ selections at: index! !

Object subclass: #View
	instanceVariableNames: 'model controller superView subViews transformation viewport window displayTransformation insetDisplayBox borderWidth borderColor insideColor boundingBox'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Framework'!

!View commentStamp: '<historical>' prior: 0!
My instances are intended to be components in a structured picture. Each View in the structured picture can contain other Views as sub-components. These sub-components are called subViews. A View can be a subView of only one View. This View is called its superView. The set of Views in a structured picture forms a hierarchy. The one View in the hierarchy that has no superView is called the topView of the structured picture. A View in a structured picture with no subViews is called a bottom View. A View and all of its subViews, and all of their subViews and so on, are treated as a unit in many operations on the View. For example, if a View is displayed, all of its subViews are displayed as well. There are several categories of operations that can be performed on a View. Among these are the following:
	
	1.	Adding subViews to a View.
	2.	Positioning subViews within a View.
	3.	Deleting subViews from a View.
	4.	Transforming a View.
	5.	Displaying a View.
	
Each View has its own coordinate system. In order to change from one coordinate system to another, each View has two transformations associated with it. The local transformation is a WindowingTransformation that maps objects in the coordinate system of the View to objects in the coordinate system of the superView of the View. The displayTransformation is a WindowingTransformation that maps objects in the coordinate system of the View to objects in the display screen coordinate system.
	
The part of the space that is to be made visible is represented by the window of the View. The window of a View is a Rectangle expressed in the coordinate system of the View. The area occupied by a View in the coordinate system of its superView is called its viewport. The viewport of a View is its window transformed by its local transformation. The region of the display screen occupied by a View is called its displayBox. The display box of a View can include a border. The width of the border expressed in display screen coordinates is called the border width of the View. The color of the border is called the border color. The region of the display box of a View excluding the border is called the inset display box. The color of the inset display box is called the inside color of the View.!

View subclass: #DisplayTextView
	instanceVariableNames: 'rule mask editParagraph centered'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Views'!

!DisplayTextView commentStamp: '<historical>' prior: 0!
I represent a view of an instance of DisplayText.!

!DisplayTextView classSide methodsFor: 'examples' stamp: ''!
example2	
	"Create a standarad system view with two parts, one editable, the other not."
	| topView aDisplayTextView |
	topView := StandardSystemView new.
	topView label: 'Text Editor'.
	aDisplayTextView := self new model: 'test string label' asDisplayText.
	aDisplayTextView controller: NoController new.
	aDisplayTextView window: (0 @ 0 extent: 100 @ 100).
	aDisplayTextView borderWidthLeft: 2 right: 0 top: 2 bottom: 2.
	topView addSubView: aDisplayTextView.

	aDisplayTextView := self new model: 'test string' asDisplayText.
	aDisplayTextView window: (0 @ 0 extent: 100 @ 100).
	aDisplayTextView borderWidth: 2.
	topView
		addSubView: aDisplayTextView
		align: aDisplayTextView viewport topLeft
		with: topView lastSubView viewport topRight.
	topView controller open

	"DisplayTextView example2"! !

!DisplayTextView classSide methodsFor: 'examples' stamp: ''!
example3	
	"Create a passive view of some text on the screen."
	| view |
	view:= self new model: 'this is a test of one line
and the second line' asDisplayText.
	view translateBy: 100@100.	
	view borderWidth: 2.
	view display.
	view release

	"DisplayTextView example3"! !

!DisplayTextView classSide methodsFor: 'examples' stamp: ''!
example4	
	"Create four passive views of some text on the screen with fat borders."
	| view |
	view:= self new model: 'this is a test of one line
and the second line' asDisplayText.
	view translateBy: 100@100.	
	view borderWidth: 5.
	view display.
	3 timesRepeat: [view translateBy: 100@100. view display].
	view release

	"DisplayTextView example4"! !

!DisplayTextView methodsFor: 'private' stamp: ''!
centerText

	self isCentered
		ifTrue: 
			[editParagraph
				align: editParagraph boundingBox center
				with: self getWindow center]! !

!DisplayTextView methodsFor: 'accessing' stamp: ''!
centered

	centered := true.
	self centerText! !

!DisplayTextView methodsFor: 'deEmphasizing' stamp: ''!
deEmphasizeView 
	"Refer to the comment in View|deEmphasizeView."

	(self controller isKindOf: ParagraphEditor)
	 	ifTrue: [controller deselect]! !

!DisplayTextView methodsFor: 'controller access' stamp: ''!
defaultController 
	"Refer to the comment in View|defaultController."

	^self defaultControllerClass newParagraph: editParagraph! !

!DisplayTextView methodsFor: 'controller access' stamp: ''!
defaultControllerClass 
	"Refer to the comment in View|defaultControllerClass."

	^ParagraphEditor! !

!DisplayTextView methodsFor: 'private' stamp: ''!
defaultRule

	^Form over! !

!DisplayTextView methodsFor: 'window access' stamp: ''!
defaultWindow 
	"Refer to the comment in View|defaultWindow."

	^self inverseDisplayTransform: (editParagraph boundingBox expandBy: 6 @ 6)! !

!DisplayTextView methodsFor: 'displaying' stamp: ''!
display 
	"Refer to the comment in View|display."

	self isUnlocked ifTrue: [self positionText].
	super display! !

!DisplayTextView methodsFor: 'displaying' stamp: ''!
displayView 
	"Refer to the comment in View|displayView."

	self clearInside.
	(self controller isKindOf: ParagraphEditor )
		ifTrue: [controller changeParagraph: editParagraph].
	editParagraph foregroundColor: self foregroundColor
				backgroundColor: self backgroundColor.
	self isCentered
		ifTrue: 
			[editParagraph displayOn: Display
				transformation: self displayTransformation
				clippingBox: self insetDisplayBox
				fixedPoint: editParagraph boundingBox center]
		ifFalse: 
			[editParagraph displayOn: Display]! !

!DisplayTextView methodsFor: 'accessing' stamp: ''!
fillColor
	"Answer an instance of class Form that is to be used as the mask when 
	displaying the receiver's model (a DisplayText)."

	^ mask! !

!DisplayTextView methodsFor: 'accessing' stamp: ''!
fillColor: aForm 
	"Set aForm to be the mask used when displaying the receiver's model."

	mask := aForm! !

!DisplayTextView methodsFor: 'initialize-release' stamp: ''!
initialize 
	"Refer to the comment in View|initialize."

	super initialize.
	centered := false! !

!DisplayTextView methodsFor: 'accessing' stamp: ''!
isCentered

	^centered! !

!DisplayTextView methodsFor: 'lock access' stamp: ''!
lock 
	"Refer to the comment in View|lock.  Must do what would be done by displaying..."

	self isUnlocked ifTrue: [self positionText].
	super lock! !

!DisplayTextView methodsFor: 'accessing' stamp: ''!
mask
	"Answer an instance of class Form that is to be used as the mask when 
	displaying the receiver's model (a DisplayText)."

	^ mask! !

!DisplayTextView methodsFor: 'model access' stamp: ''!
model: aDisplayText 
	"Refer to the comment in View|model:."

	super model: aDisplayText.
	editParagraph := model asParagraph.
	self centerText! !

!DisplayTextView methodsFor: 'private' stamp: ''!
positionText

	| box |
	box := (self displayBox insetBy: 6@6) origin extent: editParagraph boundingBox extent.
	editParagraph wrappingBox: box clippingBox: box.
	self centerText! !

!DisplayTextView methodsFor: 'accessing' stamp: ''!
rule
	"Answer a number from 0 to 15 that indicates which of the sixteen 
	display rules is to be used when copying the receiver's model (a 
	DisplayText) onto the display screen."

	rule == nil
		ifTrue: [^self defaultRule]
		ifFalse: [^rule]! !

!DisplayTextView methodsFor: 'accessing' stamp: ''!
rule: anInteger 
	"Set anInteger to be the rule used when displaying the receiver's model."

	rule := anInteger! !

!DisplayTextView methodsFor: 'displaying' stamp: ''!
uncacheBits
	"Normally only sent to a StandardSystemView, but for casees where a DisplayTextView is used alone, without a superview, in which we make this a no-op, put in so that the Character Recognizer doesn't fail.  8/9/96 sw"! !

!DisplayTextView methodsFor: 'window access' stamp: ''!
window: aWindow 
	"Refer to the comment in View|window:."

	super window: aWindow.
	self centerText! !

View subclass: #FormMenuView
	instanceVariableNames: ''
	classVariableNames: 'BorderForm FormButtons SpecialBorderForm'
	poolDictionaries: ''
	category: 'ST80-Editors'!

!FormMenuView commentStamp: '<historical>' prior: 0!
I represent a View whose subViews are Switches (and Buttons and OneOnSwitches) whose actions set the mode, color, and tool for editing a Form on the screen. The default controller of my instances is FormMenuController.!

!FormMenuView classSide methodsFor: 'class initialization' stamp: 'jm 3/27/98 14:54'!
fileOut
	"Save the FormEditor button icons."
	"FormMenuView fileOut"

	| names |
	names := 
		#('select.form' 'singlecopy.form' 'repeatcopy.form' 'line.form' 'curve.form'
		'block.form' 'over.form' 'under.form' 'reverse.form' 'erase.form' 'in.form'
		'magnify.form' 'white.form' 'lightgray.form' 'gray.form' 'darkgray.form' 'black.form'
		'xgrid.form' 'ygrid.form' 'togglegrids.form' 'out.form').
	1 to: FormButtons size do: [:i |
		(FormButtons at: i) form writeOnFileNamed: (names at: i)].
	SpecialBorderForm writeOnFileNamed: 'specialborderform.form'.
	BorderForm writeOnFileNamed: 'borderform.form'.
! !

!FormMenuView classSide methodsFor: 'accessing' stamp: 'BG 12/4/2003 12:11'!
formButtons

  ^FormButtons! !

!FormMenuView classSide methodsFor: 'class initialization' stamp: 'md 7/23/2006 16:41'!
initialize2
	"The icons for the menu are typically stored on files. In order to avoid reading them every time, they are stored in a collection in a class variable, along with their offset, tool value, and initial visual state (on or off)."
	"FormMenuView initialize"

	| offsets keys states names button |
	offsets _ OrderedCollection new: 21.
	#(0 64 96 128 160 192 256 288 320 352 420) do: [:i | offsets addLast: i@0].  "First row"
	#(0 64 96 128 160 192 256 304 352 420) do: [:i | offsets addLast: i@48].  "Second row"
	offsets _ offsets asArray.
	keys _ #($a $s $d $f $g $h $j $k $l $; $' $z $x $c $v $b $n $m $, $. $/ ).  "Keyboard"
	states _ #(
		#false #false #true #false #false #false #true #false #false #false #false
		#false #false #false #false #false #true #false #false #false #false).  "Initial button states"
	names _ 
		#('select.form' 'singlecopy.form' 'repeatcopy.form' 'line.form' 'curve.form'
		'block.form' 'over.form' 'under.form' 'reverse.form' 'erase.form' 'in.form'
		'magnify.form' 'white.form' 'lightgray.form' 'gray.form' 'darkgray.form' 'black.form'
		'xgrid.form' 'ygrid.form' 'togglegrids.form' 'out.form').  "Files of button images"
	FormButtons _ OrderedCollection new.
	1 to: 21 do: [:index |
		button _ FormButtonCache new.
		button form: (Form fromFileNamed: (names at: index)).
		button offset: (offsets at: index).
		button value: (keys at: index).
		button initialState: (states at: index).
		FormButtons addLast: button].
	SpecialBorderForm  _ Form fromFileNamed: 'specialborderform.form'.
	BorderForm _ Form fromFileNamed: 'borderform.form'.
! !

!FormMenuView methodsFor: 'controller access' stamp: ''!
defaultControllerClass 
	"Refer to the comment in View|defaultControllerClass."

	^FormMenuController! !

!FormMenuView methodsFor: 'private' stamp: 'BG 12/4/2003 14:22'!
makeButton: index

	| buttonCache button |
	buttonCache := (FormButtons at: index) shallowCopy.
	buttonCache form: (FormButtons at: index) form copy.
	button := Button newOff.
	button onAction: [model changeTool: buttonCache value].
	self makeViews: buttonCache for: button.
! !

!FormMenuView methodsFor: 'private' stamp: 'BG 12/4/2003 14:23'!
makeColorConnections: indexInterval

	| connector buttonCache button aSwitchView |
	connector := Object new.  "a dummy model for connecting dependents"
	indexInterval do: [:index |
	buttonCache := (FormButtons at: index) shallowCopy.
	buttonCache form: (FormButtons at: index) form copy.
		buttonCache initialState = #true
			ifTrue: [button := OneOnSwitch newOn]
			ifFalse: [button := OneOnSwitch newOff].
		button onAction: [model changeTool: buttonCache value].
		button connection: connector.
		aSwitchView := self makeViews: buttonCache for: button.
		aSwitchView
			borderWidthLeft: 1 right: 0 top: 1 bottom: 1;
			action: #turnOn].
	aSwitchView borderWidth: 1.
! !

!FormMenuView methodsFor: 'private' stamp: 'BG 12/4/2003 14:23'!
makeConnections: indexInterval

	| connector buttonCache button aSwitchView |
	connector := Object new.  "a dummy model for connecting dependents."
	indexInterval do: [:index |
	buttonCache := (FormButtons at: index) shallowCopy.
	buttonCache form: (FormButtons at: index) form copy.
		buttonCache initialState = #true
			ifTrue: [button := OneOnSwitch newOn]
			ifFalse: [button := OneOnSwitch newOff].
		button onAction: [model changeTool: buttonCache value].
		button connection: connector.
		aSwitchView := self makeViews: buttonCache for: button.
		aSwitchView
			borderWidthLeft: 1 right: 0 top: 1 bottom: 1;
			action: #turnOn].
	aSwitchView borderWidth: 1.
! !

!FormMenuView methodsFor: 'initialize-release' stamp: ''!
makeFormEditorMenu

	| button buttonCache form aSwitchView aSwitchController|
	"Now get those forms into the subviews"
	self makeButton: 1.					"form source"
	self makeConnections: (2 to: 6).		"tools"
	self makeConnections: (7 to: 10).		"modes"
	self makeButton: 11.					"filing in"
	self makeButton: 12.					"bit editing"
	self makeColorConnections: (13 to: 17).		"colors"
	self makeGridSwitch: 18.					"toggle x"
	self makeGridSwitch: 19.					"toggle y"
	self makeButton: 20.					"setting grid"
	self makeButton: 21					"filing out"! !

!FormMenuView methodsFor: 'private' stamp: 'BG 12/4/2003 15:24'!
makeGridSwitch: index

	| buttonCache button |
	buttonCache := FormButtons at: index.
	buttonCache form: (FormButtons at: index) form copy.
	buttonCache initialState = #true
		ifTrue: [button := Switch newOn]
		ifFalse: [button := Switch newOff].
	button onAction: [model changeTool: buttonCache value].
	button offAction: [model changeTool: buttonCache value].
	self makeViews: buttonCache for: button.
! !

!FormMenuView methodsFor: 'private' stamp: 'BG 12/4/2003 14:23'!
makeSwitch: index

	| buttonCache button |
	buttonCache := (FormButtons at: index) shallowCopy.
	buttonCache form: (FormButtons at: index) form copy.
	buttonCache initialState = #true
		ifTrue: [button := Switch newOn]
		ifFalse: [button := Switch newOff].
	button onAction: [model changeTool: buttonCache value].
	self makeViews: buttonCache for: button.
! !

!FormMenuView methodsFor: 'private' stamp: 'jm 4/7/98 20:24'!
makeViews: cache for: aSwitch

	| form aSwitchView |
	form := cache form.
	aSwitchView := PluggableButtonView
		on: aSwitch
		getState: #isOn
		action: #switch.
	aSwitchView
		label: form;
		shortcutCharacter: cache value;
		window: (0@0 extent: form extent);
		translateBy: cache offset;
		borderWidth: 1.
	self addSubView: aSwitchView.
	^ aSwitchView
! !

!FormMenuView methodsFor: 'subView access' stamp: 'jm 4/2/98 17:29'!
subViewContainingCharacter: aCharacter
	"Answer the receiver's subView that corresponds to the key, aCharacter. 
	Answer nil if no subView is selected by aCharacter."

	self subViews reverseDo: 
		[:aSubView |
		(aSubView shortcutCharacter = aCharacter) ifTrue: [^aSubView]].
	^nil	
! !

View subclass: #FormView
	instanceVariableNames: 'rule mask'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Views'!

!FormView commentStamp: '<historical>' prior: 0!
I represent a view of a Form.!

FormView subclass: #FormHolderView
	instanceVariableNames: 'displayedForm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Views'!

!FormHolderView commentStamp: '<historical>' prior: 0!
I represent a view of a Form. Editing takes place by modifying a working version of the Form. The message accept is used to copy the working version into the Form; the message cancel copies the Form into the working version.!

!FormHolderView methodsFor: 'menu messages' stamp: ''!
accept 
	"Refer to the comment in FormView|accept."
	model
		copyBits: displayedForm boundingBox
		from: displayedForm
		at: 0 @ 0
		clippingBox: model boundingBox
		rule: Form over
		fillColor: nil.
	model changed: self! !

!FormHolderView methodsFor: 'menu messages' stamp: ''!
cancel 
	"Refer to the comment in FormView|cancel."

	displayedForm become: model deepCopy.
	displayedForm changed: self.
	self display! !

!FormHolderView methodsFor: 'model access' stamp: ''!
changeValueAt: location put: anInteger 
	"Refer to the comment in FormView|changeValueAt:put:."

	displayedForm pixelValueAt: location put: anInteger.
	displayedForm changed: self! !

!FormHolderView methodsFor: 'displaying' stamp: ''!
displayView 
	"Display the Form associated with this View according to the rule and
	fillColor specifed by this class."

	| oldOffset |
	oldOffset := displayedForm offset.
	displayedForm offset: 0@0.
	displayedForm
		displayOn: Display
		transformation: self displayTransformation
		clippingBox: self insetDisplayBox
		rule: self rule
		fillColor: self fillColor.
	displayedForm offset: oldOffset! !

!FormHolderView methodsFor: 'model access' stamp: ''!
model: aForm

	super model: aForm.
	displayedForm := aForm deepCopy! !

!FormHolderView methodsFor: 'initialize-release' stamp: ''!
release

	super release.
	displayedForm release.
	displayedForm := nil! !

!FormHolderView methodsFor: 'displaying' stamp: ''!
updateDisplay
	"The working version is redefined by copying the bits displayed in the 
	receiver's display area."

	displayedForm fromDisplay: self displayBox.
	displayedForm changed: self! !

!FormHolderView methodsFor: 'model access' stamp: ''!
workingForm
	"Answer the form that is currently being displayed--the working version 
	in which edits are carried out."

	^displayedForm! !

FormView subclass: #FormInspectView
	instanceVariableNames: 'offset'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Views'!

!FormInspectView classSide methodsFor: 'instance creation' stamp: 'sd 5/11/2003 21:36'!
openOn: aFormDictionary withLabel: aLabel
	"open a graphical dictionary in a window having the label aLabel. 
     aFormDictionary should be a dictionary containing as value a form."

     ^ DictionaryInspector
                openOn: aFormDictionary
                withEvalPane: true
                withLabel: aLabel
                valueViewClass: self! !

!FormInspectView methodsFor: 'as yet unclassified' stamp: ''!
defaultControllerClass 
	"Refer to the comment in View|defaultControllerClass."

	^  NoController! !

!FormInspectView methodsFor: 'as yet unclassified' stamp: 'di 9/23/1998 10:55'!
displayView 
	"Display the form as a value in an inspector.  8/11/96 sw"
	"Defeated form scaling for HS FormInspector.  8/20/96 di"
	| scale |
	Display fill: self insetDisplayBox fillColor: Color white.
	model selectionIndex == 0 ifTrue: [^ self].
	scale := self insetDisplayBox extent / model selection extent.
	scale := (scale x min: scale y) min: 1.
	model selection
		displayOn: Display
		transformation: (WindowingTransformation
			scale: scale asPoint
			translation: self insetDisplayBox topLeft - model selection offset)
		clippingBox: self insetDisplayBox
		rule: self rule
		fillColor: self fillColor! !

!FormView classSide methodsFor: 'examples' stamp: ''!
exampleOne
	"Frame a Form (specified by the user) with a border of 2 bits in width and display it offset 60 x 40 from the cornor of the display screen. "
	| f view |
	f := Form fromUser.
	view := self new model: f.
	view translateBy: 60 @ 40.
	view borderWidth: 2.
	view display.
	view release

	"FormView exampleOne"! !

!FormView classSide methodsFor: 'examples' stamp: ''!
exampleTwo
	"Frame a Form (specified by the user) that is scaled by 2. The border is 2 bits in width. Displays at location 60, 40."
	| f view |
	f := Form fromUser.
	view := self new model: f.
	view scaleBy: 2.0.
	view translateBy: 60 @ 40.
	view borderWidth: 2.
	view display.
	view release

	"FormView exampleTwo"! !

!FormView classSide methodsFor: 'examples' stamp: 'BG 12/5/2003 14:45'!
open: aForm named: aString
	"FormView open: ((Form extent: 100@100) borderWidth: 1) named: 'Squeak' "
	"Open a window whose model is aForm and whose label is aString."
	| topView aView |
	topView := StandardSystemView new.
	topView model: aForm.
	topView label: aString.
	topView minimumSize: aForm extent;
	          maximumSize: aForm extent.
	aView := FormView new.
	aView model: aForm.
	aView window: (aForm boundingBox expandBy: 2).
	aView borderWidthLeft: 2 right: 2 top: 2 bottom: 2.
	topView addSubView: aView.
	topView controller open! !

!FormView methodsFor: 'menu messages' stamp: ''!
accept
	"The receiver's model is set to the working version, the one in which 
	edits are carried out."

	^self! !

!FormView methodsFor: 'menu messages' stamp: ''!
cancel
	"Set the working form to be a copy of the model."

	^self! !

!FormView methodsFor: 'model access' stamp: ''!
changeValueAt: location put: anInteger
	"The receiver's model is a form which has an array of bits. Change the 
	bit at index, location, to be anInteger (either 1 or 0). Inform all objects 
	that depend on the model that it has changed."

	model pixelValueAt: location put: anInteger.
	model changed: self! !

!FormView methodsFor: 'controller access' stamp: ''!
defaultControllerClass 
	"Refer to the comment in View|defaultControllerClass."

	^  FormEditor! !

!FormView methodsFor: 'private' stamp: ''!
defaultRule 
	"The default display rule is 3=over or storing."

	^Form over! !

!FormView methodsFor: 'window access' stamp: ''!
defaultWindow 
	"Refer to the comment in View|defaultWindow."

	^(Rectangle origin: 0 @ 0 extent: model extent)
		expandBy: borderWidth! !

!FormView methodsFor: 'displaying' stamp: ''!
displayOn: aPort
	model displayOnPort: aPort at: self displayBox origin! !

!FormView methodsFor: 'displaying' stamp: 'hmm 7/21/97 20:45'!
displayView 
	"Refer to the comment in View|displayView."

	| oldOffset |
	super displayView.
	insideColor == nil ifFalse: [Display fill: self insetDisplayBox fillColor: insideColor].
	oldOffset := model offset.
	model offset: "borderWidth origin" 0@0.
	model
		displayOn: Display
		transformation: self displayTransformation
		clippingBox: self insetDisplayBox
		rule: self rule
		fillColor: self fillColor.
	model offset: oldOffset! !

!FormView methodsFor: 'accessing' stamp: ''!
fillColor
	"Answer an instance of class Form that is the mask used when displaying 
	the receiver's model (a Form) on the display screen (see BitBlt for the 
	meaning of this mask)."

	^ mask! !

!FormView methodsFor: 'accessing' stamp: ''!
fillColor: aForm 
	"Set the display mask for displaying the receiver's model to be the 
	argument, aForm."

	mask := aForm! !

!FormView methodsFor: 'accessing' stamp: ''!
mask
	"Answer an instance of class Form that is the mask used when displaying 
	the receiver's model (a Form) on the display screen (see BitBlt for the 
	meaning of this mask)."

	^ mask! !

!FormView methodsFor: 'accessing' stamp: ''!
rule
	"Answer a number from 0 to 15 that indicates which of the sixteen 
	display rules (logical function of two boolean values) is to be used when 
	copying the receiver's model (a Form) onto the display screen."

	rule == nil
		ifTrue: [^self defaultRule]
		ifFalse: [^rule]! !

!FormView methodsFor: 'accessing' stamp: ''!
rule: anInteger 
	"Set the display rule for the receiver to be the argument, anInteger."

	rule := anInteger! !

!FormView methodsFor: 'displaying' stamp: ''!
uncacheBits
	"Placed vacuously here so that when ControlManager>>restore calls uncacheBits for a project with no windows, we don't hang.  1/24/96 sw"! !

!FormView methodsFor: 'updating' stamp: ''!
update: aFormView 
	"Refer to the comment in View|update:."

	self == aFormView ifFalse: [self display]! !

!FormView methodsFor: 'displaying' stamp: ''!
updateDisplay
	"overridden by subclass"! !

!FormView methodsFor: 'window access' stamp: ''!
windowBox
	"For comaptibility with Control manager (see senders)"
	^ self insetDisplayBox! !

View subclass: #ListView
	instanceVariableNames: 'list selection topDelimiter bottomDelimiter isEmpty textStyle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Support'!

!ListView commentStamp: '<historical>' prior: 0!
I am an abstract View of a list of items. I provide support for storing a selection of one item, as well as formatting the list for presentation on the screen. My instances' default controller is ListController.!

!ListView methodsFor: 'list access' stamp: 'sw 12/10/1999 10:43'!
assuredTextStyle
	^ textStyle ifNil:
		[textStyle :=  ListParagraph standardListStyle]
! !

!ListView methodsFor: 'delimiters' stamp: ''!
bottomDelimiter
	"Answer the string used to indicate the bottom of the list."

	^bottomDelimiter! !

!ListView methodsFor: 'delimiters' stamp: ''!
bottomDelimiter: aString 
	"Set the string used to indicate the bottom of the list."

	bottomDelimiter := aString! !

!ListView methodsFor: 'display box access' stamp: ''!
boundingBox 
	"Refer to the comment in View|boundingBox."

	^list boundingBox! !

!ListView methodsFor: 'clipping box access' stamp: ''!
clippingBox
	"Answer the rectangle in which the model can be displayed--this is the 
	insetDisplayBox inset by the height of a line for an item."

	^self insetDisplayBox insetBy: 
		(Rectangle
			left: 0
			right: 0
			top: 0
			bottom: self insetDisplayBox height \\ list lineGrid)! !

!ListView methodsFor: 'displaying' stamp: ''!
deEmphasizeSelectionBox
	self displaySelectionBox! !

!ListView methodsFor: 'deEmphasizing' stamp: ''!
deEmphasizeView 
	"Refer to the comment in View|deEmphasizeView."
	^ self deEmphasizeSelectionBox! !

!ListView methodsFor: 'controller access' stamp: ''!
defaultControllerClass 
	"Refer to the comment in View|defaultControllerClass."

	^ListController! !

!ListView methodsFor: 'selecting' stamp: ''!
deselect
	"If the receiver has a selection, then it is highlighted. Remove the 
	highlighting."

	selection ~= 0 ifTrue: [Display reverse: (self selectionBox intersect: self clippingBox)]! !

!ListView methodsFor: 'displaying' stamp: ''!
display 
	"Refer to the comment in View.display."
	(self isUnlocked and: [self clippingBox ~= list clippingRectangle])
		ifTrue:  "Recompose the list if the window changed"
			[selection isNil ifTrue: [selection := 0].
			self positionList].
	super display! !

!ListView methodsFor: 'displaying' stamp: ''!
displaySelectionBox
	"If the receiver has a selection and that selection is visible on the display 
	screen, then highlight it."
	selection ~= 0 ifTrue:
		[Display reverse: (self selectionBox intersect: self clippingBox)]! !

!ListView methodsFor: 'displaying' stamp: ''!
displayView 
	"Refer to the comment in View|displayView."

	self clearInside.
	list foregroundColor: self foregroundColor
		backgroundColor: self backgroundColor.
	list displayOn: Display! !

!ListView methodsFor: 'deEmphasizing' stamp: ''!
emphasizeView 
	"List emphasis is its own inverse."
	^ self deEmphasizeView! !

!ListView methodsFor: 'selecting' stamp: ''!
findSelection: aPoint 
	"Determine which selection is displayed in an area containing the point, 
	aPoint. Answer the selection if one contains the point, answer nil 
	otherwise."

	| trialSelection |
	(self clippingBox containsPoint: aPoint) ifFalse: [^nil].
	trialSelection := aPoint y - list compositionRectangle top // list lineGrid + 1.
	topDelimiter == nil ifFalse: [trialSelection := trialSelection - 1].
	(trialSelection < 1) | (trialSelection > self maximumSelection)
		ifTrue: [^ nil]
		ifFalse: [^ trialSelection]! !

!ListView methodsFor: 'private' stamp: ''!
firstShown
	"Return the index of the top item currently visible"
	| trial |
	trial := self findSelection: self insetDisplayBox topLeft.
	^ trial == nil
		ifTrue: [1]
		ifFalse: [trial]! !

!ListView methodsFor: 'font access' stamp: 'sw 12/9/1999 18:07'!
font
	^ self assuredTextStyle fontNamed: textStyle fontNames first
! !

!ListView methodsFor: 'font access' stamp: 'jm 9/20/1998 19:44'!
font: aFontOrNil

	aFontOrNil
		ifNil: [textStyle := nil]
		ifNotNil: [
			textStyle := TextStyle fontArray: (Array with: aFontOrNil).
			textStyle gridForFont: 1 withLead: 1].
	self changed: #list.  "update display"
! !

!ListView methodsFor: 'initialize-release' stamp: ''!
initialize 
	"Refer to the comment in View|initialize."

	super initialize.
	topDelimiter := '------------'.
	bottomDelimiter := '------------'.
	isEmpty := true.
	self list: Array new! !

!ListView methodsFor: 'display box access' stamp: 'mkd 11/4/1999 14:31'!
isSelectionBoxClipped
        "Answer whether there is a selection and whether the selection is visible 
        on the screen."

        ^ selection ~= 0 and:
			[(self selectionBox intersects: 
                       (self clippingBox insetBy: (Rectangle left: 0 right: 0 top: 1 bottom: 0))) not]! !

!ListView methodsFor: 'private' stamp: ''!
lastShown
	"Return the index of the bottom item currently visible"
	| trial bottomMargin |
	bottomMargin := self insetDisplayBox height \\ list lineGrid.
	trial := self findSelection: self insetDisplayBox bottomLeft - (0@bottomMargin).
	trial == nil
		ifTrue: [trial := self findSelection: self insetDisplayBox bottomLeft
					- (0@(list lineGrid+bottomMargin))].
	^ trial == nil
		ifTrue: [list numberOfLines - 2]
		ifFalse: [trial]! !

!ListView methodsFor: 'list access' stamp: ''!
list
	"Answer the list of items the receiver displays."

	^list! !

!ListView methodsFor: 'list access' stamp: 'sw 12/9/1999 18:06'!
list: anArray 
	"Set the list of items the receiver displays to be anArray."
	| arrayCopy i |
	isEmpty := anArray isEmpty.
	arrayCopy := Array new: (anArray size + 2).
	arrayCopy at: 1 put: topDelimiter.
	arrayCopy at: arrayCopy size put: bottomDelimiter.
	i := 2.
	anArray do: [:el | arrayCopy at: i put: el. i := i+1].
	arrayCopy := arrayCopy copyWithout: nil.
	list := ListParagraph withArray: arrayCopy style: self assuredTextStyle.
	selection := 0.
	self positionList.
! !

!ListView methodsFor: 'lock access' stamp: ''!
lock
	"Refer to the comment in view|lock.  Must do at least what display would do to lock the view."

	(self isUnlocked and: [self clippingBox ~= list clippingRectangle])
		ifTrue:  "Recompose the list if the window changed"
			[self positionList].
	super lock! !

!ListView methodsFor: 'selecting' stamp: ''!
maximumSelection
	"Answer which selection is the last possible one."
	^ list numberOfLines
		- (topDelimiter == nil ifTrue: [0] ifFalse: [1])
		- (bottomDelimiter == nil ifTrue: [0] ifFalse: [1])! !

!ListView methodsFor: 'selecting' stamp: ''!
minimumSelection
	"Answer which selection is the first possible one."
	^ 1! !

!ListView methodsFor: 'selecting' stamp: ''!
moveSelectionBox: anInteger 
	"Presumably the selection has changed to be anInteger. Deselect the 
	previous selection and display the new one, highlighted."

	selection ~= anInteger
		ifTrue: 
			[self deselect.
			selection := anInteger.
			self displaySelectionBox].
	self isSelectionBoxClipped
		ifTrue: [self scrollSelectionIntoView]! !

!ListView methodsFor: 'delimiters' stamp: ''!
noBottomDelimiter
	"Set the string used to indicate the bottom of the list to be nothing."

	bottomDelimiter := nil! !

!ListView methodsFor: 'delimiters' stamp: ''!
noTopDelimiter
	"Set the string used to indicate the top of the list to be nothing."

	topDelimiter := nil! !

!ListView methodsFor: 'selecting' stamp: 'di 5/22/1998 00:25'!
numSelectionsInView
	^ self clippingBox height // self list lineGrid! !

!ListView methodsFor: 'private' stamp: ''!
positionList

	list wrappingBox: self wrappingBox clippingBox: self clippingBox ! !

!ListView methodsFor: 'list access' stamp: ''!
reset
	"Set the list of items displayed to be empty."

	isEmpty := true.
	self list: Array new! !

!ListView methodsFor: 'list access' stamp: ''!
resetAndDisplayView
	"Set the list of items displayed to be empty and redisplay the receiver."

	isEmpty
		ifFalse: 
			[self reset.
			self displayView]! !

!ListView methodsFor: 'displaying' stamp: ''!
scrollBy: anInteger 
	"Scroll up by this amount adjusted by lineSpacing and list limits"
	| maximumAmount minimumAmount amount wasClipped |
	maximumAmount := 0 max:
		list clippingRectangle top - list compositionRectangle top.
	minimumAmount := 0 min:
		list clippingRectangle bottom - list compositionRectangle bottom.
	amount := (anInteger min: maximumAmount) max: minimumAmount.
	amount ~= 0
		ifTrue: [list scrollBy: amount negated.  ^ true]
		ifFalse: [^ false]  "Return false if no scrolling took place"! !

!ListView methodsFor: 'displaying' stamp: ''!
scrollSelectionIntoView
	"Selection is assumed to be on and clipped out of view.
	Uses controller scrollView to keep selection right"
	| delta |
	(delta := self insetDisplayBox bottom - self selectionBox bottom) < 0
		ifTrue: [^ self controller scrollView: delta - (list lineGrid-1)]. "up"
	(delta := self insetDisplayBox top - self selectionBox top) > 0
		ifTrue: [^ self controller scrollView: delta + 1] "down"! !

!ListView methodsFor: 'selecting' stamp: ''!
selection
	"Answer the receiver's current selection."

	^selection! !

!ListView methodsFor: 'list access' stamp: ''!
selection: selIndex
	selection := selIndex! !

!ListView methodsFor: 'selecting' stamp: ''!
selectionBox
	"Answer the rectangle in which the current selection is displayed."

	^(self insetDisplayBox left @ (list compositionRectangle top + self selectionBoxOffset) 
		extent: self insetDisplayBox width @ list lineGrid)
		insetBy: (Rectangle left: 1 right: 1 top: 1 bottom: 0)! !

!ListView methodsFor: 'selecting' stamp: ''!
selectionBoxOffset
	"Answer an integer that determines the y position for the display box of 
	the current selection."

	^ (selection - 1 + (topDelimiter == nil ifTrue: [0] ifFalse: [1]))
		* list lineGrid! !

!ListView methodsFor: 'delimiters' stamp: ''!
topDelimiter
	"Answer the string used to indicate the top of the list."

	^topDelimiter! !

!ListView methodsFor: 'delimiters' stamp: ''!
topDelimiter: aString 
	"Set the string used to indicate the top of the list."

	topDelimiter := aString! !

!ListView methodsFor: 'updating' stamp: ''!
update: aSymbol 
	"Refer to the comment in View|update:."

	aSymbol == #list
		ifTrue: 
			[self list: model list.
			self displayView.
			^self].
	aSymbol == #listIndex
		ifTrue: 
			[self moveSelectionBox: model listIndex.
			^self]! !

!ListView methodsFor: 'private' stamp: ''!
wrappingBox

	| aRectangle |
	aRectangle := self insetDisplayBox. 
	selection = 0
		ifTrue: [^aRectangle topLeft + (4 @ 0) extent: list compositionRectangle extent]
		ifFalse: [^aRectangle left + 4 @ 
					(aRectangle top - 
						(self selectionBoxOffset 
							min: ((list height - aRectangle height 
									+ list lineGrid truncateTo: list lineGrid)
							max: 0))) 
					extent: list compositionRectangle extent]! !

ListView subclass: #PluggableListView
	instanceVariableNames: 'getListSelector getSelectionSelector setSelectionSelector getMenuSelector getMenuTitleSelector keystrokeActionSelector autoDeselect items'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Pluggable Views'!

!PluggableListView commentStamp: '<historical>' prior: 0!
A pluggable list view gets its content from the model. This allows the same kind of view to be used in different situations, thus avoiding a proliferation of gratuitous view and controller classes. Selector usage is:

		getListSel		fetch the list of items (strings) to be displayed
		getSelectionSel	get the currently selected item
		setSelectionSel	set the currently selected item (takes an argument)
		getMenuSel		get the pane-specific, 'yellow-button' menu
		keyActionSel	process a keystroke typed in this pane (takes an argument)

	Any of the above selectors can be nil, meaning that the model does not supply behavior for the given action, and the default behavior should be used. However, if getListSel is nil, the default behavior just provides an empty list, which makes for a rather dull list view!!

	The model informs a pluggable view of changes by sending #changed: to itself with getListSel or getSelectionSel as a parameter. The view informs the model of selection changes by sending setSelectionSel to it with the newly selected item as a parameter, and invokes menu and keyboard actions on the model via getMenuSel and keyActionSel.

	Pluggability allows a single model object to have pluggable list views on multiple aspects of itself. For example, an object representing one personal music library might be organized as a three-level hierarchy: the types of music, the titles within a given type, and the songs on a given title. Pluggability allows one to easily build a multipane browser for this object with separate list views for the music type, title, and song.

	AutoDeselect is a feature, normally set to true, that will tell the model that there is no selection if you click on an item that is currently selected.  If autoDeselect is false, then the model will simply be told to select the same item again.!

!PluggableListView classSide methodsFor: 'instance creation' stamp: 'md 7/13/2005 16:34'!
on: anObject list: getListSel selected: getSelectionSel changeSelected: setSelectionSel
	"Create a 'pluggable' list view on the given model parameterized by the given message selectors."

	^ self new
		on: anObject
		list: getListSel
		selected: getSelectionSel
		changeSelected: setSelectionSel
		menu: nil
		keystroke: #arrowKey:from:		"default"
! !

!PluggableListView classSide methodsFor: 'instance creation' stamp: 'md 7/13/2005 16:34'!
on: anObject list: getListSel selected: getSelectionSel changeSelected: setSelectionSel menu: getMenuSel
	"Create a 'pluggable' list view on the given model parameterized by the given message selectors."

	^ self new
		on: anObject
		list: getListSel
		selected: getSelectionSel
		changeSelected: setSelectionSel
		menu: getMenuSel
		keystroke: #arrowKey:from:		"default"

! !

!PluggableListView classSide methodsFor: 'instance creation' stamp: 'md 7/13/2005 16:33'!
on: anObject list: getListSel selected: getSelectionSel changeSelected: setSelectionSel menu: getMenuSel keystroke: keyActionSel
	"Create a 'pluggable' list view on the given model parameterized by the given message selectors."

	^ self new
		on: anObject
		list: getListSel
		selected: getSelectionSel
		changeSelected: setSelectionSel
		menu: getMenuSel
		keystroke: keyActionSel
! !

!PluggableListView methodsFor: 'initialization' stamp: 'di 4/10/98 09:56'!
autoDeselect: trueOrFalse
	"Enable/disable autoDeselect (see class comment)"
	autoDeselect := trueOrFalse.! !

!PluggableListView methodsFor: 'model access' stamp: 'di 5/6/1998 20:52'!
changeModelSelection: anInteger
	"Change the model's selected item index to be anInteger."
	| newIndex |
	newIndex := anInteger.
	(autoDeselect == nil or: [autoDeselect]) ifTrue:
		[getSelectionSelector ifNotNil:
			[(model perform: getSelectionSelector) = anInteger ifTrue:
				["Click on existing selection deselects"
				newIndex := 0]]].

	setSelectionSelector ifNotNil:
		[model perform: setSelectionSelector with: newIndex].! !

!PluggableListView methodsFor: 'controller access' stamp: 'jm 3/11/98 17:17'!
defaultControllerClass 

	^ PluggableListController
! !

!PluggableListView methodsFor: 'initialization' stamp: 'jm 9/20/1998 19:48'!
font: aFontOrNil

	super font: aFontOrNil.
	self list: self getList.  "update display"
! !

!PluggableListView methodsFor: 'model access' stamp: 'di 5/6/1998 20:52'!
getCurrentSelectionIndex
	"Answer the index of the current selection."

	getSelectionSelector == nil ifTrue: [^ 0].
	^ model perform: getSelectionSelector! !

!PluggableListView methodsFor: 'model access' stamp: ''!
getList 
	"Answer the list to be displayed."

	| lst |
	getListSelector == nil ifTrue: [^ #()].
	lst := model perform: getListSelector.
	lst == nil ifTrue: [^ #()].
	^ lst! !

!PluggableListView methodsFor: 'model access' stamp: 'tk 4/2/98 13:36'!
getListSelector
	^ getListSelector! !

!PluggableListView methodsFor: 'model access' stamp: 'sw 8/18/1998 12:07'!
getMenu: shiftKeyDown
	"Answer the menu for this text view, supplying an empty menu to be filled in. If the menu selector takes an extra argument, pass in the current state of the shift key."
	| menu aMenu |
	getMenuSelector == nil ifTrue: [^ nil].
	menu := CustomMenu new.
	getMenuSelector numArgs = 1
		ifTrue:
			[aMenu := model perform: getMenuSelector with: menu.
			getMenuTitleSelector ifNotNil: [aMenu title: (model perform: getMenuTitleSelector)].
			^ aMenu].
	getMenuSelector numArgs = 2
		ifTrue: [aMenu := model perform: getMenuSelector with: menu with: shiftKeyDown.
				getMenuTitleSelector ifNotNil: [aMenu title: (model perform: getMenuTitleSelector)].
				^ aMenu].
	^ self error: 'The getMenuSelector must be a 1- or 2-keyword symbol'! !

!PluggableListView methodsFor: 'model access' stamp: 'nk 6/29/2004 14:45'!
handleKeystroke: aChar
	"Answer the menu for this list view."

	| args aSpecialKey |

	aSpecialKey := aChar asciiValue.
	aSpecialKey < 32 ifTrue: [ self specialKeyPressed: aSpecialKey. ^nil ].
	keystrokeActionSelector ifNil: [^ nil].

	controller controlTerminate.
	(args := keystrokeActionSelector numArgs) = 1
		ifTrue: [model perform: keystrokeActionSelector with: aChar.
				^ controller controlInitialize].
	args = 2
		ifTrue: [model perform: keystrokeActionSelector with: aChar with: self.
				^ controller controlInitialize].
	^ self error: 'The keystrokeActionSelector must be a 1- or 2-keyword symbol'! !

!PluggableListView methodsFor: 'initialization' stamp: 'di 6/20/2001 09:58'!
list: arrayOfStrings
	"Set the receivers items to be the given list of strings
	The instance variable 'items' holds the original list. The instance variable 'list' is a paragraph constructed from this list."

	((items == arrayOfStrings) "fastest" or: [items = arrayOfStrings]) ifTrue: [^ self].
	items := arrayOfStrings.
	isEmpty := arrayOfStrings isEmpty.

	"add top and bottom delimiters"
	list := ListParagraph
		withArray:
			(Array streamContents: [:s |
				s nextPut: topDelimiter.
				arrayOfStrings do:
					[:item | item == nil ifFalse:
						[(item isMemberOf: MethodReference)  "A very specific fix for MVC"
							ifTrue: [s nextPut: item asStringOrText]
							ifFalse: [s nextPut: item]]].
				s nextPut: bottomDelimiter])
		 style: self assuredTextStyle.

	selection := self getCurrentSelectionIndex.
	self positionList.! !

!PluggableListView methodsFor: 'initialization' stamp: 'tk 4/12/1998 08:25'!
menu: getMenuSel

	getMenuSelector := getMenuSel! !

!PluggableListView methodsFor: 'initialization' stamp: 'sw 8/18/1998 12:04'!
menuTitleSelector: getMenuTitleSel
	getMenuTitleSelector := getMenuTitleSel! !

!PluggableListView methodsFor: 'initialization' stamp: 'di 4/10/98 09:55'!
on: anObject list: getListSel selected: getSelectionSel changeSelected: setSelectionSel menu: getMenuSel keystroke: keyActionSel

	self model: anObject.
	getListSelector := getListSel.
	getSelectionSelector := getSelectionSel.
	setSelectionSelector := setSelectionSel.
	getMenuSelector := getMenuSel.
	keystrokeActionSelector := keyActionSel.
	autoDeselect := true.
	self borderWidth: 1.
	self list: self getList.! !

!PluggableListView methodsFor: 'model access' stamp: 'sw 10/9/1998 08:24'!
setSelectionSelectorIs: aSelector
	^ aSelector == setSelectionSelector! !

!PluggableListView methodsFor: 'model access' stamp: 'nk 6/29/2004 14:42'!
specialKeyPressed: keyEvent
	"Process the up and down arrows in a list pane."
     | oldSelection nextSelection max min howMany |

	(#(1 4 11 12 30 31) includes: keyEvent) ifFalse: [ ^ false ].

     oldSelection := self getCurrentSelectionIndex.
     nextSelection := oldSelection.
     max := self maximumSelection.
     min := self minimumSelection.
     howMany := self numSelectionsInView.	"get this exactly??"

     keyEvent == 31 ifTrue:
		["down-arrow; move down one, wrapping to top if needed"
		nextSelection := oldSelection + 1.
		nextSelection > max ifTrue: [nextSelection := 1]].

     keyEvent == 30 ifTrue:
		["up arrow; move up one, wrapping to bottom if needed"
		nextSelection := oldSelection - 1.
		nextSelection < 1 ifTrue: [nextSelection := max]].

     keyEvent == 1  ifTrue: [nextSelection := 1].  "home"
     keyEvent == 4  ifTrue: [nextSelection := max].   "end"
     keyEvent == 11 ifTrue: [nextSelection := min max: (oldSelection - howMany)].  "page up"
     keyEvent == 12  ifTrue: [nextSelection := (oldSelection + howMany) min: max].  "page down"
     nextSelection = oldSelection  ifFalse:
		[model okToChange
			ifTrue:
				[self changeModelSelection: nextSelection.
				"self controller moveMarker"]].
	
	^true
			! !

!PluggableListView methodsFor: 'updating' stamp: 'di 5/25/1998 10:24'!
update: aSymbol 
	"Refer to the comment in View|update:."
	aSymbol == getListSelector ifTrue:
		[self list: self getList.
		self displayView.
		self displaySelectionBox.
		^self].
	aSymbol == getSelectionSelector ifTrue:
		[^ self moveSelectionBox: self getCurrentSelectionIndex].
! !

!PluggableListView methodsFor: 'updating' stamp: 'BG 1/22/2004 13:15'!
verifyContents
	| newItems existingSelection anIndex |
	"Called on window reactivation to react to possible structural changes.  Update contents if necessary."

	newItems := self getList.
	((items == newItems) "fastest" or: [items = newItems]) ifTrue: [^ self].
	self flash.  "list has changed beneath us; could get annoying, but hell"
	existingSelection := list stringAtLineNumber: (selection + (topDelimiter ifNil: [0] ifNotNil: [1])).  "account for cursed ------ row"
	self list: newItems.

	(newItems size > 0 and: [newItems first isKindOf: Symbol]) ifTrue:
		[existingSelection := existingSelection asSymbol].
	(anIndex := newItems indexOf: existingSelection ifAbsent: [nil])
		ifNotNil:
			[model noteSelectionIndex: anIndex for: getListSelector.]
		ifNil:
			[self changeModelSelection: 0].
	selection := 0. " to display the list without selection "
	self displayView.
	self update: getSelectionSelector.
! !

PluggableListView subclass: #PluggableListViewByItem
	instanceVariableNames: 'itemList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Pluggable Views'!

!PluggableListViewByItem methodsFor: 'as yet unclassified' stamp: 'di 5/6/1998 16:28'!
changeModelSelection: anInteger
	"Change the model's selected item to be the one at the given index."
	| item |
	setSelectionSelector ifNotNil: [
		item := (anInteger = 0 ifTrue: [nil] ifFalse: [itemList at: anInteger]).
		model perform: setSelectionSelector with: item].
! !

!PluggableListViewByItem methodsFor: 'as yet unclassified' stamp: 'di 5/6/1998 16:27'!
getCurrentSelectionIndex
	"Answer the index of the current selection."
	| item |
	getSelectionSelector == nil ifTrue: [^ 0].
	item := model perform: getSelectionSelector.
	^ itemList findFirst: [ :x | x = item]
! !

!PluggableListViewByItem methodsFor: 'as yet unclassified' stamp: 'sw 12/9/1999 18:07'!
list: arrayOfStrings
	"Set the receivers items to be the given list of strings."
	"Note: the instance variable 'items' holds the original list.
	 The instance variable 'list' is a paragraph constructed from
	 this list."

	itemList := arrayOfStrings.
	isEmpty := arrayOfStrings isEmpty.

	"add top and bottom delimiters"
	list := ListParagraph
		withArray:
			(Array streamContents: [:s |
				s nextPut: topDelimiter.
				arrayOfStrings do: [:item | item == nil ifFalse: [s nextPut: item]].
				s nextPut: bottomDelimiter])
		 style: self assuredTextStyle.

	selection := self getCurrentSelectionIndex.
	self positionList.! !

PluggableListView subclass: #PluggableListViewOfMany
	instanceVariableNames: 'getSelectionListSelector setSelectionListSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Pluggable Views'!

!PluggableListViewOfMany classSide methodsFor: 'instance creation' stamp: 'tpr 10/8/2001 20:53'!
on: anObject list: listSel primarySelection: getSelectionSel changePrimarySelection: setSelectionSel listSelection: getListSel changeListSelection: setListSel menu: getMenuSel
	^ self new
		on: anObject
		list: listSel
		primarySelection: getSelectionSel
		changePrimarySelection: setSelectionSel
		listSelection: getListSel
		changeListSelection: setListSel
		menu: getMenuSel
		keystroke: #arrowKey:from:		"default"! !

!PluggableListViewOfMany classSide methodsFor: 'instance creation' stamp: 'tpr 10/8/2001 20:52'!
on: anObject list: listSel primarySelection: getSelectionSel changePrimarySelection: setSelectionSel listSelection: getListSel changeListSelection: setListSel menu: getMenuSel keystroke: keyActionSel 
	^ self new
		on: anObject
		list: listSel
		primarySelection: getSelectionSel
		changePrimarySelection: setSelectionSel
		listSelection: getListSel
		changeListSelection: setListSel
		menu: getMenuSel
		keystroke: keyActionSel! !

!PluggableListViewOfMany methodsFor: 'displaying' stamp: 'tpr 10/4/2001 21:34'!
deEmphasizeView 
	"Refer to the comment in View|deEmphasizeView."
	selection := 0.
	1 to: self maximumSelection do:
		[:i | selection := i.
		(self listSelectionAt: i) ifTrue: [self deEmphasizeSelectionBox]].
	selection := 0! !

!PluggableListViewOfMany methodsFor: 'controller access' stamp: 'di 5/17/1998 22:48'!
defaultControllerClass 

	^ PluggableListControllerOfMany
! !

!PluggableListViewOfMany methodsFor: 'displaying' stamp: 'tpr 10/4/2001 21:34'!
highlightFrom: start to: stop
	(start == nil or: [stop == nil]) ifTrue: [^ self displayView].
	start to: stop do:
		[:i | selection := i.
		(self listSelectionAt: selection) ifTrue: [self displaySelectionBox]].
	selection := 0! !

!PluggableListViewOfMany methodsFor: 'selecting' stamp: 'tpr 10/4/2001 22:17'!
listSelectionAt: index
	getSelectionListSelector ifNil:[^false].
	^model perform: getSelectionListSelector with: index! !

!PluggableListViewOfMany methodsFor: 'selecting' stamp: 'tpr 10/4/2001 22:17'!
listSelectionAt: index put: value
	setSelectionListSelector ifNil:[^false].
	^model perform: setSelectionListSelector with: index with: value! !

!PluggableListViewOfMany methodsFor: 'selecting' stamp: 'tk 4/6/98 15:43'!
moveSelectionBox: anInteger 
	"Presumably the selection has changed to be anInteger. Deselect the 
	previous selection and display the new one, highlighted."
	selection ~= anInteger
		ifTrue: 
			[selection := anInteger.
			self displaySelectionBox]! !

!PluggableListViewOfMany methodsFor: 'initialization' stamp: 'tpr 10/8/2001 20:53'!
on: anObject list: listSel primarySelection: getSelectionSel changePrimarySelection: setSelectionSel listSelection: getListSel changeListSelection: setListSel menu: getMenuSel keystroke: keyActionSel
	"setup a whole load of pluggability options"
	getSelectionListSelector := getListSel.
	setSelectionListSelector := setListSel.
	super on: anObject list: listSel selected: getSelectionSel changeSelected: setSelectionSel menu: getMenuSel keystroke: keyActionSel
! !

!PluggableListViewOfMany methodsFor: 'displaying' stamp: 'tk 4/6/98 20:42'!
scrollBy: anInteger
	"This is a possible way to intercept what ListOfManyController did to get multiple selections to show.  Feel to replace this."

	| ans |
	ans := super scrollBy: anInteger.
"	self displaySelectionBox."
	^ ans! !

!PluggableListViewOfMany methodsFor: 'selecting' stamp: 'tk 4/6/98 15:43'!
selection
	"Have to override normal controller smarts about deselection"
	^ 0! !

!PluggableListViewOfMany methodsFor: 'updating' stamp: 'tk 4/8/98 13:12'!
update: aSymbol 
	aSymbol == getListSelector
		ifTrue: [self list: self getList.
			^ self displayView; emphasizeView].
	aSymbol == getSelectionSelector
		ifTrue: [^ self displayView; emphasizeView].
	aSymbol == #allSelections
		ifTrue: [^ self displayView; emphasizeView].
	^ super update: aSymbol! !

View subclass: #PluggableButtonView
	instanceVariableNames: 'label getStateSelector actionSelector getLabelSelector getMenuSelector shortcutCharacter askBeforeChanging triggerOnMouseDown complemented argumentsProvider argumentsSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Pluggable Views'!

!PluggableButtonView commentStamp: '<historical>' prior: 0!
A PluggableButtonView is a combination of an indicator for a boolean value stored in its model and an action button. The action of a button is often, but not always, to toggle the boolean value that it shows. Its pluggable selectors are:

		getStateSelector		fetch a boolean value from the model
		actionSelector		invoke this button's action on the model

Either of the above selectors can be nil, meaning that the model does not supply behavior for the given action, and the default behavior should be used. For example, if getStateSelector is nil, then this button shows the state of a read-only boolean that is always false.

The model informs a pluggable view of changes by sending #changed: to itself with getStateSelector as a parameter. The view tells the model when the button is pressed by sending actionSelector.

Options:
	askBeforeChanging		have model ask user before allowing change that could lose edits
	triggerOnMouseDown	do button action on mouse down (vs. up) transition
	shortcutCharacter		a place to record an optional shortcut key

!

!PluggableButtonView classSide methodsFor: 'example' stamp: 'jm 4/7/98 19:55'!
example
	"PluggableButtonView example"

	| s1 s2 s3 b1 b2 b3 topView |
	s1 := Switch new.
	s2 := Switch new turnOn.
	s3 := Switch new.
	s2 onAction: [s3 turnOff].
	s3 onAction: [s2 turnOff].
	b1 := (PluggableButtonView on: s1 getState: #isOn action: #switch) label: 'S1'.
	b2 := (PluggableButtonView on: s2 getState: #isOn action: #turnOn) label: 'S2'.
	b3 := (PluggableButtonView on: s3 getState: #isOn action: #turnOn) label: 'S3'.
	b1 borderWidth: 1.
	b2 borderWidth: 1.
	b3 borderWidth: 1.
	topView := StandardSystemView new
		label: 'Switch Test';
		addSubView: b1;
		addSubView: b2 toRightOf: b1;
		addSubView: b3 toRightOf: b2.
	topView controller open.
! !

!PluggableButtonView classSide methodsFor: 'instance creation' stamp: 'jm 8/14/1998 16:19'!
on: anObject

	^ self on: anObject getState: #isOn action: #switch
! !

!PluggableButtonView classSide methodsFor: 'instance creation' stamp: 'tk 4/29/1998 11:19'!
on: anObject getState: getStateSel action: actionSel

	^ self new
		on: anObject
		getState: getStateSel
		action: actionSel
		label: nil
		menu: nil! !

!PluggableButtonView classSide methodsFor: 'instance creation' stamp: 'sumim 2/15/2002 17:18'!
on: anObject getState: getStateSel action: actionSel getArguments: getArgumentsSel from: argsProvidor

	^ self new
		on: anObject
		getState: getStateSel
		action: actionSel
		getArguments: getArgumentsSel
		from: argsProvidor
		label: nil
		menu: nil! !

!PluggableButtonView classSide methodsFor: 'instance creation' stamp: 'tk 4/29/1998 11:19'!
on: anObject getState: getStateSel action: actionSel label: labelSel

	^ self new
		on: anObject
		getState: getStateSel
		action: actionSel
		label: labelSel
		menu: nil! !

!PluggableButtonView classSide methodsFor: 'instance creation' stamp: 'tk 4/29/1998 11:18'!
on: anObject getState: getStateSel action: actionSel label: labelSel menu: menuSel

	^ self new
		on: anObject
		getState: getStateSel
		action: actionSel
		label: labelSel
		menu: menuSel! !

!PluggableButtonView methodsFor: 'accessing' stamp: 'jrm 6/1/1998 21:53'!
action: aSymbol 
	"Set actionSelector to be the action defined by aSymbol."

	actionSelector := aSymbol
! !

!PluggableButtonView methodsFor: 'accessing' stamp: 'jm 4/2/98 18:53'!
askBeforeChanging

	^ askBeforeChanging
! !

!PluggableButtonView methodsFor: 'accessing' stamp: 'jm 4/7/98 19:17'!
askBeforeChanging: aBoolean
	"If this preference is turned on, then give the model an opportunity to ask the user before accepting a change that might cause unaccepted edits to be lost."

	askBeforeChanging := aBoolean.
! !

!PluggableButtonView methodsFor: 'private' stamp: 'nk 4/17/2004 19:49'!
centerAlignLabelWith: aPoint
	"Align the center of the label with aPoint."

	| alignPt |
	alignPt := label boundingBox center.
	(label isKindOf: Paragraph) ifTrue: 
		[alignPt := alignPt + (0@(label textStyle leading))]. 
	(label isForm)
	  ifTrue: [label offset: 0 @ 0].
	label align: alignPt with: aPoint
! !

!PluggableButtonView methodsFor: 'private' stamp: 'acg 2/23/2000 00:10'!
centerLabel
	"If there is a label, align its center with the center of the insetDisplayBox"

	label ifNotNil: 
		[self centerAlignLabelWith: self insetDisplayBox center].
! !

!PluggableButtonView methodsFor: 'displaying' stamp: 'jm 4/2/98 18:49'!
deEmphasizeView 

	self getModelState ifTrue: [self displayNormal].
! !

!PluggableButtonView methodsFor: 'controller access' stamp: 'jm 4/2/98 17:39'!
defaultControllerClass 

	^ PluggableButtonController
! !

!PluggableButtonView methodsFor: 'other' stamp: 'jm 4/7/98 19:12'!
defaultWindow
	"Return a rectangle large enough to contain this button's label. If this button is label-less, just return the standard View default window."

	label == nil
		ifTrue: [^ super defaultWindow]
		ifFalse: [^ label boundingBox expandBy: 6].
! !

!PluggableButtonView methodsFor: 'displaying' stamp: ''!
display
	"Sets the PluggableButtonView mode to 'normal', displays the border, displays the inside and, if its model is 'on', complements the inside."

	self displayBorder.
	self displayView.
! !

!PluggableButtonView methodsFor: 'displaying' stamp: 'jm 4/4/98 20:49'!
displayComplemented
	"Complement the receiver if it isn't already."

	complemented ifFalse: [
		complemented := true.
		Display reverse: self insetDisplayBox].
! !

!PluggableButtonView methodsFor: 'displaying' stamp: 'jm 4/2/98 18:43'!
displayNormal
	"Complement the receiver if its mode is 'complemented'."

	complemented ifTrue: [
		complemented := false.
		Display reverse: self insetDisplayBox].
! !

!PluggableButtonView methodsFor: 'displaying' stamp: 'acg 2/23/2000 00:18'!
displayView

	"Displays this switch and its label, if any."

	self clearInside.
	label ifNotNil: [
		(label isKindOf: Paragraph) ifTrue: [
			label foregroundColor: self foregroundColor
				 backgroundColor: self backgroundColor].
		label displayOn: Display
				at: label boundingBox topLeft
				clippingBox: self insetDisplayBox].
	complemented := false.! !

!PluggableButtonView methodsFor: 'displaying' stamp: 'jm 4/4/98 20:41'!
emphasizeView 

	self getModelState ifTrue: [self displayComplemented].
! !

!PluggableButtonView methodsFor: 'private' stamp: 'di 6/26/1998 11:04'!
getMenu: shiftKeyDown
	"Answer the menu for this view, supplying an empty menu to be filled in. If the menu selector takes an extra argument, pass in the current state of the shift key."

	| menu |
	getMenuSelector == nil ifTrue: [^ nil].
	menu := CustomMenu new.
	getMenuSelector numArgs = 1
		ifTrue: [^ model perform: getMenuSelector with: menu].
	getMenuSelector numArgs = 2
		ifTrue: [^ model perform: getMenuSelector with: menu with: shiftKeyDown].
	^ self error: 'The getMenuSelector must be a 1- or 2-keyword symbol'
! !

!PluggableButtonView methodsFor: 'private' stamp: 'jm 4/4/98 20:50'!
getModelState
	"Answer the result of sending the receiver's model the getStateSelector message."

	getStateSelector == nil
		ifTrue: [^ false]
		ifFalse: [^ model perform: getStateSelector].
! !

!PluggableButtonView methodsFor: 'initialize-release' stamp: 'jm 5/2/1998 15:20'!
initialize

 	super initialize.
	label := nil.
	getStateSelector := nil.
	actionSelector := nil.
	getLabelSelector := nil.
	getMenuSelector := nil.
	shortcutCharacter := nil.
	askBeforeChanging := false.
	triggerOnMouseDown := false.
	complemented := false.
! !

!PluggableButtonView methodsFor: 'private' stamp: 'acg 2/23/2000 00:09'!
insetDisplayBox
	"Answer the receiver's inset display box. The inset display box is the 
	intersection of the receiver's window, tranformed to display coordinates, 
	and the inset display box of the superView, inset by the border width. 
	The inset display box represents the region of the display screen in 
	which the inside of the receiver (all except the border) is displayed. If 
	the receiver is totally clipped by the display screen and its superView, 
	the resulting Rectangle will be invalid."

	insetDisplayBox ifNil: 
		[insetDisplayBox := self computeInsetDisplayBox.
		 self centerLabel].
	^insetDisplayBox! !

!PluggableButtonView methodsFor: 'accessing' stamp: 'jm 4/2/98 17:38'!
label
	"Answer the DisplayObject used as this button's label."

	^ label
! !

!PluggableButtonView methodsFor: 'accessing' stamp: 'nk 4/17/2004 19:49'!
label: aStringOrDisplayObject 
	"Label this button with the given String or DisplayObject."

	((aStringOrDisplayObject isKindOf: Paragraph)
	or: [aStringOrDisplayObject isForm])
		ifTrue: [label := aStringOrDisplayObject]
		ifFalse: [label := aStringOrDisplayObject asParagraph].
	self centerLabel.
! !

!PluggableButtonView methodsFor: 'initialize-release' stamp: 'sw 2/17/2002 05:32'!
on: anObject getState: getStateSel action: actionSel getArguments: getArgumentsSel from: argsProvidor label: labelSel menu: menuSel

	self initialize.
	self model: anObject.
	getStateSelector := getStateSel.
	actionSelector := actionSel.
	argumentsSelector := getArgumentsSel.
	argumentsProvider := argsProvidor.
	getLabelSelector := labelSel.
	getMenuSelector := menuSel! !

!PluggableButtonView methodsFor: 'initialize-release' stamp: 'tk 4/29/1998 11:18'!
on: anObject getState: getStateSel action: actionSel label: labelSel menu: menuSel

	self initialize.
	self model: anObject.
	getStateSelector := getStateSel.
	actionSelector := actionSel.
	getLabelSelector := labelSel.
	getMenuSelector := menuSel.! !

!PluggableButtonView methodsFor: 'other' stamp: 'sw 2/17/2002 05:32'!
performAction
	"Inform the model that this button has been pressed. Sent by the controller when this button is pressed."

	argumentsSelector
		ifNil:
			[actionSelector ifNotNil:
				[model perform: actionSelector]]
		ifNotNil:
			[model perform: actionSelector
				withArguments:
					(Array with: (argumentsProvider perform: argumentsSelector))]! !

!PluggableButtonView methodsFor: 'accessing' stamp: 'jm 4/2/98 17:43'!
shortcutCharacter
	"Return the Character to be used as a shortcut to turn on this switch, or nil if this switch doesn't have a keyboard shortcut."

	^ shortcutCharacter
! !

!PluggableButtonView methodsFor: 'accessing' stamp: 'jm 4/2/98 17:43'!
shortcutCharacter: aCharacter 
	"Set the character to be used as a keyboard shortcut for turning on this switch."

	shortcutCharacter := aCharacter.
! !

!PluggableButtonView methodsFor: 'displaying' stamp: 'jm 4/5/98 12:54'!
toggleMouseOverFeedback
	"Complement the label (or a portion of the displayBox if no label is defined) to show that the mouse is over this button. This feedback can be removed by a second call to this method."

	Display reverse: self insetDisplayBox fillColor: Color gray.
	Display reverse: (self insetDisplayBox insetBy: 2) fillColor: Color gray.
! !

!PluggableButtonView methodsFor: 'accessing' stamp: 'jm 4/2/98 19:26'!
triggerOnMouseDown

	^ triggerOnMouseDown
! !

!PluggableButtonView methodsFor: 'accessing' stamp: 'jm 4/7/98 19:16'!
triggerOnMouseDown: aBoolean
	"If this preference is turned on, then trigger my action immediately when the mouse goes down."

	triggerOnMouseDown := aBoolean.
! !

!PluggableButtonView methodsFor: 'other' stamp: 'tk 4/27/1998 21:23'!
update: aParameter 

	aParameter == getLabelSelector ifTrue: [
		getLabelSelector ifNotNil: [
			self label: (model perform: getLabelSelector).
			self displayView]].
	self getModelState 
		ifTrue: [self displayComplemented]
		ifFalse: [self displayNormal].
! !

!PluggableButtonView methodsFor: 'other' stamp: 'jm 4/7/98 19:12'!
window: aWindow
	"Center my label when my window changes."

	super window: aWindow.
	self centerLabel.
! !

View subclass: #StandardSystemView
	instanceVariableNames: 'labelFrame labelText isLabelComplemented savedSubViews minimumSize maximumSize collapsedViewport expandedViewport labelBits windowBits bitsValid updatablePanes'
	classVariableNames: 'CacheBits LabelStyle'
	poolDictionaries: ''
	category: 'ST80-Support'!

!StandardSystemView commentStamp: '<historical>' prior: 0!
I represent a view that has a label above its top left corner. The text in the label identifies the kind of view. In addition to a label, I add control over the maximum and minimum size of the display box of my instance. My default controller is StandardSystemController. The elements of ScheduledControllers, the sole instance of ControlManager, are usually controllers for instances of me.!

StandardSystemView subclass: #ColorSystemView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Support'!

!ColorSystemView methodsFor: 'as yet unclassified' stamp: ''!
cacheBitsAsTwoTone
	^ false! !

!ColorSystemView methodsFor: 'as yet unclassified' stamp: 'di 2/26/98 08:58'!
displayDeEmphasized 
	"Display this view with emphasis off.
	If windowBits is not nil, then simply BLT if possible."
	bitsValid
		ifTrue: [self lock.
				windowBits displayAt: self windowOrigin]
		ifFalse: [super displayDeEmphasized]
! !

StandardSystemView subclass: #FormEditorView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Editors'!

!FormEditorView methodsFor: 'as yet unclassified' stamp: ''!
cacheBitsAsTwoTone
	^ false! !

StandardSystemView subclass: #ProjectView
	instanceVariableNames: ''
	classVariableNames: 'ArmsLengthCmd'
	poolDictionaries: ''
	category: 'ST80-Support'!

!ProjectView classSide methodsFor: 'as yet unclassified' stamp: ''!
open: aProject 
	"Answer an instance of me for the argument, aProject. It is created on the
	display screen."
	| topView |
	topView := self new model: aProject.
	topView minimumSize: 50 @ 30.
	topView borderWidth: 2.
	topView controller open! !

!ProjectView classSide methodsFor: 'as yet unclassified' stamp: 'RAA 7/25/2000 10:35'!
openAndEnter: aProject 
	"Answer an instance of me for the argument, aProject. It is created on 
	the display screen."
	| topView |
	topView := self new model: aProject.
	topView minimumSize: 50 @ 30.
	topView borderWidth: 2.
	topView window: (RealEstateAgent initialFrameFor: topView world: nil).
	ScheduledControllers schedulePassive: topView controller.
	aProject
		enter: false
		revert: false
		saveForRevert: false! !

!ProjectView methodsFor: 'displaying' stamp: 'tk 4/19/2000 17:15'!
armsLengthCommand: aCommand
	"Set up a save to be done after the subproject exits to here.  displayOn: checks it."

	ArmsLengthCmd := aCommand.! !

!ProjectView methodsFor: 'displaying' stamp: ''!
cacheBitsAsTwoTone
	^ false! !

!ProjectView methodsFor: 'initialization' stamp: ''!
defaultControllerClass
	^ ProjectController! !

!ProjectView methodsFor: 'displaying' stamp: 'tk 6/21/2000 16:22'!
displayDeEmphasized
	| cmd |
	"Display this view with emphasis off.  Check for a command that
could not be executed in my subproject.  Once it is done, remove the
trigger."

	super displayDeEmphasized.
	ArmsLengthCmd ifNil: [^ self].
	ArmsLengthCmd first == model ifFalse: [^ self].	"not ours"
	cmd := ArmsLengthCmd second.
	ArmsLengthCmd := nil.
	model "project" perform: cmd.
	model "project" enter.
! !

!ProjectView methodsFor: 'displaying' stamp: 'sw 1/11/2000 15:32'!
displayView
	super displayView.
	self label = model name
		ifFalse: [self setLabelTo: model name].
	self isCollapsed ifTrue: [^ self].
	model thumbnail ifNil: [^ self].
	self insetDisplayBox extent = model thumbnail extent
		ifTrue: [model thumbnail displayAt: self insetDisplayBox topLeft]
		ifFalse: [(model thumbnail
					magnify: model thumbnail boundingBox
					by: self insetDisplayBox extent asFloatPoint / model thumbnail extent) 				displayAt: self insetDisplayBox topLeft]
! !

!ProjectView methodsFor: 'initialization' stamp: 'ar 9/27/2005 20:10'!
maybeRelabel: newLabel
	"If the change set name given by newLabel is already in use, do nothing; else relabel the view"

	(newLabel isEmpty or: [newLabel = self label])
		ifTrue: [^ self].
	(ChangeSet named: newLabel) == nil
		ifFalse: [^ self].
	self relabel: newLabel! !

!ProjectView methodsFor: 'initialization' stamp: 'ar 9/27/2005 20:10'!
relabel: newLabel
	(newLabel isEmpty or: [newLabel = self label])
		ifTrue: [^ self].
	(ChangeSet named: newLabel) == nil
		ifFalse: [self inform: 'Sorry that name is already used'.
				^ self].
	model projectChangeSet name: newLabel.
	super relabel: newLabel! !

!ProjectView methodsFor: 'displaying' stamp: 'sw 9/14/1998 13:01'!
release
	super release.
	Smalltalk garbageCollect! !

!ProjectView methodsFor: 'initialization' stamp: 'sw 1/11/2000 15:30'!
uncacheBits
	super uncacheBits.
	self label ~=  model name ifTrue: [self setLabelTo: model name]! !

!StandardSystemView classSide methodsFor: 'class initialization' stamp: ''!
cachingBits
	^ CacheBits! !

!StandardSystemView classSide methodsFor: 'class initialization' stamp: ''!
doCacheBits
	"StandardSystemView doCacheBits - Enable fast window repaint feature"
	CacheBits := true.
	ScheduledControllers unCacheWindows.
	ScheduledControllers restore! !

!StandardSystemView classSide methodsFor: 'class initialization' stamp: ''!
dontCacheBits
	"StandardSystemView dontCacheBits - Disable fast window repaint feature.
	Return true iff bits were cached, ie if space was been recovered"
	CacheBits ifFalse: [^ false].
	CacheBits := false.
	ScheduledControllers unCacheWindows.
	^ true! !

!StandardSystemView classSide methodsFor: 'class initialization' stamp: 'sw 12/6/1999 23:42'!
initialize		"StandardSystemView initialize"
	self doCacheBits.
	self setLabelStyle! !

!StandardSystemView classSide methodsFor: 'class initialization' stamp: 'nk 9/1/2004 10:26'!
setLabelStyle
	| aFont |
	"StandardSystemView setLabelStyle"
	aFont := Preferences windowTitleFont.
	LabelStyle := TextStyle fontArray: { aFont }.
	LabelStyle gridForFont: 1 withLead: 0! !

!StandardSystemView methodsFor: 'displaying' stamp: ''!
cacheBits
	| oldLabelState |
	CacheBits ifFalse: [^ self uncacheBits].
	(oldLabelState := isLabelComplemented) ifTrue: [ self deEmphasize ].
	self cacheBitsAsIs.
	(isLabelComplemented := oldLabelState) ifTrue: [ self emphasize ].
! !

!StandardSystemView methodsFor: 'displaying' stamp: ''!
cacheBitsAsIs
	CacheBits ifFalse: [^ self uncacheBits].
	windowBits := (self cacheBitsAsTwoTone and: [Display depth > 1])
		ifTrue: [ColorForm
					twoToneFromDisplay: self windowBox
					using: windowBits
					backgroundColor: self backgroundColor]
		ifFalse: [Form fromDisplay: self windowBox using: windowBits].
	bitsValid := true.
! !

!StandardSystemView methodsFor: 'displaying' stamp: ''!
cacheBitsAsTwoTone
	^ true! !

!StandardSystemView methodsFor: 'framing' stamp: 'sr 3/26/2000 03:47'!
chooseCollapsePoint
	"Answer the point at which to place the collapsed window."
	| pt labelForm beenDown offset |
	labelForm := Form fromDisplay: self labelDisplayBox.
	self uncacheBits.
	self erase.
	beenDown := Sensor anyButtonPressed.
	self isCollapsed ifTrue:
		[offset := self labelDisplayBox topLeft - self growBoxFrame topLeft.
		labelForm follow: [pt := (Sensor cursorPoint + offset max: 0@0) truncateTo: 8]
				while: [Sensor anyButtonPressed
							ifTrue: [beenDown := true]
							ifFalse: [beenDown not]].
		^ pt].
	^ (RealEstateAgent assignCollapseFrameFor: self) origin.
! !

!StandardSystemView methodsFor: 'framing' stamp: ''!
chooseFrame
	"Answer a new frame, depending on whether the view is currently 
	collapsed or not."
	| labelForm f |
	self isCollapsed & expandedViewport notNil
		ifTrue:
			[labelForm := bitsValid
				ifTrue: [windowBits]
				ifFalse: [Form fromDisplay: self labelDisplayBox].
			bitsValid := false.
			self erase.
			labelForm slideFrom: self labelDisplayBox origin
					to: expandedViewport origin-self labelOffset
					nSteps: 10.
			^ expandedViewport]
		ifFalse:
			[f := self getFrame.
			bitsValid := false.
			self erase.
			^ f topLeft + self labelOffset extent: f extent]! !

!StandardSystemView methodsFor: 'framing' stamp: ''!
chooseMoveRectangle
	"Ask the user to designate a new window rectangle."
	| offset p |
	offset := Sensor anyButtonPressed "Offset if draggin, eg, label"
		ifTrue: [self windowBox topLeft - Sensor cursorPoint]
		ifFalse: [0@0].
	self isCollapsed
		ifTrue: [^ self labelDisplayBox newRectFrom:
					[:f | p := Sensor cursorPoint + offset.
					p := (p max: 0@0) truncateTo: 8.
					p extent: f extent]]
		ifFalse: [^ self windowBox newRectFrom:
					[:f | p := Sensor cursorPoint + offset.
					self constrainFrame: (p extent: f extent)]]! !

!StandardSystemView methodsFor: 'clipping box access' stamp: ''!
clippingBox
	"Answer the rectangular area in which the receiver can show its label."

	^self isTopView
		ifTrue: [self labelDisplayBox]
		ifFalse: [super insetDisplayBox]! !

!StandardSystemView methodsFor: 'label access' stamp: 'di 6/16/97 12:30'!
closeBoxFrame
	^ Rectangle origin: (self labelDisplayBox leftCenter + (10@-5)) extent: (11@11)! !

!StandardSystemView methodsFor: 'framing' stamp: 'di 5/11/1999 22:09'!
collapse
	"If the receiver is not already collapsed, change its view to be that of its 
	label only."

	self isCollapsed ifFalse:
			[model modelSleep.
			(subViews ~~ nil and: [subViews size = 1 and: [subViews first isKindOf: MorphWorldView]])
				ifTrue: [subViews first deEmphasizeView].
			expandedViewport := self viewport.
			savedSubViews := subViews.
			self resetSubViews.
			labelText isNil ifTrue: [self label: nil.  bitsValid := false.].
			self window: (self inverseDisplayTransform:
					((self labelDisplayBox topLeft extent: (labelText extent x + 70) @ self labelHeight)
						 intersect: self labelDisplayBox))]! !

!StandardSystemView methodsFor: 'framing' stamp: ''!
collapseToPoint: collapsePoint
	self collapse.
	self align: self displayBox topLeft with: collapsePoint.
	collapsedViewport := self viewport.
	self displayEmphasized! !

!StandardSystemView methodsFor: 'framing' stamp: ''!
collapsedFrame
	"Answer the rectangle occupied by this window when collapsed."
	^ collapsedViewport  "NOTE may be nil"! !

!StandardSystemView methodsFor: 'clipping box access' stamp: 'BG 12/5/2003 11:13'!
constrainFrame: aRectangle
	"Constrain aRectangle, to the minimum and maximum size
	for this window"

   | adjustmentForLabel |
   adjustmentForLabel := 0 @ (labelFrame height  - labelFrame borderWidth).
	^ aRectangle origin extent:
		((aRectangle extent max: minimumSize + adjustmentForLabel)
		      min: maximumSize + adjustmentForLabel).! !

!StandardSystemView methodsFor: 'testing' stamp: ''!
containsPoint: aPoint 
	"Refer to the comment in View|containsPoint:."

	^(super containsPoint: aPoint) | (self labelContainsPoint: aPoint)! !

!StandardSystemView methodsFor: 'object fileIn' stamp: 'RAA 12/20/2000 17:49'!
convertToCurrentVersion: varDict refStream: smartRefStrm
	
	updatablePanes ifNil: [updatablePanes := #()].
	^super convertToCurrentVersion: varDict refStream: smartRefStrm.

! !

!StandardSystemView methodsFor: 'displaying' stamp: ''!
deEmphasizeForDebugger
	"Carefully de-emphasis this window because a debugger is being opened. Care must be taken to avoid invoking potentially buggy window display code that could cause a recursive chain of errors eventually resulting in a virtual machine crash. In particular, do not de-emphasize the subviews."

	self deEmphasizeView.  "de-emphasize this top-level view"
	self uncacheBits.
	Smalltalk garbageCollectMost > 1000000 ifTrue: [
		"if there is enough space, cache current window screen bits"
		self cacheBitsAsIs].
! !

!StandardSystemView methodsFor: 'displaying' stamp: 'di 10/3/97 13:18'!
deEmphasizeLabel
	"Un-Highlight the label."
	labelFrame height = 0 ifTrue: [^ self].  "no label"
	self displayLabelBackground: false.
	self displayLabelText.! !

!StandardSystemView methodsFor: 'deEmphasizing' stamp: ''!
deEmphasizeView 
	"Refer to the comment in View|deEmphasizeView."

	isLabelComplemented ifTrue:
		[self deEmphasizeLabel.
		isLabelComplemented := false]! !

!StandardSystemView methodsFor: 'controller access' stamp: ''!
defaultControllerClass 
	"Refer to the comment in View|defaultControllerClass."

	^StandardSystemController! !

!StandardSystemView methodsFor: 'displaying' stamp: ''!
display
	isLabelComplemented
		ifTrue: [self displayEmphasized]
		ifFalse: [self displayDeEmphasized]! !

!StandardSystemView methodsFor: 'displaying' stamp: 'hmm 7/21/1999 07:37'!
displayDeEmphasized 
	"Display this view with emphasis off.
	If windowBits is not nil, then simply BLT if possible,
		but force full display for top window so color is preserved."
	(bitsValid and: [controller ~~ ScheduledControllers activeController])
		ifTrue: [self lock.
				windowBits displayAt: self windowOrigin]
		ifFalse: [Display deferUpdates: true.
				super display.
				Display deferUpdates: false; forceToScreen: self windowBox.
				CacheBits ifTrue: [self cacheBitsAsIs]]
! !

!StandardSystemView methodsFor: 'displaying' stamp: ''!
displayEmphasized
	"Display with label highlighted to indicate that it is active."

	self displayDeEmphasized; emphasize.
	isLabelComplemented := true! !

!StandardSystemView methodsFor: 'displaying' stamp: 'di 5/15/1998 21:55'!
displayLabelBackground: emphasized
	"Clear or emphasize the inner region of the label"
	| r1 r2 r3 c3 c2 c1 |
	emphasized ifFalse:
		["Just clear the label if not emphasized"
		^ Display fill: (self labelDisplayBox insetBy: 2) fillColor: self labelColor].
	r1 := self labelDisplayBox insetBy: 2.
	r2 := r1 insetBy: 0@2.
	r3 := r2 insetBy: 0@3.
	c3 := self labelColor.
	c2 := c3 dansDarker.
	c1 := c2 dansDarker.
	Display fill: r1 fillColor: c1.
	Display fill: r2 fillColor: c2.
	Display fill: r3 fillColor: c3.
 
"	Here is the Mac racing stripe code
	stripes := Bitmap with: (self labelColor pixelWordForDepth: Display depth)
					with: (Form black pixelWordForDepth: Display depth).
	self windowOrigin y even ifTrue: [stripes swap: 1 with: 2].
	Display fill: (self labelDisplayBox insetBy: 3) fillColor: stripes.
"! !

!StandardSystemView methodsFor: 'displaying' stamp: ''!
displayLabelBoxes
	"closeBox, growBox."
	| aRect smallRect backColor |
	aRect := self closeBoxFrame.
	backColor := self labelColor.
	Display fill: (aRect insetBy: -2) fillColor: backColor.
	Display fillBlack: aRect.
	Display fill: (aRect insetBy: 1) fillColor: backColor.

	aRect := self growBoxFrame.
	smallRect := aRect origin extent: 7@7.
	Display fill: (aRect insetBy: -2) fillColor: backColor.
	aRect := aRect insetOriginBy: 2@2 cornerBy: 0@0.
	Display fillBlack: aRect.
	Display fill: (aRect insetBy: 1) fillColor: backColor.
	Display fillBlack: smallRect.
	Display fill: (smallRect insetBy: 1) fillColor: backColor! !

!StandardSystemView methodsFor: 'displaying' stamp: 'di 9/10/1998 09:43'!
displayLabelText
	"The label goes in the center of the window"
	| labelRect |
	labelText foregroundColor: self foregroundColor
			backgroundColor: self labelColor.
	labelRect := self labelTextRegion.
	Display fill: (labelRect expandBy: 3@0) fillColor: self labelColor.
	labelText displayOn: Display at: labelRect topLeft clippingBox: labelRect
			rule: labelText rule fillColor: labelText fillColor.
	labelText destinationForm: nil! !

!StandardSystemView methodsFor: 'displaying' stamp: 'di 8/29/97 18:57'!
displayOn: aPort
	bitsValid ifFalse:
		[^ Display clippingTo: aPort clipRect do: [super display]].
	windowBits displayOnPort: aPort at: self windowOrigin! !

!StandardSystemView methodsFor: 'displaying' stamp: 'ar 5/14/2001 23:40'!
displayRacingStripes
	"Display Racing Stripes in the label"
	| labelDisplayBox stripes top bottom left box right |
	labelDisplayBox := self labelDisplayBox.
	top := labelDisplayBox top + 3.
	bottom := labelDisplayBox bottom - 3.
	stripes := Bitmap with: (Display pixelWordFor: self labelColor)
			with: (Display pixelWordFor: Color black).
	top even ifFalse: [stripes swap: 1 with: 2].

	left := labelDisplayBox left + 3.

	box := self closeBoxFrame.
	right := box left - 2.
	Display fill: (Rectangle left: left right: right top: top bottom: bottom)
			fillColor: stripes.
	left := box right + 2.

	box := self labelTextRegion.
	right := box left - 3.
	Display fill: (Rectangle left: left right: right top: top bottom: bottom)
			fillColor: stripes.
	left := box right + 2.

	box := self growBoxFrame.
	right := box left - 2.
	Display fill: (Rectangle left: left right: right top: top bottom: bottom)
			fillColor: stripes.
	left := box right + 2.

	right := labelDisplayBox right - 3.
	Display fill: (Rectangle left: left right: right top: top bottom: bottom)
			fillColor: stripes.
! !

!StandardSystemView methodsFor: 'displaying' stamp: 'di 10/3/97 13:14'!
displayView
	"Refer to the comment in View|displayView. "
	labelFrame height = 0 ifTrue: [^ self].  "no label"
	self displayBox width = labelFrame width ifFalse:
		["recompute label width when window changes size"
		self setLabelRegion].
	(labelFrame align: labelFrame topLeft with: self windowOrigin)
		insideColor: self labelColor;
		displayOn: Display.
	self displayLabelText! !

!StandardSystemView methodsFor: 'displaying' stamp: 'di 10/3/97 13:18'!
emphasizeLabel
	"Highlight the label."
	labelFrame height = 0 ifTrue: [^ self].  "no label"
	self displayLabelBackground: true.
	self displayLabelBoxes.
	self displayLabelText.! !

!StandardSystemView methodsFor: 'deEmphasizing' stamp: ''!
emphasizeView 
	"Refer to the comment in View|emphasizeView."

	self emphasizeLabel! !

!StandardSystemView methodsFor: 'displaying' stamp: 'di 8/30/97 11:07'!
erase
	"Clear the display box of the receiver to be gray, as the screen background."
	| oldValid |
	CacheBits
		ifTrue:
			[oldValid := bitsValid.
			bitsValid := false.
			ScheduledControllers restore: self windowBox without: self.
			bitsValid := oldValid]
		ifFalse:
			[ScheduledControllers restore: self windowBox without: self]! !

!StandardSystemView methodsFor: 'framing' stamp: 'sw 10/20/1999 09:46'!
expand
	"If the receiver is collapsed, change its view to be that of all of its subviews, not its label alone."
	| newFrame |
	self isCollapsed
		ifTrue:
			[newFrame := self chooseFrame expandBy: borderWidth.
			collapsedViewport := self viewport.
			subViews := savedSubViews.
			labelFrame borderWidthLeft: 2 right: 2 top: 2 bottom: 2.
			savedSubViews := nil.
			self setWindow: nil.
			self resizeTo: newFrame.
			self displayDeEmphasized.
			model modelWakeUpIn: self]! !

!StandardSystemView methodsFor: 'framing' stamp: ''!
expandedFrame
	"Answer the rectangle occupied by this window when expanded."
	^ expandedViewport  "NOTE may be nil"! !

!StandardSystemView methodsFor: 'framing' stamp: 'sw 8/15/97 17:18'!
fullScreen
	"Expand the receiver to fill the screen.  Let the model decide how big is full -- allows for flop-out scrollbar on left if desired"

	self isCollapsed ifFalse:
		[self reframeTo: model fullScreenSize]! !

!StandardSystemView methodsFor: 'framing' stamp: ''!
getFrame
	"Ask the user to designate a rectangular area in which
	the receiver should be displayed."
	| minFrame |
	minFrame := Cursor origin showWhile: 
		[(Sensor cursorPoint extent: self minimumSize) newRectFrom:
			[:f | Sensor cursorPoint extent: self minimumSize]].
	self maximumSize <= self minimumSize ifTrue: [^ minFrame].
	^ Cursor corner showWhile:
		[minFrame newRectFrom:
			[:f | self constrainFrame: (f origin corner: Sensor cursorPoint)]]! !

!StandardSystemView methodsFor: 'label access' stamp: 'di 6/16/97 12:29'!
growBoxFrame
	^ Rectangle origin: (self labelDisplayBox rightCenter + (-22@-5)) extent: (11@11)! !

!StandardSystemView methodsFor: 'framing' stamp: 'sw 1/22/96'!
initialExtent
	"Answer the desired extent for the receiver when it is first opened on the screen.  "

	^ model initialExtent min: maximumSize max: minimumSize! !

!StandardSystemView methodsFor: 'framing' stamp: 'RAA 6/14/2000 17:27'!
initialFrame
        "Find a plausible initial screen area for the receiver, taking into account user preference, the size needed, and other windows currently on the screen.  5/22/96 sw: let RealEstateAgent do it for us"

        ^ RealEstateAgent initialFrameFor: self world: nil! !

!StandardSystemView methodsFor: 'initialize-release' stamp: 'sw 10/29/1999 12:58'!
initialize 
	"Refer to the comment in View|initialize."
	super initialize.
	labelFrame := Quadrangle new.
	labelFrame region: (Rectangle origin: 0 @ 0 extent: 50 @ self labelHeight).
	labelFrame borderWidthLeft: 2 right: 2 top: 2 bottom: 2.
	self label: nil.
	isLabelComplemented := false.
	minimumSize := 50 @ 50.
	maximumSize := Display extent.
	collapsedViewport := nil.
	expandedViewport := nil.
	bitsValid := false.
	updatablePanes := #()! !

!StandardSystemView methodsFor: 'testing' stamp: ''!
isCollapsed
	"Answer whether the receiver is collapsed (true) or expanded (false)."

	^savedSubViews ~~ nil! !

!StandardSystemView methodsFor: 'label access' stamp: ''!
label
	"Answer the string that appears in the receiver's label."
	labelText isNil
		ifTrue: [^ 'Untitled' copy]
		ifFalse: [^ labelText asString]! !

!StandardSystemView methodsFor: 'label access' stamp: 'sw 12/9/1999 17:44'!
label: aString 
	"Set aString to be the receiver's label."
	labelText := Paragraph
			withText: (Text string: ((aString == nil or: [aString isEmpty])
								ifTrue: ['Untitled' copy]
								ifFalse: [aString])
							attributes: (Array with: TextEmphasis bold))
			style: LabelStyle.
	insetDisplayBox == nil ifTrue: [^ self].  "wait for further initialization"
	self setLabelRegion! !

!StandardSystemView methodsFor: 'label access' stamp: ''!
labelColor
	"Answer the color to use as the background for the receiver's label.  By default, this is the same as the background color of the window, but need not be.  7/16/96 sw"

	^ self backgroundColor! !

!StandardSystemView methodsFor: 'testing' stamp: ''!
labelContainsPoint: aPoint 
	"Answer TRUE if aPoint is in the label box."

	^self labelDisplayBox containsPoint: aPoint! !

!StandardSystemView methodsFor: 'label access' stamp: ''!
labelDisplayBox
	"Answer the rectangle that borders the visible parts of the receiver's label 
	on the display screen."

	^ labelFrame region
		align: labelFrame topLeft
		with: self windowOrigin! !

!StandardSystemView methodsFor: 'label access' stamp: ''!
labelFrame
	^labelFrame! !

!StandardSystemView methodsFor: 'label access' stamp: 'sw 12/9/1999 17:47'!
labelHeight
	^ ((LabelStyle fontAt: 1) height + 4) max: 20! !

!StandardSystemView methodsFor: 'label access' stamp: ''!
labelOffset
	^ 0 @ (self labelHeight-2)! !

!StandardSystemView methodsFor: 'label access' stamp: 'sr 3/26/2000 04:26'!
labelText
	^labelText! !

!StandardSystemView methodsFor: 'label access' stamp: ''!
labelTextRegion
	labelText == nil ifTrue: [^ self labelDisplayBox center extent: 0@0].
	^ (labelText boundingBox
			align: labelText boundingBox center
			with: self labelDisplayBox center)
		intersect: (self labelDisplayBox insetBy: 35@0)! !

!StandardSystemView methodsFor: 'displaying' stamp: 'RAA 6/14/2000 17:27'!
makeMeVisible

        | newLoc portRect |
        ((Display boundingBox insetBy: (0@0 corner: self labelHeight asPoint))
                containsPoint: self displayBox topLeft) ifTrue: [^ self "OK -- my top left is visible"].

        "window not on screen (probably due to reframe) -- move it now"
        newLoc := self isCollapsed
                ifTrue: [RealEstateAgent assignCollapsePointFor: self]
                ifFalse: [(RealEstateAgent initialFrameFor: self world: nil) topLeft].
        portRect := newLoc + self labelOffset
                                extent: self windowBox extent - self labelOffset.
        self resizeTo: portRect.
        self setLabelRegion.
! !

!StandardSystemView methodsFor: 'size' stamp: ''!
maximumSize
	"Answer a point representing the maximum width and height of the 
	receiver."

	^maximumSize! !

!StandardSystemView methodsFor: 'size' stamp: ''!
maximumSize: aPoint 
	"Set the argument, aPoint, to be the maximum width and height of the 
	receiver."

	maximumSize := aPoint! !

!StandardSystemView methodsFor: 'size' stamp: ''!
minimumSize
	"Answer a point representing the minimum width and height of the 
	receiver."

	^minimumSize! !

!StandardSystemView methodsFor: 'size' stamp: ''!
minimumSize: aPoint 
	"Set the argument, aPoint, to be the minimum width and height of the 
	receiver."

	minimumSize := aPoint! !

!StandardSystemView methodsFor: 'initialize-release' stamp: ''!
model: aModel
	"Set the receiver's model.  For a Standard System View, we also at this time get the default background color set up.  7/30/96 sw"
	super model: aModel.
	self setDefaultBackgroundColor! !

!StandardSystemView methodsFor: 'framing' stamp: ''!
moved
	"The user has moved the receiver; after a new view rectangle is chosen, this method is called to allow certain views to take note of the change.  6/10/96 sw" ! !

!StandardSystemView methodsFor: 'framing' stamp: ''!
newFrame: frameChangeBlock
	self reframeTo: (self windowBox newRectFrom:
		[:f | self constrainFrame: (frameChangeBlock value: f)])! !

!StandardSystemView methodsFor: 'label access' stamp: 'di 10/3/97 14:20'!
noLabel
	"A label of zero height indicates no label"
	labelFrame height > 0
		ifTrue: [labelFrame region: (labelFrame bottomLeft + (0@1) extent: labelFrame width@0).
				labelFrame borderWidth: 0.
				self uncacheBits]! !

!StandardSystemView methodsFor: 'framing' stamp: 'di 10/22/1998 16:15'!
reframePanesAdjoining: subView along: side to: aDisplayBox 
	| newBox delta newRect minDim theMin |
	newRect := aDisplayBox.
	theMin := 16.
	"First check that this won't make any pane smaller than theMin screen dots"
	minDim := ((subViews select: [:sub | sub displayBox bordersOn: subView displayBox along: side])
		collect: [:sub | sub displayBox adjustTo: newRect along: side])
			inject: 999 into: [:was :rect | (was min: rect width) min: rect height].
	"If so, amend newRect as required"
	minDim < theMin ifTrue:
		[delta := minDim - theMin.
		newRect := newRect withSide: side setTo: 
				((newRect perform: side) > (subView displayBox perform: side)
					ifTrue: [(newRect perform: side) + delta]
					ifFalse: [(newRect perform: side) - delta])].
	"Now adjust all adjoining panes for real"
	subViews do:
		[:sub | (sub displayBox bordersOn: subView displayBox along: side) ifTrue:
			[newBox := sub displayBox adjustTo: newRect along: side.
			sub window: sub window viewport:
				(sub transform: (sub inverseDisplayTransform: newBox)) rounded]].
	"And adjust the growing pane itself"
	subView window: subView window viewport:
			(subView transform: (subView inverseDisplayTransform: newRect)) rounded.

	"Finally force a recomposition of the whole window"
	viewport := nil.
	self resizeTo: self viewport.
	self uncacheBits; displayEmphasized! !

!StandardSystemView methodsFor: 'framing' stamp: 'BG 12/4/2003 13:14'!
reframeTo: newFrame
	"Reframe the receiver to the given screen rectangle.  
	Repaint difference after the change.  "
	| oldBox newBox portRect |
	self uncacheBits.
	oldBox := self windowBox.
	portRect := newFrame topLeft + self labelOffset
				corner: newFrame corner.
	self setWindow: nil.
	self resizeTo: portRect.
	self setLabelRegion.
	newBox := self windowBox.
	(oldBox areasOutside: newBox) do:
		[:rect | ScheduledControllers restore: rect].
	self displayEmphasized! !

!StandardSystemView methodsFor: 'label access' stamp: 'di 6/10/1998 13:18'!
relabel: aString 
	"A new string for the label.  Window is assumed to be active.
	Window will redisplay only if label bar has to grow."
	| oldRegion oldWidth |
	(model windowReqNewLabel: aString) ifFalse: [^ self].
	oldRegion := self labelTextRegion.
	oldWidth := self insetDisplayBox width.
	self label: aString.
	Display fill: ((oldRegion merge: self labelTextRegion) expandBy: 3@0)
			fillColor: self labelColor.
	self insetDisplayBox width = oldWidth
		ifTrue: [self displayLabelText; emphasizeLabel]
		ifFalse: [self uncacheBits; displayEmphasized].
! !

!StandardSystemView methodsFor: 'initialize-release' stamp: 'jm 8/20/1998 18:29'!
release

	model windowIsClosing.
	self isCollapsed ifTrue: [savedSubViews do: [:v | v release]].
	super release.
! !

!StandardSystemView methodsFor: 'framing' stamp: ''!
resize
	"Determine the rectangular area for the receiver, adjusted to the 
	minimum and maximum sizes."
	| f |
	f := self getFrame.
	self resizeTo: (f topLeft + self labelOffset extent: f extent)
! !

!StandardSystemView methodsFor: 'framing' stamp: ''!
resizeInitially
	"Determine the rectangular area for the receiver, adjusted to the 
	minimum and maximum sizes."
	self resizeTo: self initialFrame
! !

!StandardSystemView methodsFor: 'framing' stamp: 'di 4/6/98 15:12'!
resizeMinimumCenteredAt: aPoint 
	"Determine the rectangular area for the receiver, adjusted so that it is 
	centered a position, aPoint."
	| aRectangle |
	aRectangle := 0 @ 0 extent: self minimumSize.
	aRectangle := aRectangle align: aRectangle center with: aPoint.
	self resizeTo: aRectangle! !

!StandardSystemView methodsFor: 'framing' stamp: 'di 4/6/98 15:29'!
resizeTo: aRectangle
	"Resize this view to aRectangle"

	"First get scaling right inside borders"
	self window: (self window insetBy: borderWidth)
		viewport: (aRectangle insetBy: borderWidth).

	"Then ensure window maps to aRectangle"
	window := transformation applyInverseTo: aRectangle! !

!StandardSystemView methodsFor: 'label access' stamp: 'sw 1/19/2001 20:13'!
setLabel: aLabel
	"For compatibility with morphic"

	self relabel: aLabel! !

!StandardSystemView methodsFor: 'label access' stamp: 'di 10/3/97 13:35'!
setLabelRegion
	"Always follows view width"

	labelFrame region: (0 @ 0 extent: self displayBox width @ self labelHeight).
	labelFrame borderWidth: 2! !

!StandardSystemView methodsFor: 'label access' stamp: 'sumim 2/8/2002 14:36'!
setLabelTo: aString 
	"Force aString to be the new label of the receiver, bypassing any logic about whether it is acceptable and about propagating information about the change."

	| oldRegion oldWidth |
	self label: aString.
	self controller isControlActive ifFalse: [^ self].
	oldRegion := self labelTextRegion.
	oldWidth := self insetDisplayBox width.
	Display fill: ((oldRegion merge: self labelTextRegion) expandBy: 3@0)
			fillColor: self labelColor.
	self insetDisplayBox width = oldWidth
		ifTrue: [self displayLabelText; emphasizeLabel]
		ifFalse: [self uncacheBits; displayEmphasized]! !

!StandardSystemView methodsFor: 'private' stamp: ''!
setTransformation: aTransformation 
	"Override to support label size changes "
	super setTransformation: aTransformation.
	self label: self label! !

!StandardSystemView methodsFor: 'updating' stamp: 'sw 10/29/1999 12:57'!
setUpdatablePanesFrom: getSelectors
	| aList aPane |
	"Set my updatablePanes inst var to the list of panes which are list panes with the given get-list selectors.  Order is important here!!  Note that the method is robust in the face of panes not found, but a warning is printed in the transcript in each such case"

	aList := OrderedCollection new.
	getSelectors do:
		[:sel | aPane := self subViewSatisfying:
				[:pane | (pane isKindOf: PluggableListView) and: [pane getListSelector == sel]].
			aPane
				ifNotNil:
					[aList add: aPane]
				ifNil:
					[Transcript cr; show: 'Warning: view ', sel, ' not found.']].
	updatablePanes := aList asArray! !

!StandardSystemView methodsFor: 'framing' stamp: ''!
standardWindowOffset
	^ Preferences standardWindowOffset! !

!StandardSystemView methodsFor: 'private' stamp: 'di 10/21/1998 16:12'!
subviewWithLongestSide: sideBlock near: aPoint 
	| theSub theSide theLen box |
	theLen := 0.
	subViews do:
		[:sub | box := sub insetDisplayBox.
		box forPoint: aPoint closestSideDistLen:
			[:side :dist :len |
			(dist <= 5 and: [len > theLen]) ifTrue:
				[theSub := sub.
				theSide := side.
				theLen := len]]].
	sideBlock value: theSide.
	^ theSub! !

!StandardSystemView methodsFor: 'displaying' stamp: ''!
uncacheBits
	windowBits := nil.
	bitsValid := false.! !

!StandardSystemView methodsFor: 'updating' stamp: 'sw 10/29/1999 21:20'!
updatablePanes
	"Answer the list of panes, in order, which might be sent the #verifyContents message upon window activation or expansion."
	^ updatablePanes ifNil: [updatablePanes := #()]! !

!StandardSystemView methodsFor: 'updating' stamp: 'sw 1/11/2000 15:30'!
update: aSymbol
	aSymbol = #relabel
		ifTrue: [^ self setLabelTo: model labelString].
	^ super update: aSymbol! !

!StandardSystemView methodsFor: 'displaying' stamp: ''!
windowBits
	^ windowBits! !

!StandardSystemView methodsFor: 'framing' stamp: ''!
windowBox
	^ self displayBox merge: self labelDisplayBox! !

!StandardSystemView methodsFor: 'framing' stamp: 'di 10/3/97 14:19'!
windowOrigin
	^ (self isCollapsed or: [labelFrame height = 0  "no label"])
		ifTrue: [self displayBox topLeft]
		ifFalse: [self displayBox topLeft - self labelOffset]! !

View subclass: #StringHolderView
	instanceVariableNames: 'displayContents hasUnacceptedEdits askBeforeDiscardingEdits'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Support'!

!StringHolderView commentStamp: '<historical>' prior: 0!
I am a View of a String that is an aspect of a more structured object. This String should not be changed by any editing unless the user issues the accept command. Thus my instances provide a working copy of the String. This copy is edited. When the user issues the accept command, the String is copied from the working version; or if the user issues the cancel command, the working version is restored from the String. StringHolderController is my default controller. It is initialized specially by passing the string viewed which is then converted to a Paragraph for editing.!

StringHolderView subclass: #FillInTheBlankView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Support'!

!FillInTheBlankView commentStamp: '<historical>' prior: 0!
I am a view of a FillInTheBlank. I display a query and an editable view of the user's reply string.
!

!FillInTheBlankView classSide methodsFor: 'instance creation' stamp: 'jm 4/28/1998 08:35'!
multiLineOn: aFillInTheBlank message: queryString centerAt: aPoint answerHeight: answerHeight
	"Answer an instance of me on aFillInTheBlank asking the question queryString. Allow the reply to be multiple lines, and make the user input view the given height."

	| messageView answerView topView |
	messageView := DisplayTextView new
		model: queryString asDisplayText;
		borderWidthLeft: 2 right: 2 top: 2 bottom: 0;
		controller: NoController new.
	messageView
		window: (0@0 extent: (messageView window extent max: 200@30));
		centered.
	answerView := self new
		model: aFillInTheBlank;
		window: (0@0 extent: (messageView window width@answerHeight));
		borderWidth: 2.
	topView := View new model: aFillInTheBlank.
	topView controller: ModalController new.
	topView addSubView: messageView.
	topView addSubView: answerView below: messageView.
	topView align: topView viewport center with: aPoint.
	topView window:
		(0 @ 0 extent:
			(messageView window width) @
			  (messageView window height + answerView window height)).
	topView translateBy:
		(topView displayBox amountToTranslateWithin: Display boundingBox).
	^ topView
! !

!FillInTheBlankView classSide methodsFor: 'instance creation' stamp: 'jm 4/28/1998 08:22'!
on: aFillInTheBlank message: queryString centerAt: aPoint
	"Answer an instance of me on aFillInTheBlank for a single line of input in response to the question queryString."

	aFillInTheBlank acceptOnCR: true.
	^ self
		multiLineOn: aFillInTheBlank
		message: queryString
		centerAt: aPoint
		answerHeight: 40
! !

!FillInTheBlankView classSide methodsFor: 'instance creation' stamp: 'jdr 6/4/2000 15:03'!
requestPassword: aFillInTheBlank message: queryString centerAt: aPoint answerHeight: answerHeight
	"Answer an instance of me on aFillInTheBlank asking the question queryString. Allow the reply to be multiple lines, and make the user input view the given height."

	| messageView answerView topView myPar pwdFont myArray myStyle |
	aFillInTheBlank acceptOnCR: true.
	messageView := DisplayTextView new
		model: queryString asDisplayText;
		borderWidthLeft: 2 right: 2 top: 2 bottom: 0;
		controller: NoController new.
	messageView
		window: (0@0 extent: (messageView window extent max: 200@30));
		centered.
	answerView := self new
		model: aFillInTheBlank;
		window: (0@0 extent: (messageView window width@answerHeight));
		borderWidth: 2.
	" now answerView to use the password font"
	myPar := answerView displayContents.
	pwdFont := (StrikeFont passwordFontSize: 12).
	myArray := Array new: 1.
	myArray at: 1 put: pwdFont.
	myStyle := TextStyle fontArray: myArray.
	myPar setWithText: (myPar text) style: myStyle.

	topView := View new model: aFillInTheBlank.
	topView controller: ModalController new.
	topView addSubView: messageView.
	topView addSubView: answerView below: messageView.
	topView align: topView viewport center with: aPoint.
	topView window:
		(0 @ 0 extent:
			(messageView window width) @
			  (messageView window height + answerView window height)).
	topView translateBy:
		(topView displayBox amountToTranslateWithin: Display boundingBox).
	^ topView
! !

!FillInTheBlankView methodsFor: 'controller access' stamp: 'jm 4/28/1998 06:37'!
defaultControllerClass

	^ FillInTheBlankController
! !

StringHolderView subclass: #PluggableTextView
	instanceVariableNames: 'getTextSelector setTextSelector getSelectionSelector getMenuSelector hasEditingConflicts'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Pluggable Views'!

!PluggableTextView commentStamp: '<historical>' prior: 0!
A PluggableTextView gets its content from the model. This allows the same kind of view to be used in different situations, thus avoiding a proliferation of gratuitous view and controller classes. See the class comment for PluggableListView.

Selectors are:

		getTextSel		fetch the original text from the model
		setTextSel		submit new text to the model when user "accepts"
		getSelectionSel	get the current text selection range
		getMenuSel		get the pane-specific, 'yellow-button' menu

	Any of the above selectors can be nil, meaning that the model does not supply behavior for the given action, and the default behavior should be used. For example, if setTextSel is nil then this view is consider read-only.

	The model informs a pluggable view of changes by sending #changed: to itself with getTextSel as a parameter. The view informs the model of selection changes by sending setTextSel to it with the newly selected item as a parameter, and invokes menu actions on the model via getMenuSel.
!

!PluggableTextView classSide methodsFor: 'instance creation' stamp: 'jm 3/29/98 07:24'!
on: anObject text: getTextSel accept: setTextSel

	^ self on: anObject
		text: getTextSel
		accept: setTextSel
		readSelection: nil
		menu: nil
! !

!PluggableTextView classSide methodsFor: 'instance creation' stamp: 'jm 3/29/98 07:24'!
on: anObject text: getTextSel accept: setTextSel readSelection: getSelectionSel menu: getMenuSel

	^ self new on: anObject
		text: getTextSel
		accept: setTextSel
		readSelection: getSelectionSel
		menu: getMenuSel
! !

!PluggableTextView methodsFor: 'object fileIn' stamp: 'RAA 12/20/2000 17:48'!
convertToCurrentVersion: varDict refStream: smartRefStrm
	
	hasEditingConflicts ifNil: [hasEditingConflicts := false].
	^super convertToCurrentVersion: varDict refStream: smartRefStrm.

! !

!PluggableTextView methodsFor: 'initialization' stamp: 'jm 3/29/98 07:24'!
defaultControllerClass 

	^ PluggableTextController
! !

!PluggableTextView methodsFor: 'model access' stamp: 'di 6/26/1998 11:06'!
getMenu: shiftKeyDown
	"Answer the menu for this text view, supplying an empty menu to be filled in. If the menu selector takes an extra argument, pass in the current state of the shift key."

	| menu |
	getMenuSelector == nil ifTrue: [^ nil].
	menu := CustomMenu new.
	getMenuSelector numArgs = 1
		ifTrue: [^ model perform: getMenuSelector with: menu].
	getMenuSelector numArgs = 2
		ifTrue: [^ model perform: getMenuSelector with: menu with: shiftKeyDown].
	^ self error: 'The getMenuSelector must be a 1- or 2-keyword symbol'! !

!PluggableTextView methodsFor: 'model access' stamp: 'sw 1/16/1999 14:31'!
getSelection
	"Answer the model's selection interval."

	getSelectionSelector == nil ifTrue: [^ 1 to: 0].  "null selection"
	^ getSelectionSelector ifNotNil: [model perform: getSelectionSelector]
! !

!PluggableTextView methodsFor: 'model access' stamp: 'tk 4/1/98 08:32'!
getText 
	"Answer the list to be displayed."
	| txt |
	getTextSelector == nil ifTrue: [^ Text new].
	txt := model perform: getTextSelector.
	txt == nil ifTrue: [^ Text new].
	self hasUnacceptedEdits: false.	"clean now"
	^ txt! !

!PluggableTextView methodsFor: 'model access' stamp: 'tk 3/31/98 15:58'!
getTextSelector
	"This is sent to the model to find out what text to display"

	^ getTextSelector! !

!PluggableTextView methodsFor: 'updating' stamp: 'sw 10/29/1999 21:03'!
hasEditingConflicts
	"Return true if a conflicting edit to the same code (typically) is known to have occurred after the current contents started getting edited"

	^ hasEditingConflicts == true! !

!PluggableTextView methodsFor: 'updating' stamp: 'sw 10/29/1999 21:04'!
hasEditingConflicts: aBoolean
	hasEditingConflicts := aBoolean! !

!PluggableTextView methodsFor: 'updating' stamp: 'sw 10/29/1999 21:04'!
hasUnacceptedEdits: aBoolean
	super hasUnacceptedEdits: aBoolean.
	aBoolean ifFalse: [hasEditingConflicts := false]! !

!PluggableTextView methodsFor: 'initialization' stamp: 'sw 10/29/1999 21:02'!
initialize 
	super initialize.
	hasEditingConflicts := false! !

!PluggableTextView methodsFor: 'model access' stamp: 'jm 5/3/1998 19:29'!
isReadOnlyView

	^ setTextSelector == nil
! !

!PluggableTextView methodsFor: 'model access' stamp: 'jm
 8/20/1998 11:55'!
model: aLockedModel 
	"Refer to the comment in View|model:."
 
	self model: aLockedModel controller: controller.
	self editString: self getText.
! !

!PluggableTextView methodsFor: 'initialization' stamp: 'tk 4/6/98 10:51'!
on: anObject text: getTextSel accept: setTextSel readSelection: getSelectionSel menu: getMenuSel

	self model: anObject.
	getTextSelector := getTextSel.
	setTextSelector := setTextSel.
	getSelectionSelector := getSelectionSel.
	getMenuSelector := getMenuSel.
	self borderWidth: 1.
	self editString: self getText.
	self setSelection: self getSelection.

! !

!PluggableTextView methodsFor: 'controller access' stamp: 'BG 11/26/2003 16:06'!
selectionInterval

  ^self controller selectionInterval! !

!PluggableTextView methodsFor: 'initialization' stamp: 'jm 3/29/98 07:25'!
setSelection: sel

	controller selectFrom: sel first to: sel last.
! !

!PluggableTextView methodsFor: 'model access' stamp: 'di 3/10/98 13:51'!
setText: textToAccept from: ctlr
	"Inform the model of text to be accepted, and return true if OK.
	Any errors should be reported to the controller, ctlr."
	setTextSelector == nil ifTrue: [^ true].
	setTextSelector numArgs = 2
		ifTrue: [^ model perform: setTextSelector with: textToAccept with: ctlr]
		ifFalse: [^ model perform: setTextSelector with: textToAccept]! !

!PluggableTextView methodsFor: 'updating' stamp: 'tk 5/23/2001 12:26'!
update: aSymbol
	"Refer to the comment in View|update:. Do nothing if the given symbol does not match any action. "

	aSymbol == #wantToChange ifTrue:
			[self canDiscardEdits ifFalse: [self promptForCancel].  ^ self].
	aSymbol == #flash ifTrue: [^ controller flash].
	aSymbol == getTextSelector ifTrue: [^ self updateDisplayContents].
	aSymbol == getSelectionSelector ifTrue: [^ self setSelection: self getSelection].
	aSymbol == #clearUserEdits ifTrue: [^ self hasUnacceptedEdits: false].
	(aSymbol == #autoSelect and: [getSelectionSelector ~~ nil]) ifTrue:
			[ParagraphEditor abandonChangeText.	"no replacement!!"
			^ controller setSearch: model autoSelectString;
					againOrSame: true].
	aSymbol == #appendEntry ifTrue:
			[^ controller doOccluded: [controller appendEntry]].
	aSymbol == #clearText ifTrue:
			[^ controller doOccluded:
				[controller changeText: Text new]].
	aSymbol == #bs ifTrue:
			[^ controller doOccluded:
				[controller bsText]].
	aSymbol == #codeChangedElsewhere ifTrue:
			[^ self hasEditingConflicts: true]

! !

!PluggableTextView methodsFor: 'model access' stamp: 'di 4/27/1998 12:46'!
updateDisplayContents

	self editString: self getText.
	self displayView.
	self setSelection: self getSelection.
! !

!StringHolderView classSide methodsFor: 'instance creation' stamp: ''!
container
	"Answer an instance of me with a new instance of StringHolder as the 
	model."

	^self container: StringHolder new! !

!StringHolderView classSide methodsFor: 'instance creation' stamp: ''!
container: aContainer 
	"Answer an instance of me whose model is aContainer. Give it a 2-dot 
	border."

	| aCodeView |
	aCodeView := self new model: aContainer.
	aCodeView borderWidthLeft: 2 right: 2 top: 2 bottom: 2.
	^aCodeView! !

!StringHolderView classSide methodsFor: 'instance creation' stamp: 'ar 9/27/2005 20:48'!
open
	"Create a standard system view of a workspace on the screen."

	self open: StringHolder new label: 'Workspace'! !

!StringHolderView classSide methodsFor: 'instance creation' stamp: ''!
open: aStringHolder 
	"Create a standard system view of the argument, aStringHolder, as viewed 
	by an instance of me. The view has label 'StringHolder'."

	self open: aStringHolder label: 'StringHolder'! !

!StringHolderView classSide methodsFor: 'instance creation' stamp: 'sma 4/30/2000 10:15'!
open: aStringHolder label: labelString 
	"NOTE this should be in the model class, and all senders so redirected,
	in order that the view class can be discarded in a morphic world."

	"Create a standard system view of the model, aStringHolder, as viewed by 
	an instance of me. The label of the view is aString."
	| aStringHolderView topView |

	Smalltalk isMorphic ifTrue: [^ aStringHolder openAsMorphLabel: labelString].

	aStringHolderView := self container: aStringHolder.
	topView := StandardSystemView new.
	topView model: aStringHolderView model.
	topView addSubView: aStringHolderView.
	topView label: labelString.
	topView minimumSize: 100 @ 50.
	topView controller open! !

!StringHolderView methodsFor: 'updating' stamp: 'jm 3/24/98 14:38'!
askBeforeDiscardingEdits: aBoolean
	"Set the flag that determines whether the user should be asked before discarding unaccepted edits."

	askBeforeDiscardingEdits := aBoolean.
! !

!StringHolderView methodsFor: 'updating' stamp: 'tk 4/13/1998 22:58'!
canDiscardEdits
	"Return true if this view either has no text changes or does not care."

	^ (hasUnacceptedEdits & askBeforeDiscardingEdits) not
! !

!StringHolderView methodsFor: 'deEmphasizing' stamp: ''!
deEmphasizeView 
	"Refer to the comment in View|deEmphasizeView."

	(self controller isKindOf: ParagraphEditor)
	 	ifTrue: [controller deselect]! !

!StringHolderView methodsFor: 'controller access' stamp: ''!
defaultController 
	"Refer to the comment in View|defaultController."

	^self defaultControllerClass newParagraph: displayContents! !

!StringHolderView methodsFor: 'controller access' stamp: ''!
defaultControllerClass 
	"Refer to the comment in View|defaultControllerClass."

	^StringHolderController! !

!StringHolderView methodsFor: 'displaying' stamp: ''!
display 
	"Refer to the comment in View.display."
	(self isUnlocked and: [self insetDisplayBox ~= displayContents clippingRectangle])
		ifTrue:  "Recompose the text if the window changed"
				[self positionDisplayContents. 
				(self controller isKindOf: ParagraphEditor)
					ifTrue: [controller recomputeSelection]].
	super display! !

!StringHolderView methodsFor: 'controller access' stamp: ''!
displayContents

	^displayContents! !

!StringHolderView methodsFor: 'displaying' stamp: 'hmm 6/18/2000 19:24'!
displayView 
	"Refer to the comment in View|displayView."

	Display deferUpdatesIn: self displayBox while: [
		self clearInside.
		(self controller isKindOf: ParagraphEditor)
			ifTrue: [controller display]
			ifFalse: [displayContents display]]! !

!StringHolderView methodsFor: 'model access' stamp: ''!
editString: aString 
	"The paragraph to be displayed is created from the characters in aString."

	displayContents := Paragraph withText: aString asText
		style: TextStyle default copy
		compositionRectangle: (self insetDisplayBox insetBy: 6 @ 0)
		clippingRectangle: self insetDisplayBox
		foreColor: self foregroundColor backColor: self backgroundColor.
	(self controller isKindOf: ParagraphEditor)
		ifTrue: [controller changeParagraph: displayContents]! !

!StringHolderView methodsFor: 'model access' stamp: 'sma 5/28/2000 23:25'!
getMenu: shiftKeyState
	^ nil! !

!StringHolderView methodsFor: 'updating' stamp: 'jm 3/24/98 17:49'!
hasUnacceptedEdits
	"Return true if this view has unaccepted edits."

	^ hasUnacceptedEdits
! !

!StringHolderView methodsFor: 'updating' stamp: 'tk 4/13/1998 17:17'!
hasUnacceptedEdits: aBoolean
	"Set the hasUnacceptedEdits flag to the given value."

	hasUnacceptedEdits := aBoolean.
! !

!StringHolderView methodsFor: 'initialize-release' stamp: 'jm 3/24/98 14:39'!
initialize 
	"Refer to the comment in View|initialize."

	super initialize.
	displayContents := '' asParagraph.
	hasUnacceptedEdits := false.
	askBeforeDiscardingEdits := true.
! !

!StringHolderView methodsFor: 'displaying' stamp: ''!
lock
	"Refer to the comment in view|lock.  Must do at least what display would do to lock the view."
	(self isUnlocked and: [self insetDisplayBox ~= displayContents clippingRectangle])
		ifTrue:  "Recompose the text if the window changed"
				[self positionDisplayContents. 
				(self controller isKindOf: ParagraphEditor)
					ifTrue: [controller recomputeSelection]].
	super lock! !

!StringHolderView methodsFor: 'model access' stamp: ''!
model: aLockedModel 
	"Refer to the comment in View|model:."
 
	super model: aLockedModel.
	self editString: model contents! !

!StringHolderView methodsFor: 'displaying' stamp: ''!
positionDisplayContents
	"Presumably the text being displayed changed so that the wrapping box 
	and clipping box should be reset."

	displayContents 
		wrappingBox: (self insetDisplayBox insetBy: 6 @ 0)
		clippingBox: self insetDisplayBox! !

!StringHolderView methodsFor: 'updating' stamp: 'dgd 9/21/2003 17:42'!
promptForCancel
	"Ask if it is OK to cancel changes to text"
	| okToCancel stripes |
	self topView isCollapsed ifTrue:
		[(self confirm: 'Changes have not been saved.
Is it OK to cancel those changes?' translated) ifTrue: [model clearUserEditFlag].
		^ self].
	stripes := (Form extent: 16@16 fromStipple: 16r36C9) bits.
	Display border: self insetDisplayBox width: 4
			rule: Form reverse fillColor: stripes.
	okToCancel := self confirm: 'Changes have not been saved.
Is it OK to cancel those changes?' translated.
	Display border: self insetDisplayBox width: 4
			rule: Form reverse fillColor: stripes.
	okToCancel ifTrue:
		[self updateDisplayContents.
		model clearUserEditFlag].
! !

!StringHolderView methodsFor: 'updating' stamp: 'di 4/21/1998 11:30'!
update: aSymbol
	"Refer to the comment in View|update:."
	aSymbol == #wantToChange ifTrue: [^ self promptForCancel].
	aSymbol == #clearUserEdits ifTrue: [^ self hasUnacceptedEdits: false].
	aSymbol == #flash ifTrue: [^ controller flash].
	self updateDisplayContents! !

!StringHolderView methodsFor: 'updating' stamp: ''!
updateDisplayContents
	"Make the text that is displayed be the contents of the receiver's model."

	self editString: model contents.
	self displayView! !

!View methodsFor: 'subView inserting' stamp: ''!
addSubView: aView 
	"Remove aView from the tree of Views it is in (if any) and adds it to the 
	rear of the list of subViews of the receiver. Set the superView of aView 
	to be the receiver. It is typically used to build up a hierarchy of Views 
	(a structured picture). An error notification is generated if aView is the 
	same as the receiver or its superView, and so on."

	self addSubView: aView ifCyclic: [self error: 'cycle in subView structure.']! !

!View methodsFor: 'subView inserting' stamp: ''!
addSubView: aSubView above: lowerView
	"Adds aView (see View|addSubView:) so that it lies above topView."

	self addSubView: aSubView
		align: aSubView viewport bottomLeft
		with: lowerView viewport topLeft! !

!View methodsFor: 'subView inserting' stamp: ''!
addSubView: aSubView align: aPoint1 with: aPoint2 
	"Add aView to the receiver's list of subViews (see View|addSubView:) 
	and translate aView so that aPoint1 coincides with aPoint2. It is typically 
	used to build up a hierarchy of Views (a structured picture). Normally, 
	aPoint1 is a point on aView's viewport (e.g. aView viewport topLeft), 
	and aPoint2 is either an arbitrary point in the receiver's coordinate 
	system or a point on the receiver's window (e.g., self window topRight)."

	self addSubView: aSubView.
	aSubView align: aPoint1 with: aPoint2! !

!View methodsFor: 'subView inserting' stamp: ''!
addSubView: aSubView below: lowerView
	"Add the argument, aSubView, (see View|addSubView:) so that it lies 
	below the view, topView."

	self addSubView: aSubView
		align: aSubView viewport topLeft
		with: lowerView viewport bottomLeft! !

!View methodsFor: 'subView inserting' stamp: ''!
addSubView: aView ifCyclic: exceptionBlock 
	"Remove aView from the tree of Views it is in (if any) and add it to the 
	rear of the list of subViews of the receiver. Set the superView of aView 
	to be the receiver. It is typically used to build up a hierarchy of Views 
	(a structured picture). An error notification is generated if aView is the 
	same as the receiver or its superView, and so on."

	(self isCyclic: aView)
		ifTrue: [exceptionBlock value]
		ifFalse: 
			[aView removeFromSuperView.
			subViews addLast: aView.
			aView superView: self]! !

!View methodsFor: 'subView inserting' stamp: ''!
addSubView: aSubView toLeftOf: rightView
	"Adds aView (see addSubView:) so that it lies to the right of rightView."

	self addSubView: aSubView
		align: aSubView viewport topRight
		with:  rightView viewport topLeft! !

!View methodsFor: 'subView inserting' stamp: ''!
addSubView: aSubView toRightOf: leftView
	"Add the argument, aSubView, (see View|addSubView:) so that it lies to 
	the right of the view, leftView."

	self addSubView: aSubView
		align: aSubView viewport topLeft
		with: leftView viewport topRight! !

!View methodsFor: 'subView inserting' stamp: ''!
addSubView: aView viewport: aViewport 
	"Add aView to the receiver's list of subViews (see View|addSubView:) and 
	applies to aView a scale and translation computed from its window and 
	aViewport (such that its window fills aViewport)."

	self addSubView: aView.
	aView window: aView window viewport: aViewport! !

!View methodsFor: 'subView inserting' stamp: ''!
addSubView: aView window: aWindow viewport: aViewport 
	"Add aView to the receiver's list of subViews (see View|addSubView:) 
	and applies to aView a scale and translation computed from aWindow 
	and aViewport (such that aWindow fills aViewport)."

	self addSubView: aView.
	aView window: aWindow viewport: aViewport! !

!View methodsFor: 'transforming' stamp: ''!
align: aPoint1 with: aPoint2 
	"Add a translation of (aPoint2 - aPoint1) to the receiver's local 
	transformation. The point in the receiver's coordinate system that 
	previously was transformed to aPoint1 in the superView's coordinate 
	system will now be transformed to aPoint2 in the superView's coordinate 
	system. Other points will be translated by the same amount. It is 
	normally used when adding subViews to their superView in order to 
	line up the Viewport of one subView with that of another subView (see 
	View|addSubView:align:with:). aPoint1 and aPoint2 are usually points on 
	the viewports that are to be aligned. For example, 'subView2 align: 
	subView2 viewport topLeft with: subView1 viewport topRight' would be 
	used to place the viewport of subView2 next to the viewport of 
	subView1 with the topLeft and topRight corners, respectively, 
	coinciding. It is also possible to align the viewport of a subView with 
	the window of the superView, e.g., 'subView align: subView viewport 
	center with: superView window center'. View|align:with: assumes that 
	the view has been properly scaled, if necessary, to match its superView 
	(see View|scaleBy:). Typically, the coordinate systems of the receiver 
	and its superView will differ only by a translation offset so that no 
	scaling is necessary."

	self setTransformation: (transformation align: aPoint1 with: aPoint2)! !

!View methodsFor: 'display box access' stamp: ''!
apparentDisplayBox
	^self insetDisplayBox expandBy: 2 @ 2! !

!View methodsFor: 'bordering' stamp: 'sw 11/2/1998 15:34'!
backgroundColor
	Display depth <= 2 ifTrue: [^ Color white].
	insideColor ifNotNil: [^ Color colorFrom: insideColor].
	^ superView == nil
		ifFalse: [superView backgroundColor]
		ifTrue:	[Color white]! !

!View methodsFor: 'bordering' stamp: 'tk 10/21/97 12:31'!
backgroundColor: aColor
	Display depth = 1 ifTrue:
		[(aColor ~= nil and: [aColor isTransparent not]) ifTrue:
			["Avoid stipple due to attempts to match non-whites"
			^ insideColor := Color white]].
	insideColor := aColor! !

!View methodsFor: 'bordering' stamp: ''!
borderWidth
	"Answer either 0, indicating no border, or a Rectangle whose left value is 
	the width in display coordinates of the receiver's left border. Right, top, 
	and bottom widths are analogous. The border width is initially 0. A 
	View with a border width of 0 will not have any border displayed."

	^borderWidth! !

!View methodsFor: 'bordering' stamp: ''!
borderWidth: anInteger
	"Set the four border widths of the receiver to anInteger."

	self
		borderWidthLeft: anInteger
		right: anInteger
		top: anInteger
		bottom: anInteger! !

!View methodsFor: 'bordering' stamp: ''!
borderWidthLeft: anInteger1 right: anInteger2 top: anInteger3 bottom: anInteger4
	"Set the border widths of the receiver. These arguments represent the left, 
	right, top, and bottom border widths."

	borderWidth :=
			Rectangle
				left: anInteger1
				right: anInteger2
				top: anInteger3
				bottom: anInteger4.
	self unlock! !

!View methodsFor: 'display box access' stamp: ''!
boundingBox
	"Answer the bounding box which for the default case is the rectangular 
	area surrounding the bounding boxes of all the subViews."

	boundingBox ~~ nil
		ifTrue: [^boundingBox]
		ifFalse: [^self computeBoundingBox]! !

!View methodsFor: 'testing' stamp: 'tk 4/13/1998 22:58'!
canDiscardEdits
	"Return true if this pane is not dirty."

	^ true
! !

!View methodsFor: 'clearing' stamp: ''!
clear
	"Use the border color to paint the display box (including the border, see 
	View|displayBox) of the receiver."

	borderColor ~= nil ifTrue: [self clear: Color black]! !

!View methodsFor: 'clearing' stamp: ''!
clear: aColor 
	"Use aColor to paint the display box (including the border, see 
	View|displayBox) of the receiver."

	aColor ~= nil ifTrue: [Display fill: self displayBox fillColor: aColor]! !

!View methodsFor: 'clearing' stamp: ''!
clearInside
	"Use the inside color to paint the inset display box (excluding the border, 
	see View|insetDisplayBox) of the receiver."

	self clearInside: self backgroundColor! !

!View methodsFor: 'clearing' stamp: ''!
clearInside: aColor 
	"Use aColor to paint the inset display box (excluding the border, see 
	View|insetDisplayBox) of the receiver."

	aColor ~~ nil ifTrue: [Display fill: self insetDisplayBox fillColor: aColor]! !

!View methodsFor: 'miscellaneous' stamp: ''!
clipRect
	^ superView clipRect! !

!View methodsFor: 'miscellaneous' stamp: ''!
clipRect: r
	superView clipRect: r! !

!View methodsFor: 'displaying' stamp: ''!
clippingTo: rect do: aBlock

	superView clippingTo: rect do: aBlock! !

!View methodsFor: 'display box access' stamp: ''!
computeBoundingBox
	"Answer the minimum Rectangle that encloses the bounding boxes of the 
	receiver's subViews. If the receiver has no subViews, then the bounding 
	box is the receiver's window. Subclasses should redefine 
	View|boundingBox if a more suitable default for the case of no subViews 
	is available."

	| aRectangle |
	subViews isEmpty ifTrue: [^self getWindow].
	aRectangle := self firstSubView transform: self firstSubView boundingBox.
	subViews do: 
		[:aView | 
		aRectangle := aRectangle merge: (aView transform: aView boundingBox).].
	^aRectangle expandBy: borderWidth! !

!View methodsFor: 'private' stamp: ''!
computeDisplayTransformation
	"Answer a WindowingTransformation that transforms the coordinate 
	system of the View into that of the display screen. The transformation is 
	computed by composing the View's transformation with all transformations 
	along its superView chain. It is sent by View|displayTransformation when
	the View is unlocked (see View|unlock)."

	self isTopView
		ifTrue: [^transformation]
		ifFalse: [^superView displayTransformation compose: transformation]! !

!View methodsFor: 'private' stamp: ''!
computeInsetDisplayBox
	"Compute the View's inset display box by intersecting the superView's
	inset display box with the View's window transformed to display
	coordinates and then inseting the result by the border width. It is sent by 
	View|insetDisplayBox if the inset display box is nil.

	The insetDisplayBox points are truncated to prevent sending floating point numbers to QuickDraw which will die."

	self isTopView
		ifTrue:
			[^((self displayTransform: self getWindow) insetBy: borderWidth) truncated]
		ifFalse:
			[^(superView insetDisplayBox
				intersect: (self displayTransform: self getWindow)) truncated
						insetBy: borderWidth]! !

!View methodsFor: 'testing' stamp: ''!
containsPoint: aPoint
	"Answer whether aPoint is within the receiver's display box. It is sent to 
	a View's subViews by View|subViewAt: in order to determine which 
	subView contains the cursor point (so that, for example, control can be 
	pass down to that subView's controller)."

	^ self insetDisplayBox containsPoint: aPoint! !

!View methodsFor: 'controller access' stamp: ''!
controller
	"If the receiver's controller is nil (the default case), answer an initialized 
	instance of the receiver's default controller. If the receiver does not 
	allow a controller, answer the symbol #NoControllerAllowed."

	controller == nil ifTrue: [self controller: self defaultController].
	^controller! !

!View methodsFor: 'controller access' stamp: ''!
controller: aController 
	"Set the receiver's controller to aController. #NoControllerAllowed can be 
	specified to indicate that the receiver will not have a controller. The 
	model of aController is set to the receiver's model."

	self model: model controller: aController! !

!View methodsFor: 'deEmphasizing' stamp: ''!
deEmphasize
	"Modify the emphasis (highlighting, special tabs) of the receiver. This 
	includes objects such as labels, lines, and boxes. Typically used so that 
	the receiver is not presented as active. Do this for the receiver and then 
	for each of the receiver's subViews."

	self deEmphasizeView.
	self deEmphasizeSubViews! !

!View methodsFor: 'deEmphasizing' stamp: ''!
deEmphasizeForDebugger
	"Overridden by StandardSystemView. This default behavior does nothing."
! !

!View methodsFor: 'deEmphasizing' stamp: ''!
deEmphasizeSubViews
	"Send the deEmphasize message to each of the receiver's subviews."

	subViews do: [:aSubView | aSubView deEmphasize]! !

!View methodsFor: 'deEmphasizing' stamp: ''!
deEmphasizeView
	"Subclasses should redefine View|deEmphasizeView in order to modify 
	the emphasis (highlighting, special tabs) of particular objects associated 
	with the View such as labels, lines, and boxes."

	^self! !

!View methodsFor: 'controller access' stamp: ''!
defaultController
	"Answer an initialized instance of the receiver's default controller. 
	Subclasses should redefine this message only if the default controller 
	instances need to be initialized in a nonstandard way."

	^self defaultControllerClass new! !

!View methodsFor: 'controller access' stamp: ''!
defaultControllerClass
	"Answer the class of the default controller for the receiver. Subclasses 
	should redefine View|defaultControllerClass if the class of the default 
	controller is not Controller."

	^Controller! !

!View methodsFor: 'window access' stamp: ''!
defaultWindow
	"Build the minimum Rectangle that encloses all the windows of the 
	receiver's subViews. The answer is a Rectangle obtained by expanding 
	this minimal Rectangle by the borderWidth of the receiver. If the 
	receiver has no subViews, then a Rectangle enclosing the entire display 
	screen is answered. It is used internally by View methods if no window 
	has been specified for the View. Specialized subclasses of View should 
	redefine View|defaultWindow to handle the default case for instances 
	that have no subViews."

	| aRectangle |
	subViews isEmpty ifTrue: [^DisplayScreen boundingBox].
	aRectangle := self firstSubView viewport.
	subViews do: [:aView | aRectangle := aRectangle merge: aView viewport].
	^aRectangle expandBy: borderWidth! !

!View methodsFor: 'displaying' stamp: 'hmm 6/27/2000 07:07'!
display
	"Display the receiver's border, display the receiver, then display the 
	subViews of the receiver. Can be sent to the top View of a structured 
	picture in order to display the entire structure, or to any particular View 
	in the structure in order to display that View and its subViews. It is 
	typically sent in response to an update request to a View."

	Display deferUpdatesIn: self displayBox while: [
		self displayBorder.
		self displayView.
		self displaySubViews]! !

!View methodsFor: 'displaying' stamp: ''!
displayBorder
	"Display the receiver's border (using the receiver's borderColor)."

	borderWidth = 0
		ifTrue:
			[insideColor == nil
				ifFalse: 
					[Display fill: self displayBox fillColor: self backgroundColor]]
		ifFalse:
			[Display
				border: self displayBox
				widthRectangle: borderWidth
				rule: Form over
				fillColor: self foregroundColor.
			insideColor == nil ifFalse:
				[Display fill: self insetDisplayBox fillColor: self backgroundColor]]! !

!View methodsFor: 'display box access' stamp: ''!
displayBox
	"Answer the receiver's inset display box (see View|insetDisplayBox) 
	expanded by the borderWidth. The display box represents the region of 
	the display screen in which the receiver (including the border) is 
	displayed. If the receiver is totally clipped by the display screen and its 
	superView, the resulting Rectangle will be invalid."

	^self insetDisplayBox expandBy: borderWidth! !

!View methodsFor: 'displaying' stamp: ''!
displayClippingTo: rect

	| bigRect |
	bigRect := rect insetBy: -1.
	self clippingTo: bigRect do: [Display clippingTo: bigRect do: [self display]]
! !

!View methodsFor: 'displaying' stamp: ''!
displayDeEmphasized
	self display; deEmphasize! !

!View methodsFor: 'displaying' stamp: ''!
displaySubViews
	"Display all the subViews of the receiver."

	subViews do: [:aSubView | aSubView display]! !

!View methodsFor: 'display transformation' stamp: ''!
displayTransform: anObject 
	"Apply the display transformation of the receiver to anObject (see 
	View|displayTransformation) and answer the resulting scaled, translated 
	object. It is normally applied to Rectangles, Points, and other objects with 
	coordinates defined in the View's local coordinate system in order to get 
	a corresponding object in display coordinates."

	^(self displayTransformation applyTo: anObject) rounded! !

!View methodsFor: 'display transformation' stamp: ''!
displayTransformation
	"Answer a WindowingTransformation that is the result of composing all 
	local transformations in the receiver's superView chain with the 
	receiver's own local transformation. The resulting transformation 
	transforms objects in the receiver's coordinate system into objects in the 
	display screen coordinate system."

	displayTransformation == nil
		ifTrue: [displayTransformation := self computeDisplayTransformation].
	^displayTransformation! !

!View methodsFor: 'displaying' stamp: ''!
displayView
	"Subclasses should redefine View|displayView in order to display 
	particular objects associated with the View such as labels, lines, and 
	boxes."

	^self! !

!View methodsFor: 'displaying' stamp: ''!
displayViewDeEmphasized
	self displayView; deEmphasizeView! !

!View methodsFor: 'deEmphasizing' stamp: ''!
emphasize
	"Modify the emphasis (highlighting, special tabs) of the receiver. This 
	includes objects such as labels, lines, and boxes. Typically used so that 
	the receiver is presented as active. Do this for the receiver and then 
	for each of the receiver's subViews."

	self emphasizeView.
	self emphasizeSubViews! !

!View methodsFor: 'deEmphasizing' stamp: ''!
emphasizeSubViews
	"Send the emphasize message to each of the receiver's subviews."

	subViews do: [:aSubView | aSubView emphasize]! !

!View methodsFor: 'deEmphasizing' stamp: ''!
emphasizeView
	"Subclasses should redefine View|emphasizeView in order to modify 
	the emphasis (highlighting, special tabs) of particular objects associated 
	with the View such as labels, lines, and boxes."

	^self! !

!View methodsFor: 'subView access' stamp: ''!
firstSubView
	"Answer the first subView in the receiver's list of subViews if it is not 
	empty, else nil."

	subViews isEmpty
		ifTrue: [^nil]
		ifFalse: [^subViews first]! !

!View methodsFor: 'indicating' stamp: ''!
flash
	"Cause the inset display box (the display box excluding the border, see 
	View|insetDisplayBox) of the receiver to complement twice in succession."

	Display flash: self insetDisplayBox! !

!View methodsFor: 'bordering' stamp: 'sw 11/2/1998 15:35'!
foregroundColor
	borderColor ifNotNil: [^ Color colorFrom: borderColor].
	^ superView == nil
		ifFalse: [superView foregroundColor]
		ifTrue:	[Color black]! !

!View methodsFor: 'bordering' stamp: ''!
foregroundColor: aColor
	borderColor := aColor! !

!View methodsFor: 'private' stamp: ''!
getController
	"Answer the View's controller if one exists. nil indicates that the default
	controller is to be used."

	^controller! !

!View methodsFor: 'private' stamp: ''!
getViewport
	"Answer the Rectangle representing the View's viewport (in the
	coordinate system of the superclass). If no viewport has been specified,
	the View's window transformed into the superView's coordinate system is
	saved and returned. It should be used by methods of View and subclasses
	(instead of directly referring to the viewport) unless it is known that a
	viewport actually exists. It should not be used outside of View or
	subclasses because the viewport is not sharable."

	viewport == nil ifTrue: [viewport := (self transform: self getWindow) truncated].
	^viewport! !

!View methodsFor: 'private' stamp: ''!
getWindow
	"Answer the Rectangle that represents the window of this View. If no
	window has been specified, a default window (see View|defaultWindow)
	is created, saved, and returned. Should be used by methods of View and
	subclasses to access the View window instead of directly accessing the
	field unless it is known that a window actually exists. It is not to be used
	outside of View (or subclasses) because the window is not sharable.
	View|window should be used for outside access to the window."

	window == nil ifTrue: [self setWindow: self defaultWindow].
	^window! !

!View methodsFor: 'miscellaneous' stamp: ''!
gridSpacing
	^ superView gridSpacing! !

!View methodsFor: 'testing' stamp: 'jm 3/17/98 13:35'!
hasUnacceptedEdits
	"Return true if this view has unaccepted edits."

	^ false
! !

!View methodsFor: 'indicating' stamp: ''!
highlight
	"Cause the inset display box (the display box excluding the border, see 
	View|insetDisplayBox) of the receiver to complement."

	Display reverse: self insetDisplayBox! !

!View methodsFor: 'initialize-release' stamp: ''!
initialize
	"Initialize the state of the receiver. Subclasses should include 'super 
	initialize' when redefining this message to insure proper initialization."

	self resetSubViews.
	transformation := WindowingTransformation identity.
	self borderWidth: 0! !

!View methodsFor: 'display box access' stamp: 'acg 2/23/2000 00:08'!
insetDisplayBox
	"Answer the receiver's inset display box. The inset display box is the 
	intersection of the receiver's window, tranformed to display coordinates, 
	and the inset display box of the superView, inset by the border width. 
	The inset display box represents the region of the display screen in 
	which the inside of the receiver (all except the border) is displayed. If 
	the receiver is totally clipped by the display screen and its superView, 
	the resulting Rectangle will be invalid."

	insetDisplayBox ifNil: [insetDisplayBox := self computeInsetDisplayBox].
	^insetDisplayBox! !

!View methodsFor: 'window access' stamp: ''!
insetWindow
	"Answer a Rectangle that is obtained by insetting the receiver's window 
	rectangle by the border width."

	^self getWindow insetBy: borderWidth! !

!View methodsFor: 'bordering' stamp: ''!
insideColor: aColor 
	^ self backgroundColor: aColor! !

!View methodsFor: 'displaying' stamp: ''!
inspectFirstSubView
	subViews notNil ifTrue:
		[subViews size > 0 ifTrue:
			[(subViews at: 1) inspect]]! !

!View methodsFor: 'displaying' stamp: ''!
inspectModel
	model notNil
		ifTrue: [^ model inspect]
		ifFalse: [self flash]! !

!View methodsFor: 'displaying' stamp: ''!
inspectView
	^self inspect! !

!View methodsFor: 'display transformation' stamp: ''!
inverseDisplayTransform: aPoint 
	"Answer a Point that is obtained from the argument, aPoint, by applying 
	to it the inverse of the receiver's display transformation. It is typically 
	used by the Controller of the receiver in order to convert a point in 
	display coordinates, such as the cursor point, to the local coordinate 
	system of the receiver."

	^self displayTransformation applyInverseTo: aPoint! !

!View methodsFor: 'private' stamp: ''!
isCyclic: aView 
	"Answer true if aView is the same as this View or its superView, false 
	otherwise."

	self == aView ifTrue: [^true].
	self isTopView ifTrue: [^false].
	^superView isCyclic: aView! !

!View methodsFor: 'lock access' stamp: ''!
isLocked
	"Answer whether the receiver is locked. A View is 'locked' if its display 
	transformation and inset display box are defined. If these are undefined, 
	the View is 'unlocked'. The display transformation and inset display box 
	become undefined when the transformation of the View (or the 
	transformation of a View in its superView chain) is changed, or when 
	the superView of the View is changed, or any other change to the View 
	that affects the display screen coordinates of the View. The locking and 
	unlocking of a View is handled automatically by the internal methods of 
	the View, but can also be done explicitly if desired (see View|lock, and 
	View|unlock)."

	displayTransformation == nil | (insetDisplayBox == nil)
		ifTrue: [^false]
		ifFalse: [^true]! !

!View methodsFor: 'testing' stamp: ''!
isObscured

	| topController displayRect |
	(topController := self topView controller)
		== ScheduledControllers activeController
			ifTrue: [^false].
	displayRect := self insetDisplayBox.
	ScheduledControllers scheduledControllers do: [:ctrlr |
		ctrlr == topController ifTrue: [^false].
		(displayRect intersects: ctrlr view insetDisplayBox)
			ifTrue: [^true]].
	self error: 'not in ScheduledControllers'.
	^false! !

!View methodsFor: 'superView access' stamp: ''!
isTopView
	"Answer whether the receiver is a top view, that is, if it has no 
	superView."

	^superView == nil! !

!View methodsFor: 'lock access' stamp: ''!
isUnlocked
	"Answer whether the receiver is unlocked. See comment in 
	View|isLocked."

	^displayTransformation == nil & (insetDisplayBox == nil)! !

!View methodsFor: 'subView access' stamp: ''!
lastSubView
	"Answer the last subView in the receiver's list of subViews if it is not 
	empty, else nil."

	subViews isEmpty
		ifTrue: [^nil]
		ifFalse: [^subViews last]! !

!View methodsFor: 'lock access' stamp: ''!
lock
	"'Lock' the receiver and all of its subViews (see View|isLocked). This has 
	the effect of computing and storing the display transformation (see 
	View|displayTransformation) and inset display box (see 
	View|insetDisplayBox) of the receiver and all its subViews. The locking 
	and unlocking of a View is handled automatically by the internal 
	methods of the View, but can also be done explicitly if desired."

	self isLocked ifTrue: [^self].
	displayTransformation := self computeDisplayTransformation.
	insetDisplayBox := self computeInsetDisplayBox.
	subViews do: [:aSubView | aSubView lock]! !

!View methodsFor: 'displaying' stamp: ''!
maximumSize
	"Answer the maximum size of the receiver."

	^ 10000 @ 10000
	! !

!View methodsFor: 'displaying' stamp: ''!
minimumSize
	"Answer the minimum size of the receiver."
	^ 10 @ 10
	! !

!View methodsFor: 'model access' stamp: ''!
model
	"Answer the receiver's model."

	^model! !

!View methodsFor: 'model access' stamp: ''!
model: aModel 
	"Set the receiver's model to aModel. The model of the receiver's controller 
	is also set to aModel."

	self model: aModel controller: controller! !

!View methodsFor: 'controller access' stamp: ''!
model: aModel controller: aController 
	"Set the receiver's model to aModel, add the receiver to aModel's list of 
	dependents, and set the receiver's controller to aController. Subsequent 
	changes to aModel (see Model|change) will result in View|update: 
	messages being sent to the receiver. #NoControllerAllowed for the value 
	of aController indicates that no default controller is available; nil for the 
	value of aController indicates that the default controller is to be used 
	when needed. If aController is neither #NoControllerAllowed nor nil, its 
	view is set to the receiver and its model is set to aModel."

	model ~~ nil & (model ~~ aModel)
		ifTrue: [model removeDependent: self].
	aModel ~~ nil & (aModel ~~ model)
		ifTrue: [aModel addDependent: self].
	model := aModel.
	aController ~~ nil
		ifTrue: 
			[aController view: self.
			aController model: aModel].
	controller := aController! !

!View methodsFor: 'miscellaneous' stamp: ''!
nestedViewport

	"The viewport size used to control scaling of nested user views."

	^ (0@0 extent: self viewport extent)
			insetBy: 16 @ 16! !

!View methodsFor: 'miscellaneous' stamp: ''!
printViewSpecOn: strm nested: level
	"Print window and viewport specs
	of this and all nested views."
	strm crtab: level; nextPutAll: self class name.
	strm crtab: level; nextPutAll: 'window: '; print: self window.
	strm crtab: level; nextPutAll: 'viewport: '; print: self viewport.
	strm crtab: level; nextPutAll: 'displayBox: '; print: self displayBox.
	strm crtab: level; nextPutAll: 'border: '; print: self borderWidth.
	subViews do: [:v | v printViewSpecOn: strm nested: level+1]! !

!View methodsFor: 'initialize-release' stamp: ''!
release
	"Remove the receiver from its model's list of dependents (if the model
	exists), and release all of its subViews. It is used to break possible cycles
	in the receiver and should be sent when the receiver is no longer needed.
	Subclasses should include 'super release.' when redefining release."

	model removeDependent: self.
	model := nil.
	controller release.
	controller := nil.
	subViews ~~ nil ifTrue: [subViews do: [:aView | aView release]].
	subViews := nil.
	superView := nil! !

!View methodsFor: 'subView removing' stamp: ''!
releaseSubView: aView 
	"Delete aView from the receiver's list of subViews and send it the 
	message 'release' (so that it can break up cycles with subViews, etc.)."

	self removeSubView: aView.
	aView release! !

!View methodsFor: 'subView removing' stamp: ''!
releaseSubViews
	"Release (see View|releaseSubView:) all subViews in the receiver's list of 
	subViews."

	subViews do: [:aView | aView release].
	self resetSubViews! !

!View methodsFor: 'subView removing' stamp: ''!
removeFromSuperView
	"Delete the receiver from its superView's collection of subViews."

	superView ~= nil ifTrue: [superView removeSubView: self]! !

!View methodsFor: 'subView removing' stamp: ''!
removeSubView: aView 
	"Delete aView from the receiver's list of subViews. If the list of subViews 
	does not contain aView, create an error notification."

	subViews remove: aView.
	aView superView: nil.
	aView unlock! !

!View methodsFor: 'subView removing' stamp: ''!
removeSubViews
	"Delete all the receiver's subViews."

	subViews do: 
		[:aView | 
		aView superView: nil.
		aView unlock].
	self resetSubViews! !

!View methodsFor: 'subView access' stamp: ''!
resetSubViews
	"Set the list of subviews to an empty collection."
	
	subViews := OrderedCollection new! !

!View methodsFor: 'transforming' stamp: ''!
scale: aScale translation: aTranslation 
	"The x component of aScale (a Point) specifies the scale (translation) in 
	the x direction; the y component specifies the scale (translation) in the y 
	direction. aScale can optionally be an instance of Integer or Float in 
	order to specify uniform scaling in both directions. Create a new local 
	transformation for the receiver with a scale factor of aScale and a 
	translation offset of aTranslation. When the transformation is applied (see 
	View|transform:), the scale is applied first, followed by the translation. It 
	is typically used when building a superView from its subViews in order 
	to line up the viewports of the subViews in the desired way. If no 
	scaling is required between subView and superView, then 
	View|align:with: is often more convenient to use."

	self setTransformation:
		(WindowingTransformation scale: aScale translation: aTranslation)! !

!View methodsFor: 'transforming' stamp: ''!
scaleBy: aScale 
	"The x component of aScale (a Point) specifies the scale in the x 
	direction; the y component specifies the scale in the y direction. aScale 
	can, optionally, be an instance of Integer or Float in order to specify 
	uniform scaling in both directions. Scales the View by aScale. The scale 
	is concatenated with the current transformation of the receiver and is 
	applied when View|transform is sent. This happens automatically in the 
	process of displaying the receiver, for example."

	self setTransformation: (transformation scaleBy: aScale)! !

!View methodsFor: 'scrolling' stamp: ''!
scrollBy: aPoint 
	"The x component of aPoint specifies the amount of scrolling in the x 
	direction; the y component specifies the amount of scrolling in the y 
	direction. The amounts are specified in the receiver's local coordinate 
	system. Scroll the receiver up or down, left or right. The window of the 
	receiver is kept stationary and the subViews and other objects in the 
	receiver are translated relative to it. Scrolling doesn't change the 
	insetDisplayBox or the viewport since the change in the transformation 
	is canceled by the change in the window. In other words, all display 
	objects in the view, except the window, are translated by the scrolling 
	operation.
	Note: subclasses may override to return false if no scrolling takes place."

	| aRectangle |
	aRectangle := insetDisplayBox.
	transformation := transformation scrollBy: aPoint.
	window := self getWindow translateBy: aPoint x negated @ aPoint y negated.
	self unlock.
	insetDisplayBox := aRectangle.
	^ true! !

!View methodsFor: 'morphic compatibility' stamp: 'mdr 1/24/2000 17:27'!
setBalloonText: aString
	"Unfortunately we just ignore this help text because we are not morphic"
! !

!View methodsFor: 'initialize-release' stamp: ''!
setDefaultBackgroundColor
	"Obtain the background color from the receiver's model, unless the #uniformWindowColors preference is set to true, in which case obtain it from generic Object; and install it as the receiver's background color.  5/1/96 sw"

	| colorToUse |
	colorToUse := Preferences uniformWindowColors
		ifTrue:
			[Object new defaultBackgroundColor]
		ifFalse:
			[model defaultBackgroundColor].
	self backgroundColor: colorToUse! !

!View methodsFor: 'private' stamp: ''!
setTransformation: aTransformation 
	"Set the View's local transformation to aTransformation, unlock the View 
	(see View|unlock), and set the viewport to undefined (this forces it to be 
	recomputed when needed). Should be used instead of setting the 
	transformation directly."

	transformation := aTransformation.
	self unlock.
	viewport := nil! !

!View methodsFor: 'private' stamp: ''!
setWindow: aWindow 
	"Set the View's window to aWindow and unlock the View (see
	View|unlock). View|setWindow should be used by methods of View and
	subclasses to set the View window (rather than directly setting the
	instance variable) to insure that the View is unlocked."

	window := aWindow.
	viewport := nil.
	self unlock! !

!View methodsFor: 'miscellaneous' stamp: 'JMM 10/21/2003 18:12'!
stretchFrame: newFrameBlock startingWith: startFrame 
	"Track the outline of a newFrame as long as mouse drags it.
	Maintain max and min constraints throughout the drag"
	| frame newFrame click delay |
	delay := Delay forMilliseconds: 10.
	frame := startFrame origin extent: ((startFrame extent min: self maximumSize)
											max: self minimumSize).
	Display border: frame width: 2 rule: Form reverse fillColor: Color gray.
	click := false.
	[click and: [Sensor noButtonPressed]] whileFalse: 
		[delay wait.
		click := click | Sensor anyButtonPressed.
		newFrame := newFrameBlock value: frame.
		newFrame := newFrame topLeft extent: ((newFrame extent min: self maximumSize)
											max: self minimumSize).
		newFrame = frame ifFalse:
			[Display border: frame width: 2 rule: Form reverse fillColor: Color gray.
			Display border: newFrame width: 2 rule: Form reverse fillColor: Color gray.
			frame := newFrame]].
	Display border: frame width: 2 rule: Form reverse fillColor: Color gray.
	^ frame! !

!View methodsFor: 'subView access' stamp: ''!
subViewContaining: aPoint 
	"Answer the first subView that contains aPoint within its window and 
	answer nil, otherwise. It is typically sent from a Controller in order to 
	determine where to pass control (usually to the Controller of the View 
	returned by View|subViewContaining:)."

	subViews reverseDo: 
		[:aSubView | 
		(aSubView displayBox containsPoint: aPoint) ifTrue: [^aSubView]].
	^nil! !

!View methodsFor: 'subView access' stamp: ''!
subViewSatisfying: aBlock
	"Return the first subview that satisfies aBlock, or nil if none does.  1/31/96 sw"

	^ subViews detect: [:aView | aBlock value: aView] ifNone: [nil]! !

!View methodsFor: 'basic control sequence' stamp: ''!
subViewWantingControl
	"Answer the first subView that has a controller that now wants control."

	subViews reverseDo: 
		[:aSubView | aSubView controller isControlWanted ifTrue: [^aSubView]].
	^nil! !

!View methodsFor: 'subView access' stamp: ''!
subViews
	"Answer the receiver's collection of subViews."

	^subViews! !

!View methodsFor: 'superView access' stamp: ''!
superView
	"Answer the superView of the receiver."

	^superView! !

!View methodsFor: 'private' stamp: ''!
superView: aView 
	"Set the View's superView to aView and unlock the View (see
	View|unlock). It is sent by View|addSubView: in order to properly set all
	the links."

	superView := aView.
	self unlock! !

!View methodsFor: 'subView access' stamp: ''!
textEditorView
	"Return the first view in the receiver whose controller is a ParagraphEdior, or nil if none.  1/31/96 sw"

	(controller isKindOf: ParagraphEditor) ifTrue: [^ self].
	^ self subViewSatisfying:
		[:v | v textEditorView ~~ nil]! !

!View methodsFor: 'superView access' stamp: ''!
topView
	"Answer the root of the tree of Views in which the receiver is a node. 
	The root of the tree is found by going up the superView path until 
	reaching a View whose superView is nil."

	superView == nil
		ifTrue: [^self]
		ifFalse: [^superView topView]! !

!View methodsFor: 'transforming' stamp: ''!
transform: anObject 
	"Apply the local transformation of the receiver to anObject and answer 
	the resulting transformation. It is used to get the superView coordinates 
	of an object. For example, the viewport is equal to the window 
	transformed."

	^transformation applyTo: anObject! !

!View methodsFor: 'transforming' stamp: ''!
transformation
	"Answer a copy of the receiver's local transformation."

	^transformation copy! !

!View methodsFor: 'transforming' stamp: ''!
transformation: aTransformation 
	"Set the receiver's local transformation to a copy of aTransformation, 
	unlock the receiver (see View|unlock) and set the viewport to undefined 
	(this forces it to be recomputed when needed)."

	self setTransformation: aTransformation copy! !

!View methodsFor: 'transforming' stamp: ''!
translateBy: aPoint 
	"Translate the receiver by aPoint. The translation is concatenated with 
	the current transformation of the receiver and is applied when 
	View|transform is sent. This happens automatically in the process of 
	displaying the receiver."

	self setTransformation: (transformation translateBy: aPoint)! !

!View methodsFor: 'lock access' stamp: ''!
unlock
	"Unlock the receiver and all of its subViews (see View|isUnlocked). This 
	has the effect of forcing the display transformation (see 
	View|displayTransformation) and inset display box (see 
	View|insetDisplayBox) of the receiver and all its subViews to be 
	recomputed the next time they are referenced. The locking and 
	unlocking of a View is handled automatically by the internal methods of 
	the View, but can also be done explicitly if desired."

	self isUnlocked ifTrue: [^self].
	displayTransformation := nil.
	insetDisplayBox := nil.
	subViews do: [:aSubView | aSubView unlock]! !

!View methodsFor: 'updating' stamp: ''!
update
	"Normally sent by the receiver's model in order to notify the receiver of 
	a change in the model's state. Subclasses implement this message to do 
	particular update actions. A typical action that might be required is to 
	redisplay the receiver."

	self update: self! !

!View methodsFor: 'updating' stamp: ''!
update: aParameter 
	"Normally sent by the receiver's model in order to notify the receiver of 
	a change in the model's state. Subclasses implement this message to do 
	particular update actions. A typical action that might be required is to 
	redisplay the receiver."

	^self! !

!View methodsFor: 'viewport access' stamp: ''!
viewport
	"Answer a copy of the receiver's viewport."

	^self getViewport copy! !

!View methodsFor: 'window access' stamp: ''!
window
	"Answer a copy of the receiver's window."

	^self getWindow copy! !

!View methodsFor: 'window access' stamp: ''!
window: aWindow 
	"Set the receiver's window to a copy of aWindow."

	self setWindow: aWindow copy! !

!View methodsFor: 'transforming' stamp: ''!
window: aWindow viewport: aViewport 
	"Set the receiver's window to aWindow, set its viewport to aViewport, and 
	create a new local transformation for the receiver based on aWindow and 
	aViewport. The receiver is scaled and translated so that aWindow, when 
	transformed, coincides with aViewport. It is used to position a subView's 
	window within some specific region of its superView's area. For example, 
	'subView window: aRectangle1 viewport: aRectangle2' sets subView's 
	window to aRectangle1, its viewport to aRectangle2, and its local 
	transformation to one that transforms aRectangle1 to aRectange2."

	self window: aWindow.
	self setTransformation:
		(WindowingTransformation window: aWindow viewport: aViewport).
	self getViewport! !

Object subclass: #WindowingTransformation
	instanceVariableNames: 'scale translation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Framework'!

!WindowingTransformation commentStamp: '<historical>' prior: 0!
My instances are used to transform objects from a source coordinate system to a destination coordinate system. Each instance contains a scale and a translation which can be applied to objects that respond to scaleBy: and translateBy:. It can be created with a default identity scale and translation, or with a specified scale and translation, or with a scale and translation computed from a window (a Rectangle in the source coordinate system) and a viewport (a Rectangle in the destination coordinate system). In applying a WindowingTransformation to an object, the object is first scaled (around the origin of the source coordinate system) and then translated. WindowingTransformations can be composed to form a single compound transformation.!

!WindowingTransformation classSide methodsFor: 'instance creation' stamp: ''!
identity
	"Answer an instance of me with no scaling (nil) and no translation 
	(0@0)."

	^self new setScale: nil translation: 0 @ 0! !

!WindowingTransformation classSide methodsFor: 'instance creation' stamp: ''!
scale: aScale translation: aTranslation 
	"Answer an instance of me with a scale factor of aScale and a translation 
	offset of aTranslation. When the transformation is applied (see 
	WindowingTransformation|apply:), the scale is applied first, followed by 
	the translation."

	^self new setScale: aScale translation: aTranslation! !

!WindowingTransformation classSide methodsFor: 'instance creation' stamp: ''!
window: aWindow viewport: aViewport 
	"Answer an instance of me with a scale and translation based on 
	aWindow and aViewport. The scale and translation are computed such 
	that aWindow, when transformed, coincides with aViewport."

	| scale translation |
	aViewport width = aWindow width & (aViewport height = aWindow height)
		ifTrue:
			[scale := nil]
		ifFalse:
			[scale := aViewport width asFloat / aWindow width asFloat
						@ (aViewport height asFloat / aWindow height asFloat)].
	scale == nil
		ifTrue: [translation := aViewport left - aWindow left
								@ (aViewport top - aWindow top)]
		ifFalse: [translation := aViewport left - (scale x * aWindow left)
								@ (aViewport top - (scale y * aWindow top))].
	^self new setScale: scale translation: translation! !

!WindowingTransformation methodsFor: 'transforming' stamp: ''!
align: point1 with: point2 
	"Answer a WindowingTransformation with the same scale as the receiver 
	and with a translation of (aPoint2 - aPoint1). It is normally used when 
	the source and destination coordinate systems are scaled the same (that 
	is, there is no scaling between them), and is then a convenient way of 
	specifying a translation, given two points that are intended to coincide."

	^self translateBy: point2 - point1! !

!WindowingTransformation methodsFor: 'applying transform' stamp: ''!
applyInverseTo: anObject 
	"Apply the inverse of the receiver to anObject and answer the result. 
	Used to map some object in destination coordinates to one in source 
	coordinates."

	| transformedObject |
	transformedObject := anObject translateBy: translation x negated @ translation y negated.
	scale == nil
		ifFalse: [transformedObject := transformedObject scaleBy: 1.0 / scale x @ (1.0 / scale y)].
	^transformedObject! !

!WindowingTransformation methodsFor: 'applying transform' stamp: ''!
applyTo: anObject 
	"Apply the receiver to anObject and answer the result. Used to map some 
	object in source coordinates to one in destination coordinates."

	| transformedObject |
	scale == nil
		ifTrue: [transformedObject := anObject]
		ifFalse: [transformedObject := anObject scaleBy: scale].
	transformedObject := transformedObject translateBy: translation.
	^transformedObject! !

!WindowingTransformation methodsFor: 'private' stamp: ''!
checkScale: aScale
	"Convert aScale to the internal format of a floating-point Point."

 	| checkedScale |
	checkedScale := aScale asPoint.
	^checkedScale x asFloat @ checkedScale y asFloat! !

!WindowingTransformation methodsFor: 'applying transform' stamp: ''!
compose: aTransformation 
	"Answer a WindowingTransformation that is the composition of the 
	receiver and aTransformation. The effect of applying the resulting 
	WindowingTransformation to an object is the same as that of first 
	applying aTransformation to the object and then applying the receiver to 
	its result."

	| aTransformationScale newScale newTranslation |
	aTransformationScale := aTransformation scale.
	scale == nil
		ifTrue: 
			[aTransformation noScale
				ifTrue: [newScale := nil]
				ifFalse: [newScale := aTransformationScale].
			newTranslation := translation + aTransformation translation]
		ifFalse: 
			[aTransformation noScale
				ifTrue: [newScale := scale]
				ifFalse: [newScale := scale * aTransformationScale].
			newTranslation := translation + (scale * aTransformation translation)].
	^WindowingTransformation scale: newScale translation: newTranslation! !

!WindowingTransformation methodsFor: 'transforming' stamp: ''!
noScale
	"Answer true if the identity scale is in effect; answer false, otherwise."

	^scale == nil! !

!WindowingTransformation methodsFor: 'printing' stamp: ''!
printOn: aStream 
	"Refer to the comment in Object|printOn:."

	aStream nextPutAll: self class name, ' scale: ';
		print: scale; nextPutAll: ' translation: ';
		print: translation! !

!WindowingTransformation methodsFor: 'transforming' stamp: ''!
scale
	"Answer a copy of the point that represents the current scale of the 
	receiver."

	scale == nil
		ifTrue: [^1.0 @ 1.0]
		ifFalse: [^scale copy]! !

!WindowingTransformation methodsFor: 'transforming' stamp: ''!
scaleBy: aScale 
	"Answer a WindowingTransformation with the scale and translation of 
	the receiver both scaled by aScale."

	| checkedScale newScale newTranslation |
	aScale == nil
		ifTrue: 
			[newScale := scale.
			newTranslation := translation]
		ifFalse: 
			[checkedScale := self checkScale: aScale.
			scale == nil
				ifTrue: [newScale := checkedScale]
				ifFalse: [newScale := scale * checkedScale].
			newTranslation := checkedScale * translation].
	^WindowingTransformation scale: newScale translation: newTranslation! !

!WindowingTransformation methodsFor: 'scrolling' stamp: ''!
scrollBy: aPoint 
	"Answer a WindowingTransformation with the same scale as the receiver 
	and with a translation of the current translation plus aPoint scaled by 
	the current scale. It is used when the translation is known in source 
	coordinates, rather than scaled source coordinates (see 
	WindowingTransformation|translateBy:). An example is that of scrolling 
	objects with respect to a stationary window in the source coordinate 
	system. If no scaling is in effect (scale = nil), then 
	WindowingTransformation|translateBy: and 
	WindowingTransformation|scrollBy: are equivalent."

	| newTranslation |
	scale == nil
		ifTrue: [newTranslation := aPoint]
		ifFalse: [newTranslation := scale * aPoint].
	^self translateBy: newTranslation! !

!WindowingTransformation methodsFor: 'private' stamp: ''!
setScale: aScale translation: aTranslation 
	"Sets the scale to aScale and the translation to aTranslation."

	scale := aScale.
	translation := aTranslation! !

!WindowingTransformation methodsFor: 'transforming' stamp: ''!
translateBy: aPoint 
	"Answer a WindowingTransformation with the same scale as the receiver 
	and with a translation of the current translation plus aPoint. It is used 
	when the translation is known in scaled source coordinates, rather than 
	source coordinates (see WindowingTransformation|scrollBy:). If no scaling 
	is in effect (scale = nil), then WindowingTransformation|translateBy: and 
	WindowingTransformation|scrollBy: are equivalent."

	^WindowingTransformation scale: scale translation: translation + aPoint! !

!WindowingTransformation methodsFor: 'transforming' stamp: ''!
translation
	"Answer a copy of the receiver's translation."

	^translation copy! !

ParagraphEditor subclass: #StringHolderController
	instanceVariableNames: ''
	classVariableNames: 'CodeYellowButtonMenu CodeYellowButtonMessages'
	poolDictionaries: ''
	category: 'ST80-Support'!

!StringHolderController commentStamp: '<historical>' prior: 0!
I represent a ParagraphEditor for a single paragraph of text, omitting alignment commands. I provide items in the yellow button menu so that the text selection can be evaluated and so that the contents of the model can be stored or restored.
	doIt	evaluate the text selection as an expression
	printIt	same as doIt but insert a description of the result after the selection
	accept	store the contents of the StringHolder into the model
	cancel	store the contents of the model into the StringHolder!

StringHolderController subclass: #FillInTheBlankController
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Support'!

!FillInTheBlankController commentStamp: '<historical>' prior: 0!
I am the controller for a FillInTheBlankView. Based on a flag in the view, I can either accept the input string when a carriage return is typed, or I can allow multiple lines of input that is accepted by either typing enter or by invoking the 'accept' command.
!

!FillInTheBlankController methodsFor: 'other' stamp: 'jm 5/6/1998 15:13'!
accept

	super accept.
	model done: true.
! !

!FillInTheBlankController methodsFor: 'other' stamp: 'sw 1/31/2000 14:47'!
cancel

	model setResponseForCancel.
	super cancel.
	model done: true.
! !

!FillInTheBlankController methodsFor: 'basic control sequence' stamp: 'th 9/17/2002 16:46'!
controlInitialize

	model acceptOnCR ifFalse: [^ super controlInitialize].
	self setMark: self markBlock stringIndex.
	self setPoint: self pointBlock stringIndex.
	self initializeSelection.
	beginTypeInBlock := nil.
! !

!FillInTheBlankController methodsFor: 'basic control sequence' stamp: 'jm 5/6/1998 15:11'!
controlTerminate

	| topController |
	super controlTerminate.
	topController := view topView controller.
	topController ifNotNil: [topController close].
! !

!FillInTheBlankController methodsFor: 'other' stamp: 'jm 4/28/1998 06:25'!
dispatchOnCharacter: char with: typeAheadStream
	"Accept the current input if the user hits the carriage return or the enter key."

	(model acceptOnCR and:
	 [(char = Character cr) | (char = Character enter)])
		ifTrue: [
			sensor keyboard.  "absorb the character"
			self accept.
			^ true]
		ifFalse: [
			^ super dispatchOnCharacter: char with: typeAheadStream].
! !

!FillInTheBlankController methodsFor: 'control defaults' stamp: 'sma 3/11/2000 14:45'!
isControlActive
	^ self isControlWanted! !

!FillInTheBlankController methodsFor: 'control defaults' stamp: 'sma 3/11/2000 14:45'!
isControlWanted
	^ model done not! !

!FillInTheBlankController methodsFor: 'other' stamp: 'jm 4/28/1998 08:01'!
processYellowButton
	"Suppress yellow-button menu if acceptOnCR is true."

	model acceptOnCR ifFalse: [^ super processYellowButton].
! !

StringHolderController subclass: #PluggableTextController
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Pluggable Views'!

!PluggableTextController methodsFor: 'as yet unclassified' stamp: 'dgd 9/21/2003 17:47'!
accept 
	view hasUnacceptedEdits ifFalse: [^ view flash].
	view hasEditingConflicts ifTrue:
		[(self confirm: 
'Caution!! This method may have been
changed elsewhere since you started
editing it here.  Accept anyway?' translated) ifFalse: [^ self flash]].

	(view setText: paragraph text from: self) ifTrue:
		[initialText := paragraph text copy.
		view ifNotNil: [view hasUnacceptedEdits: false]]    .
! !

!PluggableTextController methodsFor: 'transcript' stamp: 'di 6/3/1998 20:46'!
appendEntry
	"Append the text in the model's writeStream to the editable text. "
	self deselect.
	paragraph text size > model characterLimit ifTrue:
		["Knock off first half of text"
		self selectInvisiblyFrom: 1 to: paragraph text size // 2.
		self replaceSelectionWith: Text new].
	self selectInvisiblyFrom: paragraph text size + 1 to: paragraph text size.
	self replaceSelectionWith: model contents asText.
	self selectInvisiblyFrom: paragraph text size + 1 to: paragraph text size! !

!PluggableTextController methodsFor: 'transcript' stamp: 'sma 3/15/2000 21:39'!
bsText
	self changeText: (self text copyFrom: 1 to: (self text size - 1 max: 0))! !

!PluggableTextController methodsFor: 'transcript' stamp: 'di 5/13/1998 14:16'!
changeText: aText
	"The paragraph to be edited is changed to aText."
	paragraph text: aText.
	self resetState.
	self selectInvisiblyFrom: paragraph text size + 1 to: paragraph text size.
	self selectAndScroll.
	self deselect! !

!PluggableTextController methodsFor: 'transcript' stamp: 'di 6/3/1998 20:42'!
doOccluded: actionBlock
	| paneRect rectSet bottomStrip |
	paneRect := paragraph clippingRectangle.
	paragraph withClippingRectangle: (paneRect withHeight: 0)
		do: [actionBlock value.
			self scrollIn: paneRect].
	view topView isCollapsed ifTrue: [^ self].
	rectSet := self visibleAreas.
	bottomStrip := paneRect withTop: paragraph compositionRectangle bottom + 1.
	rectSet do:
		[:rect |
		(bottomStrip intersects: rect) ifTrue:
			["The subsequent displayOn should clear this strip but it doesnt"
			Display fill: (bottomStrip intersect: rect)
					fillColor: paragraph backgroundColor].
		paragraph withClippingRectangle: rect
				do: [paragraph displayOn: Display]]! !

!PluggableTextController methodsFor: 'transcript' stamp: 'th 9/20/2002 11:26'!
scrollIn: scrollRect
	"Altered from selectAndScroll so can use with null clipRect"
	"Scroll until the selection is in the view and then highlight it."
	| deltaY |
	deltaY := self stopBlock top - scrollRect top.
	deltaY >= 0 
		ifTrue: [deltaY := self stopBlock bottom - scrollRect bottom max: 0].
						"check if stopIndex below bottom of scrollRect"
	deltaY ~= 0 
		ifTrue: [self scrollBy: (deltaY abs + paragraph lineGrid - 1) * deltaY sign]! !

!PluggableTextController methodsFor: 'accessing-selection' stamp: 'th 9/19/2002 18:37'!
selectForTopFrom: start to: stop
	"Deselect, then select the specified characters inclusive.
	 Be sure the selection is in view."

	self selectFrom: start to: stop scroll: #selectAndScrollToTop! !

!PluggableTextController methodsFor: 'accessing-selection' stamp: 'th 9/19/2002 18:37'!
selectFrom: start to: stop
	"Deselect, then select the specified characters inclusive.
	 Be sure the selection is in view."

	self selectFrom: start to: stop scroll: #selectAndScroll! !

!PluggableTextController methodsFor: 'accessing-selection' stamp: 'th 9/19/2002 18:35'!
selectFrom: start to: stop scroll: scrollCommand
	"Deselect, then select the specified characters inclusive.
	 Be sure the selection is in view."

	(start = self startIndex and: [stop + 1 = self stopIndex]) ifFalse:
		[view superView ifNotNil: [self deselect].
		self selectInvisiblyFrom: start to: stop].
	view superView ifNotNil: [self perform: scrollCommand]! !

!PluggableTextController methodsFor: 'as yet unclassified' stamp: 'tk 3/31/98 20:49'!
userHasEdited
	"Note that the user has edited my text."

	view hasUnacceptedEdits: true! !

!PluggableTextController methodsFor: 'as yet unclassified' stamp: 'tk 3/31/98 20:49'!
userHasNotEdited
	"Note that my text is free of user edits."

	view hasUnacceptedEdits: false! !

!PluggableTextController methodsFor: 'transcript' stamp: 'di 5/7/1998 22:23'!
visibleAreas
	"Transcript dependents last controller visibleAreas"
	| visibleAreas rect remnants myTopController |
	myTopController := self view topView controller.
	visibleAreas := Array with: view insetDisplayBox.
	myTopController view uncacheBits.
	ScheduledControllers scheduledWindowControllers do:
		[:c | c == myTopController ifTrue: [^ visibleAreas].
		rect := c view windowBox.
		remnants := OrderedCollection new.
		visibleAreas do: [:a | remnants addAll: (a areasOutside: rect)].
		visibleAreas := remnants].
	^ visibleAreas! !

PluggableTextController subclass: #ReadOnlyTextController
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ST80-Pluggable Views'!

!ReadOnlyTextController methodsFor: 'as yet unclassified' stamp: 'di 5/7/1998 13:41'!
accept 
	"Overridden to allow accept of clean text"
	(view setText: paragraph text from: self) ifTrue:
		[initialText := paragraph text copy.
		view ifNotNil: [view hasUnacceptedEdits: false]].
! !

!ReadOnlyTextController methodsFor: 'as yet unclassified' stamp: 'di 5/7/1998 14:02'!
userHasEdited
	"Ignore this -- I stay clean"! !

!ReadOnlyTextController methodsFor: 'as yet unclassified' stamp: 'di 5/7/1998 13:45'!
zapSelectionWith: aText
	view flash  "no edits allowed"! !

!StringHolderController methodsFor: 'menu messages' stamp: 'jm 3/18/98 20:53'!
accept 
	"Refer to the comment in ParagraphEditor|accept."

	super accept.
	model contents: paragraph string.
	self userHasNotEdited.
! !

!StringHolderController methodsFor: 'compiler access' stamp: ''!
bindingOf: aString
	^model bindingOf: aString! !

!StringHolderController methodsFor: 'menu messages' stamp: 'jm 3/18/98 20:54'!
cancel 
	"Refer to the comment in ParagraphEditor|cancel."

	super cancel.
	self userHasNotEdited.
! !

!StringHolderController methodsFor: 'accessing' stamp: 'di 6/21/2001 10:32'!
changeText: aText
	"The paragraph to be edited is changed to aText."
	paragraph text: aText.
	self resetState.
	self selectInvisiblyFrom: paragraph text size + 1 to: paragraph text size.
	self selectAndScroll.
	self deselect! !

!StringHolderController methodsFor: 'private' stamp: 'jm 3/18/98 20:43'!
closeTypeIn
	"Note edit if something actually was typed."

	beginTypeInBlock ~~ nil ifTrue: [self userHasEdited].
	super closeTypeIn.
! !

!StringHolderController methodsFor: 'edit flag' stamp: 'di 10/9/1998 15:41'!
hasUnacceptedEdits: aBoolean
	^ view hasUnacceptedEdits: aBoolean! !

!StringHolderController methodsFor: 'accessing' stamp: ''!
model: aModel

	super model: aModel.
	view displayContents == nil
		ifFalse: [self changeParagraph: view displayContents]! !

!StringHolderController methodsFor: 'menu messages' stamp: 'tk 4/13/1998 23:14'!
performMenuMessage: aSelector
	"Intercept #again so the model does not get locked by keying the search text."

	| hadEdits |
	hadEdits := view canDiscardEdits not.
	super performMenuMessage: aSelector.
	(hadEdits not and:
	 [aSelector == #again and:
	 [(UndoMessage sends: #undoAgain:andReselect:typedKey:) and:
	 [UndoMessage arguments at: 3]]])
		ifTrue: [self userHasNotEdited].
! !

!StringHolderController methodsFor: 'edit flag' stamp: 'tk 4/13/1998 23:09'!
userHasEdited
	"Note that the user has edited my text."

	view hasUnacceptedEdits: true
! !

!StringHolderController methodsFor: 'edit flag' stamp: 'tk 4/13/1998 23:08'!
userHasNotEdited
	"Note that my text is free of user edits."

	model clearUserEditFlag
! !

!StringHolderController methodsFor: 'private' stamp: 'jm 3/18/98 20:45'!
zapSelectionWith: aText
	"Note edit except during typeIn, which notes edits at close."

	super zapSelectionWith: aText.
	beginTypeInBlock == nil ifTrue: [self userHasEdited].
! !
StringHolder initialize!
BitEditor initialize!
Controller initialize!
FormEditor initialize!
ListParagraph initialize!
PopUpMenu initialize!
StandardSystemController initialize!
StandardSystemView initialize!
